!{\src2tex{textfont=tt}}
!!****f* ABINIT/gaussian_quadratures
!! NAME
!! gaussian_quadratures
!!
!! FUNCTION
!!
!! COPYRIGHT
!!  Copyright (C) 2007 ABINIT group (MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  func= user defined real double precision function
!!  method= string of characters defining the quadrature method (see below)
!!  accuracy_in= accuracy required in terms of !!! 
!!  xmin,xmax= range of integration
!!  nx_in
!!
!! OUTPUT
!!  
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!
!! CHILDREN
!!      coeffs_gausslegint,leave_new,memerr,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine gaussian_quadratures(func,xmin,xmax,nx_in,accuracy_in,method,quad)

 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nx_in
 real(dp),external :: func
 real(dp),intent(in) :: accuracy_in,xmax,xmin
 real(dp),intent(out) :: quad
 character(len=*),intent(in) :: method

!Local variables-------------------------------
!scalars
 integer,parameter :: NMAX=10
 integer :: istat,itrial,ix,n,nx
 real(dp) :: accuracy,old_quad
 logical :: ok=.false.
 character(len=500) :: message
!arrays
 real(dp),allocatable :: wx(:),xx(:)

! *************************************************************************

 nx=nx_in
 if (nx <= 0 ) nx=50

 accuracy=accuracy_in
 if (accuracy <=0) accuracy= 1.e-2

 write(*,*)'accuracy= ',accuracy
 quad=zero
 
 select case(trim(method)) 

  case ('gauss-legendre')

   do itrial=1,NMAX

    allocate(wx(nx),xx(nx),stat=istat)
    if(istat/=0) call memerr('coeffs_gausslegint','wx/xx',2*nx,'dp')

    call coeffs_gausslegint(xmin,xmax,xx,wx,nx)
    quad=zero
    do ix=1,nx
     quad=quad+wx(ix)*func(xx(ix))
    end do 
    write(*,*) itrial,quad

    nx=nx*2
    deallocate(wx,xx)
 
    if ( (abs(quad-old_quad) <= accuracy*abs(old_quad)) .and. (itrial > 1) ) then 
     ok=.true.
     exit
    end if
    old_quad=quad

   end do

  case default 

   write(message,'(4a)')ch10,&
&   ' gaussian_quadratures : BUG -',ch10,&
&   method,' not implemented',ch10
   call wrtout(6,message,'COLL')
   call leave_new('COLL')

 end select

 if (ok) then 
  write(message,'(4a,i4,a,f8.6,3a,es16.8,a,es16.8,a)')ch10,&
&  ' gaussian_quadratures : ',ch10,&
&  '  converged at iteration = ',itrial,' with accuracy <= ',100*accuracy,' %',ch10,&
&  '  final value            = ',quad,' +- ',abs(quad-old_quad),ch10
  call wrtout(6,message,'COLL')
 else 
  write(message,'(4a,i4,3a,es16.8,a,es16.8)')ch10,   &
&  ' gaussian_quadratures : ERROR- ',ch10,           &
&  '  not converged after ',NMAX,' iterations',ch10, &
&  '  last variation ',abs(quad-old_quad),' is larger than accuracy required ',accuracy*abs(old_quad)
  call wrtout(6,message,'COLL')
  call leave_new('COLL')
 end if 

end subroutine gaussian_quadratures
!!***
