!{\src2tex{textfont=tt}}
!!****f* ABINIT/strainsym
!! NAME
!! strainsym
!!
!! FUNCTION
!! For given order of point group, symmetrizes the strain tensor,
!! then produce primitive vectors based on the symmetrized strain.
!!
!! COPYRIGHT
!! Copyright (C) 2007 ABINIT group (XG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!! nsym=order of group.
!! rprimd(3,3)= primitive vectors, to be symmetrized
!! rprimd0(3,3)= reference primitive vectors, already symmetrized
!! symrel(3,3,nsym)=symmetry operators in terms of action on primitive translations
!!
!! OUTPUT
!! rprimd_symm(3,3)= symmetrized primitive vectors
!!
!! SIDE EFFECTS
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine strainsym(nsym,rprimd0,rprimd,rprimd_symm,symrel)

 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_11util
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nsym
!arrays
 integer,intent(in) :: symrel(3,3,nsym)
 real(dp),intent(in) :: rprimd(3,3),rprimd0(3,3)
 real(dp),intent(out) :: rprimd_symm(3,3)

!Local variables-------------------------------
!scalars
 integer :: ii,jj,kk,isym,mu,nu
 real(dp) :: summ,tmp
!arrays
 real(dp) :: gprimd0(3,3),scaling(3,3),scaling_symm(3,3)
 integer :: symrec(3,3,nsym)

!**************************************************************************

!DEBUG
!write(6,*)' strainsym : enter '
!write(6, '(a,9f8.4)')' rprimd0=',rprimd0
!write(6, '(a,9f8.4)')' rprimd =',rprimd
!do isym=1,nsym
! write(6, '(a,i3,a,9i2)' )' isym,symrel(:,:,isym)=',isym,'   ',symrel(:,:,isym)
!enddo
!ENDDEBUG

 call matr3inv(rprimd0,gprimd0)

!Produce the (strain + delta) tensor in reduced coordinates, called scaling
 do ii=1,3
  do jj=1,3
   scaling(ii,jj)=zero
   do kk=1,3
    scaling(ii,jj)=scaling(ii,jj)+rprimd(kk,ii)*gprimd0(kk,jj)
   end do
  end do
 end do

!DEBUG
!write(6, '(a,9f8.4)')' scaling     =',scaling
!ENDDEBUG
 
 scaling(:,:)=scaling(:,:)/dble(nsym)
 scaling_symm(:,:)=zero

!Get the symmetry matrices in terms of reciprocal basis (inverse transpose of symrel)
 do isym=1,nsym
  call mati3inv(symrel(:,:,isym),symrec(:,:,isym))
 end do
 
!Loop over all symmetry operations:
 do isym=1,nsym
  do mu=1,3
   do nu=1,3
    summ=zero
    do ii=1,3
     tmp=scaling(ii,1)*symrel(1,nu,isym)+scaling(ii,2)*symrel(2,nu,isym)+&
&        scaling(ii,3)*symrel(3,nu,isym)
     summ=summ+symrec(ii,mu,isym)*tmp
    end do
    scaling_symm(mu,nu)=scaling_symm(mu,nu)+summ
   end do
  end do
 end do

!DEBUG
!scaling_symm(:,:)=scaling(:,:)*nsym
!write(6, '(a,9f8.4)')' scaling_symm=',scaling_symm
!ENDDEBUG

!Generates the new primitive vectors from symmetrized scaling 
 do ii=1,3
  do jj=1,3
   rprimd_symm(ii,jj)=zero
   do kk=1,3
    rprimd_symm(ii,jj)=rprimd_symm(ii,jj)+rprimd0(ii,kk)*scaling_symm(jj,kk)
   end do
  end do
 end do

!DEBUG
!rprimd_symm(:,:)=rprimd(:,:)
!ENDDEBUG

!DEBUG
!write(6,*)' strainsym : exit '
!write(6, '(a,9f8.4)')' rprimds=',rprimd_symm
!ENDDEBUG

end subroutine strainsym
!!***
