!{\src2tex{textfont=tt}}
!!****f* ABINIT/setnoccmmp
!! NAME
!! setnoccmmp
!!
!! FUNCTION
!! LDA/GGA+U only:
!! Impose value of density matrix nocc_m_mp;
!! Then symetrize it.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 ABINIT group (MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt.
!!
!! INPUTS
!!  dimdmat=first dimension of dmatpawu array
!!  dmatpawu(dimdmat,dimdmat,nsppol,natpawu)=input density matrix to be copied into noccmpp
!!  indsym(4,nsym,natom)=indirect indexing array for atom labels
!!  natom=number of atoms in cell
!!  natpawu=number of atoms on which PAW+U is applied
!!  nspden=number of spin-density components
!!  nsppol=number of independant spin components
!!  nsym=number of symmetry elements in space group
!!  ntypat=number of atom types
!!  paw_ij(natom) <type(paw_ij_type)>=paw arrays given on (i,j) channels
!!  pawtab(ntypat) <type(pawtab_type)>=paw tabulated starting data
!!  symafm(nsym)=(anti)ferromagnetic part of symmetry operations
!!  typat(natom)=type for each atom
!!  usepawu=1 if PAW+U is activated
!!
!! OUTPUT
!!   paw_ij(natom)%noccmmp(2*pawtab(itypat)%lpawu+1,2*pawtab(itypat)%lpawu+1,nspden)=density matrix
!!
!! PARENTS
!!      scfcv
!!
!! CHILDREN
!!      wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine setnoccmmp(dimdmat,dmatpawu,indsym,natom,natpawu,nspden,nsppol,nsym,&
&                     ntypat,paw_ij,pawtab,symafm,typat,usepawu)

 use defs_basis
  use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
#endif
!End of the abilint section

 implicit none

!Arguments ---------------------------------------------
!scalars
 integer,intent(in) :: dimdmat,natom,natpawu,nspden,nsppol,nsym,ntypat,usepawu
!arrays
 integer,intent(in) :: indsym(4,nsym,natom),symafm(nsym),typat(natom)
 real(dp) :: dmatpawu(dimdmat,dimdmat,nsppol,natpawu)
 type(paw_ij_type),intent(inout) :: paw_ij(natom)
 type(pawtab_type),intent(in) :: pawtab(ntypat)

!Local variables ---------------------------------------
!scalars
 integer :: at_indx,iafm,iatom,iatpawu,ilm,irot,ispden,itypat,jlm,lpawu
 logical :: antiferro
 character(len=500) :: message
!arrays
 integer :: nsym_used(2)
 real(dp) :: sumocc(2)
!no_abirules
  type noccmmp_at
   real(dp),pointer :: noccmmp(:,:,:)
  end type
  type(noccmmp_at) :: tmp(natom)

! *********************************************************************

 if (usepawu==0) return
 antiferro=(nspden==2.and.nsppol==1)

 write(message,'(6a)') ch10,'Occupation matrix for correlated orbitals is kept constant',ch10,&
&                           'and equal to initial one !',ch10,&
&                           '----------------------------------------------------------'
 call wrtout(6,message,'COLL')

!Impose value of nocc_mmp from dmatpu
 iatpawu=0
 do iatom=1,natom
  lpawu=pawtab(typat(iatom))%lpawu
  if (lpawu/=-1) then
   iatpawu=iatpawu+1
   allocate(tmp(iatom)%noccmmp(2*lpawu+1,2*lpawu+1,nsppol))
   tmp(iatom)%noccmmp(1:2*lpawu+1,1:2*lpawu+1,1:nsppol)=&
&             dmatpawu(1:2*lpawu+1,1:2*lpawu+1,1:nsppol,iatpawu)
  end if
 end do

!Loops over atoms
 do iatom=1,natom
  lpawu=pawtab(typat(iatom))%lpawu
  if (lpawu/=-1) then

!  Loop over spin components
   do ispden=1,nsppol

!   Loops over components of nocc_mmp
    do jlm=1,2*lpawu+1
     do ilm=1,2*lpawu+1

      nsym_used(1:2)=0
      sumocc(1:2)=zero

!     Accumulate values of nocc_mmp over symmetries
      do irot=1,nsym
       if ((symafm(irot)/=1).and.(.not.antiferro)) cycle
       iafm=1;if ((antiferro).and.(symafm(irot)==-1)) iafm=2
       nsym_used(iafm)=nsym_used(iafm)+1
       at_indx=indsym(4,irot,iatom)
       sumocc(iafm)=sumocc(iafm)+tmp(at_indx)%noccmmp(ilm,jlm,ispden)
      end do ! End loop over symmetries

!     Store new values of nocc_mmp
      paw_ij(iatom)%noccmmp(ilm,jlm,ispden)=sumocc(1)/nsym_used(1)

!     Antiferromagnetic case: has to fill up "down" component of nocc_mmp
      if (antiferro.and.nsym_used(2)>0) paw_ij(iatom)%noccmmp(ilm,jlm,2)=sumocc(2)/nsym_used(2)

     end do !ilm
    end do !jlm
   end do ! ispden

!  Printing of new nocc_mmp
   do ispden=1,nspden
    write(message,'(2a,i3,a,i1,a)') ch10,'== Atom ',iatom,&
&            ' == Imposed occupation matrix for spin ',ispden,' =='
    call wrtout(6,message,'COLL')
    do ilm=1,2*lpawu+1
     write(message,'(12(1x,9(1x,f10.5)))') (paw_ij(iatom)%noccmmp(ilm,jlm,ispden),jlm=1,2*lpawu+1)
     call wrtout(6,message,'COLL')
    end do
   end do

  end if
 end do ! iatom

!Memory deallocation
 do iatom=1,natom
  lpawu=pawtab(typat(iatom))%lpawu
  if (lpawu/=-1) deallocate(tmp(iatom)%noccmmp)
 end do

end subroutine setnoccmmp
!!***
