!{\src2tex{textfont=tt}}
!!****f* abinit/prep_getghc
!! NAME
!! prep_getghc
!!
!! FUNCTION
!! this routine prepares the data to the call of getghc.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 ABINIT group (FBottin,MT,GZ)
!! this file is distributed under the terms of the
!! gnu general public license, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  blocksize= size of block for FFT
!!  cwavef(2,npw*nspinor*ndat)=planewave coefficients of wavefunction.
!!  dimffnl=second dimension of ffnl (1+number of derivatives)
!!  dtfil <type(datafiles_type)>=variables related to files
!!  ffnl(npw,dimffnl,lmnmax,ntypat)=nonlocal form factors on basis sphere.
!!  gs_hamk <type(gs_hamiltonian_type)>=all data for the hamiltonian at k
!!  gvnlc=matrix elements <G|Vnonlocal|C>
!!  kg_k(3,npw_k)=reduced planewave coordinates.
!!  kinpw(npw)=(modified) kinetic energy for each plane wave (hartree)
!!  icall = order of call of this routine in lobpcgccwf
!!  lmnmax=if useylm=1, max number of (l,m,n) comp. over all type of psps
!!        =if useylm=0, max number of (l,n)   comp. over all type of psps
!!  matblk=dimension of the array ph3d
!!  mgfft=maximum size of 1d ffts
!!  mpi_enreg=informations about mpi parallelization
!!  mpsang= 1+maximum angular momentum for nonlocal pseudopotentials
!!  mpssoang= 1+maximum (spin*angular momentum) for nonlocal pseudopotentials
!!  natom=number of atoms in cell.
!!  nband_k=number of bands at this k point for that spin polarization
!!  nbdblock=
!!  npw_k=number of plane waves at this k point
!!  nspinor=number of spinorial components of the wavefunctions
!!  ntypat=number of types of atoms in unit cell.
!!  nvloc=final dimension of vlocal (usually 1, but 4 for non-collinear)
!!  n4,n5,n6 used for dimensionning of vlocal
!!  ph3d(2,npw,matblk)=3-dim structure factors, for each atom and plane wave.
!!  prtvol=control print volume and debugging output
!!  vlocal(n4,n5,n6,nvloc)= local potential in real space, on the augmented fft grid
!!
!! OUTPUT
!!  gwavef=(2,npw*nspinor*ndat)=matrix elements <G|H|C>.
!!  swavef=(2,npw*nspinor*ndat)=matrix elements <G|S|C>.
!!
!! SIDE EFFECTS
!!
!! PARENTS
!!      lobpcgccwf,lobpcgwf
!!
!! CHILDREN
!!      fourwf,sphereboundary,timab,xallgather_mpi,xallgatherv_mpi
!!      xalltoallv_mpi,xcomm_init
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine prep_getghc(cwavef,dimffnl,dtfil,ffnl_gather,gs_hamk,gvnlc,gwavef,swavef,iblock,icall,istwf_k,kg_k_gather,&
& kinpw_gather,lmnmax,matblk,blocksize,mgfft,mpi_enreg,mpsang,mpssoang,natom,nbdblock,nband_k,dimtabs,npw_k,&
& nspinor,ntypat,nvloc,n4,n5,n6,ph3d_gather,prtvol,sij_opt,vlocal)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_14wfs, except_this_one => prep_getghc
 use interfaces_lib01hidempi
#else
 use defs_xfuncmpi
#endif
!End of the abilint section

 implicit none

#if defined MPI_FFT
          include 'mpif.h'
#endif
!Arguments ------------------------------------
 type(gs_hamiltonian_type) :: gs_hamk
 integer :: sij_opt
 integer :: blocksize
 integer :: dimffnl,iblock,icall,istwf_k,lmnmax,matblk,mgfft,mpsang,mpssoang,n4,n5
 integer :: n6,natom,nband_k,nbdblock,npw_k,nspinor,ntypat,nvloc,prtvol
 type(datafiles_type) :: dtfil
 type(mpi_type) :: mpi_enreg
 real(dp) :: gvnlc(2,npw_k*nspinor*blocksize)
 real(dp) :: vlocal(n4,n5,n6,nvloc),cwavef(2,npw_k*nspinor*blocksize),gwavef(2,npw_k*nspinor*blocksize)
 real(dp) :: swavef(2,npw_k*nspinor*blocksize)
 integer :: dimtabs
 integer :: kg_k_gather(3,dimtabs)
 real(dp) :: ffnl_gather(dimtabs,dimffnl,lmnmax,ntypat)
 real(dp) :: kinpw_gather(dimtabs)
 real(dp) :: ph3d_gather(2,dimtabs,matblk)

!Local variables-------------------------------
 character(len=500) :: message
 integer :: spaceComm=0
 integer :: ier
 integer :: old_me_g0,old_paral_level,tim_getghc
 real(dp) :: tsec(2)

!local variables for mpialltoallv
!FB Is cleaned
 real(dp), allocatable :: swavef_alltoall(:,:)
 real(dp), allocatable :: cwavef_alltoall(:,:),gwavef_alltoall(:,:),gvnlc_alltoall(:,:)
 integer :: iproc,ndatarecv
 integer,  allocatable :: recvcounts(:)
 integer,  allocatable :: sendcounts(:),sdispls(:),rdispls(:)
 integer,  allocatable :: sendcountsloc(:),sdisplsloc(:),recvcountsloc(:),rdisplsloc(:)
!no_abirules
!correspondence with abinit. here for real wf but in complex mode
!this is the index of a given band

! *************************************************************************

 tim_getghc=6
 old_paral_level= mpi_enreg%paral_level
 mpi_enreg%paral_level=3
 call xcomm_init(mpi_enreg,spaceComm)
 if(mpi_enreg%mode_para=='b') spaceComm=mpi_enreg%comm_band

 allocate(sendcountsloc(blocksize))
 allocate(sdisplsloc(blocksize))
 allocate(sdispls(blocksize))
 allocate(sendcounts(blocksize))
 allocate(recvcountsloc(blocksize))
 allocate(rdisplsloc(blocksize))
 allocate(rdispls(blocksize))
 allocate(recvcounts(blocksize))
 call timab(546,1,tsec)
 call xallgather_mpi(npw_k,recvcounts,spaceComm,ier)
 call timab(546,2,tsec)
 rdispls(1)=0
 do iproc=2,blocksize
  rdispls(iproc)=rdispls(iproc-1)+recvcounts(iproc-1)
 end do
 ndatarecv=rdispls(blocksize)+recvcounts(blocksize)

!FB The transposition are no more performed within prep_fourwf but above.
!FB We have to check that the dimension mpw is identical the the one computed above. TO BE CLEANED
 if (ndatarecv /= dimtabs) then
  write(message, '(a,a,a,a,i8,a,i8)' ) ch10,&
&      ' prep_fourwf: BUG -',ch10,&
&      '  ndatarecv',ndatarecv,' /= dimtabs ', dimtabs
  call wrtout(06,message,'PERS')
  call leave_new('PERS')
 endif


 sendcounts(:)=npw_k
 do iproc=1,blocksize
  sdispls(iproc)=(iproc-1)*npw_k
 end do

 allocate(cwavef_alltoall(2,ndatarecv*nspinor))
 allocate(gwavef_alltoall(2,ndatarecv*nspinor))
 allocate(swavef_alltoall(2,ndatarecv*nspinor))
 allocate(gvnlc_alltoall(2,ndatarecv*nspinor))
 recvcountsloc(:)=recvcounts(:)*2*nspinor
 rdisplsloc(:)=rdispls(:)*2*nspinor
 sendcountsloc(:)=sendcounts(:)*2*nspinor
 sdisplsloc(:)=sdispls(:)*2*nspinor
 call timab(545,1,tsec)
 call xalltoallv_mpi(cwavef,sendcountsloc,sdisplsloc,cwavef_alltoall,&
&         recvcountsloc,rdisplsloc,spaceComm,ier)
 call timab(545,2,tsec)

 call getghc(cwavef_alltoall,dimffnl,ffnl_gather,dtfil%filstat,gwavef_alltoall,&
& swavef_alltoall(:,:),gs_hamk,gvnlc_alltoall,kg_k_gather,&
& kinpw_gather,lmnmax,matblk,mgfft,mpi_enreg,mpsang,mpssoang,natom,1,ndatarecv,nspinor,ntypat,&
& nvloc,n4,n5,n6,ph3d_gather,prtvol,sij_opt,tim_getghc,vlocal)

 if (gs_hamk%istwf_k==2) mpi_enreg%me_g0=old_me_g0
 call timab(545,1,tsec)
 if (sij_opt==1) then
  call xalltoallv_mpi(swavef_alltoall,recvcountsloc,rdisplsloc,swavef,&
  &         sendcountsloc,sdisplsloc,spaceComm,ier)
 endif
 call xalltoallv_mpi(gwavef_alltoall,recvcountsloc,rdisplsloc,gwavef,&
 &         sendcountsloc,sdisplsloc,spaceComm,ier)
 call xalltoallv_mpi(gvnlc_alltoall,recvcountsloc,rdisplsloc,gvnlc,&
 &         sendcountsloc,sdisplsloc,spaceComm,ier)
 call timab(545,2,tsec)

 mpi_enreg%paral_level= old_paral_level
 deallocate(sendcounts,recvcounts,sdispls,rdispls)
 deallocate(sendcountsloc,sdisplsloc)
 deallocate(recvcountsloc,rdisplsloc)
 deallocate(cwavef_alltoall,gwavef_alltoall,gvnlc_alltoall)
 deallocate(swavef_alltoall)
end subroutine prep_getghc
!!***
