!{\src2tex{textfont=tt}}
!!****f* ABINIT/wvl_nl_gradient
!! NAME
!! wvl_nl_gradient
!!
!! FUNCTION
!! Compute the non local part of the wavefunction gradient.
!!
!! COPYRIGHT
!! Copyright (C) 2005-2007 ABINIT group (DC)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine wvl_nl_gradient(dtset, grnl, mpi_enreg, occ, psps, rprimd, wvl, xcart)

  use defs_basis
  use defs_datatypes
#if defined HAVE_BIGDFT
  use libbigdft
#endif

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_lib01hidempi
#else
 use defs_xfuncmpi
#endif
!End of the abilint section

  implicit none

  !Arguments ------------------------------------
  type(dataset_type), intent(inout)      :: dtset
  type(MPI_type), intent(inout)          :: mpi_enreg
  type(pseudopotential_type), intent(in) :: psps
  type(wvl_data), intent(in)             :: wvl
  real(dp), intent(inout)                :: grnl(3, dtset%natom)
  real(dp), intent(in)                   :: occ(dtset%mband * dtset%nkpt * &
       & dtset%nsppol)
  real(dp), intent(in)                   :: rprimd(3, 3)
  real(dp), intent(in)                   :: xcart(3, dtset%natom)

  !Local variables-------------------------------
  integer               :: ia, igeo, spaceComm, ierr
  character(len=500)    :: message
  real(dp), allocatable :: gxyz(:,:)



  ! Compute forces
  write(message, '(a,a)' ) 'wvl_nl_gradient(): compute non-local part to gradient.'
  call wrtout(6,message,'COLL')

  ! Nullify output arrays.
  grnl(:, :) = zero
  
  allocate(gxyz(3, dtset%natom))
  gxyz(:,:) = zero
  ! Add the nonlocal part of the forces to grtn (BigDFT routine)
#if defined HAVE_BIGDFT
  call nonlocal_forces(mpi_enreg%me, mpi_enreg%nproc, &
       & dtset%wvl_internal%nSize(1), dtset%wvl_internal%nSize(2), &
       & dtset%wvl_internal%nSize(3), wvl%projectors%nbox_c, wvl%projectors%nbox_f, &
       & dtset%ntypat, dtset%natom, dtset%mband, wvl%wfs%mbandp, &
       & wvl%projectors%istart, wvl%projectors%nprojel, wvl%projectors%nproj, &
       & dtset%typat, psps%gth_params%psppar(0:4,0:4,:), psps%pspcod, occ, &
       & wvl%wfs%keys%nseg(0), wvl%wfs%keys%nseg(1), wvl%wfs%keys%nvctr(0), &
       & wvl%wfs%keys%nvctr(1), wvl%projectors%keys%nseg, wvl%projectors%keys%nvctr, &
       & wvl%projectors%proj, wvl%wfs%keys%keyg, wvl%wfs%keys%keyv, &
       & wvl%projectors%keys%keyg, wvl%projectors%keys%keyv, wvl%wfs%psi, xcart, &
       & psps%gth_params%radii_cf, dtset%wvl_cpmult, dtset%wvl_fpmult, &
       & dtset%wvl_hgrid, gxyz)
#else
  write(message, '(a,a,a,a)' ) ch10,&
       &  ' wvl_nl_gradient: BigDFT library is not compiled.', ch10, &
       &  '   Action, used the flag --enable-bigdft when configuring.'
  call wrtout(6,message,'COLL')
  call leave_new('COLL')
#endif

  if (mpi_enreg%paral_compil_fft == 1) then
     call xcomm_world(mpi_enreg, spaceComm)
     call xsum_mpi(gxyz, spaceComm, ierr)
  end if

  ! Forces should be in reduced coordinates.
  do ia = 1, dtset%natom, 1
     do igeo = 1, 3, 1
        grnl(igeo, ia) = - rprimd(1, igeo) * gxyz(1, ia) - &
             & rprimd(2, igeo) * gxyz(2, ia) - &
             & rprimd(3, igeo) * gxyz(3, ia)
     end do
  end do
  deallocate(gxyz)

end subroutine wvl_nl_gradient
