!{\src2tex{textfont=tt}}
!!****f* ABINIT/calc_coh
!! NAME
!! calc_coh
!!
!! FUNCTION
!!
!! COPYRIGHT
!! Copyright (C) 2005-2007 ABINIT group (FB)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!! iqibz= index of the irreducible q-point in the array qibz which is related by a symmetry operation
!!  to the point q considered in the calling subroutine. iqibz is also used to check whether the q-point is 
!!  zero or not in order to treat the integrable coulombian singularity
!! jb,kb= left and righ band indeces definining the left and right states where the partial contribution to 
!!  the matrix element of $\Sigma_{COH}$ is evaluated
!! ngfft1,ngfft1a,ngfft2,ngfft3=integers defining the FFT grid
!! npwc=number of plane waves in $\tilde epsilon^{-1}$
!! npwx=number of G vectors in the arrays gvec and qbzpg  
!! nr=number of points in real space
!! i_sz=contribution arising from the integrable coulombian singularity at q==0 (see csigme for the method used)
!! gvec(3,npwx)= G vectors in reduced coordinates 
!! qbzpg(npwx)= q+G in reduced coordinates 
!! epsm1q_o(npwc,npwc)= contain $\tilde epsilon^{-1}(q,w=0) - \delta_{G Gp}$ for this particular q-point
!! wfg2_jk(nr)= Fourier Transform of $\u_{jb k}^*(r) u_{kb k}$ 
!!
!! OUTPUT
!! sigcohme= partial contribution to the matrix element of $<jb k \sigma|\Sigma_{COH} | kb k \sigma>$ 
!!  arising from a single q-point
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      csigme
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine calc_coh(nr,npwx,npwc,ngfft1a,ngfft1,ngfft2,ngfft3,&
& gvec,epsm1q_o,qbzpg,i_sz,iqibz,jb,kb,wfg2_jk,sigcohme)

 use defs_basis

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: iqibz,jb,kb,ngfft1,ngfft1a,ngfft2,ngfft3,npwc,npwx,nr
 real(dp),intent(in) :: i_sz
 complex,intent(out) :: sigcohme
!arrays
 integer,intent(in) :: gvec(3,npwx)
 real(dp),intent(in) :: qbzpg(npwx)
 complex,intent(in) :: epsm1q_o(npwc,npwc),wfg2_jk(nr)

!Local variables-------------------------------
!scalars
 integer :: ig,ig4,ig4x,ig4y,ig4z,igp
!arrays
 complex :: epsm1q_qpg2(npwc,npwc)

! *************************************************************************

!DEBUG
!write(6,*)' calc_coh : enter '
!ENDDEBUG

 sigcohme= (0.0,0.0)

! Set up \epsilon^{-1} / |q+G|.|q+G'|
  do igp=1,npwc
   do ig=1,npwc
    epsm1q_qpg2(ig,igp)= epsm1q_o(ig,igp)/(qbzpg(ig)*qbzpg(igp))
   end do
  end do

! Treat the case q ==> 0 adequately
! TODO here we could use the improved technique we discussed with NH
  if (iqibz==1) then
   if (jb==kb) then
    epsm1q_qpg2(1,1)= epsm1q_o(1,1)*i_sz
    epsm1q_qpg2(2:npwc,1)= (0.0,0.0)
    epsm1q_qpg2(1,2:npwc)= (0.0,0.0)
   else
    epsm1q_qpg2(:,1)= (0.0,0.0)
    epsm1q_qpg2(1,:)= (0.0,0.0)
   end if
  end if

   do igp=1,npwc
    do ig=1,npwc
     ig4x= modulo(gvec(1,igp)-gvec(1,ig),ngfft1)
     ig4y= modulo(gvec(2,igp)-gvec(2,ig),ngfft2)
     ig4z= modulo(gvec(3,igp)-gvec(3,ig),ngfft3)
     ig4= 1+ig4x+ig4y*ngfft1a+ig4z*ngfft1a*ngfft2

     sigcohme= sigcohme + half*wfg2_jk(ig4)*epsm1q_qpg2(ig,igp)
    end do
   end do

end subroutine calc_coh
!!***
