!{\src2tex{textfont=tt}}
!!****f* ABINIT/cchi0
!! NAME
!! cchi0
!!
!! FUNCTION
!! Main calculation of the independent-particle susceptibility chi0 for qq!=0
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (GMR, VO, LR, RWG, MG, RShaltaf)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  ep= datatype gathering differening parameters related to the calculation of the inverse dielectric matrix
!!  energy(nkibz,nbnds,nsppol)=KS energies
!!  gwenergy(nkibzm,nbnds,nsppol)=GW energies, for self-consistency purposes
!!  grottbm1(npwvec,2,nop)=  contains the index  (IS^{-1}) G 
!!  igfft(npwvec,5,5,5)=index of G-G0 planewaves (see cigfft routine)
!!  localrdwf= input variable (for parallel case) if 1, the KSS file is local to each machine
!!  kbz(3,nkbz)=k-point coordinates, full Brillouin zone
!!  kibz(3,nkibz)=k-point coordinates, irred Brillouin zone
!!  ktab(nkbz)=mapping between a k-point in the BZ (array kbz) and the
!!   corresponding irreducible point in the array kibz
!!  ktabi(nkbz)= -1 if inversion has to be considered for this particulat k-point, 1 if not
!!  ktabr(nr,nkbz)= index of (R^{-1})(r-t) in the FFT array where R=\transpose S^{-1} and 
!!   k= IS k_irred (I is either the identity or the inversion)
!!  ktabp(nkbz)= phase factor associated to tnons e^{-i 2 \pi k\cdot R{^-1}t}
!!  mpi_enreg= datatype gathering information on parallelism, variables used 
!!   |gwpara= 0 no parallelism 
!!   |        1 parallelism is over k-points
!!   |        2 bands are spread btw processors
!!  nbnds=number of bands
!!  nbvw=number of bands in the arrays wfrv
!!  nbv=number of valence bands
!!  ngfft1,ngfft1a,ngfft2,ngfft3=FFT grid dimensions
!!  nkbz=number of k points in full Brillouin zone
!!  nkibz=number of k points in irreducible Brillouin zone
!!  nkibzm=maximum number of k points in irreducible Brillouin zone
!!  nomega=number of frequencies
!!  nop=number of symmetries 
!!  npwsigx=number of planewaves for sigma exchange (input variable)
!!  npwvec=dimension of igfft
!!  npwwfn=number of planewaves for wavefunctions (input variable)
!!  nsppol=1 for unpolarized, 2 for spin-polarized
!!  nr=number of points in FFT grid
!!  occ(nkibzm,nbnds,nsppol)=occupation numbers, for each k point in IBZ, each band and spin
!!  op(3,3,nop) :: symmetry operation in reciprocal space
!!  omega(nomega)=frequencies
!!  qq(3)=reciprocal space coordinates of the q wavevector
!!  lt_q= little group datatype
!!  kkweight(nomega,ep%nomegasf)=frequency dependent weights for Kramers Kronig transform
!!  gptab(npwsigx,npwsigx)= index of the indendent pair (G1,G2) such as (T1,T2)= S (G1,G2) 
!!  gptabo(npwsigx,npwsigx)= index of the symmetry operation S in the array op such as (T1,T2)= S (G1,G2)
!!  ip2fp(2,ngpi)= index of G1 and G2 in the array gvec for each ngpi independent pair 
!!  phgt(npwsigx,nop)= phase factors e^{-iG \cdot t} needed to symmetrize oscillator matrix elements and epsilon 
!!   (FIXME to be better described)
!!  wfr(nr,min_band_proc:max_band_proc,nkibz,nsppol) = (optional) wavefunctions in real space 
!!  wfrv(npwwfn,nbvw,nkibz,nsppol)= (optional) array containing fully and partially occupied states in t space
!!
!! OUTPUT
!!  chi0(npwsigx,npwsigx,nomega)=independent-particle susceptibility matrix for wavevector qq,
!!   and frequencies defined by omega
!!
!! TODO
!!  suppress the use of nkibzm, should be actually equal to nkibz
!!  eliminate energy (superceded by gwenergy)
!!  remove a lot of input variables since they are contained in ep%
!!
!! PARENTS
!!      screening
!!
!! CHILDREN
!!      assemblychi0,leave_new,rho_tw_g,timab,wrtout,xcomm_init
!!      xmaster_init,xme_init,xsum_master
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine cchi0(qq,ep,nomega,omega,npwvec,npwsigx,npwwfn,nkibz,kibz,nkibzm,nbnds,&
& nbv,nbvw,nsppol,occ,ktab,ktabr,ktabi,ktabp,kbz,nkbz,ngfft1,ngfft1a,ngfft2,ngfft3,igfft,nr,&
& energy,gwenergy,etadelta,chi0,mpi_enreg,nop,op,grottbm1,phgt,ngpi,gptab,gptabo,ip2fp,&
& lt_q,kkweight,min_band_proc,max_band_proc,nbnds_per_proc,localrdwf,&
& wfr,wfrv) !optional

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_15gw, except_this_one => cchi0
 use interfaces_lib01hidempi
#else
 use defs_xfuncmpi
#endif
!End of the abilint section

implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: localrdwf,max_band_proc,min_band_proc,nbnds,nbnds_per_proc,nbvw,ngfft1
 integer,intent(in) :: ngfft1a,ngfft2,ngfft3,ngpi,nkbz,nkibz,nkibzm,nomega,nop
 integer,intent(in) :: npwsigx,npwvec,npwwfn,nr,nsppol
 real(dp),intent(in) :: etadelta
 type(MPI_type),intent(inout) :: mpi_enreg
 type(epsilonm1_parameters),intent(in) :: ep
 type(little_group),intent(in) :: lt_q
!arrays
 integer,intent(in) :: grottbm1(npwvec,2,nop),igfft(npwvec,5,5,5),ktab(nkibzm)
 integer,intent(in) :: gptab(npwsigx,npwsigx),gptabo(npwsigx,npwsigx) 
 integer,intent(in) :: ip2fp(2,ngpi) 
 integer,intent(in) :: ktabi(nkibzm),ktabr(nr,nkibzm),nbv(nsppol)
 real(dp),intent(in) :: energy(nkibz,nbnds,nsppol)
 real(dp),intent(in) :: gwenergy(nkibz,nbnds,nsppol)
 real(dp),intent(in) :: kbz(3,nkbz),kibz(3,nkibz)
 real(dp),intent(in) :: occ(nkibz,nbnds,nsppol),qq(3),op(3,3)
 complex,intent(in) :: omega(nomega)
 complex,intent(in) :: ktabp(nkbz)
 complex,intent(in) :: phgt(npwsigx,nop)
 complex,intent(in),optional :: wfr(nr,min_band_proc:max_band_proc,nkibz,nsppol)
 complex,intent(in),optional :: wfrv(nr,nbvw,nkibz,nsppol)
 complex,intent(in),optional :: kkweight(nomega,ep%nomegasf) !to be optional or not to be? 
 complex,intent(out) :: chi0(npwsigx,npwsigx,nomega)

!Local variables ------------------------------
!scalars
 integer,parameter :: unitwfr=26
 integer :: i,ii,ib,ibc,ibv,ig,ig01a,ig02a,ig03a,igp,ik,ikibz,ikp,ikpibz,io,ir,is
 integer :: istat,jb_proc_rank,max_con,max_val,min_con,min_val,nkibzq,nrb,tim_fourdp
 integer :: master,spaceComm,ierr,me
 real(dp) :: eckp,f_occ,factocc,ockp,weight
 complex :: dd
 complex(dpc) :: egwdiff
 logical :: i_read,master_cast,is_metal
 logical :: bands_read,kkp_read
 logical :: wfr_in_mem,use_tr
 character(len=500) :: message
!arrays
 integer :: g0(3)
 real(dp) :: kmq(3),tsec(2)
 complex,allocatable :: rhotw(:),rhotwg(:)
 complex,allocatable,target :: wfnr(:,:,:,:),wfnrk(:,:),wfnrkp(:,:)
 complex,pointer :: wfnrckp(:),wfnrvk(:)
 complex(dpc),allocatable :: den(:)

!BEGIN SPECTRAL
! this is for the implementation of the spectral function 
 integer :: iomegal,iomegar,iosf,iosfp,method
 integer :: my_wl,my_wr
 real(dp) :: omegal,omegar,wl,wr,dummy
 real(dp) :: domega,domega_min,domega_max
 complex :: num,ct
 complex,allocatable :: chi0sf(:,:,:)
!END SPECTRAL

! *************************************************************************
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 logical :: findkp
#endif
!End of the abilint section

 write(message,'(3a)')ch10,' cchi0 : enter ',ch10
 call wrtout(06,message,'COLL')

 call timab(331,1,tsec)

 ! Initialize MPI related quantities
 call xcomm_init(mpi_enreg,spaceComm) !Init mpi_comm 
 call xme_init(mpi_enreg,me)          !Init me 
 call xmaster_init(mpi_enreg,master)  !Init master

 ! Organize the I/O of proc
 i_read=.false. ; master_cast=.false.
 if (localrdwf==1 .or. me==0) i_read=.true.
 if (localrdwf==0 .and. mpi_enreg%nproc> 1) master_cast=.true.

 ! Here we check all the possibilities to avoid bugs
 if ((present(wfrv) .and. .not.present(wfr))) then 
  write(message,'(5a)')ch10,&
&  ' cchi0 : BUG -',ch10,&
&  ' if wfrv is present then also wfr must be in input ',ch10
  call wrtout(6,message,'COLL') ; call leave_new('COLL')
 end if

 wfr_in_mem=.false.
 if (present(wfr)) then 
  wfr_in_mem=.true.
  write(message,'(2a)')ch10,' wfrk in real space are in memory'
  call wrtout(06,message,'COLL')
 end if 

 min_val=1 ; max_val=nbnds
 min_con=1 ; max_con=nbnds

 use_tr=.false.
 if (present(wfrv)) then 
  use_tr=.true.
  write(message,'(4a)')ch10,&
&  ' valence states in real space are in memory',ch10,&
&  ' using faster equation based on time reversal symmetry' 
  call wrtout(06,message,'COLL')
  ! Faster algorithm based on time reversal (at the moment only in para run)
  ! Note that special care has to be used in case of metals and/or spin
  ! since wfrv can contain unoccupied states 
  min_val=1      ; max_val=nbvw
  min_con=nbvw+1 ; max_con=nbnds
 end if 

 if (me==0) then 
  write(6,*)' use symmetries    = ',lt_q%sym_flag
  write(6,*)' use time reversal = ',use_tr
  write(6,*)' spectral method   = ',ep%spmeth
 end if 

 tim_fourdp=1
 nrb=ngfft1*ngfft2*ngfft3 !set up fft factor

 allocate(rhotwg(npwsigx))

 if (ep%spmeth==0) then 
  allocate(den(nomega))
  write(message,'(2a)')' calculating chi0(q,omega,G,G")',ch10
  call wrtout(06,message,'COLL')
 else 
  write(message,'(2a)')' calculating Im chi0(q,omega,G,G")',ch10
  call wrtout(06,message,'COLL')
  my_wl=1 ; my_wr=ep%nomegasf
  if (mpi_enreg%gwpara==2) then 
   write(message,'(3a)')ch10,' cchi0 : distributing frequencies between processors ',ch10
   call wrtout(06,message,'COLL')
   if (.not.(present(wfrv))) then 
    write(message,'(a)')' cchi0 : BUG : valence wfs in r space are not in memory'
    call wrtout(6,message,'COLL') ; call leave_new('COLL')
   end if 
   ! Calculate min and max transition energy for this processor
   ! NOTE : in metals the contribution coming from the fully and partially 
   ! occupied stated is calculated at the end, the contribution due to the 
   ! fully unoccupied states is negligible and is not evaluated (check the threshold)
   domega_min=  huge(0.0_dp)
   domega_max= -huge(0.0_dp)
   !do ikibz = 1,nkibz ! This approach does not work, dont know why 
   !kmq(:) = kibz(:,ikibz)-qq(:)
   do ik=1,nkbz
    ikibz=ktab(ik)
    kmq(:)=kbz(:,ik)-qq(:)
    ! Find kp=k-q-G0 and also G0
    if (.not.findkp(nkbz,kbz,ikp,kmq,g0)) then
     write(message,'(4a)')ch10,&
&     ' cchi0 : ERROR - ',ch10,'  kp=k-q-G0 not found'
     call wrtout(06,message,'COLL')
     write(message,'(a,i4,a,3f12.6,2a,6x,3f12.6)')&
&     ' ikibz = ',ik,' k = ',(kbz(i,ik),i=1,3),ch10,' k-q = ',(kmq(i),i=1,3)
     call wrtout(06,message,'COLL') ; call leave_new('COLL') !in this case the weight 1.0/nkbz is wrong
    end if 
    ikpibz=ktab(ikp)
    do ibc=nbvw+1,nbnds           
     do ibv=1,nbvw
      do is=1,nsppol
       if ( (mpi_enreg%proc_distrb(ikpibz,ibc,is)-mpi_enreg%me)==0 .and. &
&           (mpi_enreg%proc_distrb(ikibz, ibv,is)-mpi_enreg%me)==0) then 
        domega= gwenergy(ikpibz,ibc,is)-gwenergy(ikibz,ibv,is)
        domega_min= min(domega_min,domega)
        domega_max= max(domega_max,domega)
       end if
      end do 
     end do 
    end do
   end do
   my_wr=-999
   do io=1,ep%nomegasf !the array is dimensioned using nomegasf+1
    if (ep%omegasf(io)>domega_max) then
     my_wr=io+1 
     exit
    end if 
   end do
   if (my_wr==ep%nomegasf+1) my_wr=ep%nomegasf
   my_wl=-999
   do io=ep%nomegasf,1,-1
    if (ep%omegasf(io)<=domega_min) then  !check metals
     my_wl=io
     exit
    end if 
   end do
  end if 
!FIXME this is for metallic systems (presently only master works)
  if (me==0) my_wl=1
!FIXME
  write(message,'(2a,2i5,a,i4,3a,f8.2,3a,f8.2,2a)')ch10,&
&  ' cchi0 : allocating chi0sf using my_wl and my_wr = ',my_wl,my_wr,' ( ',my_wr-my_wl+1,' )',ch10,&
&  '  domega_min = ',domega_min*Ha_eV,' [eV] ',ch10,&
   '  domega_max = ',domega_max*Ha_eV,' [eV] ',ch10
  call wrtout(06,message,'PERS')
  if (my_wl<=0 .or. my_wr<=0) then 
   write(message,'(2(a,i5))')' my_wl= ',my_wl,' my_wr= ',my_wr
   call wrtout(06,message,'PERS') ; call leave_new('COLL')
  end if
  allocate(chi0sf(npwsigx,npwsigx,my_wl:my_wr),stat=istat)
   if(istat/=0) call memerr('cchi0','chi0sf',npwsigx**2*(my_wr-my_wl+1)*ep%nomegasf,'spc')
  chi0sf=(0.,0.)
 end if 

 bands_read=.false. ; kkp_read=.false.
 if (.not.wfr_in_mem) then
  ! Wavefunctions are not passed in input, they are on disk 
  write(message,'(4a)')ch10,&
&  ' cchi0 : reading wavefunctions from file',ch10,&
&  '  trying to allocate wavefunctions in real space'
  call wrtout(06,message,'COLL')
  allocate(wfnr(nr,nbnds,nkibz,nsppol),stat=istat)
  if (istat==0) then
   bands_read=.true.
   write(message,'(a)')'  allocated ' ; call wrtout(06,message,'COLL')
   do is=1,nsppol
    do ikibz=1,nkibz
     if (i_read) read(unitwfr,rec=ikibz+nkibz*(is-1)) ((wfnr(ir,ib,ikibz,is),ir=1,nr),ib=1,nbnds)
     if (master_cast) call xcast_mpi(wfnr,master,spaceComm,ierr)
    end do
   end do 
   write(message,'(2a)')' cchi0 : real space wavefunctions are now in memory',ch10
   call wrtout(06,message,'COLL')
  else
   kkp_read=.true.
   write(message,'(6a)')ch10,&
&   ' cchi0 : WARNING - ',ch10,&
&   '  failed in allocating the full block of wavefunctions ',ch10,&
&   '  allocating single k-point (slower)'
   call wrtout(06,message,'COLL')
   allocate(wfnrk(nr,nbnds),stat=istat)
    if(istat/=0) call memerr('chi0','wfnrk',nr*nbnds,'spc')
   allocate(wfnrkp(nr,nbnds),stat=istat)
    if(istat/=0) call memerr('chi0','wfnrkp',nr*nbnds,'spc')
  end if
 end if !of if (.not.wfr_in_mem)

 chi0(:,:,:)=(0.0,0.0) !zero chi0

 ! Setup weights (2 for spin unpolarized sistem, 1 for polarized)
 ! f_occ is used to normalize the occupation factors to one
 if (nsppol==1) then 
  weight=2.0/nkbz ; f_occ=0.5     
 else if (nsppol==2) then 
  weight=1.0/nkbz ; f_occ=1.0
 else 
  write(message,'(4a)')ch10,&
&  ' cchi0: BUG -',ch10,&
&  ' wrong value for nsppol'
  call wrtout(06,message,'COLL') ; call leave_new('COLL')
 end if

 nkibzq=nkbz
 if (lt_q%sym_flag/=0) nkibzq=sum(lt_q%ibzq(:))
 write(message,'(a,i6,a)')' calculation status ( ',nkibzq,' to be completed) :'
 call wrtout(06,message,'COLL')
 !
 ! Loop over k-points in BZ
 !
 do ik=1,nkbz
  ! Only k in IBZq, this coding is safer since lt_q%ibzq(ik) might be not allocated 
  if (lt_q%sym_flag/=0 ) then ; if (lt_q%ibzq(ik)/=1) cycle ; end if 
  ikibz=ktab(ik)
  ! Identify kp and G0 where kp=k-q-G0, form k-q
  kmq(:)= kbz(:,ik)-qq(:)
  ! Find kp=k-q-G0 and also G0
  if (.not.findkp(nkbz,kbz,ikp,kmq,g0)) then
   write(message,'(4a)')ch10,&
&   ' cchi0 : ERROR - ',ch10,'  kp=k-q-G0 not found'
   call wrtout(06,message,'COLL')
   write(message,'(a,i4,a,3f12.6,2a,6x,3f12.6)')&
&   ' ik = ',ik,' k = ',(kbz(i,ik),i=1,3),ch10,' k-q = ',(kmq(i),i=1,3)
   ! leave because the normalisation factor weight=1.0/nkbz is wrong
   call wrtout(06,message,'COLL') ; call leave_new('COLL') 
  end if
  ikpibz=ktab(ikp)
  ig01a=g0(1)+3
  ig02a=g0(2)+3
  ig03a=g0(3)+3
  !
  ! Loop over spin and calculate $\Chi_{up,up} + \Chi_{down,down}$
  !
  do is=1,nsppol 
   ! Parallelization over k-points in the full BZ
   ! Note that spin para is not active but the check should be done here
   ! array proc_distrb does not depend on band index (there is a check in screening)
   if (mpi_enreg%nproc>1 .and. mpi_enreg%gwpara==1) then 
    if ((mpi_enreg%proc_distrb(ik,1,is)-mpi_enreg%me)/=0) cycle
!DEBUG !for the time being do not comment this part
if (any(mpi_enreg%proc_distrb(ik,:,is)==-999)) then 
 write(*,*)'cchi0 : wrong distrb' 
 call leave_new('COLL')
end if 
!ENDDEBUG
   end if
   write(message,'(a,i6,a,i6,a,i2,a,i6)')&
&   ' cchi0 : ik = ',ik,' / ',nkbz,'   is = ',is,'  done by processor ',mpi_enreg%me
   call wrtout(06,message,'COLL')
   !
   if (kkp_read) then
    ! Read wfn(r) for the associated ibz k-points and spin
    ! If we want to use time reversal then the occupied states must be passed as input while 
    ! unoccupied states can be read from disk
    if (i_read) then
     read(unitwfr,rec=ikibz + nkibz*(is-1)) ((wfnrk(ir,ib),ir=1,nr),ib=1,nbnds)
     read(unitwfr,rec=ikpibz+ nkibz*(is-1)) ((wfnrkp(ir,ib),ir=1,nr),ib=1,nbnds)
    end if
    if (master_cast) then
     call xcast_mpi(wfnrk,master,spaceComm,ierr)
     call xcast_mpi(wfnrkp,master,spaceComm,ierr)
    end if
   end if
   !
   ! Loop over conduction states
   !
   do ibc=min_con,max_con
    if (mpi_enreg%nproc>1 .and. mpi_enreg%gwpara==2) then
     if (minval(abs(mpi_enreg%proc_distrb(:,ibc,is)-mpi_enreg%me))/=0) cycle
    end if
    if (use_tr .and. ep%spmeth==0) then
     eckp=gwenergy(ikibz,ibc,is)
     ockp=occ(ikibz,ibc,is)
    else
     eckp=gwenergy(ikpibz,ibc,is)
     ockp=occ(ikpibz,ibc,is)
    end if
    if (.not.(wfr_in_mem)) then
     if (kkp_read)   wfnrckp => wfnrkp(:,ibc)
     if (bands_read) wfnrckp => wfnr(:,ibc,ikpibz,is)
    end if
    !
    ! Loop over valence states
    !
    do ibv=min_val,max_val
     if (use_tr .and. ep%spmeth==0) then
      factocc=f_occ*(occ(ikpibz,ibv,is)-ockp)
     else
      factocc=f_occ*(ockp-occ(ikibz,ibv,is))
     end if
     ! Continue the loop only if abs(factocc)/=0 
     if (abs(factocc)<0.01) cycle
     if (use_tr .and. ep%spmeth==0) then
      egwdiff=gwenergy(ikpibz,ibv,is)-eckp
     else
      egwdiff=eckp-gwenergy(ikibz,ibv,is)
     end if
     !
     ! Spectral method
     !
     if (ep%spmeth/=0) then 
      if (real(egwdiff)<0) cycle
      call approxdelta(ep%nomegasf+1,ep%omegasf(1:ep%nomegasf+1),real(egwdiff),&
&      ep%spsmear,iomegal,iomegar,wl,wr,ep%spmeth)
     else 
      !
      ! Adler-Wiser Equation
      ! Add the small imaginary of the T-O response function only for non-zero real frequencies
      !
      ! MG What about metals?
      ! removed etadelta in case of imaginary omegas, now results in parallel and sequential agree
      ! there are differences in the third digit for large matrix elements but this might be due to 
      ! single precision and summation over procs and/or aliasing problems 
      ! required modification to deal with metals
      if (use_tr) then
       ! Use time reversal symmetry 
       do io=1,nomega
        if (real(omega(io))>0.001) then 
         den(io)= factocc/(omega(io)+egwdiff-(0.,1.)*egwdiff/abs(egwdiff)*etadelta)&
&                -factocc/(omega(io)-egwdiff+(0.,1.)*egwdiff/abs(egwdiff)*etadelta)
        else 
         den(io)= factocc/(omega(io)+egwdiff)&
&                -factocc/(omega(io)-egwdiff)
        end if 
       end do
      else
       ! Old implementation
       do io=1,nomega
        if (real(omega(io))>0.001) then
         den(io)= factocc/(omega(io)+egwdiff-(0.,1.)*egwdiff/abs(egwdiff)*etadelta)
        else
         den(io)= factocc/(omega(io)+egwdiff)
        end if
       end do
      end if
     end if !of ep%spmeth 
     !
     if (.not.(wfr_in_mem)) then
      if (kkp_read)   wfnrvk => wfnrk(:,ibv)
      if (bands_read) wfnrvk => wfnr(:,ibv,ikibz,is)
     end if
     !
     ! Form rho-twiddle(r)=u^*_ckp(r)*u_vk(r)
     !
     if (wfr_in_mem) then
      if (use_tr .and. ep%spmeth==0) then !note that here valence is on the left side
       call rho_tw_g(npwsigx,nr,nrb,ngfft1a,ngfft1,ngfft2,ngfft3,igfft(:,ig01a,ig02a,ig03a),&
&       wfrv(:,ibv,ikpibz,is),ktabi(ikp),ktabr(:,ikp),ktabp(ikp),              &
&       wfr (:,ibc,ikibz ,is),ktabi(ik ),ktabr(:,ik ),ktabp(ik ),rhotwg,tim_fourdp)
      else if (ep%spmeth/=0 .and. mpi_enreg%gwpara==2) then 
       call rho_tw_g(npwsigx,nr,nrb,ngfft1a,ngfft1,ngfft2,ngfft3,igfft(:,ig01a,ig02a,ig03a),&
&       wfr (:,ibc,ikpibz,is),ktabi(ikp),ktabr(:,ikp),ktabp(ikp),              &
&       wfrv(:,ibv,ikibz ,is),ktabi(ik ),ktabr(:,ik ),ktabp(ik ),rhotwg,tim_fourdp)
      else
       call rho_tw_g(npwsigx,nr,nrb,ngfft1a,ngfft1,ngfft2,ngfft3,igfft(:,ig01a,ig02a,ig03a),&
&       wfr(:,ibc,ikpibz,is),ktabi(ikp),ktabr(:,ikp),ktabp(ikp),               &
&       wfr(:,ibv,ikibz, is),ktabi(ik ),ktabr(:,ik ),ktabp(ik ),rhotwg,tim_fourdp)
      end if
     else ! if they were read from disk file..
      call rho_tw_g(npwsigx,nr,nrb,ngfft1a,ngfft1,ngfft2,ngfft3,igfft(:,ig01a,ig02a,ig03a),&
&      wfnrckp,ktabi(ikp),ktabr(:,ikp),ktabp(ikp),                            &
&      wfnrvk ,ktabi(ik ),ktabr(:,ik ),ktabp(ik ),rhotwg,tim_fourdp)
     end if

     if (ep%spmeth/=0) then 
      !
      ! Spectral method 
      !
      call assemblychi0sf(npwvec,npwsigx,lt_q%ninv,lt_q%nop,lt_q%nkbz,&
&      lt_q%wtksym(:,:,ik),lt_q%iumklp(:,:,ik),lt_q%fumklp,phgt,grottbm1,&
&      rhotwg,factocc,my_wl,iomegal,wl,my_wr,iomegar,wr,lt_q%sym_flag,ep%nomegasf,chi0sf)
     else 
      !
      ! Traditional method 
      !
      if (lt_q%sym_flag/=0) then 
       ! Use symmetries to reconstruct the full BZ
       call assemblychi0_sym(npwvec,npwsigx,lt_q%ninv,lt_q%nop,lt_q%nkbz,lt_q%wtksym(:,:,ik),&
&       lt_q%iumklp,lt_q%fumklp,phgt,grottbm1,nomega,chi0,rhotwg,den) 
      else 
        ! No symmetries : sum over the full BZ accumulating 
        ! chi0(G,Gp,io)=chi0(G,Gp,io)+(rhotwg(G)*rhotwg(Gp))*den(io)
       call assemblychi0(npwsigx,nomega,chi0,rhotwg,den)
      end if
     end if 
    end do !ibv
   end do !ibc
  end do !is
 end do !ik

 if (ep%spmeth==0) then 
  if (.not. use_tr) then 
   !
   ! Use old slow method. Sum all the contributions from the different procs
   !
   call xsum_master(chi0,master,spaceComm,ierr)
   chi0(:,:,:)=weight*chi0(:,:,:)
  else 
   ! 
   ! Use equation based on time-reversal 
   ! Calculate the contribution coming from the fully and partial occupied states
   !
   ! For the moment only master works
   if (me==0) then 
    is_metal=.false.
    do ik=1,nkbz
     ! Only k in IBZq, this coding is safer since lt_q%ibzq(ik) might be not allocated 
     if (lt_q%sym_flag/=0 ) then ; if (lt_q%ibzq(ik)/=1) cycle ; end if 
     ikibz=ktab(ik)
     ! Identify kp and G0 where kp=k-q-G0, form k-q
     kmq(:)= kbz(:,ik)-qq(:)
     ! Find kp=k-q-G0 and also G0
     if (.not.findkp(nkbz,kbz,ikp,kmq,g0)) then
      write(message,'(4a)')ch10,&
&     ' cchi0 : ERROR - ',ch10,'  kp=k-q-G0 not found by master'
      call wrtout(06,message,'COLL')
      write(message,'(a,i4,a,3f12.6,2a,6x,3f12.6)')&
&      ' ik = ',ik,' k = ',(kbz(i,ik),i=1,3),ch10,' k-q = ',(kmq(i),i=1,3)
      ! leave because the normalisation factor weight=1.0/nkbz is wrong
      call wrtout(06,message,'COLL') ; call leave_new('COLL') 
     end if
     ikpibz=ktab(ikp)
     ig01a=g0(1)+3
     ig02a=g0(2)+3
     ig03a=g0(3)+3
     !
     ! Loop over spin and calculate $\Chi_{up,up} + \Chi_{down,down}$
     !
     do is=1,nsppol 
      do ibc=min_val,max_val
       eckp=gwenergy(ikibz,ibc,is)
       ockp=occ(ikibz,ibc,is)
       ! Note the starting point for ibv
       do ibv=ibc,max_val
        factocc=f_occ*(occ(ikpibz,ibv,is)-ockp)
        if (abs(factocc)<0.01) cycle
        is_metal=.true.
        egwdiff=gwenergy(ikpibz,ibv,is)-eckp
        do io=1,nomega
         if (real(omega(io))>0.001) then 
          den(io)= factocc/(omega(io)+egwdiff-(0.,1.)*egwdiff/abs(egwdiff)*etadelta)&
&                 -factocc/(omega(io)-egwdiff+(0.,1.)*egwdiff/abs(egwdiff)*etadelta)
         else 
          den(io)= factocc/(omega(io)+egwdiff)&
&                 -factocc/(omega(io)-egwdiff)
         end if 
        end do
        ! this is to avoid the double counting
        if (ibv==ibc) den=den*half
        call rho_tw_g(npwsigx,nr,nrb,ngfft1a,ngfft1,ngfft2,ngfft3,igfft(:,ig01a,ig02a,ig03a),&
&        wfrv(:,ibv,ikpibz,is),ktabi(ikp),ktabr(:,ikp),ktabp(ikp),              &
&        wfrv(:,ibc,ikibz ,is),ktabi(ik ),ktabr(:,ik ),ktabp(ik ),rhotwg,tim_fourdp)
        !double counting
        !if (ibv==ibc) rhotwg=rhotwg*sqrt(half)
        if (lt_q%sym_flag/=0) then 
         ! Use symmetries to reconstruct the full BZ
         call assemblychi0_sym(npwvec,npwsigx,lt_q%ninv,lt_q%nop,lt_q%nkbz,lt_q%wtksym(:,:,ik),&
&         lt_q%iumklp,lt_q%fumklp,phgt,grottbm1,nomega,chi0,rhotwg,den) 
        else 
         ! No symmetries : sum over the full BZ accumulating 
         ! chi0(G,Gp,io)=chi0(G,Gp,io)+(rhotwg(G)*rhotwg(Gp))*den(io)
         call assemblychi0(npwsigx,nomega,chi0,rhotwg,den)
        end if
       end do !ibv
      end do !ibc
     end do !is
    end do !ik
    if (is_metal) then 
     write(message,'(2a)')ch10,&
&     ' cchi0 : system is metallic, metallic contribution added'
     call wrtout(06,message,'COLL')
    end if
   end if !of me==0  
   call leave_test(mpi_enreg)
   call xsum_master(chi0,master,spaceComm,ierr)
   chi0(:,:,:)=weight*chi0(:,:,:)
  end if !of use_tr
 else
  !
  ! Spectral method : perform Hilbert transform 
  ! master calculates the contribution in case of metallic|spin polarized systems 
  !
  ! first coding
  if (me==0 .and. mpi_enreg%gwpara==2 .and. ep%spmeth/=0) then 
   do ik=1,nkbz
    if (lt_q%sym_flag/=0 ) then ; if (lt_q%ibzq(ik)/=1) cycle ; end if 
    ikibz=ktab(ik)
    kmq(:)= kbz(:,ik)-qq(:)
    if (.not.findkp(nkbz,kbz,ikp,kmq,g0)) then 
     write(message,'(4a)')ch10,&
&     ' cchi0 : ERROR - ',ch10,'  kp=k-q-G0 not found by master'
     call wrtout(06,message,'COLL')
     write(message,'(a,i4,a,3f12.6,2a,6x,3f12.6)')&
&    ' ik = ',ik,' k = ',(kbz(i,ik),i=1,3),ch10,' k-q = ',(kmq(i),i=1,3)
     call wrtout(06,message,'COLL') ; call leave_new('COLL') !the weight=1.0/nkbz is wrong
    end if
    ikpibz=ktab(ikp)
    ig01a=g0(1)+3
    ig02a=g0(2)+3
    ig03a=g0(3)+3
    do is=1,nsppol 
     do ibc=min_val,max_val
      eckp=gwenergy(ikpibz,ibc,is)
      ockp=occ(ikpibz,ibc,is)
      do ibv=min_val,max_val
       factocc=f_occ*(ockp-occ(ikibz,ibv,is))
       if (abs(factocc)<0.01) cycle
       egwdiff=eckp-gwenergy(ikibz,ibv,is)
       if (real(egwdiff)<0) cycle
       write(*,*)'metallic system'
       call approxdelta(ep%nomegasf+1,ep%omegasf(1:ep%nomegasf+1),real(egwdiff),&
&      ep%spsmear,iomegal,iomegar,wl,wr,ep%spmeth)
       call rho_tw_g(npwsigx,nr,nrb,ngfft1a,ngfft1,ngfft2,ngfft3,igfft(:,ig01a,ig02a,ig03a),&
&       wfrv(:,ibc,ikpibz,is),ktabi(ikp),ktabr(:,ikp),ktabp(ikp),              &
&       wfrv(:,ibv,ikibz ,is),ktabi(ik ),ktabr(:,ik ),ktabp(ik ),rhotwg,tim_fourdp)
       call assemblychi0sf(npwvec,npwsigx,lt_q%ninv,lt_q%nop,lt_q%nkbz,&
&       lt_q%wtksym(:,:,ik),lt_q%iumklp(:,:,ik),lt_q%fumklp,phgt,grottbm1,&
&       rhotwg,factocc,my_wl,iomegal,wl,my_wr,iomegar,wr,lt_q%sym_flag,ep%nomegasf,chi0sf)
      end do !ibv
     end do !ibc
    end do !is
   end do !ik
  end if !of me==0 

  call leave_test(mpi_enreg)
  write(message,'(2a)')ch10,' cchi0 : performing Hilbert transform ...'
  call wrtout(06,message,'COLL')
  
! first coding, the loop over ig igp could be optimised taking into account symmetries 
  if (ep%spmeth==1) then !use triangular approximant
   write(message,'(a)')' using triangular approximant'
   call wrtout(06,message,'COLL')
   do ig=1,npwsigx
    do igp=1,npwsigx
     do io=1,nomega 
      ct=(0.0,0.0)
      do iosf=my_wl,my_wr
       ct= ct + kkweight(io,iosf)*chi0sf(ig,igp,iosf)
      end do
      chi0(ig,igp,io)= ct
     end do 
    end do
   end do 
!DEBUG
!call symf12(npwsigx,nomega,chi0,omega,nop,op,ngpi,gptab,gptabo,ip2fp,phgt,2)
!ENDDEBUG
  else !taylor expansion of the gaussian
   write(message,'(a)')' using Taylor expansion '
   call wrtout(06,message,'COLL')
   do ig=1,npwsigx
    do igp=1,npwsigx
     do io=1,nomega 
      ct=(0.0,0.0)
      do iosf=my_wl,my_wr
       !if(iosf==io .or. iosf==io+1 .or. iosf==io-1) cycle
       ct= ct + kkweight(io,iosf)*chi0sf(ig,igp,iosf)
      end do
      chi0(ig,igp,io)= ct
     end do 
    end do
   end do 
  !call symf12(npwsigx,nomega,chi0,omega,nop,op,ngpi,gptab,gptabo,ip2fp,phgt,2)
  end if 
  !
  ! Collect from all the proc and add weight  
  !
  call leave_test(mpi_enreg)
  !call xsum_master(chi0,master,spaceComm,ierr)
  !this is safer, the size of the packet might exceed the max allowed by MPI
  !If I would be able to determine this max value then ... (ask MPI people)
  do io=1,nomega
   call xsum_master(chi0(:,:,io),master,spaceComm,ierr)
  end do 
  chi0(:,:,:)=weight*chi0(:,:,:)
 end if 

 ! Now we have chi0(q,G,Gp,omega)
 ! Impose Hermiticity (valid only for zero or purely imaginary omega)
 ! MG what about metals, where we have poles around zero?
 do io=1,nomega
  if (abs(real(omega(io)))<0.00001) then
   do igp=1,npwsigx
    do ig=1,igp-1
     chi0(igp,ig,io)=conjg(chi0(ig,igp,io))
    end do
   end do
  end if
 end do 

!DEBUG 
!write(*,*)'after hermit'
!call symf12(npwsigx,nomega,chi0,omega,nop,op,ngpi,gptab,gptabo,ip2fp,phgt,2) !only check
!ENDDEBUG

 ! Deallocate memory 
 if (allocated(wfnr  )) deallocate(wfnr)
 if (allocated(wfnrk )) deallocate(wfnrk)
 if (allocated(wfnrkp)) deallocate(wfnrkp)
 if (allocated(rhotwg)) deallocate(rhotwg)
 if (allocated(den   )) deallocate(den)  
 if (allocated(chi0sf)) deallocate(chi0sf)

 call timab(331,2,tsec)

 write(message,'(a)')' cchi0 : exit '
 call wrtout(06,message,'COLL')

 end subroutine cchi0
!!***
