!{\src2tex{textfont=tt}}
!!****f* ABINIT/cchi0q0
!! NAME
!! cchi0q0
!!
!! FUNCTION
!! Calculate chi0 in the limit q-->0
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (GMR, VO, LR, RWG, MG, RShaltaf)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  b1(3),b2(3),b3(3)=the three primitive vectors in reciprocal space
!!  ep= datatype gathering differening parameters related to the calculation of 
!!   the inverse dielectric matrix
!!  energy(nkibz,nbnds,nsppol)=KS energies
!!  fnl= MG FIXME should be described (put an equation) but in a nutshell is required to 
!!   apply the gradient of the commutator using real spherical harmonics
!!  gradvnl =(grad_K + grad_K') Vnl(K,K') in reciprocal lattice units
!!  grottb(npwvec,2,nop)= contains the index of (IS) G where I is either the identity or the inversion 
!!  grottbm1(npwvec,2,nop)=  contains the index  (IS^{-1}) G 
!!  gwenergy(nkibzm,nbnds,nsppol)=GW energies, for self-consistency purposes
!!  igfft(npwvec,5,5,5)=index of G-G0 planewaves (see cigfft routine)
!!  inclvkb=flag to include (or not) the grad of Vkb
!!  irottb(nr,nop)= contains the index in the FFT array of (R^-1)(r-t), 
!!   where R is the symmetry operations in real space corresponding to S_iop=\transpose R^-1
!!   and t the associate fractional translation 
!!  localrdwf= input variable (for parallel case) if 1, the KSS file is local to each machine
!!  lt_q= little group datatype
!!  mpi_enreg= datatype gathering information on parallelism, variables used 
!!   |gwpara= 0 no parallelism 
!!   |         1 parallelism is over k-points
!!   |         2 bands are spread btw processors
!!  nbvw=number of bands in the (optional) arrays wfrv,wfgv
!!  kbz(3,nkbz)=k-point coordinates, full Brillouin zone
!!  ktab(nkbz)= table giving for each k-point in the BZ (kBZ), the corresponding 
!!   irreducible point (kIBZ), where kBZ= (IS) kIBZ and I is either the inversion or the identity
!!  ktabi(nkbz)= for each point in the BZ defines whether inversion  has to be 
!!   considered in the relation kBZ=(IS) kIBZ (1 => only S; -1 => -S)  
!!  ktabo(nkbzmx)= the symmetry operation S that takes kIBZ to each kBZ
!!  ktabp(nkbz)= phase factor associated to tnons e^{-i 2 \pi k\cdot R{^-1}t}
!!  mpsang=1+maximum angular momentum for nonlocal pseudopotential
!!  natom=number of atoms 
!!  nbnds=number of bands
!!  nbv=number of valence bands
!!  ngfft1,ngfft1a,ngfft2,ngfft3=FFT grid dimensions
!!  nkbz=number of k points in full Brillouin zone
!!  nkibz=number of k points in irreducible Brillouin zone
!!  nkibzm=maximum number of k points in irreducible Brillouin zone
!!  nomega=number of frequencies
!!  nop=number of symmetry operations
!!  npwsigx=number of planewaves for sigma exchange (input variable)
!!  npwvec=dimension of igfft
!!  npwwfn=number of planewaves for wavefunctions (input variable)
!!  nr=number of points of FFT grid
!!  nsppol=1 for unpolarized, 2 for spin-polarized
!!  occ(nkibzm,nbnds,nsppol)=occupation numbers, for each k point in IBZ, and each band
!!  omega(nomega)=frequencies
!!  op(3,3,nop)=symmetry operations in reciprocal space
!!  qq(3)=reciprocal space coordinates of the q wavevector
!!  ucvol=unit cell volume
!!  wtk(nkibz)=weights for k points (input variable)
!!  kkweight(nomega,ep%nomegasf)=frequency dependent weights for Kramers Kronig transform
!!  phgt(npwsigx,nop)= phase factors e^{-iG \cdot t} needed to symmetrize oscillator matrix elements and epsilon 
!!  wfg(npwwfn,min_band_proc:max_band_proc,nkibz,nsppol)= (optional) wfs in real space FIXME (to be better described)
!!  wfr(nr,min_band_proc:max_band_proc,nkibz,nsppol)= (optional)  wfs in G space  FIXME (to be better described)
!!  wfgv(npwwfn,nbvw,nkibz,nsppol)= (optional) array containing fully and partially occupied states in G space
!!  wfrv(nr,nbvw,nkibz,nsppol) = (optional) array containing unoccupied states in real space
!!
!! OUTPUT
!!  chi0(npwsigx,npwsigx,nomega)=independent-particle susceptibility matrix for wavevector qq,
!!   and frequencies defined by omega
!!
!! NOTES
!!  The terms "head", "wings" and "body" of chi(G,Gp) refer to
!!  G=Gp=0, either G or Gp=0, and neither=0 respectively
!!
!! TODO 
!!  Remove a lot of input variables since they are contained in ep%
!!
!! PARENTS
!!      screening
!!
!! CHILDREN
!!      approxdelta,assemblychi0,assemblychi0q0_sym,assemblychi0sfq0,dosym
!!      leave_new,leave_test,matrginv,memerr,pclock,rho_tw_g,wrtout,xcast_mpi
!!      xcomm_init,xmaster_init,xme_init,xsum_master
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine cchi0q0(qq,ep,nomega,omega,gvec,npwvec,npwsigx,npwwfn,op,nop,kibz,&
& nkibz,nkibzm,nbnds,nbv,nbvw,nsppol,occ,ktab,ktabr,ktabi,ktabo,ktabp,kbz,nkbz,irottb,&
& ngfft1,ngfft1a,ngfft2,ngfft3,igfft,nr,energy,gwenergy,etadelta,chi0,b1,b2,b3,ucvol,&
& wtk,grottb,inclvkb,gradvnl,mpi_enreg,grottbm1,phgt,ngpi,gptab,gptabo,ip2fp,&
& lt_q,natom,mpsang,fnl,fnld,kkweight,min_band_proc,max_band_proc,nbnds_per_proc,localrdwf,&
& wfr,wfg,wfrv,wfgv) !optional

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_11util
 use interfaces_15gw, except_this_one => cchi0q0
 use interfaces_lib01hidempi
#else
 use defs_xfuncmpi
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: natom,mpsang
 integer,intent(in) :: inclvkb,localrdwf,max_band_proc,min_band_proc,nbnds,nbnds_per_proc,nbvw
 integer,intent(in) :: ngfft1,ngfft1a,ngfft2,ngfft3,ngpi,nkbz,nkibz,nkibzm,nomega
 integer,intent(in) :: nop,npwsigx,npwvec,npwwfn,nr,nsppol
 real(dp),intent(in) :: etadelta,ucvol
 type(MPI_type),intent(inout) :: mpi_enreg
 type(little_group),intent(in) :: lt_q
 type(epsilonm1_parameters),intent(in) :: ep
!arrays
 integer,intent(in) :: grottb(npwvec,2,nop),grottbm1(npwvec,2,nop)
 integer,intent(in) :: gptab(npwsigx,npwsigx),gptabo(npwsigx,npwsigx) 
 integer,intent(in) :: ip2fp(2,ngpi) 
 integer,intent(in) :: gvec(3,npwvec),igfft(npwvec,5,5,5),irottb(nr,nop)
 integer,intent(in) :: ktab(nkibzm),ktabi(nkibzm),ktabo(nkibzm)
 integer,intent(in) :: ktabr(nr,nkibzm),nbv(nsppol)
 real(dp),intent(in) :: b1(3),b2(3),b3(3)
 real(dp),intent(in) :: energy(nkibz,nbnds,nsppol)
 real(dp),intent(in) :: gwenergy(nkibz,nbnds,nsppol)
 real(dp),intent(in) :: kbz(3,nkibzm),kibz(3,nkibz)
 real(dp),intent(in) :: occ(nkibz,nbnds,nsppol)
 real(dp),intent(in) :: op(3,3,nop),qq(3),wtk(nkibz)
 complex,intent(in) :: gradvnl(3,npwwfn,npwwfn,nkibz),omega(nomega)
! FIXME new input variables, should be described
 complex,intent(in) :: fnl(npwwfn,mpsang*mpsang,natom,nkibz),fnld(3,npwwfn,mpsang*mpsang,natom,nkibz)
 complex,intent(in) :: ktabp(nkbz)
 complex,intent(in) :: phgt(npwsigx,nop)
 complex,intent(in),optional :: wfg(npwwfn,min_band_proc:max_band_proc,nkibz,nsppol)
 complex,intent(in),optional :: wfr(nr,min_band_proc:max_band_proc,nkibz,nsppol)
 complex,intent(in),optional :: wfgv(npwwfn,nbvw,nkibz,nsppol)
 complex,intent(in),optional :: wfrv(nr,nbvw,nkibz,nsppol)
 complex,intent(in),optional :: kkweight(nomega,ep%nomegasf) !FIXME Do I really need the opt intent?
 complex,intent(out) :: chi0(npwsigx,npwsigx,nomega)

!Local variables ------------------------------
!scalars
 integer,parameter :: unitwfg=25,unitwfr=26
 integer :: ib,ibc,ibv,ig,igp,iinv,ik,ikbz,ikibz,io,iop,ir,is,istat
 integer :: jb_proc_rank,max_con,max_val,min_con,min_val,nrb,nkibzq,tim_fourdp
 integer :: iat,ilm,ii
 integer :: master,spaceComm,ierr,me,my_wl,my_wr
 real(dp) :: domega,domega_min,domega_max,f_occ,factocc,weight
 complex(dpc) :: ediff,egwdiff
 complex :: ct,ct2,cta1,cta4,tempct
!MG the if statements related to these logical variables have been commented
 logical,parameter :: onlyantiresonant=.false.,noantiresonant=.false.
 logical :: i_read,i_cast,is_metal,use_tr
 logical :: wfrg_in_mem
 character(len=500)::message
!arrays
 real(dp) :: opinv(3,3),qcart(3),qrot(3)
 complex :: cta2(3),cta3(3),rhotwx(3)
 complex,allocatable :: rhotwg(:),wfg_k(:,:)
 complex,allocatable :: wfng(:,:),wfnrk(:,:),wfr_k(:,:)
 complex,allocatable :: wfr_v(:,:)
 complex,allocatable :: fnlkslm(:),fnldkslm(:,:)
 complex(dpc),allocatable :: den(:)

! This is for the spectral method
 integer :: iomegal,iomegar,iosf,method
 real(dp) :: omegal,omegar,wl,wr,smear
 complex :: num
 complex,allocatable :: chi0sf(:,:,:)
!END SPECTRAL

! *************************************************************************
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 complex :: dotproductqrc
#endif
!End of the abilint section

 write(message,'(3a)')ch10,' cchi0q0 : enter',ch10 
 call wrtout(06,message,'COLL')

 write(6,*)' dimension rhotwx=',size(rhotwx,1)

 ! Initialize MPI related quantities
 call xcomm_init(mpi_enreg,spaceComm) !Init mpi_comm
 call xme_init(mpi_enreg,me)          !Init me
 call xmaster_init(mpi_enreg,master)  !Init master

 ! Organize the I/O of proc
 i_read=.false. ; i_cast=.false.
 if (me==0 .or. localrdwf==1) i_read=.true.  !here there is a possible crash since default is 1
 if (mpi_enreg%nproc>1 .and. localrdwf==0) i_cast=.true.

 if ((     present(wfr) .and. .not.present(wfg)) .or.& 
&    (.not.present(wfr) .and.      present(wfg))) then
  write(message,'(5a)')ch10,&
&  ' cchi0q0 : BUG -',ch10,&
&  ' if wfr is passed in input then wfg must be present ',ch10
  call wrtout(6,message,'COLL') ; call leave_new('COLL')
 end if

 if ((     present(wfrv) .and. .not.present(wfgv)) .or.& 
&    (.not.present(wfrv) .and.      present(wfgv))) then
  write(message,'(5a)')ch10,&
&  ' cchi0q0 : BUG -',ch10,&
&  ' if wfrv is passed in input then wfgv must be present ',ch10
  call wrtout(6,message,'COLL') ; call leave_new('COLL')
 end if

 wfrg_in_mem=.false.
 if (present(wfr) .and. present(wfg)) then   
  wfrg_in_mem=.true.
  write(message,'(2a)')ch10,' wavefunctions in r and G space are in memory'
  call wrtout(06,message,'COLL')
 end if 

 min_val=1 ; max_val=nbnds
 min_con=1 ; max_con=nbnds

 use_tr=.false.
 if (present(wfrv) .and. present(wfgv)) then 
  ! Faster algorithm based on time reversal (at the moment only in para run)
  ! Note that special care has to be used in case of metals, moreover keep in mind
  ! that wfrv could also contain unoccupied states 
  use_tr=.true.
  write(message,'(5a)')ch10,&
&  ' valence states in real and reciprocal space are in memory',ch10,&
&  ' using faster equation based on time reversal symmetry',ch10
  call wrtout(06,message,'COLL')
  min_val=1      ; max_val=nbvw         
  min_con=nbvw+1 ; max_con=nbnds
 end if 

 if (me==0) then 
  write(6,*)'symmetrization flag = ',lt_q%sym_flag
  write(6,*)'use time reversal   = ',use_tr
  write(6,*)'spectral method     = ',ep%spmeth
 end if 

 tim_fourdp=1
 nrb=ngfft1*ngfft2*ngfft3
 
 ! Setup weight (2 for spin unpolarized systems, 1 for polarized)
 ! f_occ is set outside the loops and is used to normalize the occupation factors to one
 if (nsppol==1) then 
  weight=2.0/nkbz ; f_occ=0.5
 else if (nsppol==2) then 
  weight=1.0/nkbz ; f_occ=1.0
 else 
  write(message,'(2a)')ch10,&
& ' cchi0: BUG wrong value for nsppol '
  call wrtout(6,message,'COLL') ; call leave_new('COLL')
 end if

 if (.not.(wfrg_in_mem)) then
  write(message,'(3a)')ch10,&
&  ' cchi0q0 : wavefunctions are not in memory, reading from file',ch10
  call wrtout(6,message,'COLL') 
  allocate(wfng(npwwfn,nbnds),stat=istat)
   if(istat/=0) call memerr('chi0q0','wfng',npwwfn*nbnds,'spc')
  allocate(wfnrk(nr,nbnds),stat=istat)
   if(istat/=0) call memerr('chi0q0','wfnrk',nr*nbnds,'spc')
 end if

 allocate(rhotwg(npwsigx),stat=istat)
  if(istat/=0) call memerr('chi0q0','rhotwg',npwsigx,'spc')

 if (ep%spmeth==0) then 
  allocate(den(nomega))
  write(message,'(2a)')' calculating chi0(q=(0,0,0),omega,G,G")',ch10
  call wrtout(06,message,'COLL')
 else 
  write(message,'(2a)')' calculating Im chi0(q=(0,0,0),omega,G,G")',ch10
  call wrtout(06,message,'COLL')
  my_wl=1 ; my_wr=ep%nomegasf
  if (mpi_enreg%gwpara==2) then 
   write(message,'(3a)')ch10,' cchi0q0 : distributing frequencies between processors ',ch10
   call wrtout(06,message,'COLL')
   if (.not.(present(wfrv) .and. present(wfgv)))then 
    write(message,'(a)')' valence wfs in r-space are not in memory, not coded yet'
    call wrtout(06,message,'COLL') ; call leave_new('COLL')
   end if 
   !
   ! Calculate min and max transition energy for this processor
   ! NOTE : in metals the contribution coming from the fully and partially 
   ! occupied stated is calculated at the end, the contribution due to the 
   ! fully unoccupied states is negligible and is not evaluated (check the threshold)
   !
   domega_min=  huge(0.0_dp)
   domega_max= -huge(0.0_dp)
   !do ikibz = 1,nkibz ! This approach does not work, dont know why 
   !kmq(:) = kibz(:,ikibz)-qq(:)
   do ik=1,nkbz
    ikibz=ktab(ik)
    do ibc=nbvw+1,nbnds           
     do ibv=1,nbvw
      do is=1,nsppol
       if ( (mpi_enreg%proc_distrb(ikibz,ibc,is)-mpi_enreg%me)==0 .and. &
&           (mpi_enreg%proc_distrb(ikibz,ibv,is)-mpi_enreg%me)==0) then 
        domega= gwenergy(ikibz,ibc,is)-gwenergy(ikibz,ibv,is)
        domega_min= min(domega_min,domega)
        domega_max= max(domega_max,domega)
       end if
      end do 
     end do 
    end do
   end do
   my_wr=-999
   do io=1,ep%nomegasf 
    ! The array is dimensioned using nomegasf+1
    if (ep%omegasf(io)>domega_max) then
     my_wr=io+1 
     exit
    end if 
   end do
   if (my_wr==ep%nomegasf+1) my_wr=ep%nomegasf
   my_wl=-999
   do io=ep%nomegasf,1,-1
    if (ep%omegasf(io)<=domega_min) then  !check metals
     my_wl=io
     exit
    end if 
   end do
  end if 
!FIXME this is for metallic systems (presently only master works)
  if (me==0) my_wl=1
!FIXME
  write(message,'(2a,2i5,a,i4,3a,f8.2,3a,f8.2,2a)')ch10,&
&  ' cchi0q0: allocating chi0sf using my_wl and my_wr = ',&
&  my_wl,my_wr,' ( ',my_wr-my_wl+1,' )',ch10,&
&  '  domega_min = ',domega_min*Ha_eV,' [eV] ',ch10,&
   '  domega_max = ',domega_max*Ha_eV,' [eV] ',ch10
  call wrtout(06,message,'PERS')
  allocate(chi0sf(npwsigx,npwsigx,my_wl:my_wr),stat=istat)
   if(istat/=0) call memerr('cchi0q0','chi0sf',npwsigx**2*(my_wr-my_wl+1)*ep%nomegasf,'spc')
  chi0sf=(0.,0.)
!WARNING in case of metals I need to be sure that master has a sufficiently large my_wr
!moreover my_wl must be 1 
 end if 

 if (inclvkb==2) allocate(fnlkslm(npwwfn),fnldkslm(3,npwwfn),stat=istat)
  if (istat/=0) call memerr('chi0q0','fnlkslm fnldkslm',4*npwwfn,'spc')

 nkibzq=nkbz
 if (lt_q%sym_flag/=0) nkibzq=sum(lt_q%ibzq(:))
 write(message,'(a,i6,a)')' calculation status ( ',nkibzq,' to be completed) :'
 call wrtout(06,message,'COLL')

 chi0(:,:,:)=(0.,0.) 
 !
 ! Loop over k-points in BZ
 !
 do ikbz=1,nkbz 
  !
  ! If we use symmetries then sum only over k in the IBZ_q 
  ! this coding is safer since lt_q%ibzq(ik) might be not allocated 
  !
  if (lt_q%sym_flag/=0) then ; if (lt_q%ibzq(ikbz)/=1) cycle ; end if
  !
  ikibz=ktab(ikbz)
  iop=ktabo(ikbz)
  iinv=(3-ktabi(ikbz))/2
  !
  ! Loop over spin calculate polarizability: $\chi_{up,up} + \chi_{down,down}$
  !
  do is=1,nsppol
   ! Parallelization over k-points in the full BZ
   ! Note that spin para is not active but the check should be done here
   ! array proc_distrb does not depend on band index (there is a check in screening)
   if (mpi_enreg%nproc>1 .and. mpi_enreg%gwpara==1) then
    if ( (mpi_enreg%proc_distrb(ikbz,1,is)-mpi_enreg%me)/=0 ) cycle
!DEBUG for the time being do not comment
    if (any(mpi_enreg%proc_distrb(ikbz,:,is)==-999)) then 
     write(*,*)'cchi0q0 : wrong distrb' 
     call leave_new('COLL')
    end if 
!ENDDEBUG
   end if
   write(message,'(a,i6,a,i6,a,i2,a,i6)')&
&   ' cchi0q0 : ik = ',ikbz,' / ',nkbz,'   is = ',is,'  done by processor ',mpi_enreg%me
   call wrtout(06,message,'COLL')

   if (.not.(wfrg_in_mem)) then
    ! Read wfn(G) and wfn(r) for the associated ibz k-point and spin
    ! If we want to use time reversal then occupied states must be passed as input while 
    ! unoccupied states can be read from disk
    if (i_read) then
     read(unitwfg,rec=ikibz+nkibz*(is-1)) ((wfng(ig,ib),ig=1,npwwfn),ib=1,nbnds)
     read(unitwfr,rec=ikibz+nkibz*(is-1)) ((wfnrk(ir,ib),ir=1,nr),ib=1,nbnds)
    end if
    if (i_cast) then
     call xcast_mpi(wfng,master,spaceComm,ierr)
     call xcast_mpi(wfnrk,master,spaceComm,ierr)
    end if
   end if
   !
   ! Conventions: 1) a symmetry in real space acts as R_t f(r) = f(R^-1(r-t))
   !              2) S=\transpose R^-1 
   !              3) kbz=S kibz 
   !
   !  The wavefunctions for the k-point in the BZ are:
   !
   !  u(G,b,kbz)= u (S^-1G,b,kibz)* e^{-i(kbz+G)*t) 
   !            = u*(S^-1G,b,kibz)* e^{i(kbz+G)*t) for inversion
   !  u(r,b,kbz)= e^{-ikbz*t} u (R^-1(r-t),b,kibz) 
   !            = e^{ ikbz*t} u*(R^-1(r-t),b,kibz) for inversion
   !
   !  The gradient of Vnl(K,K'') for the k-point in the BZ should be:
   !   gradvnl(G,Gp,kbz)=R gradvnl(R^-1G,R^-1Gp,kibz)

!MG FIXME should check the expression in case of non zero tnons but Im lazy now 
   do ibc=min_con,max_con
    if (mpi_enreg%nproc>1 .and. mpi_enreg%gwpara==2) then
     if (minval(abs(mpi_enreg%proc_distrb(:,ibc,is)-mpi_enreg%me))/=0) cycle
    end if     
    do ibv=min_val,max_val
     if (use_tr .and. ep%spmeth==0) then 
      factocc= f_occ*(occ(ikibz,ibv,is)-occ(ikibz,ibc,is))
     else 
      factocc=f_occ*(occ(ikibz,ibc,is)-occ(ikibz,ibv,is))
     end if       
     ! Continue the loop only if factocc !=0
     if (abs(factocc)<0.01) cycle
     !if (noantiresonant .and. factocc<tol8) cycle
     !if (onlyantiresonant .and. factocc>tol8) cycle
     if (use_tr .and. ep%spmeth==0) then
      ediff   =   energy(ikibz,ibv,is) -  energy(ikibz,ibc,is)
      egwdiff = gwenergy(ikibz,ibv,is) -gwenergy(ikibz,ibc,is)
     else
      ediff   =   energy(ikibz,ibc,is) -   energy(ikibz,ibv,is)
      egwdiff = gwenergy(ikibz,ibc,is) - gwenergy(ikibz,ibv,is)
     end if
     !
     ! Spectral method
     !
     if (ep%spmeth/=0) then
      if (real(egwdiff)<0) cycle
      call approxdelta(ep%nomegasf+1,ep%omegasf(1:ep%nomegasf+1),real(egwdiff),&
&      ep%spsmear,iomegal,iomegar,wl,wr,ep%spmeth)
     else 
      !
      ! Standard Implementation
      ! Add the small imaginary of the T-O response function only for non zero real frequencies
      ! MG What about metals?
      ! removed etadelta in case of imaginary omegas, now results in parallel and sequential agree
      ! there are differences in the third digit for large matrix elements but this might be due to 
      ! single precision and summation over procs and/or aliasing problems 
      ! it required modification to deal with metals
      !
      if (use_tr) then
       do io=1,nomega 
        if (real(omega(io))>0.001) then 
         den(io)= factocc/(omega(io)+egwdiff-(0.,1.)*egwdiff/abs(egwdiff)*etadelta)&
&                -factocc/(omega(io)-egwdiff+(0.,1.)*egwdiff/abs(egwdiff)*etadelta)
        else 
         den(io)= factocc/(omega(io)+egwdiff)&
&                -factocc/(omega(io)-egwdiff)
        end if 
       end do
      else
       do io=1,nomega
        if (real(omega(io))>0.001) then
         den(io)=factocc/(omega(io)+egwdiff-(0.,1.)*egwdiff/abs(egwdiff)*etadelta)
        else
         den(io)=factocc/(omega(io)+egwdiff)
        end if
       end do
      end if
     end if !of ep%spmeth
     !
     ! Calculate rho-twiddle(G)
     ! MG Maybe there is a more clever way to rewrite all this stuff 
     ! Remember that modifications are needed for metals
     !
     if (wfrg_in_mem) then
      if (use_tr .and. ep%spmeth==0) then !note that here valence is on the left side
       call rho_tw_g(npwsigx,nr,nrb,ngfft1a,ngfft1,ngfft2,ngfft3,igfft(:,3,3,3),          &
&                    wfrv(:,ibv,ikibz,is),iinv,ktabr(:,ikbz),ktabp(ikbz),                 &
&                    wfr (:,ibc,ikibz,is),iinv,ktabr(:,ikbz),ktabp(ikbz),rhotwg,tim_fourdp)
      else if (ep%spmeth/=0 .and. mpi_enreg%gwpara==2) then 
       call rho_tw_g(npwsigx,nr,nrb,ngfft1a,ngfft1,ngfft2,ngfft3,igfft(:,3,3,3),          &
&                    wfr (:,ibc,ikibz,is),iinv,ktabr(:,ikbz),ktabp(ikbz),                 &
&                    wfrv(:,ibv,ikibz,is),iinv,ktabr(:,ikbz),ktabp(ikbz),rhotwg,tim_fourdp)
      else
       call rho_tw_g(npwsigx,nr,nrb,ngfft1a,ngfft1,ngfft2,ngfft3,igfft(:,3,3,3),          &
&                    wfr(:,ibc,ikibz,is),iinv,ktabr(:,ikbz),ktabp(ikbz),                  &
&                    wfr(:,ibv,ikibz,is),iinv,ktabr(:,ikbz),ktabp(ikbz),rhotwg,tim_fourdp)
      end if
     else ! if they were read from disk file..
      call rho_tw_g(npwsigx,nr,nrb,ngfft1a,ngfft1,ngfft2,ngfft3,igfft(:,3,3,3),   &
&                   wfnrk(:,ibc),iinv,ktabr(:,ikbz),ktabp(ikbz),                  &
&                   wfnrk(:,ibv),iinv,ktabr(:,ikbz),ktabp(ikbz),rhotwg,tim_fourdp)
     end if 
     !
     ! Set up rho-twiddle(G=0) using small vector q instead of zero and k.p perturbation theory
     !
     rhotwx(:)=(0.0,0.0)
     !FIXME Check these equation in case of band parallelsim and spectral
     ! First term -iq \cdot <c,k|\nabla|v,k>, Cancellation of phases in case of non-symmorphic operations
     do ig=1,npwwfn
      if (wfrg_in_mem) then
       if (use_tr .and. ep%spmeth==0) then
        !MG here I have inverted conduction and valence to use the equation with time reversal symmetry
        ct=conjg(wfgv(ig,ibv,ikibz,is))*wfg(ig,ibc,ikibz,is)
       else if (ep%spmeth/=0 .and. mpi_enreg%gwpara==2) then 
        !Added treatment in case of spectral method
        ct=conjg(wfg(ig,ibc,ikibz,is))*wfgv(ig,ibv,ikibz,is)
       else
        ct=conjg(wfg(ig,ibc,ikibz,is))*wfg(ig,ibv,ikibz,is)
       end if
      else
       ct=conjg(wfng(ig,ibc))*wfng(ig,ibv)
      end if
      rhotwx(:)=rhotwx(:)+gvec(:,ig)*ct
     end do !ig
     !
     ! Second term <c,k|[Vnl,iqr]|v,k>
     !FIXME check the phase in case of simmorphic 
     !FIXME check the order in case of spectral method
     !
     if (inclvkb==1) then
!DEBUG
!     write(6,*)' dimensions gradvnl :',size(gradvnl,1),size(gradvnl,2),size(gradvnl,3),size(gradvnl,4)
!     write(6,*)' ikibz=',ikibz
!     write(6,*)' npwwfn=',npwwfn
!     write(6,*)' dimension rhotwx=',size(rhotwx,1)
!     write(6,*)' present(wfg),present(wfgv)=',present(wfg),present(wfgv)
!ENDDEBUG
      if (wfrg_in_mem) then
       if (use_tr .and. ep%spmeth==0) then
        do ig=1,npwwfn
         do igp=1,npwwfn
          ct=conjg(wfgv(ig,ibv,ikibz,is))*wfg(igp,ibc,ikibz,is)
          rhotwx(:)=rhotwx(:)+ct*gradvnl(:,ig,igp,ikibz)
         enddo
        enddo
       else if (ep%spmeth/=0 .and. mpi_enreg%gwpara==2) then
        do ig=1,npwwfn
         do igp=1,npwwfn
          ct=conjg(wfg(ig,ibc,ikibz,is))*wfgv(igp,ibc,ikibz,is)
          rhotwx(:)=rhotwx(:)+ct*gradvnl(:,ig,igp,ikibz)
         enddo
        enddo
       else
        do ig=1,npwwfn
         do igp=1,npwwfn
          ct=conjg(wfg(ig,ibc,ikibz,is))*wfg(igp,ibv,ikibz,is)
          rhotwx(:)=rhotwx(:)+ct*gradvnl(:,ig,igp,ikibz)
         enddo
        enddo
       endif
      else
       do ig=1,npwwfn
        do igp=1,npwwfn
         ct=conjg(wfng(ig,ibc))*wfng(igp,ibv)
         rhotwx(:)=rhotwx(:)+ct*gradvnl(:,ig,igp,ikibz)
        enddo
       enddo
      endif

!      do ig=1,npwwfn
!       do igp=1,npwwfn
!        if (wfrg_in_mem) then
!         if (use_tr .and. ep%spmeth==0) then
!!MG Also here I have inverted conduction and valence bands to use the equation with time reversal sym
!          ct=conjg(wfgv(ig,ibv,ikibz,is))*wfg(igp,ibc,ikibz,is)
!         else if (ep%spmeth/=0 .and. mpi_enreg%gwpara==2) then 
!!Added treatment in case of spectral method
!          ct=conjg(wfg(ig,ibc,ikibz,is))*wfgv(igp,ibc,ikibz,is)
!         else
!          ct=conjg(wfg(ig,ibc,ikibz,is))*wfg(igp,ibv,ikibz,is)
!         end if
!        else
!         ct=conjg(wfng(ig,ibc))*wfng(igp,ibv)
!        end if
!        !Must multiply by ucvol because yet gradvnl contain 1/ucvol
!        rhotwx(:)=rhotwx(:)+ct*gradvnl(:,ig,igp,ikibz)
!       end do
!      end do

     else if (inclvkb==2) then
      ! New treatment of the gradient of the non local operator 
      ! should be possible to speed up by considering only ntypat
      ! and only the kb projectors used for each type of atom  
      ! note that inclvkb==2 should be the default approach (much much faster!)
      ! XG070103 : not yet compatible with parallelism over bands ...
      ! parallelism over bands for this option is fixed by Rshaltaf 120207 
      !FIXME check the phase in case of simmorphic 
      !FIXME check in case of  gwpara==2 
      do iat=1,natom 
       do ilm=1,mpsang*mpsang 
        fnlkslm(:)=fnl(:,ilm,iat,ikibz)
        fnldkslm(:,:)=fnld(:,:,ilm,iat,ikibz) 
        cta1=(0.,0.) ; cta2(:)=(0.,0.)
        cta4=(0.,0.) ; cta3(:)=(0.,0.) 
        ! Here we take advantage of the property Y_(l-m)=(-i^m)Y_lm^* 
        !FIXME Here I have to invert conduction and valence but I have to recheck all the equations 
        do ig=1,npwwfn
         if (wfrg_in_mem) then
          if (use_tr .and. ep%spmeth==0) then !note that here valence is on the left side
           cta1=    cta1    + wfgv(ig,ibv,ikibz,is) * fnlkslm(ig)
           cta2(:)= cta2(:) + wfg (ig,ibc,ikibz,is) * fnldkslm(:,ig)
           cta3(:)= cta3(:) + wfgv(ig,ibv,ikibz,is) * fnldkslm(:,ig)
           cta4=    cta4    + wfg (ig,ibc,ikibz,is) * fnlkslm(ig)
          else if (ep%spmeth/=0 .and. mpi_enreg%gwpara==2) then 
           cta1=    cta1    + wfg (ig,ibc,ikibz,is) * fnlkslm(ig)
           cta2(:)= cta2(:) + wfgv(ig,ibv,ikibz,is) * fnldkslm(:,ig)
           cta3(:)= cta3(:) + wfg (ig,ibc,ikibz,is) * fnldkslm(:,ig)
           cta4=    cta4    + wfgv(ig,ibv,ikibz,is) * fnlkslm(ig)
          else
           cta1=    cta1    + wfg(ig,ibc,ikibz,is) * fnlkslm(ig)
           cta2(:)= cta2(:) + wfg(ig,ibv,ikibz,is) * fnldkslm(:,ig)
           cta3(:)= cta3(:) + wfg(ig,ibc,ikibz,is) * fnldkslm(:,ig)
           cta4=    cta4    + wfg(ig,ibv,ikibz,is) * fnlkslm(ig)
          end if 
         else ! if they were read from disk file..
          !this is ok
          cta1=    cta1    + wfng(ig,ibc) * fnlkslm(ig)
          cta2(:)= cta2(:) + wfng(ig,ibv) * fnldkslm(:,ig)
          cta3(:)= cta3(:) + wfng(ig,ibc) * fnldkslm(:,ig)
          cta4=    cta4    + wfng(ig,ibv) * fnlkslm(ig)
         end if
        end do !ig
        rhotwx(:)= rhotwx(:) + conjg(cta1)*cta2(:)+conjg(cta3(:))*cta4
       end do !ilm
      end do !iat
     end if

     if (ep%spmeth/=0) then 
      !
      ! Spectral method, to be consistent here we use the KS eigenvalues
      !
      rhotwx(:)= -rhotwx(:)/ediff
      if (lt_q%sym_flag==0) then !no symmetries 
       !FIXME this should be done inside assemblychi0sfq0
       !rhotwg(1)= R^-1q*rhotwx_ibz
       !rhotwg(1)=-R^-1q*conjg(rhotwx_ibz) for inversion
!FIXME My equation reads as -iq \cdot <cSk|\nabla|vSk> = -i \transpose S q \cdot <ck|\nabla\|vk>  
       opinv(:,:)=op(:,:,iop)
       call matrginv(opinv,3,3)
       call dosym(opinv,iinv,qq,qrot)
       rhotwg(1)=dotproductqrc(qrot,rhotwx,b1,b2,b3)
       if(iinv==2) rhotwg(1)=conjg(rhotwg(1))
      end if 
      call assemblychi0sfq0(qq,npwvec,npwwfn,npwsigx,lt_q%ninv,lt_q%nop,lt_q%op,&
&      lt_q%nkbz,lt_q%wtksym(:,:,ikbz),phgt,grottbm1,factocc,my_wl,iomegal,wl,  &
&      my_wr,iomegar,wr,lt_q%sym_flag,rhotwx,b1,b2,b3,rhotwg,ep%nomegasf,chi0sf)
     else 
      !
      ! Traditional method, to be consistent here we use the KS eigenvalues
      !
      rhotwx(:)= -rhotwx(:)/ediff
      if (lt_q%sym_flag==0) then 
       !
       ! No symmetries : sum over the full BZ accumulating 
       ! chi0(G,Gp,io)=chi0(G,Gp,io)+(rhotwg(G)*rhotwg(Gp))*den(io)
       ! but symmetrizing the non-analytic term:
       !  rhotwg(1)= R^-1q*rhotwx_ibz
       !  rhotwg(1)=-R^-1q*conjg(rhotwx_ibz) for inversion
       !
!FIXME My equation reads as -iq \cdot <cSk|\nabla|vSk> = -i \transpose S q \cdot <ck|\nabla\|vk>  
       opinv(:,:)=op(:,:,iop)
       call matrginv(opinv,3,3)
       call dosym(opinv,iinv,qq,qrot)
       rhotwg(1)=dotproductqrc(qrot,rhotwx,b1,b2,b3)
       if(iinv==2) rhotwg(1)=conjg(rhotwg(1))
       call assemblychi0(npwsigx,nomega,chi0,rhotwg,den)
      else  
       ! Use symmetries to reconstruct the full BZ
       ! Reconstruct also the non-analytic term
       call assemblychi0q0_sym(qq,npwvec,npwwfn,npwsigx,lt_q%ninv,lt_q%nop,lt_q%op,&
&       lt_q%nkbz,lt_q%wtksym(:,:,ikbz),phgt,grottbm1,nomega,chi0,rhotwx,rhotwg,den,b1,b2,b3) 
      end if  
     end if !of spectral_flag
     !
    end do !ibv
   end do !ibc
  end do !is
 end do !ikibz

 if (ep%spmeth==0) then 
  if (.not. use_tr) then 
   !
   ! Use old slow method. Sum all the contributions from the different procs
   !
   call xsum_master(chi0,master,spaceComm,ierr)
   chi0(:,:,:)=weight*chi0(:,:,:)
  else 
   ! 
   ! Use equation based on time-reversal 
   ! Calculate the contribution coming from the fully and partial occupied states
   !
   ! For the moment only master works
   is_metal=.false.
   if (me==0) then 
    do ikbz=1,nkbz 
     if (lt_q%sym_flag/=0) then ; if (lt_q%ibzq(ikbz)/=1) cycle ; end if
     ikibz=ktab(ikbz)
     iop=ktabo(ikbz)
     iinv=(3-ktabi(ikbz))/2
     do is=1,nsppol
      do ibc=min_val,max_val
       ! Note starting point for ibv!!
       do ibv=ibc,max_val
        factocc= f_occ*(occ(ikibz,ibv,is)-occ(ikibz,ibc,is))
        if (abs(factocc)<0.01) cycle
        is_metal=.true.
        ediff   =   energy(ikibz,ibv,is) -  energy(ikibz,ibc,is)
        egwdiff = gwenergy(ikibz,ibv,is) -gwenergy(ikibz,ibc,is)
        do io=1,nomega 
         if (real(omega(io))>0.001) then 
          den(io)= factocc/(omega(io)+egwdiff-(0.,1.)*egwdiff/abs(egwdiff)*etadelta)&
&                 -factocc/(omega(io)-egwdiff+(0.,1.)*egwdiff/abs(egwdiff)*etadelta)
         else 
          den(io)= factocc/(omega(io)+egwdiff)&
&                 -factocc/(omega(io)-egwdiff)

         end if 
        end do
        ! this is to avoid the double counting, not necessary but oh well
        if (ibv==ibc) den=den*half
        call rho_tw_g(npwsigx,nr,nrb,ngfft1a,ngfft1,ngfft2,ngfft3,igfft(:,3,3,3),          &
&                     wfrv(:,ibv,ikibz,is),iinv,ktabr(:,ikbz),ktabp(ikbz),                 &
&                     wfrv(:,ibc,ikibz,is),iinv,ktabr(:,ikbz),ktabp(ikbz),rhotwg,tim_fourdp)

        rhotwx(:)=(0.0,0.0)
        !FIXME Check these equation in case of band parallelsim and spectral
        ! First term -iq \cdot <c,k|\nabla|v,k>, Cancellation of phases in case of non-symmorphic operations
        do ig=1,npwwfn
         ct=conjg(wfgv(ig,ibv,ikibz,is))*wfgv(ig,ibc,ikibz,is)
         rhotwx(:)=rhotwx(:)+gvec(:,ig)*ct
        end do
! FIXME WARNING Second term <c,k|[Vnl,iqr]|v,k> is not included yet, can make 

        rhotwx(:)=-rhotwx(:)/ediff
        if (lt_q%sym_flag==0) then 
         !
         ! No symmetries : sum over the full BZ accumulating 
         ! chi0(G,Gp,io)=chi0(G,Gp,io)+(rhotwg(G)*rhotwg(Gp))*den(io)
         ! but symmetrizing the non-analytic term:
         !  rhotwg(1)= R^-1q*rhotwx_ibz
         !  rhotwg(1)=-R^-1q*conjg(rhotwx_ibz) for inversion
         !
!FIXME My equation reads as -iq \cdot <cSk|\nabla|vSk> = -i \transpose S q \cdot <ck|\nabla\|vk>  
         opinv(:,:)=op(:,:,iop)
         call matrginv(opinv,3,3)
         call dosym(opinv,iinv,qq,qrot)
         rhotwg(1)=dotproductqrc(qrot,rhotwx,b1,b2,b3)
         if(iinv==2) rhotwg(1)=conjg(rhotwg(1))
         call assemblychi0(npwsigx,nomega,chi0,rhotwg,den)
        else  
         ! Use symmetries to reconstruct the full BZ
         ! Reconstruct also the non-analytic term
         call assemblychi0q0_sym(qq,npwvec,npwwfn,npwsigx,lt_q%ninv,lt_q%nop,lt_q%op,&
&         lt_q%nkbz,lt_q%wtksym(:,:,ikbz),phgt,grottbm1,nomega,chi0,rhotwx,rhotwg,den,b1,b2,b3) 
        end if  
       end do !ibv
      end do !ibc
     end do !is
    end do !ikibz
    if (is_metal) then 
     write(message,'(2a)')ch10,&
&     ' cchi0q0 : system is metallic, metallic contribution added'
     call wrtout(06,message,'COLL')
    end if
   end if ! of me==0
   call leave_test(mpi_enreg)
   call xsum_master(chi0,master,spaceComm,ierr)
   chi0(:,:,:)=weight*chi0(:,:,:)
  end if !of use_tr
 else
  ! Only master calculates the contribution in metallic/spin polarized systems 
  ! first coding
  if (me==0 .and. mpi_enreg%gwpara==2 .and. ep%spmeth/=0) then 
   do ikbz=1,nkbz 
    if (lt_q%sym_flag/=0) then ; if (lt_q%ibzq(ikbz)/=1) cycle ; end if
    ikibz=ktab(ikbz)
    iop=ktabo(ikbz)
    iinv=(3-ktabi(ikbz))/2
    do is=1,nsppol
     do ibc=min_val,max_val
      do ibv=min_val,max_val
       factocc=f_occ*(occ(ikibz,ibc,is)-occ(ikibz,ibv,is))
       if (abs(factocc)<0.01) cycle
       ediff   =   energy(ikibz,ibc,is) -   energy(ikibz,ibv,is)
       egwdiff = gwenergy(ikibz,ibc,is) - gwenergy(ikibz,ibv,is)
       if (real(egwdiff)<0) cycle
       call approxdelta(ep%nomegasf+1,ep%omegasf(1:ep%nomegasf+1),real(egwdiff),&
&       ep%spsmear,iomegal,iomegar,wl,wr,ep%spmeth)
       !write(*,*)'metallic system',wl,wr
       call rho_tw_g(npwsigx,nr,nrb,ngfft1a,ngfft1,ngfft2,ngfft3,igfft(:,3,3,3),          &
&                    wfrv(:,ibc,ikibz,is),iinv,ktabr(:,ikbz),ktabp(ikbz),                 &
&                    wfrv(:,ibv,ikibz,is),iinv,ktabr(:,ikbz),ktabp(ikbz),rhotwg,tim_fourdp)
       rhotwx(:)=(0.0,0.0)
       do ig=1,npwwfn
        ct=conjg(wfgv(ig,ibc,ikibz,is))*wfgv(ig,ibv,ikibz,is)
        rhotwx(:)=rhotwx(:)+gvec(:,ig)*ct
       end do ! ig
!FIXME add treatment of the Second term <c,k|[Vnl,iqr]|v,k>
       rhotwx(:)= -rhotwx(:)/ediff
       !write(77,*)rhotwx
       if (lt_q%sym_flag==0) then !no symmetries 
        !FIXME this should be done inside assemblychi0sfq0
        !rhotwg(1)= R^-1q*rhotwx_ibz
        !rhotwg(1)=-R^-1q*conjg(rhotwx_ibz) for inversion
!FIXME My equation reads as -iq \cdot <cSk|\nabla|vSk> = -i \transpose S q \cdot <ck|\nabla\|vk>  
        opinv(:,:)=op(:,:,iop)
        call matrginv(opinv,3,3)
        call dosym(opinv,iinv,qq,qrot)
        rhotwg(1)=dotproductqrc(qrot,rhotwx,b1,b2,b3)
        if(iinv==2) rhotwg(1)=conjg(rhotwg(1))
        !write(78,*)rhotwg
       end if 
       call assemblychi0sfq0(qq,npwvec,npwwfn,npwsigx,lt_q%ninv,lt_q%nop,lt_q%op,lt_q%nkbz,&
&       lt_q%wtksym(:,:,ikbz),phgt,grottbm1,factocc,my_wl,iomegal,wl,my_wr,iomegar,wr,    &
&       lt_q%sym_flag,rhotwx,b1,b2,b3,rhotwg,ep%nomegasf,chi0sf)
       end do !ibv
      end do !ibc
     end do !is
    end do !ikibz
   end if 

  call leave_test(mpi_enreg)
  write(message,'(2a)')ch10,&
&  ' cchi0q0 : performing Hilbert transform...'
  call wrtout(06,message,'COLL')
 
  !call xsum_master(chi0sf,master,spaceComm,ierr)
  !note there are small difference btw para and seq over k-point 
  !in particular change in the sign for matrix elements < 10d-6 
  !dont know why, summing chi0sf before entering the loops does not improve the results and it is slower

! first coding, the loop over ig igp could be optimised taking into account symmetries 
  do ig=1,npwsigx
   do igp=1,npwsigx
    do io=1,nomega 
     ct=(0.0,0.0)
     do iosf=my_wl,my_wr
      !if(iosf==io .or. iosf==io+1 .or. iosf==io-1) cycle
      ct = ct+ kkweight(io,iosf)*chi0sf(ig,igp,iosf)
     end do
     chi0(ig,igp,io)= ct
    end do 
   end do
  end do 

  ! Collect from all the proc and add weight  
  ! call symf12(npwsigx,nomega,chi0,omega,nop,op,ngpi,gptab,gptabo,ip2fp,phgt,2)
  !call leave_test(mpi_enreg)
  !call xsum_master(chi0,master,spaceComm,ierr)
  !this is safer, the size of the packet could exceed the max allowed by MPI
  !If I could determine this max value ...
  do io=1,nomega
   call xsum_master(chi0(:,:,io),master,spaceComm,ierr)
  end do 
  chi0(:,:,:)= weight*chi0(:,:,:)
 end if 

 ! Deallocate memory 
 deallocate(rhotwg)
 if(inclvkb==2) deallocate(fnlkslm,fnldkslm)

 if (allocated(wfng  )) deallocate(wfng)
 if (allocated(wfnrk )) deallocate(wfnrk)
 if (allocated(den   )) deallocate(den)
 if (allocated(chi0sf)) deallocate(chi0sf)

 call pclock(180)

 write(message,'(a)')' cchi0q0 : exit '
 call wrtout(06,message,'COLL')

 end subroutine cchi0q0
!!***
