!{\src2tex{textfont=tt}}
!!****f* ABINIT/csigme
!! NAME
!! csigme
!!
!! FUNCTION
!! Calculating diagonal and off-diagonale matrix elements of the self-energy operator
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (FB, GMR, VO, LR, RWG, MG, RShaltaf)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  dtfil=filenames (only dtfil%filsrc is used if the screening must be read from file)
!!  efermi=Fermi energy
!!  en(sp%nk,sp%nb,sp%nsppol)=KS energies for k-points, bands and spin
!!  ep=epsilonm1_parameters (see the definition of this structured datatype)
!!  er=epsilonm1_results (see the definition of this structured datatype)
!!  grottb(npwvec,2,nop)= contains the index of (IS) G where I is the identity or the inversion
!!   operation and G is one of the npwvec vectors in reciprocal space
!!  gvec(3,npwvec)=integer coordinates of each plane wave in reciprocal space
!!  igfft(npwvec,5,5,5)=index of G-G0 planewaves (see cigfft routine)
!!  lt_k datatype containing information on the little group
!!  jk=k point where to calculate the matrix element (might be redundant with sp)
!!  kbz(3,sp%nkbz)=k-point coordinates, full Brillouin zone
!!  kibz(3,sp%nk)=k-point coordinates, irreducible Brillouin zone
!!  ktab(sp%nkbz)= table giving for each k-point in the BZ (kBZ), the corresponding
!!   irreducible point (kIBZ), where kBZ= (IS) kIBZ and I is either the inversion or the identity
!!  ktabp(sp%nkbz)= phase factor associated to tnons
!!  ktabi(sp%nkbz)= for each k-point in the BZ defines whether inversion has to be
!!   considered in the relation kBZ=(IS) kIBZ (1 => only S; -1 => -S)
!!  ktabr(nr,sp%nkbz)= index of R^{-1}(r-t) in the FFT array where kBZ=(IS) kIBZ
!!  minbnd,maxbnd=indices of the lowest and highest bands for which the calculation
!!    of the matrix element is to be done
!!  mpi_enreg= datatype gathering information on parallelism, variables used 
!!   |gwpara= 0 no parallelism 
!!   |        1 parallelism is over k-points                                     
!!   |        2 bands are spread btw processors
!!  ngfft1,ngfft1a,ngfft2,ngfft3=FFT grid dimensions
!!  nop=number of symmetry operations
!!  nr=number of points of FFT grid
!!  occ(sp%nk,sp%nb,sp%nsppol)=occupation numbers, for each k point in IBZ, each band and spin
!!  op(3,3,nop)=symmetry operations in reciprocal space
!!  phsgt(sp%npwc,nop)=phase factors associated to non-symmorphic operations : 
!!   phsgt(G,iop) = exp{-i2\pi S G \cdot tnons(:,iop)}
!!  q=q points where $\tilde\epsilon^{-1}$ has been computed
!!  qbz(3,sp%nqbz)=coordinates of all q points in BZ
!!  qpg(sp%npwx,sp%nq)=|q+G| norm
!!  qtab(sp%nqbz)= table giving for each q-point in the BZ (qBZ), the corresponding
!!   irreducible point (qIBZ), where qBZ= (IS) qIBZ and I is either the inversion or the identity operation
!!  qtabi(sp%nqbz)= for each q-point in the BZ defines whether the inversion has to be
!!   considered in the relation qBZ=(IS) qIBZ (1 => only S; -1 => -S)
!!  qtabo(sp%nqbz)= the symmetry operation S that takes qIBZ to each qBZ
!!  sp=sigma_parameters (see the definition of this structured datatype)
!!  ucvol=unit cell volume
!!  wfr(nr,sp%nb,sp%nk,sp%nsppol)=wavefunctions in real space
!!  nslice_qpt=if different from sp%nq it defines the number of slices of the SCR matrix that are read 
!!   from file and stored in memory, at the moment only nslice_qpt=1 is coded 
!!
!! OUTPUT
!!  sr=sigma_results (see the definition of this structured datatype)
!!
!! NOTES
!!  The treatment of the divergence of Gygi+Baldereschi (PRB 1986) is included.
!!  The calculation of energy derivative is based on finite elements.
!!
!! TODO
!!  One should eliminate the automatic arrays, and use allocatable ones
!!  The space group should be passed to the sub in order to set up bz_geometry_factor
!!  or better approaches should be coded to deal with the integrable coulombian singularity
!!
!! PARENTS
!!      sigma
!!
!! CHILDREN
!!      calc_coh,calc_sig_noppm,calc_sig_ppm,cgemv,cggfft,diago_hamilt,distrb2
!!      findqg0,mpi_barrier,prep_coh,rho_tw_g,timab,xcomm_init,xmaster_init
!!      xme_init,xsum_mpi
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine csigme(dtfil,sp,ep,jk,minbnd,maxbnd,q,gvec,npwvec,op,nop,phsgt,&
& kibz,ktab,ktabr,ktabi,ktabp,kbz,ngfft1,ngfft1a,ngfft2,ngfft3,igfft,nr,en,occ,qbz,qtab,qtabi,qtabo,&
& qpg,ucvol,grottb,wfr,efermi,er,sr,mpi_enreg,ppmodel,b1,b2,b3,nbnds_per_proc,min_band_proc, &
& max_band_proc,lt_k,nslice_qpt)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_15gw, except_this_one => csigme
 use interfaces_lib01hidempi
#else
 use defs_xfuncmpi
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: jk,max_band_proc,maxbnd,min_band_proc,minbnd
 integer,intent(in) :: nbnds_per_proc,ngfft1,ngfft1a,ngfft2,ngfft3,nop,npwvec
 integer,intent(in) :: nr,nslice_qpt,ppmodel
 real(dp),intent(in) :: efermi,ucvol
 type(MPI_type),intent(inout) :: mpi_enreg
 type(epsilonm1_parameters),intent(in) :: ep
 type(epsilonm1_results),intent(in) :: er
 type(little_group),intent(in) :: lt_k
 type(sigma_parameters),intent(in) :: sp
 type(sigma_results),intent(inout) :: sr
!arrays
 integer,intent(in) :: grottb(npwvec,2,nop),gvec(3,npwvec),ktab(sp%nkbz)
 integer,intent(in) :: ktabi(sp%nkbz),ktabr(nr,sp%nkbz),qtab(sp%nqbz)
 integer,intent(in) :: qtabi(sp%nqbz),qtabo(sp%nqbz)
 integer,target :: igfft(npwvec,5,5,5)
 real(dp),intent(in) :: b1(3),b2(3),b3(3)
 real(dp),intent(in) :: en(sp%nk,sp%nb,sp%nsppol)
 real(dp),intent(in) :: kbz(3,sp%nkbz),kibz(3,sp%nk)
 real(dp),intent(in) :: occ(sp%nk,sp%nb,sp%nsppol)
 real(dp),intent(in) :: op(3,3,nop),q(3,sp%nq),qbz(3,sp%nqbz)
 real(dp),intent(in) :: qpg(sp%npwx,sp%nq)
 complex,intent(in) :: ktabp(sp%nkbz),phsgt(sp%npwc,nop)
 complex,intent(in) :: wfr(nr,min_band_proc:max_band_proc,sp%nk,sp%nsppol)

!Local variables ------------------------------
! bz_geometry_factor: (see gwa.pdf, appendix A.4)
! sphere=7.79, fcc=7.44, sc=6.188, bcc=6.946, wz=5.255 (Lu Fu-Fa)
!scalars
 integer :: ff,i,ib,ib1,ib2,ierr,ig,ig1,iggp,igp,ii,ii1,iik,iiq,ik,ikbz,ikibz,io
 integer :: ioe0j,iop,iopk,iopq,iq,iq0bz,iqbz,iqibz,is,isless,isppol,istat,j,jb,jb_proc_rank
 integer :: jik,jj,jkbz,jkibz,k_index1,k_index2,kb,master,me
 integer :: nb1gw,nb2gw,nkpt,nomega,npwc1,npwc2,nrb,nsize,spaceComm,theta_fact
 integer :: tim_fourdp,twofm1,zb
 real(dp),parameter :: bz_geometry_factor=7.44,domegareal=0.5/Ha_eV
 real(dp) :: a,b,den,den_coh,den_sex,domegasi,e0i,e0j,fact_sp,i_sz,imdsigmee0
 real(dp) :: inv_den,kincontrib,otw,q0vol,redsigmee0,reomegame0i,smrt
 real(dp) :: sqrt_i_sz,theta_mu_minus_e0i,tol_empty,twofm1_zcut,z2
 complex :: ccoh,csex,ct,dct,dsigc,full_den,idelta,ieta,inv_full_den,num,numf
 complex :: rhotwgdp_igp,sigc,sigcohme,sigxme,twofm1_idelta,z0,zz
 complex,external :: cdotc
 logical :: cohsex,kinclb,read_epsq,verbose
 character(len=500) :: message
!arrays
 integer :: g0(3),g01(3)
 integer,allocatable :: nband_t(:)
 integer,pointer :: igfftg0(:)
 real(dp) :: e0pde(sp%nomegasrd,minbnd:maxbnd,sp%nsppol),ki(3),kj(3),kjmki(3),kk(3)
 real(dp) :: qbzpg(sp%npwx),scme(sp%nomegasrd),tsec(2)
 real(dp),allocatable :: omegame0i(:),otq(:,:)
!FIXME there are automatic arrays
 complex :: rhotwg(sp%npwx),rhotwg_ki(sp%npwx,minbnd:maxbnd)
 complex :: rhotwgdpcc(sp%npwx),rhotwgp(sp%npwx),sigsex(sp%npwc)
 complex,allocatable :: botsq(:,:),botsq_conjg_transp(:,:),eig(:,:),eig_1(:),epsm1q(:,:,:)
 complex,allocatable :: epsm1q_trcc(:,:,:),ket(:,:),ket1(:,:),ket2(:,:)
 complex,allocatable :: rhotwgdpccc(:),sigccoh(:,:)
 complex,allocatable :: sigckin(:),sigcme2(:,:),sigcme_3(:),sigcme_new(:)
 complex,allocatable :: sigcsex(:,:),sigctmp(:)
 complex,allocatable :: wfr_jb(:,:,:),wfr_temp(:,:)
 complex,allocatable :: wfg2(:,:,:,:)
 complex,allocatable :: sigcme_tmp(:,:,:,:),sigxme_tmp(:,:,:)

!MG060926 added (or modified) variables
!MG for symmetrization
integer :: wtqp,wtqm,nqibzk,indx,ncplx(sp%nsppol),wcplx,icx,sumcxtab
integer :: cplx(maxbnd-minbnd+1,sp%nsppol)
integer,allocatable :: cxtab(:,:,:)
complex,allocatable :: dummy_xme(:,:,:),dummy_cme(:,:,:,:)
integer,allocatable :: ndwn(:,:),nup(:,:)

!MG these temporary variables are for the analytical continuation
logical :: analytic_continuation=.false.,average_real=.true.
integer :: nomegasi
complex,allocatable :: omegasi(:),sp_omegasi(:),sct(:)
complex,allocatable :: sigcme_ac(:,:)
complex :: omegame0i_ac,omegame0i2_ac 
complex,allocatable :: ket_ac(:,:)
real(dp) :: ldi

!should become allocatable, used  only for AC
real(dp) :: z(ep%nomegaei), zw(ep%nomegaei)
real(dp) :: omegap2(ep%nomegaei),omegap(ep%nomegaei)
complex,allocatable :: epsm1cqwz2(:,:,:)
complex,allocatable :: integr(:,:,:)
!ENDMG

!MG added to code the reading of espilon from file (slower but required if SCR is huge)
integer,parameter :: localrdwf=0 !should become input variable when the implementations is stable
type(datafiles_type),intent(in) :: dtfil
real(dp) :: q_dummy(3,sp%nq)
complex :: omega_dummy(ep%nomega)
complex,allocatable :: epsm1q_tmp(:,:,:,:)
!ENDMG

! *************************************************************************

#ifdef VMS
!DEC$ ATTRIBUTES ALIAS:'CDOTC' :: cdotc
!DEC$ ATTRIBUTES ALIAS:'CGEMV' :: cgemv
#endif
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 complex :: dpade_approx
 real(dp) :: linfit
 complex :: newton_raphson_step
 complex :: pade_approx
#endif
!End of the abilint section

 if (sr%nomega/=sp%nomegasr) then
  write(message,'(4a)')ch10,&
& ' csigme : BUG - ',ch10,&
& '  sr%nomega/=sp%nomegasr '
  call wrtout(6,message,'COLL')
  call leave_new('COLL')
 end if

 read_epsq= .false.
 if (nslice_qpt/=sp%nq) then 
  read_epsq=.true.
  write(message,'(2a,i4,2a)')ch10,&
&  ' csigme : reading ',nslice_qpt,' slices from file (slower)',ch10
  call wrtout(6,message,'COLL')
 end if 

 call timab(421,1,tsec)
 call timab(422,1,tsec)

 call xcomm_init(mpi_enreg,spaceComm) !Init mpi_comm
 call xme_init(mpi_enreg,me)          !Init me
 call xmaster_init(mpi_enreg,master)  !Init master

!******************************************************************************************* 

 write(message,'(2a)')ch10,ch10
 call wrtout(6,message,'COLL')
 if (mod(sp%gwcalctyp,10)==0) write(message,'(a)')' standard GW with PPM'
 if (mod(sp%gwcalctyp,10)==1) write(message,'(a)')' standard GW without PPM (analytical continuation)'
 if (mod(sp%gwcalctyp,10)==2) write(message,'(a)')' standard GW without PPM (contour deformation)'
 if (mod(sp%gwcalctyp,10)==5) write(message,'(a)')' Hartree-Fock calculation'
 if (mod(sp%gwcalctyp,10)==6) write(message,'(a)')' Screened Exchange calculation'
 if (mod(sp%gwcalctyp,10)==7) write(message,'(a)')' COHSEX calculation'
 if (mod(sp%gwcalctyp,10)==8) write(message,'(a)')' model GW with PPM'
 if (mod(sp%gwcalctyp,10)==9) write(message,'(a)')' model GW without PPM'
 call wrtout(6,message,'COLL')

 if (sp%gwcalctyp<10) then
  if (sp%gwcalctyp==1) then 
   write(message,'(a)')' Newton Raphson method '
  else 
   write(message,'(a)')' Perturbative Calculation'
  end if 
 else if (sp%gwcalctyp<20) then
  write(message,'(a)')' Self-Consistent on Energies only'
 else
  write(message,'(a)')' Self-Consistent on Energies and Wavefunctions'
 end if
 call wrtout(6,message,'COLL')

!MG this should be removed, for the moment used only to develop
 if (mod(sp%gwcalctyp,10)==1) then 
  if (sp%gwcalctyp/=1) then 
   write(*,*)'not yet tested'
   call leave_new('COLL')
  else 
   analytic_continuation=.true.
  end if 
 end if

! Decomposition of Sigma_c into Coulomb-hole (coh) and screened-exchange (sex)
 cohsex= .false. 
 if (mod(sp%splitsigc,2)==1) cohsex= .true. 
! Kinetic (kin) and Coulomb (clb)
 kinclb= .false.
 if (int(sp%splitsigc/2)==1) kinclb= .true.

! Delta to avoid poles 
 idelta= cmplx(0,sp%zcut)

! min and Max band index for GW corrections (for this k-point)
 ib1= minbnd    ; ib2= maxbnd
! min and max value over k-points and spin (use to dimension the arrays)
  nb1gw= sp%minbdgw ; nb2gw= sp%maxbdgw

! Analytic integration of q^{-2} over the volume element:
! $\int_V d^3q 1/q^2 = bz_geometric_factor V^(-2/3)$
 q0vol= (8* pi*pi*pi)/(ucvol*sp%nkbz) ! V_BZ / N_k
 i_sz= bz_geometry_factor * q0vol**(-two_thirds)
 sqrt_i_sz= sqrt(i_sz)

! FFT factor
 nrb= ngfft1*ngfft2*ngfft3 
 tim_fourdp= 2

! Print info
  write(message,'(2a,3f8.3,2a,i3,a,i3,a)')ch10,&
& ' Calculating <nk|Sigma|nk> at k = ',(kbz(i,jk),i=1,3),ch10,&
& ' bands n = from ',ib1,' to ',ib2,ch10
  call wrtout(6,message,'COLL')

 jkbz= jk
 jkibz= ktab(jkbz)
 jik= (3-ktabi(jkbz))/2
 kj(:)= kbz(:,jkbz)

!DEBUG
!write(*,*)associated(sr%sigxme),size(sr%sigxme),shape(sr%sigxme)
!write(*,*)associated(sr%sigcme),size(sr%sigcme),shape(sr%sigcme)
!stop
!ENDDEBUG

! Diagonal elements of $\Sigma_x$ and $\Sigma_c(\omega)$
 sr%sigxme(ib1:ib2,jkibz,:)= zero
 sr%sigcme(ib1:ib2,jkibz,:,:)= (0.,0.) !this is single precision complex (we could switch to dpc)

! RS Not every proc may have the wavefunctions that correspond to the block of bands (minband:maxband) 
! MG now using standard convention for proc_distrb, NOTE however that we are still not consistent since 
! in the gwpara==1 case we use nkbz instead of nkibz TO BE FIXED
 allocate(wfr_jb(nr,ib1:ib2,sp%nsppol))
 wfr_jb= (0.0,0.0)

 do jb=ib1,ib2
  do is=1,sp%nsppol
   if (mpi_enreg%nproc>1 .and. mpi_enreg%gwpara==2) then
    jb_proc_rank= mpi_enreg%proc_distrb(jkibz,jb,is)
    if (me==jb_proc_rank) wfr_jb(:,jb,is)= wfr(:,jb,jkibz,is)
    call xcast_mpi(wfr_jb,jb_proc_rank,spaceComm,ierr)
    !read(666)wfr_jb(:,jb,is)
   else
    wfr_jb(:,jb,is)= wfr(:,jb,jkibz,is) !k-para or sequential case
    !write(666)wfr_jb(:,jb,is)
   end if
!CHECKME abinis and abinip with 2 proc give slightly different results for wfr_kb
   !if (me==0) write(666,*)wfr_jb(:,jb,is)
  end do !is 
 end do !jb

 if (mpi_enreg%gwpara==2) then
  call leave_test(mpi_enreg)
 end if 

! Setup frequencies around the KS\QP eigenvalues to compute the derivative (note the spin)
! maybe it is better to force an even sp%nomegasrd so we have the KS\QP eigenvalue in the middle of the interval
 ioe0j= sp%nomegasrd/2+1
 do is=1,sp%nsppol
  do jb=ib1,ib2
   do io=1,sp%nomegasrd
    e0pde(io,jb,is)= sr%egw(jb,jkibz,is)+sp%deltae*(io-ioe0j)
    sr%omegasrd(jb,jkibz,io,is)= e0pde(io,jb,is)
   end do
  end do
 end do

! Initialize quantities needed by AC 
 if (analytic_continuation) then
  nomegasi= sp%nomegasi
  allocate(omegasi(nomegasi),sp_omegasi(nomegasi))
! Logarithmic mesh along the imaginary axis
  !domegasi= (sp%omegasimax/sp%omegasimin)**(1.0d0 / (sp%nomegasi-1))
  !omegasi(1)= 0
  !ldi= domegasi
  !do io=2,nomegasi
  ! omega(io) =cmplx(0,ldi*sp%omegasimin)
  ! omegasi= ldi*domegasi
  !end do
  !sp_omega(:)= omegasi(:)
! Linear mesh along the imaginary axis
  domegasi= sp%omegasimax/(nomegasi-1)
  do io=1,nomegasi
   omegasi(io)= cmplx(0,(io-1)*domegasi)
   sp_omegasi(io)= omegasi(io) !simple precision, should use a better instruction
  end do
  sr%omegasi(:)= omegasi(:)
  write(message,'(3a)')ch10,&
&  ' csigme : calculating Sigma at imaginary frequencies [eV] (Fermi Level set to 0) ',ch10
  call wrtout(6,message,'COLL')
  !call wrtout(ab_out,message,'COLL')
  do io=1,nomegasi
   write(message,'(2(f10.3,2x))') omegasi(io)*Ha_eV
   call wrtout(ab_out,message,'COLL')
  end do

! Calculate Gauss-Legendre quadrature knots and weights
  call coeffs_gausslegint(0.0d0,1.0d0,z,zw,ep%nomegaei)
  !if we want to use the same SCR file both for AC and CD we must use the same grid
  !first frequencies are always real 
  do io=1,ep%nomegaei
   if (abs(aimag(er%omega(ep%nomegaer+io))-(1/z(io)-1)) > 0.0001) then 
    write(message,'(7a)')ch10,&
&    ' csigme : ERROR - ',ch10,&
&    '  frequencies found in SCR file are not compatible with analytic continuation',ch10,&
&    '  verify the frequencies in the SCR file',ch10
    call wrtout(6,message,'COLL')
    call leave_new('COLL')
   end if 
  end do
! To calculate \int_0^\infty domegap f(omegap) we calculate \int_0^1 dz f(1/z - 1)/z^2
  omegap(:)= 1/z(:) - 1
  omegap2(:)= omegap(:) * omegap(:)
! Allocate other arrays needed 
  allocate(epsm1cqwz2(sp%npwc,sp%npwc,ep%nomegaei),stat=istat)
  allocate(integr(sp%npwc,sp%npwc,sp%nomegasi),stat=istat)
   if(istat/=0) call memerr('csigme','integr',sp%npwc**2*sp%nomegasi,'spc')
 end if !of analytical continuation

 nomega= sp%nomegasr+sp%nomegasrd

 !sigcme2 is used to accumulate the diagonal matrix elements over k-points and considered bands
 !used only in case of ppmodel (TODO save memory)
 allocate(sigcme2(nomega,ib1:ib2),sigcme_3(nomega))
 sigcme2(:,:)= (0.,0.)
 sigcme_3(:)= (0.,0.)

!FIXME I think here I should use minbnd maxbnd
! These arrays are used to accumulate the matrix elements of \Sigma_c over k-points and bands
! Note that for AC the number of frequencies is differents (only imaginary freq)
 if (analytic_continuation) then 
  allocate(sigcme_tmp(nomegasi,ib1:ib2,ib1:ib2,sp%nsppol))
  allocate(ket(sp%npwc,nomegasi),sigctmp(nomegasi))
 else
  allocate(sigcme_tmp(nomega,ib1:ib2,ib1:ib2,sp%nsppol))
  allocate(ket(sp%npwc,nomega),sigctmp(nomega))
 end if 

 allocate(sigxme_tmp(ib1:ib2,ib1:ib2,sp%nsppol))

 sigxme_tmp(:,:,:)=   (0.,0.)
 sigcme_tmp(:,:,:,:)= (0.,0.)

! FIXME This quantities are only used for model GW if I am not wrong
 allocate(ket1(sp%npwc,nomega),ket2(sp%npwc,nomega))
 allocate(omegame0i(nomega))

! Calculation of the decomposition of the correlation part of sigma 
! into Coulomb-hole (coh) and screened-exchange (sex), GMR on 04/05/05
!FIXME should use ib1 and ib2
 if (cohsex) then
  allocate(sigccoh(sp%nb,sp%nsppol))
  allocate(sigcsex(sp%nb,sp%nsppol))
  sigccoh(:,:)= zero 
  sigcsex(:,:)= zero 
 end if

! Calculation of the Kinetic and Coulomb contributions to the bandgap energy.
!MG FIXME this array is not used anymore (YMN on 07/04/04)
 if (kinclb) then
  allocate(sigckin(sp%nb))
  sigckin(:)= zero
 end if
!END MG

! Note related to the symmetrization in sigma:
!
! In case of Sigma calculation the unitary transformation relating wavefunctions
! at symmetric k-points should be taken into account during the symmetrization 
! of the oscillator matrix elements (maybe this will be implemented in the future).
! In case of G_oW_o and GW_o calculations however it is possible to make an invariant by just 
! including all the degenerate states in the sigma calculation and averaging the QP
! corrections in the degenerate subset at the end of the run
! here we divide the states where the QP energies are required into complexes, note
! however that this approach is not based on group theory, and it will fail in 
! in case of accidental degeneracies, in the sense that GW will not remove the degeneracy 

 nqibzk= sp%nkbz
 if (lt_k%sym_flag/=0) then
  nqibzk= 0
  do ii=1,lt_k%nkbz
   nqibzk= nqibzk+lt_k%ibzq(ii)
  end do

! Evaluate number of complex and number of bands in each complex
  allocate(cxtab(maxbnd-minbnd+1,sp%nsppol,maxbnd-minbnd+1))
  cxtab(:,:,:)= 0

  do isppol=1,sp%nsppol
   do ib=1,maxbnd-minbnd+1
    do jb=1,maxbnd-minbnd+1
     !the tolerance is a little bit arbitrary (0.001 eV)
     !it could be reduced, in particular in case of nearly accidental degeneracies
     if (abs(en(jkibz,ib-1+minbnd,isppol)-en(jkibz,jb-1+minbnd,isppol))<0.001/Ha_ev) then
      cxtab(ib,isppol,jb)= 1
     end if
    end do
   end do
  end do 
  
  if (any(cxtab(:,:,:)/=0)) then 
   write(message,'(2a,3f8.3,a)')ch10,' found degenerate states at k-point  = ',(kbz(i,jk),i=1,3),ch10
   call wrtout(6,message,'COLL')
   do isppol=1,sp%nsppol
    do ib=1,maxbnd-minbnd+1
     do jb=ib+1,maxbnd-minbnd+1
      if (cxtab(ib,isppol,jb)==1) then 
       write(message,'(a,i2,a,i4,a,i4)')' (spin ',isppol,')',ib-1+minbnd,' <====> ',jb-1+minbnd
       call wrtout(6,message,'COLL')
       !If two states do not belong to the same complex => matrix elements of v_xc should differ
       !if (abs(sr%vxcme(ib,jkibz,isppol)-sr%vxcme(jb,jkibz,isppol))>abs(tol6*sr%vxcme(jb,jkibz,isppol))) then 
!        write(message,'(11a)')ch10,&
!&        ' csigme : COMMENT -',ch10,&
!&        ' It seems that at this k-point an accidental degeneracy occurs ',ch10,&
!&        ' In this case using symsigma=1 will lead to spurious effects.    ',ch10,&
!&        ' The GW calculation indeed would not be able to remove this degeneracy.',ch10,&
!&        ' In order to remove this deficiency, please run your calculation using symchi=0',ch10
!         call wrtout(6,message,'COLL')
!         call wrtout(ab_out,message,'COLL')
!         !call leave_new('COLL')
!        end if
      end if 
     end do 
    end do 
   end do 
  end if 

 end if !of sym_flag

 call timab(422,2,tsec)

 write(message,'(2a,i6,a)')ch10,' calculation status ( ',nqibzk,' to be completed):'
 call wrtout(6,message,'COLL')

!MG this quantity is needed only in case of ppmodel 3
 allocate(eig(sp%npwc,sp%npwc))

! Prepare the static COHSEX calculation: calculate the Fourier transform of \u_b1k^*(r)\u_b2k(r)
! Note that there is no need to take into account the phases arising from non-symmorphic operations
! They cancel each other since the wavefunctions are evaluated at the same k-point 
! Note moreover that the external k-points are supposed to be in the IBZ 
! prep_coh will not work properly if an arbitrary k-point is passed in input
 if (mod(sp%gwcalctyp,10)==7) then
  allocate(wfg2(nr,ib1:ib2,ib1:ib2,sp%nsppol))
  wfg2= (0.,0.)
!MG OLD call, not suitable for parallelism
!  call prep_coh(sp%nb,nr,sp%nsppol,wfr(:,:,jkibz,:),ktabr(:,jkibz),minbnd,maxbnd,&
!&  ngfft1,ngfft2,ngfft3,tim_fourdp,wfg2,mpi_enreg)
!NEW CALL Note that wfr_jb(:,jb,is)= wfr(:,jb,jkibz,is), sp%nb is not used and should be removed
  call prep_coh(sp%nb,nr,sp%nsppol,wfr_jb(:,:,:),ktabr(:,jkibz),ib1,ib2,&
&  ngfft1,ngfft2,ngfft3,tim_fourdp,wfg2,mpi_enreg)
 end if
 
 if (ppmodel==1 .or. ppmodel==2) then
  npwc1= sp%npwc ; npwc2= sp%npwc
 else if (ppmodel==4) then
  npwc1= sp%npwc ; npwc2= 1
 else ! for ppmodel3 and noppmodel
  npwc1= 1       ; npwc2= 1
 end if

!MG check this, they are only used for ppmodels
 allocate(botsq(sp%npwc,npwc1),otq(sp%npwc,npwc2))

!MG FIXME note we can reduce the number of omegas (only imaginary)
! concerning other arrays it seems that epsm1q_trcc is used only for model GW
! now it is very important to optimize the memory usage!
 if (analytic_continuation) then 
  allocate(epsm1q(sp%npwc,sp%npwc,ep%nomega))
 end if

 if ( mod(sp%gwcalctyp,10)==2 .or. &
&     mod(sp%gwcalctyp,10)==6 .or. &
&     mod(sp%gwcalctyp,10)==7 .or. &
&     mod(sp%gwcalctyp,10)==9    ) then
  allocate(epsm1q(sp%npwc,sp%npwc,ep%nomega),stat=istat) 
   if(istat/=0) call memerr('csigme','epsm1q',sp%npwc*2*ep%nomega,'spc')
  !allocate(epsm1q_trcc(sp%npwc,sp%npwc,ep%nomega),stat=istat)
  !if(istat/=0) call memerr('csigme','epsm1q_trcc',sp%npwc*2*ep%nomega,'spc')
 end if

 if (mod(sp%gwcalctyp,10)==9) then
  !MG should check wheter I need also epsm1q
  allocate(epsm1q_trcc(sp%npwc,sp%npwc,ep%nomega),stat=istat)
   if(istat/=0) call memerr('csigme','epsm1q_trcc',sp%npwc*2*ep%nomega,'spc')
 end if 

! Correct normalization of theta_mu_minus_e0, in case of nsppol==2 occ $\in [0,1]$
 if (sp%nsppol==1) then
  fact_sp= 0.5 ; tol_empty= 0.01  !below this value the state is assumed empty
 else if (sp%nsppol==2) then
  fact_sp= 1.0 ; tol_empty= 0.005 !to be consistent and obtain similar results if a metallic
 else                             !spin unpolarized system is treated using nsppol==2
  write(message,'(3a)')&
&  ' csigme : BUG ',ch10,&
&  '  nsppol must be 1 or 2'
  call wrtout(6,message,'COLL')
  call leave_new('COLL')
 end if

! Loop over k_i in BZ
 do ikbz=1,sp%nkbz

! Parallelization over k-points and spin, in case of spin there is another check in the inner loop
  if (mpi_enreg%gwpara==1) then
   if (minval(abs(mpi_enreg%proc_distrb(ikbz,:,:) - me))/=0) cycle
   if (any(mpi_enreg%proc_distrb(ikbz,:,:)==-999)) stop '-999'
  end if

  call timab(423,1,tsec)

  ikibz= ktab(ikbz)
  iik= (3-ktabi(ikbz))/2
  ki(:)= kbz(:,ikbz)

! Identify q and G0 where q+G0=k_j-k_i
  kjmki(:)= kj(:)-ki(:)
  call findqg0(iq,g0,kjmki,sp%nqbz,qbz) 

! If symmetries are used then sum only over the irreducible wedge IBZ_k
! the following coding is safer (crash under gfortran)
  if (lt_k%sym_flag/=0) then
   if (lt_k%ibzq(iq)/=1) cycle !only q in the IBZ_k
   wtqp= 0 
   wtqm= 0
   do jj=1,lt_k%nop
    wtqp= wtqp+lt_k%wtksym(1,jj,iq)
    wtqm= wtqm+lt_k%wtksym(2,jj,iq)
   end do
  end if

  write(message,'(a,i6,a,i6,a,i3)')&
&  ' csigme : ik = ',ikbz,' / ',sp%nkbz,'  done by processor ',me
  call wrtout(06,message,'COLL')

  iqibz= qtab(iq)
  iopq= qtabo(iq)
  iiq= (3-qtabi(iq))/2
  igfftg0 => igfft(:,g0(1)+3,g0(2)+3,g0(3)+3)

  if (read_epsq) then  
! Read epsilon^{-1} from file (slower)
   allocate(epsm1q_tmp(sp%npwc,sp%npwc,ep%nomega,1),stat=istat) 
    if(istat/=0) call memerr('csigme','epsm1q_tmp',sp%npwc*2*ep%nomega,'spc')
   verbose=.false.
   call rdscr(dtfil,sp%npwc,sp%nq,1,ep%nomega,q_dummy,omega_dummy,epsm1q_tmp,mpi_enreg,localrdwf,verbose,iqa=iqibz)
!DEBUG
   if (any(abs(q_dummy(:,iqibz)-q(:,iqibz))>tol6)) then 
    write(message,'(a)')' BUG while reading screening from file q_dummy/=q '
    call wrtout(6,message,'COLL')
    call leave_new('COLL')
   end if 
   !if (iqibz==1) write(111,*)epsm1q_tmp
!ENDDEBUG
  end if 

! Calculate plasmonpole parameters or full \tilde\epsilon^-1 for this qBZ (converting from qIBZ to qBZ)
! MG NOTE that here we are not taking into account a possible umklapp vector!
! in this case the symmetrized equation is different since we have to shift the G vectors
! There is however a check in sigma
!
! Symmetry properties: if q_bz=Sq_ibz+G0 then:
!
!  \epsilon^{-1}_{SG1-G0, SG2-G0} (q_bz) = e^{+iS(G2-G1)\cdot t} \epsilon^{-1}_{G1,G2)}   (q)
!
!  If time reversal symmetry can be used then :
!  \epsilon^{-1}_{G1, G2} (-q_bz) = e^{+i(G1-G2)\cdot t} \epsilon^{-1}_{-S^{-1}(G1+Go),-S^{-1}(G2+G0)}^* (q)

  if (mod(sp%gwcalctyp,10)==0 .or. &
&     mod(sp%gwcalctyp,10)==8) then

   if (ppmodel==1 .or. ppmodel==2) then
    do jj=1,sp%npwc
     do ii=1,sp%npwc
      !botsq(grottb(i,iiq,iopq),grottb(j,iiq,iopq))=er%bigomegatwsq(i,j,iqibz)
      !otq(grottb(i,iiq,iopq),grottb(j,iiq,iopq))=er%omegatw(i,j,iqibz)
      !MG needed in case of non-symmorphic operations
      !plasmon pole frequencies otq are invariant under symmetry 
      botsq(grottb(ii,iiq,iopq),grottb(jj,iiq,iopq))= er%bigomegatwsq(ii,jj,iqibz)*phsgt(ii,iopq)*conjg(phsgt(jj,iopq))
      otq(grottb(ii,iiq,iopq),grottb(jj,iiq,iopq))= er%omegatw(ii,jj,iqibz)  
     end do
    end do
   else if (ppmodel==3) then
    !Symmetry properties in case of non-symmorphic operations
    !for notations see pag 22 of Quasiparticle Calculations in solid (Aulbur et al)
    !
    ! \omega^2_{ii}(q_bz) = \omega^2_{ii}(q)    (otq array)
    ! \alpha_{ii}(q_bz) = \alpha_{ii}(q)    (botq array
    ! \Phi_{SG - G0}(q_bz) = \Phi_{G}(q) e^{-iSG \cdot t}  (eigenvector of e^{-1}, eig array) 
    !
    do ii=1,sp%npwc  ! ii: DM bands
     otq(ii,1)= er%omegatw(ii,1,iqibz)
     botsq(ii,1)= er%bigomegatwsq(ii,1,iqibz)
     do jj=1,sp%npwc
      !eig(grottb(j,iiq,iopq),ii)=er%eigpot(j,ii,iqibz)
      eig(grottb(jj,iiq,iopq),ii)= er%eigpot(jj,ii,iqibz)*phsgt(jj,iopq)
     end do
    end do
   else if (ppmodel==4) then
    !Symmetry properties in case of non-symmorphic operations
    !for notations see pag 23 of Quasiparticle Calculations in solid (Aulbur et al)
    !
    ! Symmetry properties: if q_bz=Sq_ibz+G0 then:
    !
    ! \omega^2_{ii}(q_bz) = \omega^2_{ii}(q)    (otq array)
    ! y_{SG_-G0}(q_bz) = y_{G}(q) e^{-iSG \cdot t} where y=Lx 
    !
    do ii=1,sp%npwc  ! ii: DM bands
     otq(ii,1)= er%omegatw(ii,1,iqibz)
     do jj=1,sp%npwc
      !botsq(grottb(j,iiq,iopq),ii)=er%bigomegatwsq(j,ii,iqibz) ! ii is the band index of DM
      botsq(grottb(jj,iiq,iopq),ii)= er%bigomegatwsq(jj,ii,iqibz)*phsgt(jj,iopq)
     end do
    end do
   end if !of if ppmodel

! Take into account time reversal symmetry 
   if (iiq==2) then
    botsq(:,:)= conjg(botsq(:,:))
    eig(:,:)= conjg(eig(:,:))
   end if

  else if (mod(sp%gwcalctyp,10)==1 .or. &
&          mod(sp%gwcalctyp,10)==2 .or. &
&          mod(sp%gwcalctyp,10)==9) then

! Numerical integration or model GW using the contour deformation technique or Analytic Continuation
! in case of AC we should symmetrize only the imaginary frequencies

  if (read_epsq) then  !if epsilon was read from file
   do io=1,ep%nomega
    do ii=1,sp%npwc
     do jj=1,sp%npwc
      !epsm1q(grottb(i,iiq,iopq),grottb(j,iiq,iopq),io)=er%epsm1(i,j,io,iqibz)
      !MG needed in case of non-symmorphic operations
      epsm1q(grottb(ii,iiq,iopq),grottb(jj,iiq,iopq),io)= epsm1q_tmp(ii,jj,io,1)*phsgt(ii,iopq)*conjg(phsgt(jj,iopq))
     end do
    end do
   end do
   deallocate(epsm1q_tmp)
  else 
   do io=1,ep%nomega
    do ii=1,sp%npwc
     do jj=1,sp%npwc
      !epsm1q(grottb(i,iiq,iopq),grottb(j,iiq,iopq),io)=er%epsm1(i,j,io,iqibz)
      !MG needed in case of non-symmorphic operations
      epsm1q(grottb(ii,iiq,iopq),grottb(jj,iiq,iopq),io)= er%epsm1(ii,jj,io,iqibz)*phsgt(ii,iopq)*conjg(phsgt(jj,iopq))
     end do
    end do
   end do
  end if

! Take into account time reversal symmetry 
   if(iiq==2) epsm1q(:,:,:)= conjg(epsm1q(:,:,:))
! Minus exchange contribution
   do io=1,ep%nomega
    do ii=1,sp%npwc
     epsm1q(ii,ii,io)= epsm1q(ii,ii,io)-1.
    end do
   end do

! In case of AC, prepare the first term numerically: \Sum w_i 1/z_i^2 f(1/z_i - 1)
! Note that the first frequencies are always real, so skip them
   if (mod(sp%gwcalctyp,10)==1) then 
    do iop=1,ep%nomegaei
     z2 = z(iop)*z(iop)
     epsm1cqwz2(:,:,iop)= zw(iop)*epsm1q(:,:,ep%nomegaer+iop)/z2
    end do
   end if 

  else if (mod(sp%gwcalctyp,10)==6 .or. &
&          mod(sp%gwcalctyp,10)==7) then

! SEX or COHSEX, only omega==0 is needed here
   do ii=1,sp%npwc
    do jj=1,sp%npwc
     !epsm1q(grottb(i,iiq,iopq),grottb(j,iiq,iopq),1)=er%epsm1(i,j,1,iqibz)
     epsm1q(grottb(ii,iiq,iopq),grottb(jj,iiq,iopq),1)= er%epsm1(ii,jj,1,iqibz)*phsgt(ii,iopq)*conjg(phsgt(jj,iopq))
    end do
   end do

! Take into account time reversal symmetry 
   if(iiq==2) epsm1q(:,:,1)= conjg(epsm1q(:,:,1))
! Minus exchange contribution
   do ii=1,sp%npwc
    epsm1q(ii,ii,1)= epsm1q(ii,ii,1)-1.
   end do

  else if (mod(sp%gwcalctyp,10)==9) then
!FIXME Here I have to derive the equations needed in case of non-symmorphic operations
! Here there is something strange in the sense that this part is never executed
! should check previous version of the code
! For model GW create transpose(conjg(epsm1q))
   do io=1,ep%nomega
    epsm1q_trcc(:,:,io)= transpose(conjg(epsm1q(:,:,io)))
   end do

  end if !gwcalctyp

! Set up table of |q+G| in BZ
! MG NOTE that also in this case we are not taking into account umklapp processes
  do ig=1,sp%npwx
   qbzpg(grottb(ig,iiq,iopq))= qpg(ig,iqibz)
  end do

  call timab(423,2,tsec)

! Loop over bands (summing over these bands)
  do ib=1,sp%nb

! Inner loop over spin
!   if nsppol==2 ==>  0<=occ<= 1,
!   tol_empty is set to 0.1 if nsspol==1 or 0.05 if nsppol==2
!   to be consistent and obtain the same results if a metallic
!   spin unpolarized system is treated using nsppol==2

   do isppol=1,sp%nsppol

! Parallelism over spin (the processor has this k-point but what about spin?)
    if (mpi_enreg%gwpara==1) then
     !if (minval(abs(mpi_enreg%proc_distrb(ikbz,:,isppol)-mpi_enreg%me))/=0) cycle
     if (mpi_enreg%proc_distrb(ikbz,ib,isppol)/=me) cycle
     if (any(mpi_enreg%proc_distrb(ikbz,:,:)==-999)) stop '-999'
    end if

    if (mpi_enreg%gwpara==2) then !note ikibz
     if (mpi_enreg%proc_distrb(ikibz,ib,isppol)/=me) cycle
    end if

! Skip empty state ib for HF, SEX, and COHSEX
    if ( occ(ikibz,ib,isppol)< tol_empty .and. &
&      ( mod(sp%gwcalctyp,10)==5 .or.          &
&        mod(sp%gwcalctyp,10)==6 .or.          &
&        mod(sp%gwcalctyp,10)==7) ) cycle

    if (analytic_continuation) then
     integr(:,:,:)= (0.0,0.0)
     do io=1,nomegasi
      !here en are calculated wrt the fermi level 
      omegame0i_ac= omegasi(io)-en(ikibz,ib,isppol)
      omegame0i2_ac= omegame0i_ac*omegame0i_ac
! Calculate integral over omegap with Gauss-Legendre quadrature
! -1/pi \int domegap  epsm1c * (omega - e0i) / ( (omega - e0i)^2 + omegap^2)
      do iop=1,ep%nomegaei
       do iggp=0,sp%npwc*sp%npwc-1
        ig=iggp/sp%npwc+1
        igp= iggp-(ig-1)*sp%npwc+1
        ! \int domegap   epsm1c / ( (omega - e0i)^2 + omegap^2)
        integr(ig,igp,io)= integr(ig,igp,io) + epsm1cqwz2(ig,igp,iop)/(omegame0i2_ac + omegap2(iop))
       end do
      end do
       integr(:,:,io)= integr(:,:,io)*omegame0i_ac
     end do
     integr(:,:,:)= -integr(:,:,:)/pi
    end if

! Calculate all rho-twiddle(G) at once <k-q, ib s| e^{-i(q+G).r} | s jb k >
    do jb=ib1,ib2

     call rho_tw_g(sp%npwx,nr,nrb,ngfft1a,ngfft1,ngfft2,ngfft3,igfftg0, &
&     wfr(:,ib,ikibz,isppol),iik,ktabr(:,ikbz),ktabp(ikbz),             &
&     wfr_jb(:,jb,isppol),jik,ktabr(:,jkbz),ktabp(jkbz),                &
&     rhotwg_ki(:,jb),tim_fourdp)

! Divide rho-twiddle(G) by |q+G|
     rhotwg_ki(:,jb)= rhotwg_ki(:,jb)/qbzpg(:)

! Treat analytically the case q --> 0
     if (ikbz==jkbz) then
      if (ib==jb) then
       rhotwg_ki(1,jb)= cmplx(sqrt_i_sz,0.0)
      else
       rhotwg_ki(1,jb)= (0.0,0.0) !ignore always
      end if
     end if

    end do !jb got all the matrix elements from minbnd up to maxbnd

    theta_mu_minus_e0i= fact_sp*occ(ikibz,ib,isppol)

! Starting point to evaluate the derivative of Sigma and the Spectral function
    e0i= en(ikibz,ib,isppol)

! Frequencies for the spectral function, e0i=en(ikibz,ib,isppol)
!FIXME the interval is not centered on eoi! WHY?
    if (sp%nomegasr>0) omegame0i(1:sr%nomega)= real(sr%omega(1:sr%nomega))-e0i

    do kb=ib1,ib2 !right band index

     call timab(424,1,tsec)

! Find frequencies $\omega$ - $\epsilon_in$ to evaluate the derivative of $\Sigma$ taking into account spin
! Note that we have stored e_KS+ Delta \omega in sr%omegasrd so now we have to subtract e_KS
! this is not required in case of AC 
     do io=sp%nomegasr+1,nomega
      omegame0i(io)= real(sr%omegasrd(kb,jkibz,io-sp%nomegasr,isppol)) - e0i
     end do 

     rhotwgp(:)= rhotwg_ki(:,kb)

! First calculate the ket  \Sigma | \phi_k >
     ket(:,:)=  (0.,0.)
     ket1(:,:)= (0.,0.)
     ket2(:,:)= (0.,0.)

! GW calculation WITH Plasmon-Pole Model
! Note that ppmodel 3 or 4 work only in case of standard perturbative approach!
     if (mod(sp%gwcalctyp,10)==0) then 

      call calc_sig_ppm(sp%npwc,nomega,rhotwgp,botsq,otq,omegame0i,sp%zcut,theta_mu_minus_e0i,&
&      ket,ppmodel,eig,sigcme_3,sp%npwx,npwc1,npwc2)

      if (ppmodel==3 .or. ppmodel==4) then
! Applying symmetries (not completely sure but the automatic tests are correct CHECKME)
       if (lt_k%sym_flag==0) then
        sigcme2(:,kb)= sigcme2(:,kb)+sigcme_3(:) 
       else
        sigcme2(:,kb)= sigcme2(:,kb)+&
&        (wtqp+wtqm)*real(sigcme_3(:)) + (wtqp-wtqm)*(0.,1.)*aimag(sigcme_3(:))
       end if
      end if

     else if (mod(sp%gwcalctyp,10)==1) then 
! GW calculation using analytical continuation
     
      ket(:,:)= (0.,0.)

! Store \sum_Gp rhotw_G integr_GGp(omegasi) rhotw_Gp must be multiplied later by 4*pi/(ucvol*nkbz)
!FIXME this can be optimized
      do io=1,nomegasi
       do ig=1,sp%npwc
        ct= (0.,0.)
        do igp=1,sp%npwc
         ct= ct + integr(ig,igp,io)*rhotwgp(igp)
        end do
        ket(ig,io)= ct 
       end do
      end do

     else if (mod(sp%gwcalctyp,10)==2) then 
! GW calculation: numerical integration with contour deformation

      call calc_sig_noppm(sp%npwc,nomega,ep%nomega,ep%nomegaer,ep%nomegaei,rhotwgp(1:sp%npwc),&
&      er%omega,epsm1q,omegame0i,theta_mu_minus_e0i,ket)

     else if (mod(sp%gwcalctyp,10)==6 .or. &
&             mod(sp%gwcalctyp,10)==7) then
! SEX or COHSEX, here only the SEX part is considered

      call cgemv('N',sp%npwc,sp%npwc,(1.,0.),epsm1q(:,:,1),sp%npwc,rhotwgp(1:sp%npwc),1,(0.,0.),sigsex,1)

      sigsex(:)= -theta_mu_minus_e0i*sigsex(:)
     
      do io=1,nomega !SEX is energy independent
       ket(:,io)= sigsex(:)
      end do

     else if (mod(sp%gwcalctyp,10)==8) then 
! MODEL GW calculation WITH Plasmon-Pole Model

      allocate(sigcme_new(nomega))
! Calculate \Sigma(E_k) | k > to obtain < j | \Sigma(E_k) | k >
      call calc_sig_ppm(sp%npwc,nomega,rhotwgp(1:sp%npwc),botsq,otq,omegame0i,sp%zcut,&
&      theta_mu_minus_e0i,ket1,ppmodel,eig,sigcme_new,sp%npwx,npwc1,npwc2)
      !calculate < k | \Sigma(E_k) to obtain < k | \Sigma(E_k) | j >^*
      if(sp%gwcalctyp==28) then
       if(ppmodel/=1 .and. ppmodel/=2)then ! This is needed to have sp%npwc=npwc1=npwc2
        write(message,'(6a)')ch10,&
&        ' csigme : ERROR -',ch10,&
&        '  At present, gwcalctyp=28 cannot be used with ppmodel=3 or ppmodel=4.',&
&        '  Sorry for this ...',&
&        '  Action : use another Plasmon Pole Model with gwcalctyp=28.'
        call wrtout(6,message,'COLL')
        call leave_new('COLL')
       endif
       allocate(botsq_conjg_transp(npwc1,sp%npwc))
!      Keep the next two lines separate from the call, otherwise gfortran messes
!      up
       botsq_conjg_transp=transpose(botsq)
       botsq_conjg_transp=conjg(botsq_conjg_transp)
       call calc_sig_ppm(sp%npwc,nomega,rhotwgp(1:sp%npwc),botsq_conjg_transp,&
&       transpose(otq),omegame0i,sp%zcut,theta_mu_minus_e0i,ket2,&
&       ppmodel,eig,sigcme_3,sp%npwx,npwc1,npwc2)
       deallocate(botsq_conjg_transp)
       ket(:,:)=(ket1(:,:)+ket2(:,:))*0.5
      else
       ket(:,:)= ket1(:,:)
      end if

      deallocate(sigcme_new)

     else if (mod(sp%gwcalctyp,10)==9) then 
! MODEL GW calculation numerical integration

! Calculate \Sigma(E_k) | k > to obtain < j | \Sigma(E_k) | k >
      call calc_sig_noppm(sp%npwc,nomega,ep%nomega,ep%nomegaer,ep%nomegaei,rhotwgp(1:sp%npwc),&
&           er%omega,epsm1q,omegame0i,theta_mu_minus_e0i,ket1)

      if (sp%gwcalctyp==29) then
! Calculate < k | \Sigma(E_k) to obtain < k | \Sigma(E_k) | j >^*

       call calc_sig_noppm(sp%npwc,nomega,ep%nomega,ep%nomegaer,ep%nomegaei,rhotwgp(1:sp%npwc),&
&            er%omega,epsm1q_trcc,omegame0i,theta_mu_minus_e0i,ket2)

       ket(:,:)= (ket1(:,:)+ket2(:,:))*0.5

      else
       ket(:,:)= ket1(:,:)
      end if

     end if !of if (sp%gwcalctyp...)

!FB we could use hermitianity for HF, SEX, COHSEX to reduce next loop to:   do jb=minbnd,kb
     do jb=ib1,ib2  !left band index

! Check self-consistent or not? if not ==> only diagonal elements
      if (sp%gwcalctyp<20 .and. jb/=kb) cycle

      rhotwg(:)= rhotwg_ki(:,jb)

      if (analytic_continuation) then 
! Calculate the braket < \phi_j | \Sigma | \phi_k >
       do io=1,nomegasi
        sigctmp(io)= cdotc(sp%npwc,rhotwg(1:sp%npwc),1,ket(:,io),1)
       end do 
      else
       do io=1,nomega
        sigctmp(io) = cdotc(sp%npwc,rhotwg(1:sp%npwc),1,ket(:,io),1)
       end do
      end if 

      !Set up bare exchange matrix element
      !must be multiplied later by 4*PI/(ucvol*nkbz)
      sigxme=-cdotc(sp%npwx,rhotwg,1,rhotwgp,1)*theta_mu_minus_e0i
      !MG060926 added spin index

! Symmetrization of \Sigma_x matrix elements
      if (lt_k%sym_flag==0) then
       sigxme_tmp(jb,kb,isppol)= sigxme_tmp(jb,kb,isppol) + sigxme
      else
       sigxme_tmp(jb,kb,isppol)= sigxme_tmp(jb,kb,isppol) + &
&       (wtqp+wtqm)*real(sigxme)+ (wtqp-wtqm)*(0.,1.)*aimag(sigxme)
      end if

      call timab(424,2,tsec)
      call timab(425,1,tsec)

! The static COH is calculated only once, it is independent wrt the band index ib used to sum over bands
      if (mod(sp%gwcalctyp,10)==7 .and. ib==1) then
       call calc_coh(nr,sp%npwx,sp%npwc,ngfft1a,ngfft1,ngfft2,ngfft3,&
&       gvec,epsm1q(:,:,1),qbzpg,i_sz,iqibz,jb,kb,wfg2(:,jb,kb,isppol),sigcohme)
       !static COH is energy independent
       sigctmp(:)= sigctmp(:)+sigcohme
      end if

! Symmetrize matrix elements (works well only in case of standard perturbative approach)
! Must be multiplied later by 4*pi/(ucvol*nkbz)
      if (lt_k%sym_flag==0) then
       sigcme_tmp(:,jb,kb,isppol)= sigcme_tmp(:,jb,kb,isppol) + sigctmp(:)
      else
       sigcme_tmp(:,jb,kb,isppol)= sigcme_tmp(:,jb,kb,isppol) + &
&      (wtqp+wtqm)*real(sigctmp(:)) + (wtqp-wtqm)*(0.,1.)*aimag(sigctmp(:))
      end if

! Decomposition Sigma_c into Coulomb-hole (coh) and screened-exchange (sex) (GMR)
!MG FIXME the case nsppol==2 is not well tested Do not understand why > 0.5
      if (theta_mu_minus_e0i>0.5) then
       ff= 1
      else
       ff= 0
      end if

      twofm1= 2*ff-1
      twofm1_zcut= twofm1*sp%zcut
      twofm1_idelta= twofm1*idelta
      !Now, introduce rhotwgdpcc, for speed reasons
      rhotwgdpcc(:)= conjg(rhotwg(:))
      reomegame0i= omegame0i(ioe0j)

      if ((cohsex) .and. (io==sr%nomega+ioe0j)) then
       ccoh= 0
       csex= 0
       do igp =1,sp%npwc
        rhotwgdp_igp= rhotwg(igp)
        do ig =1,sp%npwc
         if (ppmodel==3 .and. ig/=igp) cycle
         otw= dble(otq(ig,igp)) ! in principle otw -> otw - ieta
         num= rhotwgdpcc(ig)*botsq(ig,igp)*rhotwgdp_igp
         den_coh= reomegame0i-otw
         if (den_coh**2>sp%zcut**2) then
          ccoh= ccoh + num/(den_coh*otw)
         else ! if den is small
          ccoh= ccoh + num*cmplx(den_coh,twofm1_zcut)/ &
&                ((den_coh**2+twofm1_zcut**2)*otw)
         end if
         den_sex= reomegame0i**2-otw**2
         if (den_sex**2>sp%zcut**2) then
          csex= csex - 2*ff*num/den_sex
         else ! if den is small
          csex= csex -2*ff*num*cmplx(den_sex,twofm1_zcut)/ &
&                      ((den_sex**2+twofm1_zcut**2)*otw)
         end if
        end do !ig
       end do !igp

       !Must be multiplied later by 4*pi/(ucvol*nkbz).
       sigccoh(jb,isppol)= sigccoh(jb,isppol) + ccoh/two
       sigcsex(jb,isppol)= sigcsex(jb,isppol) + csex/two

      end if ! cohsex

      ! Calculation of the kinetic contribution to the bandgap energy. (YMN on 07/04/04)
      !if ((kinetic).and.(io == sr%nomega+ioe0j)) then
      !print *,' omega = ',sr%omegasrd(jb,jkibz,io-sp%nomegasr)
      ! do igp = 1, sp%npwc
      !  do ig = 1, sp%npwc
      !   dct = zero
      !    do ig1 = 1, sp%npwc
      !     call fkin(ff,kincontrib,reomegame0i,otq(ig,ig1),otq(ig1,igp),sp%zcut)
      !     dct = dct+botsq(ig,ig1)*botsq(ig1,igp)*kincontrib
      !    end do
      !    ct = ct-rhotwgdpcc(ig)*dct*rhotwg(igp)
      !   end do
      !  end do
      ! sigckin(jb) = sigckin(jb)-ct*half  !Must be multiplied later by 4*pi/(ucvol*nkbz).
      !end if ! kinetic

      call timab(425,2,tsec)

     end do !jb used to calculate matrix elements of $\Sigma$

     !shaltaf (030406): this has to be done in a clean way later
     if (mod(sp%gwcalctyp,10)==0) then
      if (ppmodel==3 .or. ppmodel==4) then
       sigcme_tmp(:,kb,kb,isppol)= sigcme2(:,kb)
      end if
     end if

    end do !kb to calculate matrix elements of $\Sigma$
   end do !end loop over spin
  end do !ib
 end do !ikbz

! Now we have all the diagonal (off-diagonal) matrix elements for the input k-point

 deallocate(sigcme2,sigcme_3)

! Why not xsum_master, then only master works! Is it faster?
 !do is=1,sp%nsppol
 ! call xsum_mpi(sigcme_tmp(:,:,:,is),spaceComm,ierr)
 ! call xsum_mpi(sigxme_tmp(:,:,is),spaceComm,ierr)
 !end do

 !MG now we call ALLREDUCE once
 !btw there is a problem witu mpi since sigcme_tmp can be large 
 !since it is dimensioned used the full number of bands!!
 call xsum_mpi(sigcme_tmp(:,:,:,:),spaceComm,ierr)
 call xsum_mpi(sigxme_tmp(:,:,:),spaceComm,ierr)

 call timab(426,1,tsec)

! Multiply by constants
 sigxme_tmp(:,:,:)= (4*pi/(ucvol*sp%nkbz))*sigxme_tmp(:,:,:)
 sigcme_tmp(:,:,:,:)= (4*pi/(ucvol*sp%nkbz))*sigcme_tmp(:,:,:,:)

! In case of symmetrization we have to sum over the complex
! presently only diagonal terms are considered
 if (lt_k%sym_flag/=0) then

  allocate(dummy_xme(-minbnd+maxbnd+1,-minbnd+maxbnd+1,sp%nsppol))
!FIXME here there is a problem in case of AC with symmetries
  if (analytic_continuation) then 
   allocate(dummy_cme(nomegasi,-minbnd+maxbnd+1,-minbnd+maxbnd+1,sp%nsppol))
  else 
   allocate(dummy_cme(nomega,-minbnd+maxbnd+1,-minbnd+maxbnd+1,sp%nsppol))
  end if 

  dummy_cme= (0.,0.)
  dummy_xme= (0.,0.)

! Average over diagonal elements
  do is=1,sp%nsppol
   do ib=1,maxbnd-minbnd+1
    sumcxtab= 0
    do jb=1,maxbnd-minbnd+1
     if (cxtab(ib,is,jb)==1) then
      dummy_xme(ib,ib,is)= dummy_xme(ib,ib,is)+sigxme_tmp(minbnd-1+jb,minbnd-1+jb,is)
      !NOTE that frequencies should be equal, this is another reason to use a strict
      !criterion for the tollerance on eigenvalues
      dummy_cme(:,ib,ib,is)= dummy_cme(:,ib,ib,is)+sigcme_tmp(:,minbnd-1+jb,minbnd-1+jb,is)
     end if
     sumcxtab= sumcxtab+cxtab(ib,is,jb)
    end do
    dummy_xme(ib,ib,is)= dummy_xme(ib,ib,is)/sumcxtab
    dummy_cme(:,ib,ib,is)= dummy_cme(:,ib,ib,is)/sumcxtab
    !write(*,*) dummy_xme(ib,ib,is),dummy_cme(1,ib,ib,is)
   end do
  end do

  !off diagonal elements NOT SURE !FIXME
  !sigxme_tmp and sigc_me contain matrix element <n|Sigma |m>
  !in this case we sum only over the right complex for
  !the upper right part of the matrix is modified summing over the complex of |m>,
  !the lower left part is obtained summing over the complex of <n|

   allocate(ndwn(maxbnd-minbnd+1,maxbnd-minbnd+1))
   allocate(nup(maxbnd-minbnd+1,maxbnd-minbnd+1))

   do is=1,sp%nsppol
    ndwn= 0
    nup= 0

    do ib=1,maxbnd-minbnd+1
     do jb=1,maxbnd-minbnd+1

      if (jb>ib) then
       do kb=1,maxbnd-minbnd+1
        do zb=1,maxbnd-minbnd+1
         !if (zb>kb .and. cxtab(jb,is,zb)==1 .and. cxtab(ib,is,kb)==1) then
         if (zb>kb .and. cxtab(jb,is,zb)==1) then
          dummy_xme(ib,jb,is)=dummy_xme(ib,jb,is)+sigxme_tmp(minbnd-1+kb,minbnd-1+zb,is)
          dummy_cme(:,ib,jb,is)=dummy_cme(:,ib,jb,is)+sigcme_tmp(:,minbnd-1+kb,minbnd-1+zb,is)
          nup(ib,jb)=nup(ib,jb)+1
         end if
        end do
       end do
       if (nup(ib,jb)==0) nup(ib,jb)=1
       dummy_xme(ib,jb,is)=dummy_xme(ib,jb,is)/nup(ib,jb)
       dummy_cme(:,ib,jb,is)=dummy_cme(:,ib,jb,is)/nup(ib,jb)
      end if

      if (jb<ib) then
       do kb=1,maxbnd-minbnd+1
        do zb=1,maxbnd-minbnd+1
         !if (zb<kb .and. cxtab(ib,is,kb)==1 .and. cxtab(jb,is,kb)==1) then
         if (zb<kb .and. cxtab(ib,is,kb)==1) then
          dummy_xme(ib,jb,is)=dummy_xme(ib,jb,is)+sigxme_tmp(minbnd-1+kb,minbnd-1+zb,is)
          dummy_cme(:,ib,jb,is)=dummy_cme(:,ib,jb,is)+sigcme_tmp(:,minbnd-1+kb,minbnd-1+zb,is)
          ndwn(ib,jb)=ndwn(ib,jb)+1
         end if
        end do
       end do
       if (ndwn(ib,jb)==0) ndwn(ib,jb)=1
       dummy_xme(ib,jb,is)=dummy_xme(ib,jb,is)/ndwn(ib,jb)
       dummy_cme(:,ib,jb,is)=dummy_cme(:,ib,jb,is)/ndwn(ib,jb)
      end if

     end do
    end do

   end do

! Copy values
   do is=1,sp%nsppol
    do ib=1,maxbnd-minbnd+1
     do jb=1,maxbnd-minbnd+1
     !if (ib/=jb) cycle
     sigxme_tmp(ib-1+minbnd,jb-1+minbnd,is)= dummy_xme(ib,jb,is)
     !this is to check another scheme in case of AC
     if (analytic_continuation .and. average_real) cycle
     sigcme_tmp(:,ib-1+minbnd,jb-1+minbnd,is)= dummy_cme(:,ib,jb,is)
    end do
   end do
  end do

 end if

! If non self-consistent, erase all off-diagonal elements
 if (sp%gwcalctyp<20) then
  do jb=ib1,ib2
   do kb=ib1,ib2
    if (jb==kb) cycle
    sr%hhartree(jb,kb,jkibz,:)= (0.,0.)
   end do
  end do
 end if

 call diago_hamilt(minbnd,maxbnd,sp%nsppol,sr%hhartree(minbnd:maxbnd,minbnd:maxbnd,jkibz,:),&
&  sigxme_tmp(minbnd:maxbnd,minbnd:maxbnd,:),sigcme_tmp(sr%nomega+ioe0j,minbnd:maxbnd,minbnd:maxbnd,:),&
&  sr%eigvec_qp(minbnd:maxbnd,minbnd:maxbnd,jkibz,:),sr%en_qp_diago(minbnd:maxbnd,jkibz,:))

! Store final results
 do is=1,sp%nsppol
  do jb=ib1,ib2

! Store exchange diagonal element (\Sigma_x is hermitian)
  sr%sigxme(jb,jkibz,is)= real(sigxme_tmp(jb,jb,is))

!MG ADDED TO DEAL WITH AC (should be done in a cleaner way later)
  if (.not.analytic_continuation) then 

! Store correlation matrix elements calculated at energy E0
   sr%sigcmee0(jb,jkibz,is)= sigcme_tmp(sr%nomega+ioe0j,jb,jb,is)

! The values of SigC are linearly interpolated
! and the fit parameter a is assumed as the derivative dSigC/dE
  scme(:)= real(sigcme_tmp(sr%nomega+1:sr%nomega+sp%nomegasrd,jb,jb,is))
  e0pde(:,jb,is)= sr%omegasrd(jb,jkibz,:,is)

  if (sp%nomegasrd==1) then
   smrt= 0.0 ; a= 0.0
  else
   smrt= linfit(sp%nomegasrd,e0pde(:,jb,is),scme(:),a,b)
  end if

  if (smrt>0.1/Ha_eV) then
   write(message,'(4a,i4,a,i3,2a,f22.15,2a,f22.15,2a)')ch10,&
&   ' csigme : WARNING - the values of Re Sig_c are not linear ',ch10,&
&   '  band index       = ',jb,' spin = ',is,ch10,& 
&   '  root mean square = ',smrt,ch10,&
&   '  estimated slope  = ',a,ch10,&
&   '  Omega [eV] SigC [eV]' 
   call wrtout(6,message,'COLL')
   do ii=1,sp%nomegasrd
    write(*,*)e0pde(ii,jb,is)*Ha_eV,scme(ii)*Ha_eV
   end do
  end if

   sr%dsigmee0(jb,jkibz,is)= cmplx(a,0)

! Calculate Z= ( 1 - dSigma/domega(E0) )^-1
   sr%ze0(jb,jkibz,is) = 1/( 1 - sr%dsigmee0(jb,jkibz,is) )

! Calculate DeltaE_GW = E - E0 = (Sigma(E0) - V_xc) / (1 - dSigma/domega)
   sr%degw(jb,jkibz,is)= &
&   ( sr%sigxme(jb,jkibz,is)+sr%sigcmee0(jb,jkibz,is)-sr%e0(jb,jkibz,is)+sr%hhartree(jb,jb,jkibz,is) ) / &
&    ( 1 - sr%dsigmee0(jb,jkibz,is) )

   sr%egw(jb,jkibz,is)= sr%e0(jb,jkibz,is) + sr%degw(jb,jkibz,is)

! Calculate Sigma(E)= Sigma(E0) + (E-E0) dSigma/dE
   sr%sigmee(jb,jkibz,is)= sr%sigxme(jb,jkibz,is) + sr%sigcmee0(jb,jkibz,is) + &
&                            sr%degw(jb,jkibz,is) * sr%dsigmee0(jb,jkibz,is)

! Spectra of Sigma
   do io=1,sr%nomega
    sr%sigcme(jb,jkibz,io,is)= sigcme_tmp(io,jb,jb,is)
    sr%sigxcme(jb,jkibz,io,is)= sr%sigxme(jb,jkibz,is) + sr%sigcme(jb,jkibz,io,is)
   end do

   do io=1,sp%nomegasrd
    sr%sigcmesrd(jb,jkibz,io,is)= sigcme_tmp(sr%nomega+io,jb,jb,is)
    sr%sigxcmesrd(jb,jkibz,io,is)= sr%sigxme(jb,jkibz,is) + sr%sigcmesrd(jb,jkibz,io,is)
   end do

  else ! case AC
! Calculate Sigc(E0), dSigc(E0)
   zz= cmplx(sr%e0(jb,jkibz,is),0.0)
   if (real(zz)>0) then
    !Diagonal elements of sigcme_tmp
!FIXME  check what happens in case of SCGW, we need a different approach to symmetrize
    sr%sigcmee0(jb,jkibz,is) = pade_approx(nomegasi,sp_omegasi,sigcme_tmp(:,jb,jb,is),zz)
    sr%dsigmee0(jb,jkibz,is) = dpade_approx(nomegasi,sp_omegasi,sigcme_tmp(:,jb,jb,is),zz)
   else
    !Sigma(-iw)= Sigma(iw)*, avoid poles in the complex plane 
    sr%sigcmee0(jb,jkibz,is)=  pade_approx(nomegasi,conjg(sp_omegasi(:)),conjg(sigcme_tmp(:,jb,jb,is)),zz)
    sr%dsigmee0(jb,jkibz,is)= dpade_approx(nomegasi,conjg(sp_omegasi(:)),conjg(sigcme_tmp(:,jb,jb,is)),zz)
   end if

! Calculate Z= ( 1 - dSigma/domega(E0) )^-1
   sr%ze0(jb,jkibz,is)= 1/( 1 - sr%dsigmee0(jb,jkibz,is) )

! Calculate the 0 of E^0 - V_xc + Sig_x + Sig_c(z) - z, which is E^qp
! Using the Newton_Raphson method and starting from E^0
   zz= cmplx(sr%e0(jb,jkibz,is),0.0)
   do !i dont like infinite loop, should add a limit
    if (real(zz)>0) then
     sigc=   pade_approx(nomegasi,sp_omegasi,sigcme_tmp(:,jb,jb,is),zz)
     dsigc= dpade_approx(nomegasi,sp_omegasi,sigcme_tmp(:,jb,jb,is),zz)
    else
     sigc=   pade_approx(nomegasi,conjg(sp_omegasi(:)),conjg(sigcme_tmp(:,jb,jb,is)),zz)
     dsigc= dpade_approx(nomegasi,conjg(sp_omegasi(:)),conjg(sigcme_tmp(:,jb,jb,is)),zz)
    end if
    ct= sr%e0(jb,jkibz,is) - sr%vxcme(jb,jkibz,is) + sr%sigxme(jb,jkibz,is) + sigc - zz
    dct= dsigc - 1
    if (abs(ct)<0.0001/Ha_eV) exit
    zz= newton_raphson_step(zz,ct,dct)
   end do

! Store the final result (should shift everything according to efermi)
   sr%egw(jb,jkibz,is)= zz
   sr%degw(jb,jkibz,is)= sr%egw(jb,jkibz,is) - sr%e0(jb,jkibz,is)
   sr%sigmee(jb,jkibz,is)= sr%sigxme(jb,jkibz,is) + sigc

!  Spectra of Sigma, remember that sr%nomega does not contains the frequencies used to evaluate the derivative
!  each frequency is obtained using the pade_expression
   do io=1,sr%nomega
    zz= sr%omega(io)
    if (real(zz)>0) then
     sr%sigcme(jb,jkibz,io,is)= pade_approx(nomegasi,sp_omegasi,sigcme_tmp(:,jb,jb,is),zz)
    else
     sr%sigcme(jb,jkibz,io,is)= pade_approx(nomegasi,conjg(sp_omegasi(:)),conjg(sigcme_tmp(:,jb,jb,is)),zz)
    endif
    sr%sigxcme(jb,jkibz,io,is)= sr%sigxme(jb,jkibz,is) + sr%sigcme(jb,jkibz,io,is)
   end do

! Matsubara axis spectra
   do io=1,nomegasi
    sr%sigcmesi(jb,jkibz,io,is)=  sigcme_tmp(io,jb,jb,is)
    sr%sigxcmesi(jb,jkibz,io,is)= sr%sigxme(jb,jkibz,is) + sr%sigcmesi(jb,jkibz,io,is)
   end do

  end if !of Analytical continuation

 if (me==0) then
  write (6,'(i5,9f8.3)')                &
&  jb,                                  &
&  sr%e0(jb,jkibz,is)*Ha_eV,            &
&  sr%vxcme(jb,jkibz,is)*Ha_eV,         &
&  sr%sigxme(jb,jkibz,is)*Ha_eV,        &
&  real(sr%sigcmee0(jb,jkibz,is))*Ha_eV,&
&  real(sr%ze0(jb,jkibz,is)),           &
&  real(sr%dsigmee0(jb,jkibz,is)),      &
&  real(sr%sigmee(jb,jkibz,is))*Ha_eV,  &
&  real(sr%degw(jb,jkibz,is))*Ha_eV,    &
&  real(sr%egw(jb,jkibz,is))*Ha_eV

  write (6,'(i5,9f8.3)')                 &
&  jb,                                   &
&  0.0,0.0,0.0,                          &
&  aimag(sr%sigcmee0(jb,jkibz,is))*Ha_eV,&
&  aimag(sr%ze0(jb,jkibz,is)),           &
&  aimag(sr%dsigmee0(jb,jkibz,is)),      &
&  aimag(sr%sigmee(jb,jkibz,is))*Ha_eV,  &
&  aimag(sr%degw(jb,jkibz,is))*Ha_eV,    &
&  aimag(sr%egw(jb,jkibz,is))*Ha_eV
  end if 

  end do !jb
 end do !is

! Decomposition of Sigma_c into Coulomb-hole (coh) and screened-exchange (sex) (GMR)
!FIXME Check this part
 if (cohsex) then
  sigccoh(:,:)= (4*pi/(ucvol*sp%nkbz))*sigccoh(:,:)
  sigcsex(:,:)= (4*pi/(ucvol*sp%nkbz))*sigcsex(:,:)
  write (6,'(/a)') " COH and SEX decomposition of Sig_c(E0) (eV):"
  write (6,'(a5,3a10)') " band","     Sig_c","   Sig_coh","   Sig_sex"

  do is=1,sp%nsppol
   do jb=minbnd,maxbnd
    write (6,'(i5,3f10.5," (Re)")')jb,real(sr%sigcmee0(jb,jkibz,is))*Ha_eV, &
&                                     real(sigccoh(jb,is))*Ha_eV, &
&                                     real(sigcsex(jb,is))*Ha_eV
    write (6,'(5x,3f10.5," (Im)")')   aimag(sr%sigcmee0(jb,jkibz,is))*Ha_eV, &
&                                     aimag(sigccoh(jb,is))*Ha_eV, &
&                                     aimag(sigcsex(jb,is))*Ha_eV
   end do
  end do
 end if

! Calculation kinetic and Coulomb contributions to the bandgap energy (YMN)
!FIXME This part should be commented because sigckin is not calculated anymore
 if (kinclb) then
  sigckin(:)= (four*pi/(ucvol*sp%nkbz))*sigckin(:)
  write (6,'(/,a)') " Kinetic and Coulomb contributions to Sig_c(E0) (eV):"
  write (6,'(a5,3a10)') " band","     Sig_c","   Sig_c^k","   Sig_c^c"
  do jb = minbnd,maxbnd
!MG FIXME using only spin up in all the terms in the following part
   write (6,'(i5,3f10.5," (Re)")') jb,real(sr%sigcmee0(jb,jkibz,1))*Ha_eV, &
&                                     real(sigckin(jb))*Ha_eV, &
&                                    (real(sr%sigcmee0(jb,jkibz,1))-real(sigckin(jb)))*Ha_eV
   write (6,'(5x,3f10.5," (Im)")')    aimag(sr%sigcmee0(jb,jkibz,1))*Ha_eV, &
&                                     aimag(sigckin(jb))*Ha_eV, &
&                                    (aimag(sr%sigcmee0(jb,jkibz,1))-aimag(sigckin(jb)))*Ha_eV
  end do
  deallocate(sigckin)
 end if

!******************************************************************************************* 

!Deallocate memory
 deallocate(omegame0i,sigcme_tmp)
 deallocate(wfr_jb)
 deallocate(ket,ket1,ket2)
 deallocate(otq,botsq)
 
 if (mod(sp%gwcalctyp,10)==7) deallocate (wfg2)

 if (allocated(sigctmp))      deallocate(sigctmp)
 if (allocated(sigccoh))      deallocate(sigccoh)
 if (allocated(sigcsex))      deallocate(sigcsex)
 if (allocated(eig))          deallocate(eig)
 if (allocated(epsm1q))       deallocate(epsm1q)
 if (allocated(epsm1q_trcc))  deallocate(epsm1q_trcc)
 if (allocated(cxtab))        deallocate(cxtab)
 if (allocated(dummy_xme))    deallocate(dummy_xme)
 if (allocated(dummy_cme))    deallocate(dummy_cme)
 if (allocated(ndwn))         deallocate(ndwn)
 if (allocated(nup))          deallocate(nup)

! AC related quantities
 if (allocated(sigcme_ac))    deallocate(sigcme_ac)
 if (allocated(epsm1cqwz2))   deallocate(epsm1cqwz2)
 if (allocated(integr))       deallocate(integr)

 call timab(426,2,tsec)
 call timab(421,2,tsec)

!DEBUG
!write(6,*)' csigme : exit '
!ENDDEBUG

end subroutine csigme
!!***
