!{\src2tex{textfont=tt}}
!!****f* ABINIT/cutoff_sphere
!! NAME
!! cutoff_sphere
!!
!! FUNCTION
!!  Calculate the Fourier transform of the coulombian interaction with a spherical cutoff   
!!  vc(G)= \frac{4\pi}{G^2} [ 1-cos(G*R_cut) ]   (1)
!!
!! COPYRIGHT
!!  Copyright (C) 2007 ABINIT group (MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  gmet(3,3)=metric in reciprocal space
!!  gvec(3,ng)=G vectors in reduced coordinates 
!!  rcut=cutoff radius
!!  ng=number of G vectors
!!  nq=number of q-points 
!!  qpt(3,nq)=q-points where the cutoff coulombian is required
!!
!! OUTPUT
!!  vccut(nq,ng)= Fourier components of the effective coulombian interaction 
!!
!! SIDE EFFECTS
!!
!! NOTES
!!  For |q| < small and G=0 we use the limit -->0 of Eq. (1) 
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine cutoff_sphere(nq,qpt,ng,gvec,gmet,rcut,vccut)
    
 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_12geometry
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in)  :: nq,ng 
 real(dp),intent(in) :: rcut
!arrays
 integer,intent(in) :: gvec(3,ng)
 real(dp),intent(in) :: gmet(3,3),qpt(3,nq)
 real(dp),intent(out) :: vccut(nq,ng) 

!Local variables-------------------------------
 integer :: ig,igs,iq                     
 real(dp),parameter :: small=tol6     
 real(dp) :: qpg
!character(len=500) :: message                  
 
! *************************************************************************
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 real(dp) :: normv
#endif
!End of the abilint section

 do iq=1,nq
  igs=1
  if (normv(qpt(:,iq),gmet,'g')<small) then  
   ! For small q and G=0 use the limit q-->0 
   vccut(iq,1)= 2.0*pi*rcut**2
   igs=2
  end if
  do ig=igs,ng
   qpg=normv(qpt(:,iq)+gvec(:,ig),gmet,'g')
   vccut(iq,ig)= 4*pi*(1.0_dp-cos(rcut*qpg))/ qpg**2
  end do
 end do
 
end subroutine cutoff_sphere
!!***
