!{\src2tex{textfont=tt}}
!!****f* ABINIT/cutoff_table
!! NAME
!! cutoff_table
!!
!! FUNCTION
!! Create a table for each point in the FFT grid 
!!  0 if the point is outside the cutoff region
!!  1 otherwise 
!!
!!  This table is used to redefine the density in the isolated system (see cutoff_density)
!!
!! COPYRIGHT
!!  Copyright (C) 2007 ABINIT group (MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  rmet(3,3)=real space metric ($\textrm{bohr}^{2}$).
!!  cutp <type cutoff_data> 
!!   The following quantities are used :
!!   cutp%ngfft(1:3)= FFT dimensions
!!   cutp%nr= number of point in the FFT mesh
!!   cutp%boxcenter=reduced coordinates of the center of the box  (input variable
!!  (FIXME kept from tddft.F90, should write something in doc) 
!!   cutp%cutoffmode= string of characters defining the cutoff mode 
!!   cutp%rcut= cutoff radius 
!!
!! OUTPUT
!!  cutp%ctab(cutp%nr)
!!  For each point of the FFT grid gives 
!!   1 if the point in inside the cutoff region
!!   0 otherwise
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!  
!!
!! CHILDREN
!!  
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine cutoff_table(rmet,cutp)
    
 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_12geometry
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars 
 type(cutoff_data),intent(inout) :: cutp
!arrays 
 real(dp),intent(in) :: rmet(3,3) 

!Local variables-------------------------------
!scalars
 integer :: ix,iy,iz,ir,ngfft1,ngfft2,ngfft3   
 character(len=500) :: message       
!arrays
 real(dp) :: rr(3),rxy(3),rz(3) 
 
! *************************************************************************
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 real(dp) :: normv
#endif
!End of the abilint section

 ngfft1=cutp%ngfft(1) 
 ngfft2=cutp%ngfft(2) 
 ngfft3=cutp%ngfft(3)

 cutp%ctab(:)=1

 do ix=0,ngfft1-1
  do iy=0,ngfft2-1
   do iz=0,ngfft3-1

    rr(1)=dble(ix)/ngfft1
    rr(2)=dble(iy)/ngfft2
    rr(3)=dble(iz)/ngfft3
    ir= 1 + ix +iy*ngfft1 + iz*ngfft1*ngfft2
    rr(:)=rr(:)-cutp%boxcenter(:)

    select case (trim(cutp%cutoffmode))
     case('sphere')
      if (normv(rr,rmet,'r') > cutp%rcut) cutp%ctab(ir)=0
     case('cylinder') !infinite cylinder
      rxy(1)=rr(1)
      rxy(2)=rr(2)
      rxy(3)=zero
      if (normv(rxy,rmet,'r') > cutp%rcut) cutp%ctab(ir)=0
     case('surface')
      rz(:)=zero
      rz(3)=rr(3)
      if (normv(rxy,rmet,'r') > cutp%rcut) cutp%ctab(ir)=0
     case default     
     goto 1
    end select

   end do 
  end do 
 end do

 return

1  write(message,'(6a)')ch10,&
&   ' cutoff_table : BUG -',ch10,&
&   '  cutoffmode ',trim(cutp%cutoffmode),' not yet implemented'
   call wrtout(6,message,'COLL')
   call leave_new('COLL')

end subroutine cutoff_table
!!***
