!{\src2tex{textfont=tt}}
!!****f* ABINIT/cvxclda
!! NAME
!! eps1_tc
!!
!! FUNCTION
!! Starting from the RPA DM calculate TC DM
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (Rhaltaf,XG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  dtset <type(dataset_type)>=all input variables in this dataset
!!  mpi_enreg = informations about MPI parallelization.
!!  ngfft1,ngfft2,ngfft3 = size of the FFT grid.
!!  nr = total number of points on the FFT grid.
!!  rho(nr,nsppol) = the charge density on the FFT grid.
!!   (total in first half and spin-up in second half if nsppol=2)
!!  rprimd(3,3) = dimensional real space primitive translations (bohr).
!!  npweps: the size of kernel matrix
!!  igfft=array of fft index of each G vector 
!!  epsm1: RPA eps1
!! OUTPUT
!!  epsm1: TC eps1
!!  binary file with extension GWG_SCR that includes $\varepsilon^{-1}$ with vertex corrections
!!
!! NOTES
!!  
!!
!! PARENTS
!!      screening
!!
!! CHILDREN
!!      leave_new,rhohxc,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine eps1_tc(dtset,mpi_enreg,ngfft1,ngfft2,ngfft3,nr,rho,rprimd,igfft,&
& npweps,gmet,gvec,nomega,epsm1,nq,ucvol,qq,omega,b1,b2,b3,&
& dtfil,hdr,npwwfn,npwvec,nbnds)
 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_11util
 use interfaces_13xc
 use interfaces_15gw, except_this_one => eps1_tc
#else
 use defs_interfaces
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: ngfft1,ngfft2,ngfft3,nr,npweps,igfft(npweps),gvec(3,npweps),nomega,nq,npwwfn,npwvec,nbnds
 type(MPI_type),intent(inout) :: mpi_enreg
 type(dataset_type),intent(in) :: dtset
 type(datafiles_type),intent(in) :: dtfil
 type(hdr_type),intent(inout) :: hdr
!arrays
 real(dp),intent(in) :: rho(nr,dtset%nsppol),rprimd(3,3),gmet(3,3),ucvol,qq(3,nq),b1(3),b2(3),b3(3)
 complex,intent(in)::omega(nomega)
 complex,intent(inout)::epsm1(npweps,npweps,nomega,nq)
!Local variables ------------------------------
!scalars
 integer :: igp,ig,iq,ixc,nsppol,iomega,ig1,ig2,ig4,ig4x,ig4y,ig4z
 character(len=500) :: message
!arrays
 integer :: ngfft(18)
 real(dp) :: gprimd(3,3),epsilon0
 real(dp),allocatable::qplusg(:)
 complex,allocatable ::kernel(:,:,:),chi0(:,:,:),chitmp(:,:),chi(:,:,:),kxclda(:)
 character(len=80) :: title(2)
 integer,parameter::unitf=0
 logical::found
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 integer :: modx
#endif
!End of the abilint section

!************************************************************************


! calculate the exc kerenel
    ixc=dtset%ixc
    nsppol=dtset%nsppol

   allocate(kernel(npweps,npweps,nq),chi0(npweps,npweps,nomega),chitmp(npweps,npweps),chi(npweps,npweps,nomega),qplusg(npweps))

   call xc_kernel(dtset,ixc,mpi_enreg,ngfft1,ngfft2,ngfft3,nr,nsppol,rho,rprimd,igfft,npweps,gmet,kernel,gvec,nq,qq)


    write(message,'(a)')' kxclda(g)'
    call wrtout(06,message,'COLL')
    call printv(kernel(1,:,1),npweps)


    title(1)='em1 file: epsilon^-1'
    title(2)='TESTELECTRON'

! extract chi0 from RPA eps1
 
 do iq=1,nq
  do iomega=1,nomega
   call matcginv(epsm1(:,:,iomega,iq),npweps,npweps)
  end do
 end do


  do iq=1,nq   
   call cvc(nq,iq,qq,b1,b2,b3,npweps,gvec,qplusg)

   do iomega=1,nomega
    do ig=1,npweps
     epsm1(ig,ig,iomega,iq)=epsm1(ig,ig,iomega,iq)-cone
     do igp=1,npweps
      chi0(ig,igp,iomega)=(qplusg(ig)*qplusg(igp)*epsm1(ig,igp,iomega,iq))/(-4.0*pi)
    end do
   end do
   end do


  chi0(:,:,:)=chi0(:,:,:)*ucvol
   
   do iomega=1,nomega
     write (message,'(a,i2,a,i1,a)')' chi0(q=',iq,',omega=',iomega,',G,G")'
     call wrtout(06,message,'COLL')
     call printcm(chi0(:,:,iomega),npweps,npweps)
    end do


! starting from chi0 calculate chi=(1-chi0*Vc-chi0*Kernel)^-1 chi0'


  do iomega=1,nomega
   chitmp=matmul(chi0(:,:,iomega),kernel(:,:,iq))

  do ig=1,npweps
   do igp=1,npweps
      chitmp(ig,igp)=(-(4.0*pi)/(qplusg(igp)**2))*&
    &      chi0(ig,igp,iomega)-chitmp(ig,igp)
   end do
      chitmp(ig,ig)=chitmp(ig,ig)+ucvol
  end do
   
!invert (1-chi0*Vc-chi0*Kernel)
  call matcginv(chitmp(:,:),npweps,npweps)
    
!multiply for chi0 to obtain chi
  chi(:,:,iomega)=matmul(chitmp(:,:),chi0(:,:,iomega))
 
 !must multiply by ucvol (see notes-due to conversion of r->G in
 !matrix definitions)
  chi(:,:,iomega)=chi(:,:,iomega)*ucvol
  
   end do 

   do iomega=1,nomega
   write (message,'(a,i2,a,i1,a)')' chi(q=',iq,',omega=',iomega,',G,G")'
   call wrtout(06,message,'COLL')
   call printcm(chi(:,:,iomega),npweps,npweps)
   end do


! starting from chi calculate Tc-eps1

  chitmp(:,:)=czero
  epsm1(:,:,:,iq)=czero

  do iomega=1,nomega

  chitmp=matmul(kernel(:,:,iq),chi(:,:,iomega))

! Perform hermitianization

   call hermitianize(chitmp,npweps)

    do ig=1,npweps
     do igp=1,npweps
      epsm1(ig,igp,iomega,iq)=(4*pi/(qplusg(ig)*qplusg(igp)))*&
&     chi(ig,igp,iomega)/ucvol+chitmp(ig,igp)/ucvol
      !chitmp=Kxc*chi must be divided by ucvol see notes on fft r->G
     end do
     epsm1(ig,ig,iomega,iq)=epsm1(ig,ig,iomega,iq)+cone
  end do !ig
   
  end do ! iomega

! Print epsilon-twiddle^-1
    do iomega=1,nomega
     write (message,'(a,i2,a,i1,a)')' epsilon-twiddle^-1(q=',iq,',omega=',iomega,',G,G")'
     call wrtout(06,message,'COLL')
     call printcm(epsm1(:,:,iomega,iq),npweps,npweps)
    end do


   if (all(abs(qq(:,iq))<1.0e-3))then
    do iomega=1,nomega
     if(abs(real(omega(iomega)))<1.e-3) then
      if(abs(aimag(omega(iomega)))<1.e-3) then
       epsilon0=real(1.0/epsm1(1,1,iomega,iq))
        write(message,'(a,a,f8.4,a)')' dielectric constant after including',& 
&        ' vertex corrections = ',epsilon0,ch10
        call wrtout(06,message,'COLL') 
      end if
     end if
    end do
   end if
  call wrscr(dtfil,hdr,dtset,npweps,npwwfn,npwvec,nbnds,nq,nomega,qq,omega,&
&             gvec,iq,epsm1,title,unitf,hdr%nsym,dble(hdr%symrel))

 end do ! iq

 deallocate(kernel,chi0,chitmp,chi,qplusg)


end subroutine eps1_tc

