!{\src2tex{textfont=tt}}
!!****f* ABINIT/outeps
!! NAME
!! outeps
!!
!! FUNCTION
!!  Write the independent matrix elements of epsilon^{-1} in reciprocal space
!!
!! COPYRIGHT
!!  Copyright (C) 2007 ABINIT group (MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  gvec(3,npweps)= G vectors in reduced coordinates
!!  gmet(3,3)=metric tensor in reciprocal space
!!  ngpi=nuber of independent (G1,G1) pairs   
!!  gptab(npweps,npweps)= index of the indendent pair (G1,G2) such as (T1,T2)= S (G1,G2) 
!!  gptabo(npweps,npweps)= index of the symmetry operation S in the array op such as (T1,T2)= S (G1,G2)
!!  ip2fp(2,ngpi)= index of G1 and G2 in the array gvec for each ngpi independent pair
!!  npweps=number of G vectors in eps 
!!  nomega=number of frequencies
!!  title=title describing what is printed
!!  eps(npweps,npweps,nomega)=matrix to be printed
!!  omega(nomega)=frequencies
!!  prtvol= if different from 0 then all the independent pairs are written 
!!          if ==0 only first and last 50 pairs are written
!!  unt=unit number of external file
!!
!! OUTPUT
!!  
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine outeps(npweps,gvec,nomega,omega,ngpi,gptab,gptabo,ip2fp,eps,gmet,title,unt,prtvol)
    
 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_12geometry
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars 
 integer,intent(in) :: ngpi,npweps,nomega,unt,prtvol
 character(len=500),intent(in) :: title
!arrays
 integer,intent(in) :: gvec(3,npweps)
 integer,intent(in) :: gptab(npweps,npweps) 
 integer,intent(in) :: gptabo(npweps,npweps)
 integer,intent(in) :: ip2fp(2,ngpi)
 real(dp),intent(in) :: gmet(3,3)
 complex,intent(in) :: eps(npweps,npweps,nomega)
 complex,intent(in) :: omega(nomega)

!Local variables-------------------------------
!scalars
 integer :: ii,io,ip,f1,f2,enough
 integer :: npmax
 real(dp) :: e1,e2
 character(len=500) :: message         
 
! *************************************************************************
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 real(dp) :: normv
#endif
!End of the abilint section

  npmax= 50
  if (ngpi<50) npmax= ngpi
  if (prtvol/=0) npmax= huge(1)
 
  write(message,'(3a)')ch10,'# Independent matrix elements of ',trim(title)
  call wrtout(unt,message,'COLL')

  do io=1,nomega

   write(message,'(2a,i3,a,2f9.4,3a)')ch10,'#',&
&   io,'-th omega :',omega(io)*Ha_eV,' [eV]',ch10,&
    '#         G1         E1 [H]           G2         E2 [H]         Re        Im'
   call wrtout(unt,message,'COLL')
   enough=0

   do ip=1,ngpi
    enough= enough+1
    if (enough>npmax .and. enough<ngpi-npmax) cycle
    f1= ip2fp(1,ip)
    f2= ip2fp(2,ip)
    e1= normv(dble(gvec(:,f1)),gmet,'g')
    e1= half*e1**2
    e2= normv(dble(gvec(:,f2)),gmet,'g')
    e2= half*e2**2
    write(unt,'(2(3i6,4x,f4.2,2x),4x,2(f8.4,2x))')gvec(:,f1),e1,gvec(:,f2),e2,eps(f1,f2,io)
    if (enough==npmax) then 
     write(unt,*)('-',ii=1,77)
     write(unt,*)' outeps : prtvol=0, stop printing more G-G'' information' 
     write(unt,*)('-',ii=1,77)
    end if 
   end do

  end do

end subroutine outeps
!!***
