!{\src2tex{textfont=tt}}
!!****f* ABINIT/testlda
!! NAME
!! testlda
!!
!! FUNCTION
!! Test QPLDA or ABINIT LDA or KSS type file
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (GMR, VO, LR, RWG, MG, RShaltaf)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  dtset<type(dataset_type)>=all input variables for this dataset
!!  dtfil=filenames (only filkss is used)
!!  localrdwf=input variable (for parallel case). If 1, the KSS file is local to each machine
!!
!! OUTPUT
!!  i1=integer defining the file format (502 or 602 if KSS file) 
!!  i2=integer defining the file format (old file formats)
!!  ibocc(dtset%nsppol)=for each spin this array defines the band index after which the 
!!   occupations numbers are smaller that tol8 (used for screening calculations based on 
!!  the Adler-wiser epxression with time reversal or spectral method)
!!  mpsang=1+maximum angular momentum for nonlocal pseudopotential
!!  natom=number of atoms
!!  nbnd=number of bands
!!  nkpt=number of k points
!!  npw=number of plane waves
!!  nsh=number of shells
!!  nsym=number of symmetries
!!  nsppol=1 for unpolarized, 2 for spin-polarized
!!  ntypat=number of atom types
!!
!! PARENTS
!!      screening,sigma
!!
!! CHILDREN
!!      hdr_clean,hdr_io,hdr_io_netcdf
!!
!! SOURCE
#if defined HAVE_CONFIG_H
#include "config.h"
#endif

 subroutine testlda(dtset,dtfil,nsym,nkpt,nbnd,npw,nsh,i1,i2,ntypat,natom,&
& mpsang,nsppol,mpi_enreg,localrdwf,ibocc)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_13io_mpi
 use interfaces_14iowfdenpot
 use interfaces_15gw, except_this_one => testlda
 use interfaces_lib01hidempi
#else
 use defs_interfaces
 use defs_xfuncmpi
#endif
!End of the abilint section

 implicit none

#if defined MPI || defined MPI_FFT
    include 'mpif.h'
#endif

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: localrdwf
 integer,intent(out) :: i1,i2,mpsang,natom,nbnd,nkpt,npw,nsh,nsym,ntypat,nsppol
 type(datafiles_type),intent(in) :: dtfil
 type(dataset_type),intent(in) :: dtset
 type(MPI_type),intent(in) :: mpi_enreg
!arrays
 integer,intent(inout) :: ibocc(dtset%nsppol)
!Local variables-------------------------------
!scalars
 integer,parameter :: unitkss=23
 integer :: fform,ii,i10,i11,i12,i13,i14,i15,i16,i3,i4,i5,i6,i7,i8,i9,nsym2,rdwr,ios
 integer :: nel,isppol,ik,ib,is,il
 integer :: readnetcdf,status1,status2,status3
 integer:: spaceComm,me,master,ierr
 real(dp) :: efermi
 logical,parameter :: DEBUG=.false.
 logical:: read_ok
 type(hdr_type) :: hdr
 character(len=500) :: message
!arrays
 real(dp),allocatable :: temp_en(:,:,:),temp_oc(:,:,:)
 real,allocatable :: energys(:)
 real(dp),allocatable :: energyd(:)
 character(len=80) :: tit(2)
 
! *************************************************************************

 ! Initialize MPI related quantities
 call xcomm_init(mpi_enreg,spaceComm) !Init mpi_comm
 call xme_init(mpi_enreg,me)          !Init me
 call xmaster_init(mpi_enreg,master)  !Init master

 write(message,'(3a)')&
& ' testlda : testing Kohn-Sham structure file ',trim(dtfil%filkss),ch10
 call wrtout(6,message,'COLL')

 if (me==0 .or. localrdwf==1) then
  !
  ! Try the KSS abinit file format
  !
  open(unitkss,file=dtfil%filkss,status='old',form='unformatted',iostat=ios)
  if (ios/=0) then 
   write(message,'(6a)')ch10,&
&   ' testlda: ERROR- ',ch10,&
&   '  opening file: ',trim(dtfil%filkss),' as old'
   call wrtout(6,message,'COLL') ; call leave_new('COLL')
  end if
  ! Read the header of the GS wavefunction file
  rdwr=1
  readnetcdf=0 ! should become input parameter and make KSS a netcdf file as well
  if (readnetcdf==0) then
   call hdr_io(fform,hdr,rdwr,unitkss)
  else if (readnetcdf==1) then
   call hdr_io_netcdf(fform,hdr,rdwr,unitkss)
  end if
  !
  ! Read information needed to allocate some arrays
  !
  nsppol=hdr%nsppol
  ntypat=hdr%ntypat
  natom=hdr%natom
  nkpt=hdr%nkpt
  nsym=hdr%nsym
  !
  if (fform==502 .or. fform==602) then
   !
   ! KSS file successfully read
   !
   read_ok=.true.
   write(message,'(1x,47a)')('-',ii=1,47)
    call wrtout(6,message,'COLL')
   if (fform==502) write(message,'(a)')' KSS abinit double precision form'
   if (fform==602) write(message,'(a)')' KSS abinit single precision form'
   call wrtout(6,message,'COLL')
   i2=fform
   ! XG20030510 : Should be changed, later. Indeed, it is not consistent
   ! with the definition of the structured datatyp hdr, and the
   ! associated routines. Nothing should be read directly in the header
   ! of the file : everything is already in hdr ...
   write(message,'(2a,a6,a,i3)')ch10,&
&   ' generated by ABINIT ',hdr%codvsn,' header version ',hdr%headform
   call wrtout(6,message,'COLL')
   if (hdr%headform/=23 .and. hdr%headform/=34 .and. &
&      hdr%headform/=40 .and. hdr%headform/=41 .and. &
&      hdr%headform/=42 .and. hdr%headform/=44 .and. &
&      hdr%headform/=53 ) then 
    write(message,'(4a)')ch10,  &
&    ' testlda : ERROR - ',ch10,&
&    '  unknown  header version '
    call wrtout(6,message,'COLL') ; call leave_new('COLL')
   end if 
   !
   read(unitkss) tit(1)
   read(unitkss) tit(2)
   !
   write(message,'(a)')' title of file:'
    call wrtout(6,message,'COLL')
   write(message,'(1x,a79,a,1x,a79,a)')tit(1)(:79),ch10,tit(2)(:79),ch10
    call wrtout(6,message,'COLL')
   read(unitkss) nsym2,nbnd,npw,nsh,mpsang
   nsym=nsym2
   write(message,'(a,i8)')' number of atomic species      ',ntypat
    call wrtout(6,message,'COLL')
   write(message,'(a,i8)')' number of atoms               ',natom
    call wrtout(6,message,'COLL')
   write(message,'(a,i8)')' number of ang mom components  ',mpsang
    call wrtout(6,message,'COLL')
   write(message,'(a,i8)')' number of symmetry operations ',nsym
    call wrtout(6,message,'COLL')
   write(message,'(a,i8)')' number of plane waves         ',npw
    call wrtout(6,message,'COLL')
   write(message,'(a,i8)')' number of k-points            ',nkpt
    call wrtout(6,message,'COLL')
   write(message,'(a,i8)')' number of bands               ',nbnd
    call wrtout(6,message,'COLL')
   write(message,'(a,i8)')' number of spin polarisations  ',nsppol
    call wrtout(6,message,'COLL')
   write(message,'(1x,47a)')('-',ii=1,47)
    call wrtout(6,message,'COLL')
   !
   ! Perform some checks
   !
   if (hdr%nsym/=nsym2) then 
    write(message,'(3a)')ch10,&
&    ' testlda : WARNING - ',&
&    ' the code does not use the original set of symmetries : hdr%nsym/=nsym2'
    call wrtout(6,message,'COLL')
   end if 
   if (nsppol/=dtset%nsppol) then 
    write(message,'(6a)')ch10,&
&    ' testlda ERROR - ',ch10,&
&    '  the value of nsspol read in the KSS file does not agree with the value ',ch10,&
&    '  specified in the input file, please modify your input file'
    call wrtout(6,message,'COLL') ; call leave_new('COLL')
   end if 
   !
   ! In case of parallelism over bands find the band index separating 
   ! the occupied and the partially occupied from the empty states (for each spin)
   ! Each processor will store in memory the fully and the partially occupied states
   !
   !FIXME dirty coding
   if (dtset%awtr==1 .or. mpi_enreg%gwpara==2 .or. dtset%spectral/=0) then 
    !in screening this means read_val=.true.
    write(message,'(2a)')ch10,&
&    ' testlda: reading occupation numbers to calculate ibocc '
    call wrtout(06,message,'PERS')
    allocate(temp_oc(nkpt,nbnd,nsppol),temp_en(nkpt,nbnd,nsppol))
    temp_oc(:,:,:)=zero ; temp_en(:,:,:)=zero
    do isppol=1,nsppol
     do ik=1,nkpt
      do ib=1,hdr%nband(ik+nkpt*(isppol-1))
       !NOTE1: hdr%nband(nkpt*nsppol) in the GW part the number of bands 
       !must be the same for each k-point and spin
       !NOTE2: the number of bands defined in the header and the first 
       !section of the KSS file might differ if the KSS file is generated including buffer
       ii= (ik-1)*hdr%nband(ik) +(isppol-1)*nkpt*hdr%nband(ik) + ib 
       if (ib<=nbnd) temp_oc(ik,ib,isppol)=hdr%occ(ii)
      end do
     end do
    end do 
    if (fform==502) allocate(energyd(1:nbnd))
    if (fform==602) allocate(energys(1:nbnd))
    !
    ! Read from the KSS file the eigenvalues
    !
    !read(unitkss) !tit(1) 
    !read(unitkss) !tit(2)
    !read(unitkss) !nsym2,nbandkss,npwkss,nsh,mpsang
    read(unitkss)  !(((symrel2(jj,ii,isym),ii=1,3),jj=1,3),isym=1,nsym2)
    read(unitkss)  !((tnons(i,isym),i=1,3),isym=1,nsym2)
    read(unitkss)  !((gvec(i,ig),i=1,3),ig=1,ngx)
    read(unitkss)  !(shlim(i),i=1,nsh)
    read(unitkss)  !((vkbsignd(il,is),il=1,mpsang),is=1,ntypat)
    !
    do isppol=1,nsppol
     do ik=1,nkpt
      do is=1,ntypat
       do il=1,mpsang
        read(unitkss) !vkbdb(:,is,il)
        read(unitkss) !vkbdd(:,is,il)
       end do 
      end do 
      if (fform==502) then
       read(unitkss) energyd(1:nbnd)
       temp_en(ik,1:nbnd,isppol)= energyd
      else if (fform==602) then
       read(unitkss) energys(1:nbnd)
       temp_en(ik,1:nbnd,isppol)= real(energys,dp)
      end if
      do ib=1,nbnd
       read(unitkss) !wfgd(1:npwkss)
      end do
     end do !ik 
    end do !isppol
    nel=zero
    !
    ! Call fermi to calculate ibocc in case of semiconductors
    ! Recalculate occupation numbers in case of metals
    ! (if the KSS has been generated during a NSCF run occ are set to zero if I remember well)
    !
    call fermi(hdr,nbnd,nkpt,dtset%fixmom,nsppol,hdr%wtk,temp_en,temp_oc,nel,ibocc,efermi)

    ! FIXME first coding, I do not know what happens in case of SCGW 
    ! if a metal becomes semiconductor or viceversa
    ! for the moment define a limiting value for the band index for both the spins after which 
    ! we can consider the states as completely unoccupied 
    if (hdr%occopt>= 3 .and. hdr%occopt<= 7) then
     ibocc(:)= -huge(0)
     do isppol=1,nsppol
      do ik=1,nkpt
       do ib=1,nbnd
        ! Using abs to avoid problems with negative occupation numbers 
        ! tol8 is chosen to be consistent with crho.F90 and density.F90
        if (abs(temp_oc(ik,ib,isppol)) < tol8) then 
         ibocc(isppol)= max(ib,ibocc(isppol)) 
         exit 
        end if 
       end do 
      end do 
     end do 
    end if !metallic system
    !
    if (me==0) write(*,*)' Max band index for fully and partially occupied states : ',ibocc(:)
    if (fform==502) deallocate(energyd)
    if (fform==602) deallocate(energys)
    deallocate(temp_oc,temp_en)
   end if !of (mpi_enreg%gwpara==2)
   close(unitkss)
  else !of if (fform==502 .or. fform==602)
   read_ok=.false.
  end if
  !
  if (.not.read_ok) then
   !
   ! Trying the STA or the QPLDA form
   !
   rewind(unitkss)
   read(unitkss,iostat=status1) tit(1)
   if(DEBUG) print *, tit(1)
   read(unitkss,iostat=status2) tit(2)
   if(DEBUG) print *, tit(2)
   read(unitkss,iostat=status3) i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,i11,i12,i13,i14,i15,i16
   write(6,'(1x,a80,/,1x,a80)') tit(1),tit(2)
   !flags used so far:
   !i1=0 if QPLDA file is formatted, 1 if unformatted
   !i2=0 if RWG format, 1 if BF format, 2 ABINIT format
   !i3=1 if non-symmorphic operations (+vectors) included, otherwise 0
   if(DEBUG) print *, i1,i2,i3

   if(status1/=0.or.status2/=0.or.status3/=0) then
    if(DEBUG) print *, status1, status2, status3
    stop 'unknown file form, status'
   end if

   if(i2==2) then
    write(6,*) 'STA abinit form'
   else if(i2==0.or.i2==1) then
    write(6,*)'qplda cp form'
   else
    if(DEBUG) print *, 'i2 ', i2
    stop 'unknown file form, i2'
   end if

   if(i1/=1) then
    write(6,*)'formatted file'
   else
    write(6,*)'unformatted file'
   end if
   if(i3==1) then
    write(6,*)'ERROR testlda 1: check i3'
   end if

   write(6,*)'title of file:'
   write(6,'(1x,a79/1x,a79)') tit(1)(:79), tit(2)(:79)

! Skip real-space lattice vectors
   read(unitkss)
   read(unitkss) nsym
   write(6,*) 'number of symmetry operations ',nsym
! Skip symmetry operations
   read(unitkss)
   read(unitkss) npw
   write(6,*) 'number of plane waves         ',npw
! Skip g-vector
   read(unitkss)
   read(unitkss) nkpt
   write(6,*) 'number of k-points            ',nkpt
! Skip k-points
   read(unitkss)
   read(unitkss) nbnd
   write(6,*) 'number of bands               ',nbnd
!  MG 061409
!  for compatibility reasons nsppol is set to 1 in case of qplda file format
!  note however that the qplda format is not supported anymore  
!  needed a modification of the qplda format to treat spin polarized calculations!
   nsppol=1
!  read(unitkss) nsppol
   write(*,*) 'nsppol                        ',nsppol
   close(unitkss)
  end if ! read_ok

  call hdr_clean(hdr)

 end if !of (me==0 .or. localrdwf==1)

 if (mpi_enreg%nproc>1 .and. localrdwf==0) then
  write(message,'(2a)')ch10,&
&  ' testlda : master is sending data...'
  call wrtout(06,message,'COLL')
  call xcast_mpi(nsym,master,spaceComm,ierr)
  call xcast_mpi(nkpt,master,spaceComm,ierr)
  call xcast_mpi(nbnd,master,spaceComm,ierr)
  call xcast_mpi(npw,master,spaceComm,ierr)
  call xcast_mpi(nsh,master,spaceComm,ierr)
  call xcast_mpi(i1,master,spaceComm,ierr)
  call xcast_mpi(i2,master,spaceComm,ierr)
  call xcast_mpi(ntypat,master,spaceComm,ierr)
  call xcast_mpi(natom,master,spaceComm,ierr)
  call xcast_mpi(mpsang,master,spaceComm,ierr)
  call xcast_mpi(nsppol,master,spaceComm,ierr)
  call xcast_mpi(ibocc,master,spaceComm,ierr)
  call leave_test(mpi_enreg) 
 end if

 end subroutine testlda
!!***
