!{\src2tex{textfont=tt}}
!!****f* ABINIT/fedyout3
!! NAME
!! fedyout3 
!!
!! FUNCTION
!! Output Born effective charge and dielectric constant computed in finite electric fields
!!
!! COPYRIGHT
!! Copyright (C) 2004-2007 ABINIT group (XW).
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!! d2lo(2,mpert,3,mpert)=local contributions to the 2DTEs
!! d2nl(2,mpert,3,mpert)=non-local contributions to the 2DTEs
!! gprimd(3,3)=dimensional primitive translations for reciprocal space ($\textrm{bohr}^{-1}$)
!! natom=number of atoms in cell.
!! nattyp(ntypat)= # atoms of each type.
!! zion(ntypat)=charge corresponding to the atom type
!!
!!
!! OUTPUT
!!
!!
!! TO DO
!!
!! PARENTS
!!      dyout3
!!
!! CHILDREN
!!      
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

  subroutine  fedyout3(d2lo,d2nl,gprimd,iout,mpert,natom,ntypat,typat,ucvol,zion)

 use defs_basis
 use defs_datatypes

 implicit none

!Arguments ------------------------------------
integer, intent(in)  :: iout,natom,mpert,ntypat
real(dp), intent(inout) :: d2lo(2,3,mpert,3,mpert),d2nl(2,3,mpert,3,mpert)
real(dp),intent(in) :: zion(ntypat),gprimd(3,3),ucvol
integer,intent(in)  :: typat(natom)

!Local variables-------------------------------
integer :: ipert,idir,ii,jj,ipert1,ipert2,idir1,idir2,nline
real(dp) :: ter(3,3)

do ipert=1,natom
 ter(:,:)=zero
 do idir=1,3
  do ii=1,3
   do jj=1,3
    if(abs(gprimd(idir,ii))>1.0d-10)then
!      ter(idir,ii)=ter(idir,ii)+ zstar(idir,jj,ipert)*gprimd(jj,ii)
       ter(idir,ii)=ter(idir,ii)+ d2lo(1,idir,natom+5,jj,ipert)*gprimd(jj,ii)
    endif
   enddo
  enddo
! add zion to bec
  ter(idir,idir)=ter(idir,idir)+zion(typat(ipert))
 enddo
  d2lo(1,:,natom+5,:,ipert)=ter(:,:)
enddo

write(iout,*)' '
write(iout,*)' Effective charges, in cartesian coordinates,'
write(iout,*)' (in finite electric fields from electric field response) '
write(iout,*)'  if specified in the inputs, asr has been imposed'
write(iout,*)'    j1       j2             matrix element'
write(iout,*)' dir pert dir pert     real part    imaginary part'

nline=1
do ipert2=1,natom
 do idir2=1,3
  if(nline/=0)write(iout,*)' '
  nline=0
  ipert1=natom+5
   do idir1=1,3
     nline=nline+1
     write(iout,'(2(i4,i5),2(1x,f20.10))')idir1,ipert1,idir2,ipert2,&
    &      d2lo(1,idir1,ipert1,idir2,ipert2),&
    &      0.0 
   end do
  end do
 end do
 write(iout,*)' '
 write(iout,*)' '

do ipert=1,natom
 do ii=1,3
   d2lo(1,ii,ipert,1:3,natom+2) = d2lo(1,ii,ipert,1:3,natom+2) &
&                               + d2nl(1,ii,ipert,1:3,natom+2)
 end do
end do

do ipert=1,natom
 ter(:,:)=zero
 do idir=1,3
  do ii=1,3
   do jj=1,3
    if(abs(gprimd(idir,ii))>1.0d-10)then
!      ter(idir,ii)=ter(idir,ii)+ zstar(idir,jj,ipert)*gprimd(jj,ii)
       ter(idir,ii)=ter(idir,ii)+ d2lo(1,jj,ipert,idir,natom+2)*gprimd(jj,ii)
    endif
   enddo
  enddo
! add zion to bec
  ter(idir,idir)=ter(idir,idir)+zion(typat(ipert))
 enddo
  d2lo(1,:,ipert,:,natom+2)=ter(:,:)
enddo


write(iout,*)' '
write(iout,*)' Effective charges, in cartesian coordinates,'
write(iout,*)' (in finite electric fields from phonon response) '
write(iout,*)'  if specified in the inputs, asr has been imposed'
write(iout,*)'    j1       j2             matrix element'
write(iout,*)' dir pert dir pert     real part    imaginary part'

nline=1
do ipert2=1,natom
 do idir2=1,3
  if(nline/=0)write(iout,*)' '
  nline=0
  ipert1=natom+2
   do idir1=1,3
     nline=nline+1
     write(iout,'(2(i4,i5),2(1x,f20.10))')idir1,ipert1,idir2,ipert2,&
    &      d2lo(1,idir1,ipert2,idir2,ipert1),&
    &      0.0
   end do
  end do
 end do
 write(iout,*)' '
 write(iout,*)' '

 do ii=1,3
  do jj=1,3
   ter(ii,jj)=d2lo(1,ii,natom+5,jj,natom+5)
  end do
 end do
 ter(:,:)=3.1415926535_dp*4*ter(:,:)/ucvol
 
 do ii=1,3
  ter(ii,ii)=ter(ii,ii)+1.0
 end do

 write(iout,*)' '
 write(iout,*)' Dielectric tensor, in cartesian coordinates,'
 write(iout,*)' (in Finite electric fields) '
 write(iout,*)'    j1       j2             matrix element'
 write(iout,*)' dir pert dir pert     real part    imaginary part'

 ipert1=natom+5
 ipert2=natom+5
 do idir2=1,3
   do idir1=1,3
     write(iout,'(2(i4,i5),2(1x,f20.10))')idir1,ipert1,idir2,ipert2,&
    &      ter(idir1,idir2),&
    &      0.0
   end do
  end do
 write(iout,*)' '
 write(iout,*)' '

end subroutine

