!{\src2tex{textfont=tt}}
!!****f* ABINIT/screening
!! NAME
!! screening
!!
!! FUNCTION
!!
!! Calculate screening and dielectric functions
!!
!! COPYRIGHT
!! Copyright (C) 2001-2007 ABINIT group (GMR, VO, LR, RWG, MT, MG, RShaltaf)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  acell(3)=length scales of primitive translations (bohr)
!!  dtfil <type(datafiles_type)>=variables related to files
!!  rprim(3,3)=dimensionless real space primitive translations
!!
!! OUTPUT
!!  output is written in a file
!!
!! SIDE EFFECTS
!!  dtset <type(dataset_type)>=all input variables for this dataset
!!
!! NOTES
!!
!! PARENTS
!!      driver,drivergw
!!
!! CHILDREN
!!      calc_wf_qp,ccgradvnl,cchi0,cchi0q0,cigfft,ckxcldag,cvc,density,distrb2
!!      fermi,fftwfn,findnq,findq,findshells,hdr_clean,hermitianize,identk
!!      lattice,leave_new,matcginv,matrginv,metric,mkrdim,pclock,printcm,printv
!!      rdgw,rdkss,rdlda,rdldaabinit,rdqps,setmesh,setshells,surot,testlda
!!      timab,wrscr,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine screening(acell,dtfil,dtset,mpi_enreg,rprim)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_11util
 use interfaces_12geometry
 use interfaces_13xc
 use interfaces_14iowfdenpot
 use interfaces_15gw
 use interfaces_lib01hidempi
#else
 use defs_interfaces
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 type(MPI_type),intent(inout) :: mpi_enreg
 type(datafiles_type),intent(in) :: dtfil
 type(dataset_type),intent(inout) :: dtset
!arrays
 real(dp),intent(in) :: acell(3),rprim(3,3)

!Local variables ------------------------------
 character(len=4),parameter :: ctype='RPA '
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 integer :: modx
#endif
!End of the abilint section
!scalars
 integer,parameter :: NOMEGAGAUSS=30,NOMEGAREAL=201,unitem1=27,unitf=0
 integer,parameter :: unitwfg=25,unitwfr=26
 integer :: addshell,aliasing_factor,dpbytes,enforce_sym,fermiocc,i,i1,i2,i3,ia
 integer :: ib,ierr,ig,ig1,ig2,ig3,ig4,ig4x,ig4y,ig4z,ii,ik,ikq,inclvkb,io
 integer :: iomega,iop,iproc,iq,iqp,ir,is,isppol,istat,jj,localrdwf,master
 integer :: max_band_proc,me,method,min_band_proc,mpsang,natom,nband,nbcw,nbnds
 integer :: nbnds_per_proc,nbr,nbsc,nbvw,nel=0,ngfft1,ngfft1a,ngfft2,ngfft2a
 integer :: ngfft3,ngfft3a,ngpi,ngr,ninv,nkbz,nkbzx,nkibz,nkibzr,nkpt4chiq
 integer :: nomega,nop,nopr,npwvec,nq,nqptdm,nr,nscf,nshr,nsppol,ntypat,option
 integer :: prtvol,spaceComm,tim_fourdp
 real(dp),parameter :: OMEGAERMAX=100.0/Ha_eV
 real(dp) :: bzvol,domegareal,domegasf,e0,efermi,epsilon0,epsilon0nlf,etadelta
 real(dp) :: maxtrans,nu1,nu2,nu_max,nu_min,omegaplasma,ucvol,wp
 complex :: ctmp
 logical,parameter :: lda=.false.,rpa=.true.
 logical,parameter :: testelectron=.false.,testparticle=.true.
 logical :: analytic_continuation,contour_deformation,found
 logical :: ltemp,plasmon_pole_model,qeq0
 logical :: static,update_energies=.false.,read_val
 character(len=10) :: string
 character(len=500) :: message
 character(len=fnlen) :: filewfg,filewfr,filnam
 type(MPI_type) :: mpi_enreg_dummy
 type(epsilonm1_parameters) :: ep
 type(hdr_type) :: hdr
 type(little_group) :: lt_q
!arrays
 integer :: ibocc(dtset%nsppol)
 integer,allocatable :: gptab(:,:),gptabo(:,:),grottb(:,:,:),grottbm1(:,:,:)
 integer,allocatable :: gvec(:,:),igfft(:,:,:,:),insort(:),ip2fp(:,:)
 integer,allocatable :: ip2fp_dummy(:,:),irottb(:,:),ktab(:),ktabi(:),ktabo(:)
 integer,allocatable :: ktabr(:,:),nband_t(:),nbv(:),typat(:)
 real(dp) :: a1(3),a2(3),a3(3),b1(3),b2(3),b3(3),bcc2rl(3,3),brl2cc(3,3)
 real(dp) :: gmet(3,3),gprimd(3,3),qcart(3),qcc(3),qdummy(3),qtemp(3),rm1t(3)
 real(dp) :: rmet(3,3),rprimd(3,3),tsec(2)
 real(dp),allocatable :: en_qp(:,:,:),energy(:,:,:),energy_val(:,:,:)
 real(dp),allocatable :: gwenergy(:,:,:),gwenergy_val(:,:,:),kbz(:,:),kibz(:,:)
 real(dp),allocatable :: occ(:,:,:),occ_val(:,:,:),op(:,:,:),q(:,:),qplusg(:)
 real(dp),allocatable :: qptdm(:,:),rho2(:,:),rho_p(:,:),tnons(:,:)
 real(dp),allocatable :: vkb(:,:,:,:),vkbd(:,:,:,:),vkbsign(:,:),wtk(:)
 real(dp),allocatable :: xcart(:,:),xred(:,:),z(:),zw(:)
 complex,allocatable :: chitmp(:,:),fnl(:,:,:,:),fnld(:,:,:,:,:)
 complex,allocatable :: gradvnl(:,:,:,:),kkweight(:,:),ktabp(:),kxclda(:,:)
 complex,allocatable :: m_lda_to_qp(:,:,:,:),omega(:),phgt(:,:),wfg(:,:,:,:)
 complex,allocatable :: wfg_val(:,:,:,:),wfr(:,:,:,:),wfr_val(:,:,:,:)
 complex,allocatable,target :: chi0(:,:,:)
 complex,pointer :: chi(:,:,:),eps(:,:,:),epsm1(:,:,:)
 character(len=80) :: title(2)

!************************************************************************
!BEGIN EXECUTABLE SECTION

!DEBUG
!write(6,*)' screening : enter '
!call flush(6)
!ENDDEBUG

 write(message,'(7a)')&
& ' SCREENING: Calculation of the susceptibility and dielectric matrices ',ch10,ch10,&
& ' Based on a program developped by R.W. Godby, V. Olevano, G. Onida, and L. Reining.',ch10,&
& ' Incorporated in ABINIT by V. Olevano, G.-M. Rignanese, and M. Torrent.',ch10
 call wrtout(ab_out,message,'COLL') ; call wrtout(6,message,'COLL')

 ! Start clock
 call pclock(0)        
 call timab(301,1,tsec) !screening time
 call timab(302,1,tsec) !screening(1)
 call timab(331,1,tsec) !chi0

 ! Initialize MPI related quantities
 call xcomm_init(mpi_enreg,spaceComm)  !Init mpi_comm  
 call xme_init(mpi_enreg,me)           !Init me
 call xmaster_init(mpi_enreg,master)   !Init master

 ! Set up parallelization level :
 !  0 for sequential run
 !  1 parallelism is over k-points 
 !  2 conduction bands are spread btw processors
 !    each proc has occupied and partially occupied states 
 if (mpi_enreg%nproc==1) dtset%gwpara=0  
 mpi_enreg%gwpara=dtset%gwpara  
                                
 ! Define I/O :
 !  localrdwf==1 ==> all procs have access to files (default)
 !  localrdwf==0 ==> only master has access to files
 localrdwf=dtset%localrdwf       
 
 ! Set verbosity level
 prtvol=dtset%prtvol

 ! Check for calculations that are not still implemented
 ! ScGW calculations are still not compatible with parallelization over bands 
  if (mpi_enreg%gwpara==2 .and. dtset%gwcalctyp >= 10 ) then 
   write(message,'(2a,i3,2a)')ch10,&
&   ' screening : parallelism over bands and gwcalctyp ',dtset%gwcalctyp,ch10,&
&   ' are not yet compatible'
   call wrtout(6,message,'COLL') ; call leave_new('COLL')
  end if 

!******************************************************************************************* 

 ep%gwcalctyp=dtset%gwcalctyp
 write(message,'(2a,i4,a)')ch10,' GW calculation type = ',ep%gwcalctyp,ch10
 call wrtout(6,message,'COLL')

 analytic_continuation=.false. ; contour_deformation=.false.
 plasmon_pole_model=.true.     ; static=.false. 

 if (mod(ep%gwcalctyp,10)/=0 .and. mod(ep%gwcalctyp,10)/=8) plasmon_pole_model=.false.
 if (mod(ep%gwcalctyp,10)==1) analytic_continuation=.true.
 if (mod(ep%gwcalctyp,10)==2 .or. mod(ep%gwcalctyp,10)==9) contour_deformation=.true.
 if (mod(ep%gwcalctyp,10)==5 .or. mod(ep%gwcalctyp,10)==6 .or. mod(ep%gwcalctyp,10)==7) static=.true.

 dpbytes=8
#if defined T3E
 dpbytes=16
#endif

 ! Set up the parameters of the calculation
 write(message,'(a,i4,a,i4,a)')' length of a real on this machine ',dpbytes,' ; dp=',dp,ch10
 call wrtout(6,message,'COLL')

 nbnds=dtset%nband(1)
 etadelta=dtset%zcut
 write(message,'(a,f10.6)')' zcut for avoiding poles in chi0 [eV] ',etadelta*Ha_eV
 call wrtout(6,message,'COLL')

 ! Compute dimensional primitive translations rprimd (from input file)
 call mkrdim(acell,rprim,rprimd)
 ! Obtain gprimd, metrics and unit cell volume, from rprimd
 call metric(gmet,gprimd,ab_out,rmet,rprimd,ucvol)

!WARNIG here there is a possible undected error if symrel is not 
!the same as the symrel read from the KSS file, should add a check

 ! Define consistently npw, nsh, and ecut for wavefunctions
 call setshells(dtset%ecutwfn,dtset%npwwfn,dtset%nshwfn,dtset%nsym,gmet,&
& gprimd,dtset%symrel,'wfn',ucvol)
 ! Define consistently npw, nsh, and ecut for the inverse dielectri matrix
 call setshells(dtset%ecuteps,dtset%npweps,dtset%nsheps,dtset%nsym,gmet,&
& gprimd,dtset%symrel,'eps',ucvol)

 ! Calculate npwvec as the biggest between dtset%npweps and dtset%npwwfn
 npwvec=max(dtset%npwwfn,dtset%npweps)

 ! Read parameters of the KSS or QPLDA o STA file and verifify them
 ep%awtr=dtset%awtr ; ibocc(:)=0
 call testlda(dtset,dtfil,nop,nkibz,nbr,ngr,nshr,i1,i2,ntypat,natom,&
& mpsang,nsppol,mpi_enreg,localrdwf,ibocc)

 if (i2/=0 .and. i2/=1 .and. i2/=2 .and. i2/=502 .and. i2/=602) then
  write(message,'(5a)')ch10,  &
&  ' screening: ERROR -',ch10,&
&  '  Unknown format found in the screening file',ch10
  call wrtout(6,message,'COLL') ; call leave_new('COLL')
 end if

 ! If ninv==2 then assume time reversal symmetry 
 ! If read_val is true then we use a slightly modified Adler-Wiser expression 
 ! taking advantage of time-reversal (should become input)
 ninv=2 ; read_val=.false.
 if ( (ninv==2 .and. dtset%awtr==1) .or. mpi_enreg%gwpara==2 .or. dtset%spectral/=0 ) read_val=.true.

 inquire(file='nop.in',exist=ltemp)
 if (ltemp) then
  write(message,'(a)')&
&  'screening : reading symmetry operations from file nop.in'
  call wrtout(6,message,'COLL') 
  open(99,file='nop.in')
  read(99,*) nop,ninv
  close(99)
 end if

 if (npwvec>ngr) then
  write(message,'(4a)')ch10,   &
& ' screening: WARNING -',ch10,&
& '  Number of G-vectors found less then required'
  call wrtout(6,message,'COLL')
  npwvec=ngr
  if (dtset%npwwfn>ngr) dtset%npwwfn=ngr
  if (dtset%npweps>ngr) dtset%npweps=ngr
  write(message,'(2a,i6,2a,i6,2a,i6,a)')ch10,&
&  '         Calculation will proceed with npwvec = ',npwvec,ch10,      &
&  '                                       npweps = ',dtset%npweps,ch10,&
&  '                                       npwwfn = ',dtset%npwwfn,ch10
  call wrtout(6,message,'COLL')
 end if

 if (nbnds>nbr) then
  write(message,'(6a,i4,a)')ch10,&
&  ' screening: WARNING -',ch10, &
&  '  Number of bands found less then required;',ch10,&
&  '          calculation will proceed with nbnds = ',nbr,ch10
  call wrtout(6,message,'COLL')
  nbnds=nbr
 end if

 allocate(op(3,3,nop),stat=istat)
 allocate(kibz(3,nkibz),stat=istat)
 allocate(qplusg(dtset%npweps),stat=istat)
  if(istat/=0) call memerr('screening','qplusg',dtset%npweps,'dp')
 allocate(gvec(3,npwvec),stat=istat)
  if(istat/=0) call memerr('screening','gvec',3*npwvec,'i4b')

 ! Quantities used to allocate valence and conductions states in case of gwpara==2 :
 !  nbvw : maximum number of fully and partially occupied states considering the spin 
 !  nbcw : maximum number of unoccupied states considering the spin 
 nbvw=maxval(ibocc) ; nbcw=nbnds-nbvw
  write(message,'(2a,i5,a,i5,a)')ch10,&
&  ' screening : maximum band index for partially occupied states nbvw = ',nbvw,' nbcw = ',nbcw,ch10
  call wrtout(06,message,'PERS')

 ! Initialize mpi_enreg%proc_distrb according to gwpara 
 ! in case of k-point parallelism mpi_enreg%proc_distrb is redefined on the fly for each q-point
 call initmpi_gw(mpi_enreg,dtset,nkibz,nbnds,nsppol,ibocc,nbvw,nbcw,&
& min_band_proc,max_band_proc,nbnds_per_proc)

 ! Allocate KS electronic structure variables
 ! If time reversal symmetry holds then store occupied and partially occupied states 
 ! in a different array in order to use faster algorithm 
! FIXME gwpara==2 should be removed since this should be the default method
! Note that if nsppol==2 than wfg_val might also contain unoccupied states, 
! this case has to be treated inside cchi0 and cchi0q0
 !if (ninv==2 .and. dtset%gwpara==2) then 
 if (read_val) then 
  allocate(wfg_val(dtset%npwwfn,nbvw,nkibz,nsppol),stat=istat)
   if(istat/=0) call memerr('screening','wfg_val',dtset%npwwfn*nbnds_per_proc*nkibz*nsppol,'spc')
  wfg_val(:,:,:,:)=(0.,0.)
 end if 

 allocate(gwenergy(nkibz,nbnds,nsppol),stat=istat)
 allocate(  energy(nkibz,nbnds,nsppol),stat=istat)
 allocate(     occ(nkibz,nbnds,nsppol),stat=istat)
  if(istat/=0) call memerr('screening','energy',3*nbnds*nkibz*nsppol,'dp')
 gwenergy(:,:,:)=zero ; energy(:,:,:)=zero ; occ(:,:,:)=zero

 allocate(wfg(dtset%npwwfn,min_band_proc:max_band_proc,nkibz,nsppol),stat=istat)
  if(istat/=0) call memerr('screening','wfg',dtset%npwwfn*nbnds_per_proc*nkibz*nsppol,'spc')
 wfg(:,:,:,:)=(0.,0.)

 ! Read in KS band structure and information about material
 if (i2==0 .or. i2==1) then
  !
  ! Old QPLDA file
  !
  if (nsppol==2) then 
   write(message,'(4a)')ch10, &
&   ' screening ERROR- ',ch10,&
&   '  nsppol=2 and QPLDA format are not compatible' 
   call wrtout(6,message,'COLL') ; call leave_new('COLL')
  end if 
  i1=1 !file unformatted
  !WARNING non-symmorphic ops are not treated, band parallelism doesnt work
  call rdlda(10,nop,nbnds,nkibz,npwvec,dtset%npwwfn,nopr,nbr,nkibzr,ngr,&
&  title,i1,i2,i3,a1,a2,a3,op,gvec,kibz,energy,occ,wfg)
 else if (i2==2) then
  !
  ! Old format of Abinit
  !
  if (nsppol==2) then 
   write(message,'(4a)')ch10, &
&  ' screening: ERROR- ',ch10,&
&  '  nsppol=2 and old wf file format are not compatible' 
   call wrtout(6,message,'COLL') ; call leave_new('COLL')
  end if 
  !WARNING non-symmorphic ops are not treated, band parallelism doesnt work
  call rdldaabinit(10,nop,nbnds,nkibz,npwvec,dtset%npwwfn,&
&  title,a1,a2,a3,op,gvec,kibz,energy,occ,wfg)
 else if (i2==502 .or. i2==602) then
  !
  ! Read standard KSS file
  !
  allocate(typat(natom),xred(3,natom),xcart(3,natom))
  allocate(tnons(3,nop),stat=istat)
  ! Kleynmann-Bylander sign and matrix elements
  allocate(vkbsign(mpsang,ntypat))
  allocate(vkb(dtset%npwwfn,ntypat,mpsang,nkibz),stat=istat)
   if(istat/=0) call memerr('screening','vkb',dtset%npwwfn*ntypat*mpsang*nkibz,'spc')
  allocate(vkbd(dtset%npwwfn,ntypat,mpsang,nkibz),stat=istat)
   if(istat/=0) call memerr('screening','vkbd',dtset%npwwfn*ntypat*mpsang*nkibz,'spc')
  !if (ninv==2 .and. dtset%gwpara==2) then 
   if (read_val) then 
   !
   ! Report two different arrays with fully-partially occupied stated and unoccupied states 
   !
   call rdkss(dtfil,hdr,nop,nbnds,nbvw,nkibz,npwvec,nsppol,dtset%npwwfn,title,a1,a2,a3,op,&
&   gvec,kibz,energy,occ,wfg,ntypat,natom,mpsang,prtvol,typat,tnons,xred,vkbsign,vkb,vkbd,nel,&
&   mpi_enreg,localrdwf,min_band_proc,max_band_proc,&
&   wfg_val) !optional arguments
  else 
   !
   ! Old implementation: it can be used if time reversal symmetry does not hold  
   !
   call rdkss(dtfil,hdr,nop,nbnds,nbvw,nkibz,npwvec,nsppol,dtset%npwwfn,title,a1,a2,a3,op,&
&   gvec,kibz,energy,occ,wfg,ntypat,natom,mpsang,prtvol,typat,tnons,xred,vkbsign,vkb,vkbd,nel,&
&   mpi_enreg,localrdwf,min_band_proc,max_band_proc) 
  end if 
  do ia=1,natom
   xcart(:,ia)= xred(1,ia)*a1(:) + xred(2,ia)*a2(:) + xred(3,ia)*a3(:)
  end do
 end if !of i2
 ! Check whether the lattice from the input file agrees with that read in the KSS file
 if ( (any(abs(a1-rprimd(:,1))>tol6)) .or. &
      (any(abs(a2-rprimd(:,2))>tol6)) .or. &
      (any(abs(a3-rprimd(:,3))>tol6)) ) then 
  write(message,'(6a)')ch10,&
&  ' screening : ERROR - ',ch10,&
&  ' direct lattice vectors read from the KSS file ',ch10,&
&  ' differ from the values specified in the input file'
  call wrtout(6,message,'COLL')
  write(message,'(3a,3(3es16.6),3a,3(3es16.6),3a)')ch10,&
&  ' rprimd from KSS file = ',ch10,(rprimd(:,jj),jj=1,3),ch10,&
&  ' rprimd from input file = ',ch10,a1,a2,a3,ch10,ch10,&
&  ' Please, modify the lattice vectors in the input file '
  call wrtout(6,message,'COLL') ; call leave_new('COLL')
 end if 

 call pclock(10)

 ! Calculate b1, b2, b3 and ucvol, bzvol
 ! Note that here the reciprocal vectors are defined such as: a_i \cdot b_j = 2pi delta_ij
 ! This subroutine should be removed, it is better to use the Abinit convention
 call lattice(a1,a2,a3,b1,b2,b3,ucvol,bzvol)
 call findshells(b1,b2,b3,npwvec,dtset%npweps,gvec)

 call pclock(20)

 ! Set real space mesh for FFT :
 !  enforce_sym==1 enforce a FFT mesh compatible with all the symmetry operation
 !  method==0 FFT grid read from fft.in (debugging purpose)
 !  method==1 normal FFT grid 
 !  method==2 slightly augmented FFT grid  (see setmesh)
 !  method==3 augmented FFT grid 
 enforce_sym=mod(dtset%fftgw,10) 
 if (dtset%fftgw==00 .or. dtset%fftgw==01) method=0
 if (dtset%fftgw==10 .or. dtset%fftgw==11) method=1
 if (dtset%fftgw==20 .or. dtset%fftgw==21) method=2
 if (dtset%fftgw==30 .or. dtset%fftgw==31) method=3
 addshell=2 ; aliasing_factor=1 !only used in case of method 0 or 1  
 call setmesh(gmet,gvec,ngfft1,ngfft2,ngfft3,ngfft1a,ngfft2a,ngfft3a,npwvec,dtset%npweps,&
& dtset%npwwfn,nr,method,addshell,aliasing_factor,nop,dtset%symrel,tnons,enforce_sym,1)

 ! Set up tables for FFT igfft
 allocate(igfft(npwvec,5,5,5),stat=istat)
  if(istat/=0) call memerr('screening','igfft',5*5*5*npwvec,'i4b')
 call cigfft(npwvec,ngfft1a,ngfft1,ngfft2,ngfft3,gvec,igfft)

 call pclock(30)

 ! Set up table indicating rotations of r-points (R^-1)(r-t) and G vectors
 allocate(irottb(nr,nop),stat=istat)
  if(istat/=0) call memerr('screening','irottb',nr*nop,'i4b')
 allocate(grottb(npwvec,2,nop),grottbm1(npwvec,2,nop),stat=istat)
  if(istat/=0) call memerr('screening','grottb/grottbm1 ',2*(2*npwvec*nop),'i4b')
 call surot(op,tnons,nop,ninv,ngfft1,ngfft1a,ngfft2,ngfft3,nr,npwvec,gvec,grottb,irottb,grottbm1)

 call pclock(40)

 ! Set up required k-points in whole BZ; nkbzx maximum number of them
 ! FIXME here we could allocate dummy arrays with dimension nkbzm and 
 ! then copy only the portion of the table we need
 nkbzx=nkibz*nop*ninv
 allocate(kbz(3,nkbzx),wtk(nkibz),stat=istat)
 allocate(ktab(nkbzx),ktabi(nkbzx),ktabo(nkbzx),ktabr(nr,nkbzx),stat=istat)
  if(istat/=0) stop 'out of memory ktables'
 call identk(kibz,nkibz,nkbzx,nr,nop,ninv,irottb,op,kbz,ktab,ktabr,ktabi,ktabo,nkbz,wtk)

 ! Calculate phase factors for non-symmorphic operations 
 ! u_{Sk}(r)=e^{-i 2 \pi k\cdot R{^-1}t} u_k (R^{-1}(r-t))
 ! time reversal symmetry is taken into account inside rho_tw_g
 allocate(ktabp(nkbz),stat=istat)
 do ik=1,nkbz
  rm1t=matmul(transpose(op(:,:,ktabo(ik))),tnons(:,ktabo(ik)))
  ktabp(ik)= exp(-(0.,1.) * two_pi * dot_product(kibz(:,ktab(ik)),rm1t))
 end do  

 call pclock(50)

 ! Find number of q-points q=k-kfirst
 call findnq(nkbz,kbz,nop,op,nq,ninv)
 ! Find q-points q=k-kfirst
 allocate(q(3,nq))
 call findq(nkbz,kbz,nop,op,nq,q,ninv,b1,b2,b3)

 call pclock(60)
 call timab(302,2,tsec) !screening(1)
 call timab(303,1,tsec) !screening(fftwfn)

 ! Calculate KS wavefunctions in real space using FFT
 !if (ninv==2 .and. dtset%gwpara==2) then 
 if (read_val) then 
  write(message,'(2a)')ch10,' FFT valence states to real space'
  call wrtout(06,message,'COLL')
  allocate(wfr_val(nr,nbvw,nkibz,nsppol),stat=istat)
   if(istat/=0) call memerr('screening','wfr_val',nr*nbvw*nkibz*nsppol,'spc')
  wfr_val(:,:,:,:)=(0.,0.)
  tim_fourdp=4
  call fftwfn(dtset%npwwfn,1,nbvw,nkibz,nr,nsppol,wfg_val,wfr_val,&
&  igfft(:,3,3,3),ngfft1,ngfft1a,ngfft2,ngfft3,tim_fourdp,mpi_enreg)
 end if 

 allocate(wfr(nr,min_band_proc:max_band_proc,nkibz,nsppol),stat=istat)
  if(istat/=0) call memerr('screening','wfr',nr*nbnds_per_proc*nkibz*nsppol,'spc')
 wfr(:,:,:,:)=(0.,0.)
 tim_fourdp=4
 call fftwfn(dtset%npwwfn,min_band_proc,max_band_proc,nkibz,nr,nsppol,wfg,wfr,&
& igfft(:,3,3,3),ngfft1,ngfft1a,ngfft2,ngfft3,tim_fourdp,mpi_enreg)

 call timab(303,2,tsec) !screening(fftwfn)
 call timab(304,1,tsec) !FIXME this is not correct anymore

 if (ep%gwcalctyp>=10) then 
  !
  ! Self-consistent GW
  !
  if (mpi_enreg%gwpara==2) then 
   write(*,*)' WARNING - under development' 
   call leave_new('COLL')
  end if 
  ! Read QP wavefunctions of the previous step (FBruneval)
  write(message,'(a)')' screening: reading QP wavefunctions of the previous step'
  call wrtout(06,message,'COLL') !; call wrtout(ab_out,message,'COLL')
  !
  ! Read self-consistent transformation matrices and QP energies
  !
  allocate(rho_p(nr,nsppol))
  allocate(en_qp(nkibz,nbnds,nsppol),stat=istat)
   if(istat/=0) call memerr('screening','en_qp',nkibz*nbnds*nsppol,'dp')
  !memory could be optimized (only sub-block)
  allocate(m_lda_to_qp(nbnds,nbnds,nkibz,nsppol),stat=istat)
   if(istat/=0) call memerr('screening','m_lda_to_qp',nbnds**2*nkibz*nsppol,'spc')
  !TODO add a check at the beginnig, SCGW requires that
  !     bdgw does not depend on kpt, moreover the first index of bdgw must be 1
  call rdqps(dtfil,ep%gwcalctyp,nkibz,nbnds,nsppol,kibz,nscf,nr,energy,&
&  nbsc,en_qp,m_lda_to_qp,rho_p)
  energy(:,:,:)=en_qp(:,:,:)
  deallocate(rho_p)

!OLD METHOD  
!  call calc_wf_qp(nkibz,nbnds_per_proc,nr,          nsppol,m_lda_to_qp,wfr,min_band_proc,max_band_proc)
!  call calc_wf_qp(nkibz,nbnds_per_proc,dtset%npwwfn,nsppol,m_lda_to_qp,wfg,min_band_proc,max_band_proc)
!NEW METHOD

! Skip the tranformation if this is iteration number 0
! otherwise update only the wfs that are explicitly treated in the self-consistent cycle
! TODO should rationalize calc_wf_qp
 if (nscf/=0) then 
  if (mpi_enreg%gwpara==0 .or. mpi_enreg%gwpara==1) then
   call calc_wf_qp(nkibz,nbsc,nr,          nsppol,m_lda_to_qp(1:nbsc,1:nbsc,:,:),wfr(:,1:nbsc,:,:),1,nbsc)
   call calc_wf_qp(nkibz,nbsc,dtset%npwwfn,nsppol,m_lda_to_qp(1:nbsc,1:nbsc,:,:),wfg(:,1:nbsc,:,:),1,nbsc) 
  else if (mpi_enreg%gwpara==2) then  
   !here I suppose we are using the algorithm based in time reversal symmetry 
   !so that all the occupied-partially occupied states are on each proc
   write(message,'(a,a,a,a)')ch10,&
&   ' screening : ERROR -',ch10,&
&   '  Self-consistent QP GW is not yet implemented with parallelism over bands. Sorry for this.' 
   call wrtout(6,message,'COLL') 
   call leave_new('COLL')
   !now I have to find the processors that have the remaining nbsc-nbvw band, perform the transformation and 
   !BCAST the contribution to the QP wavefunctions from 1 up to nbvw 
   !in metals it might happen that nbvw is larger that nbsc
  end if 
   endif
  deallocate(m_lda_to_qp,en_qp)
 end if !of if(gwcalctyp>=10)

 if (dtset%mkmem==0) then
  write(message,'(2a)')ch10,&
& ' screening : writing wfs in real and G space on file'
  call wrtout(06,message,'COLL')
  !
  ! Open and write scratch file for wavefunctions (r,b,k,s)
  !
  filewfr=trim(dtfil%filnam_ds(5))//'_WFR'
  write(message,'(3a)')ch10,' opening file: ',trim(filewfr)
  call wrtout(06,message,'COLL')
  open(unit=unitwfr,file=filewfr,status='unknown',access='direct',recl=nr*nbnds*dpbytes)
  if (me==0 .or. localrdwf==1) then
   do is=1,nsppol
    do ik=1,nkibz
     write(unitwfr,rec=ik+nkibz*(is-1)) ((wfr(ir,ib,ik,is),ir=1,nr),ib=1,nbnds)
    end do
   end do 
  end if
  deallocate(wfr)
  !
  ! Open and write scratch file for wavefunction(G,b,k,s)
  ! needed to evaluate the head and the wings of \chi(q==0)
  !
  filewfg=trim(dtfil%filnam_ds(5))//'_WFG'
  write(message,'(3a)')ch10,' opening file: ',trim(filewfg)
  call wrtout(06,message,'COLL')
  open(unit=unitwfg,file=filewfg,status='unknown',access='direct',recl=dtset%npwwfn*nbnds*dpbytes)
  if (me==0 .or. localrdwf==1) then
   do is=1,nsppol
    do ik=1,nkibz
     write(unitwfg,rec=ik+nkibz*(is-1)) ((wfg(ig,ib,ik,is),ig=1,dtset%npwwfn),ib=1,nbnds)
    end do
   end do
  end if
  deallocate(wfg)
 end if !of mkmem==0 

 call pclock(70)
 call timab(304,2,tsec)
 call timab(305,1,tsec)

 ! For each spin nbv reports the valence band index
 ! dtset%fixmom is passed to fermi.F90 to fix the problem with newocc in case of magnetic metals
 allocate(nbv(nsppol))
 call fermi(hdr,nbnds,nkibz,dtset%fixmom,dtset%nsppol,wtk,energy,occ,nel,nbv,efermi)
 gwenergy(:,:,:)=energy(:,:,:)

 inquire(file='in.gw',exist=update_energies)
 ep%soenergy=dtset%soenergy
 if (ep%soenergy>0.1d-4) then
  write(message,'(5a,f7.3,a)')&
&  ' screening : performing a first self-consistency',ch10,&
&  '  update of the energies in W by a scissor operator',ch10,&
&  '  applying a scissor operator of [eV] : ',ep%soenergy*Ha_eV,ch10
  call wrtout(06,message,'COLL') ; call wrtout(ab_out,message,'COLL')
  do is=1,nsppol
   do ib=nbv(is)+1,nbnds
    gwenergy(:,ib,is)=energy(:,ib,is)+ep%soenergy
   end do
  end do
 else if (update_energies) then
!MG FIXME to be tested, this part should be treated clearer, maybe introducing 
!   an input variable, moreover what happens if the user asks for a scissor and 
!   an update of the energies as well? 
  write(message,'(4a)')&
&  ' screening : performing a first self-consistency',ch10,&
&  '  update of the energies in W by a previous GW calculation',ch10
  call wrtout(06,message,'COLL') ; call wrtout(ab_out,message,'COLL')
 !MG Added spin variable in the list of input variables
!FIXME in.gw should became a standard ABINIT file, 
!FIXME this will not work in band para 
!MG NOTE now should work also in case of gwpara 2 but I didnt check well 
  call rdgw(nkibz,nbnds,nbv,nsppol,kibz,gwenergy)
 end if
 !
 ! Calculate electronic density
 !
 allocate(rho2(nr,nsppol),stat=istat)
  if(istat/=0) stop 'out of memory in rho2'
 if (dtset%mkmem/=0) then 
  !
  ! WFR are in memory
  !
  !if (ninv==2 .and. dtset%gwpara==2) then 
  if (read_val) then 
   !
   ! Valence states are in memory on each proc since gwpara==2
   !
   !FIXME this sub should be cleaned 
   call density(nbnds,nkibz,nkbz,nsppol,nop,ninv,nr,ngfft1,ngfft1a,ngfft2,ngfft3,&
&   irottb,ucvol,wtk,occ,rho2,omegaplasma,mpi_enreg,1,nbvw,localrdwf,&
&   wfr=wfr_val) !optional
  else 
   !
   ! Each proc has the full set of WFR
   !
   call density(nbnds,nkibz,nkbz,nsppol,nop,ninv,nr,ngfft1,ngfft1a,ngfft2,ngfft3,&
&   irottb,ucvol,wtk,occ,rho2,omegaplasma,mpi_enreg,min_band_proc,max_band_proc,localrdwf,&
&   wfr=wfr) !optional
  end if
 else
  !
  ! WFR are on disk
  !
  call density(nbnds,nkibz,nkbz,nsppol,nop,ninv,nr,ngfft1,ngfft1a,ngfft2,ngfft3,&
&  irottb,ucvol,wtk,occ,rho2,omegaplasma,mpi_enreg,min_band_proc,max_band_proc,localrdwf)
 end if

 call pclock(80)

 ! Default values for real and imaginary frequencies
 ep%nomegaei=1 ; ep%nomegaer=1
 if (static) ep%nomegaer=0
 ! For ppmodels 2,3,4, the DM only at zero frequency is needed, in this case one can save 
 ! cpu time+memory by calculating the DM on one frequency provided that the user use nfreqre=1
 if (plasmon_pole_model .and. dtset%nfreqre==1 .and. dtset%nfreqim==0) then
  ep%nomegaer=1 ; ep%nomegaei=0
  write(message,'(7a)')ch10,&
&  ' the inverse dielectric matrix will be calculated on zero frequency only',ch10,&
&  ' please note that the calculated epsilon^-1 cannot be used ',ch10,&
&  ' to calculate QP corrections using plasmon pole model 1',ch10
  call wrtout(06,message,'COLL') ; call wrtout(ab_out,message,'COLL')
 end if 
 !
 ! Setup imaginary frequencies
 !
 if (analytic_continuation .or. contour_deformation) then
  ep%nomegaei=dtset%nfreqim
  if (ep%nomegaei==0) ep%nomegaei=NOMEGAGAUSS
 end if
 !
 ! Setup real frequencies according to the method used 
 !
 if (contour_deformation) then
  ep%nomegaer=dtset%nfreqre
  ep%omegaermax=dtset%freqremax
  if (ep%nomegaer==0) ep%nomegaer=NOMEGAREAL
  if (ep%omegaermax<0.1d-4) ep%omegaermax=OMEGAERMAX
  domegareal=ep%omegaermax/(ep%nomegaer-1)
 end if
 !
 ! Allocate and calculate the set of frequencies
 !
 nomega=ep%nomegaer+ep%nomegaei
 allocate(omega(nomega))
 ! First omega is always zero without broadening
 omega(1)=(0.,0.) 
 do io=2,ep%nomegaer
  omega(io)=cmplx((io-1)*domegareal,0.0)
 end do
 if (plasmon_pole_model .and. nomega==2) then
  e0=dtset%ppmfrq
  if (e0<0.1d-4) e0=omegaplasma
  omega(2)=cmplx(0.0,e0)
 end if
 !
 if (analytic_continuation) then
  ! Calculate Gauss-Legendre Quadrature knots and weights for AC 
  ! to calculate \int_0^\infty dx f(x) we calculate \int_0^1 dz f(1/z - 1)/z^2
  ! Note that the grid is not log as required by contour_deformation, 
  ! so we cannot use the same SCR file (should test both to understand which one is better)
  allocate(z(ep%nomegaei),zw(ep%nomegaei))
  call coeffs_gausslegint(0.0_dp,1.0_dp,z,zw,ep%nomegaei)
  do io=1,ep%nomegaei
   omega(ep%nomegaer+io)=cmplx(0.0,1/z(io)-1)
  end do
  deallocate(z,zw)
 else if (contour_deformation) then
  e0=dtset%ppmfrq
  if (e0<0.1d-4) e0 = omegaplasma
  do io=1,ep%nomegaei
   omega(ep%nomegaer+io)=cmplx(0.0,e0/3.*(exp(2./(ep%nomegaei+1)*log(4.)*io)-1.))
  end do
 end if
 !
 ! Calculation of \chi_o based on the spectral function :
 ! Setup frequency mesh for Im \chi_o and calculate frequency-dependent weights for Hilbert transform 
 ! Notice that CPU time does not depend dramatically on nomegasf, but memory does! 
 ! spmeth defines how the delta function is approximated:
 !  spmeth==1 : use Triangular approximant (Kresse method)
 !  spmeth==2 : use Gaussian method, requiring smearing (Miyake method)
 !
 ep%spmeth=dtset%spectral
 ep%spsmear=dtset%spbroad
 if (ep%spmeth/=0) then 
  write(message,'(2a,i3)')ch10,' screening : using spectral method = ',ep%spmeth
  call wrtout(06,message,'COLL')
  if (ep%spmeth==2) then 
   write(message,'(a,f8.5,a)')' with gaussian broadening = ',ep%spsmear*Ha_eV,' [eV]'
   call wrtout(06,message,'COLL')
  end if  
  ! Each proc has all the energies, maybe here we can save a bit of memory 
  ! I still have problems here, remember SiO2 for this reason Im using 1.
  ! It is better to calculate the transition k==>k-q
  ep%nomegasf=dtset%nomegasf
  maxtrans=maxval(gwenergy)-minval(gwenergy)+1./Ha_eV
  domegasf=maxtrans/ep%nomegasf
  ! Add a point to avoid boundary effects
  allocate(ep%omegasf(ep%nomegasf+1))
  write(message,'(4a,f8.2,3a,i5,2a,f8.5,3a,f8.2,2a)')ch10,&
&  ' Info on spectral method : ',ch10,&
&  '  maximum transition energy = ',maxtrans*Ha_eV,' [eV]',ch10,&
&  '  nomegasf = ',ep%nomegasf,ch10,&
&  '  domegasf = ',domegasf*Ha_eV,' [eV]',ch10,&
&  '  estimated memory required by each proc to store the imaginary part = ',&
&  2*dp*ep%nomegasf*(dtset%npweps**2/1024.0_dp/1024.0_dp)/mpi_enreg%nproc,' Mb ',ch10
  call wrtout(06,message,'COLL') ; call wrtout(ab_out,message,'COLL')
  if (dtset%userib==0) then 
   ! Use linear mesh
   write(message,'(a)')' using linear mesh' ; call wrtout(06,message,'COLL')
   do io=1,ep%nomegasf+1
    ep%omegasf(io)=domegasf*(io-1)
   end do 
  else
   ! Non-homogeneous mesh densified close to omega_p
   wp=omegaplasma 
   !wp= minval(gwenergy(:,nbv(1)+1:max_band_proc,:))-minval(gwenergy(:,:,:))
   write(message,'(a,f8.3)')' using non-linear mesh with wp [eV] =',wp*Ha_eV
   call wrtout(06,message,'COLL')
   nu_min=0.0 ! wp < maxtrans !!!
   if (maxtrans < wp ) then 
    nu_max= (wp/sqrt(2.)) * atan(sqrt(2.)*maxtrans*wp/(-maxtrans**2+wp**2))
   else 
    nu_max= (wp/sqrt(2.)) *( atan(sqrt(2.)*maxtrans*wp/(-maxtrans**2+wp**2)) + pi)
   end if 
   domegasf=(nu_max-nu_min)/(ep%nomegasf+1)
   write(*,*)' using non homogeneous mesh with wp [eV] = ',wp*Ha_eV
   write(*,*)' maxtrans [eV] = ',maxtrans*Ha_eV
   write(*,*)' nu_max = ',nu_max
   write(*,*)  -(wp/sqrt(2.)) * atan(sqrt(2.)*maxtrans*wp/(maxtrans**2-wp**2))
   write(*,*)' delta nu = ',domegasf
   ep%omegasf(1)=0.0_dp 
   ep%omegasf(ep%nomegasf+1)=maxtrans
   iq=0
   do io=2,ep%nomegasf
    nu1=domegasf*(io-1)
    nu2=tan(-sqrt(2.)*nu1/wp)
    if (nu2<0) then 
     ep%omegasf(io)= wp *(1.-sqrt(1+2*nu2**2))/(sqrt(2.)*nu2)
    else
     ep%omegasf(io)= wp * (1.+sqrt(1+2*nu2**2))/(sqrt(2.)*nu2) 
    end if 
    if (ep%omegasf(io)> maxtrans ) then  
     ep%omegasf(io)=maxtrans-0.1*iq
     iq=iq+1
    end if 
    write(102,'(i4,2x,3(f9.4,2x))')io,nu1,nu2,ep%omegasf(io)*Ha_eV
   end do 
   ! Reorder frequencies in ascending order
   allocate(insort(ep%nomegasf+1))
   do io=1,ep%nomegasf+1
    insort(io)=io
   end do
   call sort_dp(ep%nomegasf+1,ep%omegasf,insort,tol14)
   deallocate(insort)
  end if 
  !write(*,*)' mesh for Im chi_0'
  !do io=1,ep%nomegasf+1
  ! write(*,*)io,ep%omegasf(io)*Ha_eV
  !end do
  ! Calculate weights for Hilbert transform 
  allocate(kkweight(nomega,ep%nomegasf))
  call calc_kkweight(nomega,omega,ep%nomegasf,ep%omegasf(1:ep%nomegasf),etadelta,maxtrans,kkweight)
 end if  

 write(message,'(2a)')ch10,' calculating chi0 at frequencies [eV] :'
 call wrtout(06,message,'COLL') ; call wrtout(ab_out,message,'COLL')
 do io=1,nomega
  write(message,'(i3,2es16.6)')io,omega(io)*Ha_eV
  call wrtout(06,message,'COLL') ; call wrtout(ab_out,message,'COLL')
 end do

 allocate(chitmp(dtset%npweps,dtset%npweps),stat=istat)
  if(istat/=0) call memerr('screening','chitmp',dtset%npweps*dtset%npweps,'spc')

 if (lda) then
!MG FIXME to be generalized in case of spin polarized calculations
!         density.F90 or crho now calculate n_up and n_down
  if (nsppol==2) then 
   write(message,'(4a)')ch10,&
&   ' screening: ERROR- ',ch10,&
&   '  lda calculation with nsppol==2 not yet implemented'
   call wrtout(6,message,'COLL') ; call leave_new('COLL')
  end if 
 ! Calculate exchange-correlation kernel Kxclda in G-space and put it into Kxclda(1+igx+igy*...)
 allocate(kxclda(dtset%npweps,dtset%npweps),stat=istat)
 if(istat/=0) call memerr('screening','kxclda',dtset%npweps*dtset%npweps,'spc')

!FIXME using only the total charge, should modify the sub in order to treat the case nsppol==2
! call ckxcldag(ngfft1,ngfft1a,ngfft2,ngfft3,nr,rho2(:,1),kxclda)
  call xc_kernel(dtset,dtset%ixc,mpi_enreg,ngfft1,ngfft2,ngfft3,nr,nsppol,rho2,rprimd,igfft,&
&   dtset%npweps,gmet,kxclda,gvec,nq,q)

  do ig=1,dtset%npweps
!   chitmp(ig,1)=kxclda(igfft(ig,3,3,3))
    chitmp(ig,1)=kxclda(ig,1)
  end do
  write(message,'(a)')' kxclda(g)'
  call wrtout(06,message,'COLL')
  call printv(chitmp,dtset%npweps)
 end if !of if (lda)
!END FIXME

 allocate(chi0(dtset%npweps,dtset%npweps,nomega),stat=istat)
  if(istat/=0) call memerr('screening','chitmp',dtset%npweps*dtset%npweps*nomega,'spc')

 chi => chi0
 epsm1 => chi0
 eps => chi0

 write(message,'(3a)')ch10,' end of initialization',ch10
 call wrtout(6,message,'COLL')

 call pclock(90)

 ! Enable the calculations of the DM on one or more selected q points (R.Shaltaf)
 if (dtset%nqptdm>0) then
  write(message,'(6a)')ch10,&
&  ' Dielectric matrix will be calculated on some ',ch10,&
&  ' selected q points provided by the user through the input variables ',ch10,&
&  ' nqptdm and qptdm'
  call wrtout(6,message,'COLL') ; call wrtout(ab_out,message,'COLL')
  if (dtset%nqptdm>nq) then
   write(message, '(5a)')ch10, &
&   ' screening: ERROR -',ch10,&
&   '  nqptdm should not exceed the actual number of q points that already exist in BZ',ch10
   call wrtout(6,message,'COLL') ; call leave_new('COLL')
  end if
  allocate(qptdm(3,dtset%nqptdm))
  !PMA-MG fix problem with a possible larger dimension in dtset%qptdm 
  !qptdm(:,:)=dtset%qptdm(:,1:dtset%nqptdm)
  qptdm(:,1:min(dtset%nqptdm,size(dtset%qptdm,2)))=dtset%qptdm(:,1:min(dtset%nqptdm,size(dtset%qptdm,2))) 
  !check whether the q points provided by the user already exist in the set of q points'
  do iq=1,dtset%nqptdm
   found=.false.
   do iqp=1,nq
    qtemp(:)=qptdm(:,iq)-q(:,iqp)
    if (all(abs(qtemp(:))<1.0e-3)) found=.true.
    if (found) exit
   end do
   if (.not.found) then
    write(message,'(5a)')ch10,&
&    ' screening: ERROR -',ch10,&
&    '  one or more qptdm cannot be identified as regular q-points satisfying q = k - k1',ch10
    call wrtout(6,message,'COLL') ; call leave_new('COLL')
   end if
  end do !iq
 end if !of if (dtset%nqptdm>0)

 ! Allocate arrays needed to perform symmetrization (if required)
 lt_q%sym_flag=dtset%symchi
 if (lt_q%sym_flag/=0) then 
  write(*,*)' screening: switching on symmetrization'
  lt_q%nop=nop ; lt_q%ninv=ninv ; lt_q%nkbz=nkbz
  allocate(lt_q%ibzq(nkbz),lt_q%ltq(ninv,nop),lt_q%wtksym(ninv,nop,nkbz))
  allocate(lt_q%tab(nkbz),lt_q%tabi(nkbz),lt_q%tabo(nkbz),lt_q%ibzq_ind(nkbz))
  !npwvec or npweps?
  allocate(lt_q%op(3,3,lt_q%nop),lt_q%iumklp(npwvec,ninv,nop),lt_q%fumklp(ninv,nop))
  lt_q%op=op
 end if

 ! Evaluate phase factors e^{-iG \cdot t} needed to symmetrize epsilon 
 ! Do this only once, since phases due to q cancel each other 
 allocate(phgt(dtset%npweps,nop),stat=istat)
  if(istat/=0) call memerr('screening','phgt',dtset%npweps*nop,'spc')
 do ig=1,dtset%npweps
  do iop=1,nop
   phgt(ig,iop)=exp(-(0.,1.)* two_pi * dot_product(gvec(:,ig),tnons(:,iop)))
  end do 
 end do

 ! Open file for independent matrix elements of \epsilon^-1
 if (me==0) then 
  filnam=trim(dtfil%filnam_ds(4))//'_EM1'
  call isfile(filnam,'new')
  open(unitem1,file=filnam,status='unknown',form='formatted')
 end if 

 ! Allocate tables for G vectors
 allocate(gptab(dtset%npweps,dtset%npweps),gptabo(dtset%npweps,dtset%npweps),stat=istat)
  if(istat/=0) call memerr('screening','gptab/gptabo',2*dtset%npweps**2,'i4b')

 call timab(305,2,tsec)

 !-----------------------------------------------------------------------
 ! Loop over q-points
 ! Calculate the screening function and save on disc for each q-point
 !-----------------------------------------------------------------------
 write(message,'(84a)')ch10,('-',ii=1,80),ch10,&
& ' screening : beginning of the main loop',ch10
 call wrtout(06,message,'COLL')

 ! Selective q points calculation
 do iq=1,nq
  found=.false.
  if (dtset%nqptdm>0) then
   do iqp=1,dtset%nqptdm
    qtemp(:)=q(:,iq)-dtset%qptdm(:,iqp)
    if (all(abs(qtemp(:))<1.0e-3)) found=.true.
    if (found) exit ! iqp
   end do
   if (.not.found) cycle !iq
  end if

  call timab(306,1,tsec) !loop1

  qeq0=.false.
  if (all(abs(q(:,iq))<1.0e-3)) qeq0=.true.

  if (dtset%nqptdm==0) then
   write(message,'(2a,i2,a,f9.6,2(",",f9.6),a,a)')ch10,&
&   ' q-point number ',iq,'        q = (',(q(i,iq),i=1,3),') [r.l.u.]',ch10
  else
   write(message,'(2a,i2,a,f9.6,2(",",f9.6),a,a)')ch10,&
&   ' q-point number ',iqp,'        q = (',(q(i,iq),i=1,3),') [r.l.u.]',ch10
  end if
  call wrtout(06,message,'COLL')

!Here we should add the treatment of the cutoff in the coulombian

  ! Calculate qplusg(G)=|q+G| table for coulombian potential
  call cvc(nq,iq,q,b1,b2,b3,dtset%npweps,gvec,qplusg)

  call pclock(100*iq)
  call timab(306,2,tsec) !wrong

  if (lt_q%sym_flag/=0) then
   ! In case of symmetrization, find little group 
   ! For the long-wavelength limit we consider a small but non zero q,
   ! however the oscillator matrix elements are evaluated using q==0
   ! this implies that also in this case, it is possible to take advantage of symmetries
   qdummy(:)=q(:,iq)
   if (qeq0) qdummy=zero
   call setup_little_group(qdummy(:),gmet,nop,op,lt_q%ninv,nkbz,kbz,&
&   lt_q%ibzq,lt_q%ibzq_ind,lt_q%ltq,lt_q%tab,lt_q%tabo,lt_q%tabi,lt_q%wtksym,&
&   lt_q%iumklp,lt_q%fumklp,npwvec,gvec,ngfft1a,ngfft1,ngfft2,ngfft3,prtvol)
  end if

  ! Calculate the independent G-Gp pairs, TODO gptabo could be i1b
  ! used to write the independent matrix elements of epsilon^-1 in outeps 
  ! and to speed-up the KK transform in case of spectral method (not yet) 
  ! we use dtset%npweps**2 since we do not know yet the number of independent pairs
  allocate(ip2fp_dummy(2,dtset%npweps**2)) 
   if(istat/=0) call memerr('screening','ip2fp_dummy',2*dtset%npweps**2,'i4b')
  
  gptab=0 ; gptabo=0 ; ip2fp_dummy=0

  ! Here we use the small non zero q to deal with the long wavelength limit
  ! This part could be cpu consuming, should optimize the code and maybe 
  ! skip this part if we are not using the spectral method althought finding 
  ! the irreducible pairs is useful for convergence studies
  call findggp(q(:,iq),nop,op,dtset%npweps,gvec,gmet,ngpi,gptab,gptabo,ip2fp_dummy)

  allocate(ip2fp(2,ngpi),stat=istat)
   if(istat/=0) call memerr('screening','ip2fp',2*ngpi,'i4b')
  ip2fp(:,:)=ip2fp_dummy(:,1:ngpi)
  deallocate(ip2fp_dummy)

  if (mpi_enreg%gwpara==1) then
   ! Parallelization over k-points and spin
   ! If symmetries are used it is not possible to share equally the load btw procs
   ! since the number of k-points depends on the external q, we redistribute the k-points for each q
   ! note nkbz, I dont like this approach, should rewrite everything
   ! since it could lead to BUGS if this distribution is used in other subroutines where 
   ! the IBZ indexing is used
   !  XG 070614 : should be done inside initmpi_gw
   allocate(mpi_enreg%proc_distrb(nkbz,nbnds,nsppol)) !this array will be passed to chi0, 
   mpi_enreg%parareel=0
   mpi_enreg%paralbd=0
   mpi_enreg%paral_level=2
   if (lt_q%sym_flag==0) then  
    !
    ! No symmetries, divide the full BZ btw procs
    !
    allocate(nband_t(nkbz*nsppol))
    nband_t(:)=nbnds
    call distrb2(nbnds,nband_t,nkbz,nsppol,mpi_enreg)
   else if (lt_q%sym_flag/=0) then  
    !
    ! Divide the IBZ_q btw procs (parellelization might be not so efficient for particular q)
    !
    nkpt4chiq=sum(lt_q%ibzq(:)) !should became output of setup_little_group
    allocate(nband_t(nkpt4chiq*nsppol)) 
    nband_t(:)=nbnds
    mpi_enreg_dummy=mpi_enreg !sure?, CHECKME the order makes the difference
    allocate(mpi_enreg_dummy%proc_distrb(nkpt4chiq,nbnds,nsppol))
    call distrb2(nbnds,nband_t,nkpt4chiq,nsppol,mpi_enreg_dummy)
    mpi_enreg%proc_distrb=-999 !to debug
    do ikq=1,sum(lt_q%ibzq(:))
     do ik=1,nkbz
      do is=1,nsppol
       if (lt_q%ibzq_ind(ik)==ikq) mpi_enreg%proc_distrb(ik,:,is)=mpi_enreg_dummy%proc_distrb(ikq,:,is)
      end do
     end do 
    end do
    deallocate(mpi_enreg_dummy%proc_distrb)
   end if 
   deallocate(nband_t)
   !
   ! Announce the treatment of k-points by each proc
   !
   do ik=1,nkbz 
    do is=1,nsppol
!DEBUG  for the time being dont uncomment
    if (any(mpi_enreg%proc_distrb(ik,:,is)/=mpi_enreg%proc_distrb(ik,1,is))) then 
     write(*,*)'screening : wrong distrb'
     call leave_new('COLL')
    end if 
!ENDDEBUG 
     if (mpi_enreg%proc_distrb(ik,nbnds,is)/=-999) then 
      write(message,'(a,i4,a,i4,a,i4)')&
&     'P screening : treating k-point ',ik,' and spin ',is,&
&     ' by node ',mpi_enreg%proc_distrb(ik,nbnds,is)
      call wrtout(6,message,'PERS')
     end if 
    end do 
   end do 
  end if !if mpi_enreg%gwpara==1

  if (qeq0) then 
   ! Different treatment of the case q=0
   call timab(307,1,tsec)

   inclvkb=dtset%inclvkb 
   if (inclvkb==1) then 
    !
    ! Implementation based on Legendre polynomials (CPU and mem ~npwwfn^2)
    !
    allocate(gradvnl(3,dtset%npwwfn,dtset%npwwfn,nkibz),stat=istat)
     if(istat/=0) call memerr('screening','gradvnl',3*dtset%npwwfn*dtset%npwwfn*nkibz,'spc')
    call ccgradvnl(dtset%npwwfn,nkibz,b1,b2,b3,gvec,kibz,ntypat,&
&    natom,mpsang,typat,xcart,vkbsign,vkb,vkbd,gradvnl)
   else if (inclvkb==2) then 
    !
    ! Implementation based on spherical harmonics (much faster since CPU and mem ~npwwfn)
    !
    allocate(fnl(dtset%npwwfn,mpsang*mpsang,natom,nkibz),stat=istat)
      if(istat/=0) call memerr('screening','fnl',dtset%npwwfn*mpsang*mpsang*natom*nkibz,'spc')
    allocate(fnld(3,dtset%npwwfn,mpsang*mpsang,natom,nkibz),stat=istat)
     if(istat/=0) call memerr('screening','fnld',3*dtset%npwwfn*mpsang*mpsang*natom*nkibz,'spc')
    call ccgradvnl_ylm(dtset%npwwfn,nkibz,a1,a2,a3,b1,b2,b3,gvec,kibz,ntypat,natom,&
&    mpsang,typat,xcart,vkbsign,vkb,vkbd,fnl,fnld)
   end if !inclvkb

!PM-MG in 5.3.1 the dimension of k-table was not correct, we were using nkibz
!               istead of nkbz (kibz,nkibz,nkibz --> kibz,nkibz,nkbz)
   if (dtset%mkmem/=0) then
    !
    ! Wavefunctions are in memory (fast)
    !
    !if (ninv==2 .and. dtset%gwpara==2) then 
    if (read_val) then 
     !
     ! New algorithm based on time reversal
     !
     call cchi0q0(q(:,iq),ep,nomega,omega,gvec,npwvec,dtset%npweps,dtset%npwwfn,op,nop,&
&     kibz,nkibz,nkbz,nbnds,nbv,nbvw,nsppol,occ,ktab,ktabr,ktabi,ktabo,ktabp,kbz,nkbz,&
&     irottb,ngfft1,ngfft1a,ngfft2,ngfft3,igfft,nr,energy,gwenergy,etadelta,chi0,b1,b2,b3,&
&     ucvol,wtk,grottb,inclvkb,gradvnl,mpi_enreg,grottbm1,phgt,ngpi,gptab,gptabo,ip2fp,&
&     lt_q,natom,mpsang,fnl,fnld,kkweight,min_band_proc,max_band_proc,nbnds_per_proc,localrdwf,&
&     wfr=wfr,wfg=wfg,wfrv=wfr_val,wfgv=wfg_val) !optional
    else 
     !
     ! Old method, it can be used when time reversal does not hold
     !
     call cchi0q0(q(:,iq),ep,nomega,omega,gvec,npwvec,dtset%npweps,dtset%npwwfn,op,nop,&
&     kibz,nkibz,nkbz,nbnds,nbv,nbvw,nsppol,occ,ktab,ktabr,ktabi,ktabo,ktabp,kbz,nkbz,&
&     irottb,ngfft1,ngfft1a,ngfft2,ngfft3,igfft,nr,energy,gwenergy,etadelta,chi0,b1,b2,b3,&
&     ucvol,wtk,grottb,inclvkb,gradvnl,mpi_enreg,grottbm1,phgt,ngpi,gptab,gptabo,ip2fp,&
&     lt_q,natom,mpsang,fnl,fnld,kkweight,min_band_proc,max_band_proc,nbnds_per_proc,localrdwf,&
&     wfr=wfr,wfg=wfg) !optional
     end if 
   else 
    !
    ! Old method with wfs on disk, slow and obsolete
    !
    call cchi0q0(q(:,iq),ep,nomega,omega,gvec,npwvec,dtset%npweps,dtset%npwwfn,op,nop,&
&    kibz,nkibz,nkbz,nbnds,nbv,nbvw,nsppol,occ,ktab,ktabr,ktabi,ktabo,ktabp,kbz,nkbz,&
&    irottb,ngfft1,ngfft1a,ngfft2,ngfft3,igfft,nr,energy,gwenergy,etadelta,chi0,b1,b2,b3,&
&    ucvol,wtk,grottb,inclvkb,gradvnl,mpi_enreg,grottbm1,phgt,ngpi,gptab,gptabo,ip2fp,&
&    lt_q,natom,mpsang,fnl,fnld,kkweight,min_band_proc,max_band_proc,nbnds_per_proc,localrdwf)
   end if 
   !
   if (inclvkb==1) deallocate(gradvnl)
   if (inclvkb==2) deallocate(fnl,fnld)
   if (allocated(wfg)) deallocate(wfg)
   call timab(307,2,tsec)
   !
  else 
   !
   ! Calculate cchi0 at q/=0
   !
   call timab(308,1,tsec)
!PM-MG in 5.3.1 the dimension of k-table was not correct, we were using nkibz
!               istead of nkbz (nkibz,nkibz,nbnds --> nkibz,nkbz,nbnds)
   if (dtset%mkmem/=0) then
    !
    ! Wavefunctions are in memory (fast)
    !
    !if (ninv==2 .and. dtset%gwpara==2) then 
    if (read_val) then
     !
     ! New algorithm based on time reversal
     !
     call cchi0(q(:,iq),ep,nomega,omega,npwvec,dtset%npweps,dtset%npwwfn,nkibz,kibz,nkbz,nbnds,&
&     nbv,nbvw,nsppol,occ,ktab,ktabr,ktabi,ktabp,kbz,nkbz,ngfft1,ngfft1a,ngfft2,ngfft3,igfft,nr,&
&     energy,gwenergy,etadelta,chi0,mpi_enreg,nop,op,grottbm1,phgt,ngpi,gptab,gptabo,ip2fp,&
&     lt_q,kkweight,min_band_proc,max_band_proc,nbnds_per_proc,localrdwf,&
&     wfr=wfr,wfrv=wfr_val) !optional
    else 
     !
     ! Old method, can be used when time reversal does not hold
     !
     call cchi0(q(:,iq),ep,nomega,omega,npwvec,dtset%npweps,dtset%npwwfn,nkibz,kibz,nkbz,nbnds,&
&     nbv,nbvw,nsppol,occ,ktab,ktabr,ktabi,ktabp,kbz,nkbz,ngfft1,ngfft1a,ngfft2,ngfft3,igfft,nr,&
&     energy,gwenergy,etadelta,chi0,mpi_enreg,nop,op,grottbm1,phgt,ngpi,gptab,gptabo,ip2fp,&
&     lt_q,kkweight,min_band_proc,max_band_proc,nbnds_per_proc,localrdwf,&
&     wfr=wfr) !optional
    end if
   else
    !
    ! Old method with wfs on disk, slow and obsolete
    !
    call cchi0(q(:,iq),ep,nomega,omega,npwvec,dtset%npweps,dtset%npwwfn,nkibz,kibz,nkbz,nbnds,&
&    nbv,nbvw,nsppol,occ,ktab,ktabr,ktabi,ktabp,kbz,nkbz,ngfft1,ngfft1a,ngfft2,ngfft3,igfft,nr,&
&    energy,gwenergy,etadelta,chi0,mpi_enreg,nop,op,grottbm1,phgt,ngpi,gptab,gptabo,ip2fp,&
&    lt_q,kkweight,min_band_proc,max_band_proc,nbnds_per_proc,localrdwf)
   end if
   !
   call timab(308,2,tsec)
   !
  end if !of calculate cchi0 at q/=0
 
  if (mpi_enreg%gwpara==1) deallocate (mpi_enreg%proc_distrb)
  call timab(309,1,tsec)

  if (mpi_enreg%me==0) then
   ! Print chi0(q,G,Gp,omega), only master works 
   do iomega=1,nomega
    write(message,'(a,i4,a,2f9.4,a)')&
&    ' chi0(G,G'') at the ',iomega,' th omega',omega(iomega)*Ha_eV,' [eV]'
    call wrtout(06,message,'COLL')
    if (dtset%nqptdm>0)then
     write(message,'(a,i3,a,i4,a)')' chi0(q=',iqp,', omega=',iomega,', G,G'')'
    else
     write(message,'(a,i3,a,i4,a)')' chi0(q=',iq,', omega=',iomega,', G,G'')'
    end if
    call wrtout(06,message,'COLL')
    call printcm(chi0(:,:,iomega),dtset%npweps,dtset%npweps)
   end do

  call pclock(100*iq+10)

 if (rpa) then
! Calculate epsilon=(1-Vc*chi0)
   chi0(:,:,:)=chi0(:,:,:)/ucvol
   do io=1,nomega
    do ig2=1,dtset%npweps
     chitmp(:,ig2)=(-(4.0*pi)/(qplusg(:)*qplusg(ig2)))*chi0(:,ig2,io)
     chitmp(ig2,ig2)=1+chitmp(ig2,ig2)
    end do
    eps(:,:,io)=chitmp(:,:)
   end do

   do iomega=1,nomega
    write(message,'(a,i4,a,2f9.4,a)')&
&    ' epsilon(G,G'') at the ',iomega,' th omega',omega(iomega)*Ha_eV,' [eV]'
    call wrtout(06,message,'COLL')
    call printcm(eps(:,:,iomega),dtset%npweps,dtset%npweps)
   end do

   if (qeq0) epsilon0nlf=real(eps(1,1,1))

   do io=1,nomega
    call matcginv(eps(:,:,io),dtset%npweps,dtset%npweps)
   end do
 
   do iomega=1,nomega
    write(message,'(a,i4,a,2f9.4,a)')&
&    ' epsilon^-1(G,G'') at the ',iomega,' th omega',omega(iomega)*Ha_eV,' [eV]'
    call wrtout(06,message,'COLL')
    call printcm(epsm1(:,:,iomega),dtset%npweps,dtset%npweps)
   end do

   ! Write the independent matrix elements on external file (useful to perform convergence study in nomegasf)
   ! FIXME should write also the header
   write(message,'(a,3(f10.6),a)')&
&   ' epsilon^-1(G,G'') at q = ( ',(q(i,iq),i=1,3),' ) [r.l.u.]'
   call outeps(dtset%npweps,gvec,nomega,omega,ngpi,gptab,gptabo,ip2fp,epsm1,gmet,message,unitem1,prtvol)
!DEBUG
!write(*,*)'checking epsm'
!call symf12(dtset%npweps,nomega,epsm1,omega,nop,op,ngpi,gptab,gptabo,ip2fp,phgt,2) !only check
!ENDDEBUG

   ! Calculate macroscopic dielectric constant: epsmlf=1/epsm1(G=0,G''=0)
   if (qeq0) epsilon0=real(1.0/epsm1(1,1,1))

   if (qeq0 .and. .not.plasmon_pole_model) then
    filnam=trim(dtfil%filnam_ds(4))//'_ELF'
    open(69,file=filnam)
    do io=1,ep%nomegaer
     write(69,*) real(omega(io))*Ha_eV, -aimag(epsm1(1,1,io))
    end do
    close(69)
   end if

  else ! of if(rpa)
!==========================================================================================

  do iomega=1,nomega
   if(lda) then
   !calculate chi0*KxcLDA and put it in chitmp
   !!calculate chi0*Kxc in G-space
     chitmp=matmul(chi0(:,:,iomega),kxclda(:,:))
  end if
 
    if(lda) then
! Calculate chi0*KxcLDA and put it in chitmp
! calculate chi0*Kxc in G-space
    do ig1=1,dtset%npweps
     do ig2=1,dtset%npweps
     chitmp(ig1,ig2)=(-(4.0*pi)/(qplusg(ig2)**2))*&
    &      chi0(ig1,ig2,iomega)-chitmp(ig1,ig2)
     end do
    chitmp(ig1,ig1)=chitmp(ig1,ig1)+ucvol
    end do
    end if

! Calculate chi
    write(message,'(3a)')ch10,' calculating chi=(1-chi0*Vc-chi0*Kxc)^-1 chi0',ch10
    call wrtout(06,message,'COLL')

! Calculate (1-chi0*Vc-chi0*Kxc ) and put it in chitmp
    do ig1=1,dtset%npweps
     do ig2=1,dtset%npweps
      chitmp(ig1,ig2)=(-(4.0*pi)/(qplusg(ig2)**2))*chi0(ig1,ig2,iomega)-chitmp(ig1,ig2)
     end do
!MG why ucvol?
     chitmp(ig1,ig1)=chitmp(ig1,ig1)+ucvol
    end do

! Invert (1-chi0*Vc-chi0*Kxc)
     call matcginv(chitmp,dtset%npweps,dtset%npweps)

! Multiply for chi0
    chitmp=matmul(chitmp,chi0(:,:,iomega))

!must multiply by ucvol (see notes-due to conversion of r->G in matrix definitions)
    chi(:,:,iomega)=chitmp(:,:)*ucvol

      end do !iomega

! Print chi

    do iomega=1,nomega
    write (6,'(a,i2,a,i1,a)')'chi(q=',iq,',omega=',iomega,',G,G")'
    call printcm(chi(:,:,iomega),dtset%npweps,dtset%npweps)
    end do
        
!=================================================================================================
! Calculate epsilon-twiddle^-1
    write(*,*)  'calculating epsilon^-1(G,G")=1+(Vc+Kxc)*chi'
    do iomega=1,nomega
     if(.not.testparticle) then
      if(lda) then
       chitmp=matmul(kxclda(:,:),chi(:,:,iomega))
       end if
! Perform hermitianization

     call hermitianize(chitmp,dtset%npweps)

     else !(testparticle)
      !zeroes chitmp
     chitmp(:,:)=(0.0,0.0)
      end if

      do ig1=1,dtset%npweps
      epsm1(ig1,:,iomega)=(4*pi/(qplusg(ig1)*qplusg(:)*ucvol))*&
      &  chi(ig1,:,iomega)+chitmp(ig1,:)/ucvol
!chitmp=Kxc*chi  must be divided by ucvol see notes on fft r->G
        epsm1(ig1,ig1,iomega)=1.0+epsm1(ig1,ig1,iomega)
     end do ! ig1
    end do ! iomega
! Print epsilon-twiddle^-1
 do iomega=1,2
  write (message,'(a,i2,a,i1,a)')'   epsilon-twiddle^-1(q=',iq,',omega=',iomega,',G,G")'
  call wrtout(06,message,'COLL')
  call printcm(epsm1(:,:,iomega),dtset%npweps,dtset%npweps)
  end do
  
  !  Find constant epsilon0 with local field effects 
    if(qeq0) then 
    do  io=1,nomega                                                                                                        
     if(abs(aimag(omega(io)))<1.e-3)then
      epsilon0=real(1.0/epsm1(1,1,io))
     end if
    end do 
    end if

  end if !of if(rpa)
!=================================================================================
  call timab(309,2,tsec)
  call timab(310,1,tsec)

   ! Write epsilon^-1 on file
   ! on title(1) it is better to write the name of the system (e.g. Silicon bulk..)
   title(1)='em1 file: epsilon^-1'
   !string='0'
   !if (inclvkb/=0) call int2char(inclvkb,string)
   !title(1)=title(1)//' , with inclvkb= '//string
   if (testparticle) then
    title(2)='TESTPARTICLE'
   else
    title(2)='TESTELECTRON'
   end if
   title(2)(14:17)=ctype

   call wrscr(dtfil,hdr,dtset,dtset%npweps,dtset%npwwfn,npwvec,nbnds,nq,nomega,q,omega,&
&   gvec,iq,epsm1,title,unitf,nop,op)

   if (qeq0) then
    do io=1,nomega
     if (abs(real(omega(io)))<1.e-3 .and. abs(aimag(omega(io)))<1.e-3) then
       write(message,'(a,f8.4)')&
&       ' dielectric constant = ',epsilon0
       call wrtout(6,message,'COLL') ; call wrtout(ab_out,message,'COLL')
       write(message,'(a,f8.4,a)')&
&       ' dielectric constant without local fields = ',epsilon0nlf,ch10
       call wrtout(6,message,'COLL') ; call wrtout(ab_out,message,'COLL')
      end if
    end do !io
   end if !qeq0

  end if !=========================================== only master

  deallocate(ip2fp)

  call pclock(100*iq+90)
  call timab(310,2,tsec)

 end do !end of loop over q points

!******************************************************************************************* 

 ! Close and delete open files
 close(unitem1)
 if (dtset%mkmem==0) then
  close(unitwfg,status='delete') !these might be huge
  close(unitwfr,status='delete')
 end if 

 ! Deallocate arrays
 if (lda) deallocate(kxclda)
 !if (ninv==2 .and. dtset%gwpara==2) deallocate(wfg_val,wfr_val)
 if (read_val) deallocate(wfg_val,wfr_val)
 if (i2==502 .or. i2==602) deallocate(typat,xred,xcart,vkb,vkbd,vkbsign)
 if (associated(mpi_enreg%proc_distrb)) deallocate(mpi_enreg%proc_distrb)

 if (mpi_enreg%nproc==1 .or. localrdwf==1 .or. me==0) then
  if (i2==502 .or. i2==602) then
   call hdr_clean(hdr)
  end if
 end if
 if (lt_q%sym_flag/=0) then
  deallocate(lt_q%ibzq,lt_q%ltq,lt_q%wtksym)
  deallocate(lt_q%op,lt_q%tab,lt_q%tabi,lt_q%tabo,lt_q%ibzq_ind)
  deallocate(lt_q%iumklp,lt_q%fumklp)
 end if
 if (ep%spmeth/=0) then ! spectral method 
  !MG g95 crash: ep%omegasf is associated even though the array is not explicitly allocated
  if (allocated(kkweight)) deallocate(kkweight)
  if (associated(ep%omegasf)) deallocate(ep%omegasf)
 end if 

 deallocate(gptab,gptabo)
 deallocate(chitmp,chi0,energy,grottb,grottbm1)
 deallocate(gvec,gwenergy,igfft,irottb,kibz)
 deallocate(occ,omega,op,q,qplusg,rho2,nbv)
 deallocate(kbz,wtk)
 deallocate(ktab,ktabi,ktabo,ktabr,ktabp)
 deallocate(phgt)

 if (allocated(wfr  )) deallocate(wfr)
 if (allocated(wfg  )) deallocate(wfg)
 if (allocated(tnons)) deallocate(tnons)

 call pclock(9999)

 write(message,'(a)')' screening ended' 
 call wrtout(6,message,'COLL')

 call timab(301,2,tsec)

!DEBUG
!write(6,*)' sigma : end '
!call flush(6)
!ENDDEBUG

 end subroutine screening
!!***
