!{\src2tex{textfont=tt}}
!!****f* ABINIT/sigma
!! NAME
!! sigma
!!
!! FUNCTION
!!
!! Calculate self-energy operator matrix elements
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (GMR, VO, LR, RWG, MT, MG, RShaltaf)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  acell(3)=length scales of primitive translations (bohr)
!!  dtfil <type(datafiles_type)>=variables related to files
!!  dtset <type(dataset_type)>=all input variables for this dataset
!!  mpi_enreg=informations about MPI parallelization (to be completed)
!!  rprim(3,3)=dimensionless real space primitive translations
!!
!! OUTPUT
!!  output is written in a file
!!
!! PARENTS
!!      driver,drivergw
!!
!! NOTES
!!
!! files used:
!! input:
!! 10: LDA band structure and material info for sigma operator
!! 12: Epsilon-twiddle**-1 (plasmon-pole form) file EPSM1
!! stdin: parameters of calculation (incl. which
!!        k-points to calc. Sigma for) file sig.in
!! output:
!! 21: gw file, available for exciton calculation
!!
!! CHILDREN
!!      calc_ffm,calc_wf_qp,cigfft,clcqpg,cmevxclda,cppm1par,cppm2par,cppm3par
!!      cppm4par,crho,crho_augmented,csigme,distrb2,dosym,eps1_tc,fermi,fftwfn
!!      findk,findqg0,ham_changebasis,hartrham,hdr_clean,identk,identq
!!      initmpi_gw,lattice,leave_new,leave_test,mati3inv,memerr,metric,mkrdim
!!      pclock,rdgw,rdkss,rdlda,rdldaabinit,rdqps,rdscr,setmesh,setshells
!!      setup_little_group,surot,testlda,testscr,timab,write_sigma_results
!!      write_sigma_results_header,wrppm,wrqps,wrtout,xcast_mpi,xcomm_init
!!      xmaster_init,xme_init
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine sigma(acell,dtfil,dtset,mpi_enreg,rprim)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_12geometry
 use interfaces_14iowfdenpot
 use interfaces_15gw
 use interfaces_lib01hidempi
#else
 use defs_xfuncmpi
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 type(MPI_type),intent(inout) :: mpi_enreg
 type(datafiles_type),intent(in) :: dtfil
 type(dataset_type),intent(inout) :: dtset
!arrays
 real(dp),intent(in) :: acell(3),rprim(3,3)

!Local variables-------------------------------
 !complex,pointer :: wfr_jb(:,:,:,:)
!MG this is for the inclusion of non-symmorphic operations
!MG this is for the augmented FFT mesh
!MG the following added to implement the analytic continuation
! = 0.27 eV
! this is for the inclusion of the cutoff in coulombian (still under development) 
!should be filled starting from dtset%
!this is to merge parallelization over q-points with para
!END MG
!scalars
 integer,parameter :: NOMEGASI=12,iunit=10,unitem1=24
 integer :: addshell,aliasing_factor,b1gw,b2gw,enforce_sym,fform,i,i1,i2,i3,ib
 integer :: ibr,ierr,ig,ik,ikb_proc_rank,ikcalc,ikibz,ikq,io,iop,iout,iproc,iq
 integer :: iqm,ir,is,isppol,istat,j,jj,localrdwf,master,max_band_proc,me,method
 integer :: min_band_proc,mkmem,mpsang,natom,nbcw,nbnds_occ,nbnds_per_proc,nbr
 integer :: nbsc,nbvw,nbw,nel=0,ngfft1,ngfft1_temp,ngfft1a,ngfft1a_temp,ngfft1d
 integer :: ngfft1dd,ngfft2,ngfft2_temp,ngfft2a,ngfft2a_temp,ngfft2d,ngfft2dd
 integer :: ngfft3,ngfft3_temp,ngfft3a,ngfft3a_temp,ngfft3d,ngfft3dd,ngr,ninv
 integer :: nkbzx,nkibzr,nop,nopr,npwc2,npwc3,npwvec,nqbzx,nqibzx,nqpt4sigk,nr
 integer :: nr_temp,nrb,nrd,nscf,nshr,nslice_qpt,nsppol,ntypat,ppmsize,sizebdgw
 integer :: prtvol,spaceComm,tim_fourdp
 real(dp),parameter :: OMEGASIMAX=50/Ha_eV,OMEGASIMIN=0.01d0
 real(dp) :: bzvol,condbottom,domegas,efermi,efermi_qp,oldefermi,omegaplasma
 real(dp) :: ucvol,valencetop
 logical :: DEBUG=.true.,analytic_continuation,ltemp,max_found,min_found
 logical :: update_energies=.false.,use_augmented=.false.,verbose
 character(len=500) :: message
 character(len=fnlen) :: filnam
 type(MPI_type) :: mpi_enreg_dummy
 type(cutoff_data) :: cutp
 type(epsilonm1_parameters) :: ep
 type(epsilonm1_results) :: er
 type(hdr_type) :: hdr,hdr_scr
 type(little_group) :: lt_k
 type(sigma_parameters) :: sp
 type(sigma_results) :: sr
!arrays
 integer :: g0(3),ibocc(dtset%nsppol)
 integer,allocatable :: distrb(:),grottb(:,:,:),grottbm1(:,:,:),gvec(:,:)
 integer,allocatable :: igfft(:,:,:,:),irottb(:,:),ktab(:),ktabi(:),ktabo(:)
 integer,allocatable :: ktabr(:,:),nband_t(:),nbv(:),qtab(:),qtabi(:),qtabo(:)
 integer,allocatable :: symafm(:),symrel(:,:,:),symreli(:,:,:),typat(:)
 real(dp) :: a1(3),a2(3),a3(3),b1(3),b2(3),b3(3),gmet(3,3),gprimd(3,3),kgwmk(3)
 real(dp) :: rm1t(3),rmet(3,3),rprimd(3,3),sg(3),sq(3),tsec(2)
 real(dp),allocatable :: en(:,:,:),en_qp(:,:,:),kbz(:,:),kibz(:,:),occ(:,:,:)
 real(dp),allocatable :: occ_qp(:,:,:),op(:,:,:),qbz(:,:),qpg(:,:),qpoint(:,:)
 real(dp),allocatable :: qratio(:,:,:),rho(:,:),rho_p(:,:),rho_qp(:,:)
 real(dp),allocatable :: rhod(:,:),sr_gwenergy(:,:,:),tnons(:,:),vccut(:,:)
 real(dp),allocatable :: vkb(:,:,:,:),vkbd(:,:,:,:),vkbsign(:,:),wtk(:)
 real(dp),allocatable :: xred(:,:)
 complex,allocatable :: hbare(:,:,:,:),hlda(:,:,:,:),ktabp(:)
 complex,allocatable :: m_lda_to_qp(:,:,:,:),phsgt(:,:),temp_omegatw(:,:)
 complex,allocatable :: vhartr(:,:,:,:),vxc(:,:,:,:),wfg(:,:,:,:),wfr(:,:,:,:)
 complex,allocatable :: wfr_gw(:,:,:,:)
 character(len=80) :: titem1(2),title(2)
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 complex :: interp
#endif
!End of the abilint section

!************************************************************************
!BEGIN EXECUTABLE SECTION

!DEBUG
!write(6,*)' sigma : enter '
!ENDDEBUG

 write(message,'(8a)')&
& ' SIGMA: Calculation of the GW corrections ',ch10,ch10,&
& ' Based on a program developped by R.W. Godby, V. Olevano, G. Onida, and L. Reining.',ch10,&
& ' Incorporated in ABINIT by V. Olevano, G.-M. Rignanese, and M. Torrent.',ch10
 call wrtout(6,message,'COLL')
 call wrtout(ab_out,message,'COLL')

! Start clock
 call pclock(0)
 call timab(401,1,tsec)
 call timab(402,1,tsec)

! Initialize some MPI related variables
 call xcomm_init(mpi_enreg,spaceComm)  !Init mpi_comm
 call xme_init(mpi_enreg,me)           !Init me
 call xmaster_init(mpi_enreg,master)   !Init master

 ! Set up parallelization level 
 !  0 for sequential run
 !  1 parallelism is over k-points 
 !  2 conduction bands are spread btw processors
 !    each proc has occupied and partially occupied states 
 if (mpi_enreg%nproc==1) dtset%gwpara=0  
 mpi_enreg%gwpara=dtset%gwpara           

 ! Define I/O
 !  localrdwf==1 ==> all procs have access to files (default)
 !  localrdwf==0 ==> only master has access to files
 localrdwf=dtset%localrdwf    
                             
 prtvol=dtset%prtvol

 ! Not implemented yet :
 !  mkmem=0  write wfs on an external file
 !  mkmem/=0 wavefunctions are in memory
 mkmem=dtset%mkmem                

!******************************************************************************************* 

 sp%gwcalctyp=dtset%gwcalctyp
 write(message,'(a,i3)')' GW calculation type : ',sp%gwcalctyp
 call wrtout(6,message,'COLL')

 sp%splitsigc=dtset%splitsigc
 sp%ppmodel=dtset%ppmodel

 ! In case of H-F, SEX or COHSEX use Hybertsen-Louie ppm (only $\omega = 0 $)
 ! Use fake screening in case of HF calculation 
 if (mod(sp%gwcalctyp,10)==5 .or. &
&    mod(sp%gwcalctyp,10)==6 .or. &
&    mod(sp%gwcalctyp,10)==7) sp%ppmodel=2

 if (mod(sp%gwcalctyp,10)<5 .and. mod(sp%gwcalctyp,1)/=1) then
  ! If one shot GW with PPM or numerical integration (contour deformation)
  sp%nomegasrd=dtset%nomegasrd
  sp%omegasrdmax=dtset%omegasrdmax
  sp%deltae=(2*sp%omegasrdmax)/(sp%nomegasrd-1)
 else
! In case of AC no need to evaluate derivative by finite differences, why not 0? 
  sp%nomegasrd= 1
  sp%omegasrdmax= zero
  sp%deltae= zero
 endif

 write(message,'(4a,i4,2a,f10.6,2a,f10.6)')ch10,&
& ' Parameter used to calculate Sigma derivatives : ',ch10,&
& '  number of point to evaluate derivative = ',sp%nomegasrd,ch10,&
& '  frequency range [eV] = ',sp%omegasrdmax*Ha_eV,ch10,&
& '  frequency step  [eV] = ',sp%deltae*Ha_eV
 call wrtout(6,message,'COLL')

 ! In case of analytic continuation define the number of imaginary freq where calculate Sigma
 ! tests show than more than 12 freq in the Pade approximant worsen the results!
 analytic_continuation=.false.
 if (mod(sp%gwcalctyp,10)==1) then 
  analytic_continuation=.true.
  sp%nomegasi=dtset%nomegasi
  sp%omegasimax=dtset%omegasimax
  sp%omegasimin=OMEGASIMIN
  if (sp%nomegasi==0) sp%nomegasi=NOMEGASI
  if (sp%omegasimax<0.1d-4) sp%omegasimax=OMEGASIMAX
  if (sp%gwcalctyp==1) then 
   write(message,'(4a,i3,2a,f8.3,2a,f8.3,a)')ch10,&
&   ' Parameters for analytic continuation : ',ch10,&
&   '  number of points for sigma on the imaginary axis    =  ',sp%nomegasi,ch10,&
&   '  min frequency for sigma on imag axis           [eV] =  ',sp%omegasimin*Ha_eV,ch10,&
&   '  max frequency for sigma on imag axis           [eV] =  ',sp%omegasimax*Ha_eV,ch10
   call wrtout(6,message,'COLL')
  else 
   ! At the moment only one shot GW 
   write(*,*)' not coded yet' 
   call leave_new('COLL')
  end if 
 end if 

 sp%nomegasr=dtset%nfreqsp
 sp%omegasrmax=dtset%freqspmax
 if (sp%nomegasr>0) then
  domegas= 2*sp%omegasrmax/(sp%nomegasr-1)
  allocate(sr%omega(sp%nomegasr))
  do io=1,sp%nomegasr
   sr%omega(io)= -sp%omegasrmax + domegas*(io-1)
  end do
  write(message,'(4a,i8,2a,f8.3,2a,f8.3,a)')ch10,&
&  ' Parameters for spectral function : ',ch10,&
&  '  number of points    =  ',sp%nomegasr,ch10,&
&  '  max frequency  [eV] = ',sp%omegasrmax*Ha_eV,ch10,&
&  '  frequency step [eV] = ',domegas*Ha_eV,ch10
  call wrtout(6,message,'COLL')
 else
  !in indefo all these quantities are set to zero
  !sp%nomegasr=1 
  !allocate(sr%omega(sp%nomegasr))
  !sr%omega(1)=0
 end if

 sp%nb=dtset%nband(1)
 sp%zcut=dtset%zcut
 write(message,'(2a,f10.6,a)')ch10,&
& ' zcut for avoiding poles in sigma [eV] = ',sp%zcut*Ha_eV,ch10
 call wrtout(6,message,'COLL')

! MG Check for calculations that are not still implemented
! ScGW calculations are still not compatible with parallelization over bands 
!  if (mpi_enreg%gwpara==2 .and. dtset%gwcalctyp >= 10 )then 
!   write(message,'(2a,i3,2a)')ch10,&
!&   ' sigma : parallelism over bands and gwcalctyp ',dtset%gwcalctyp,ch10,&
!&   ' are not yet compatible'
!   call wrtout(6,message,'COLL')
!   call leave_new('COLL')
!  end if 

  if ( (dtset%ppmodel==3 .or. dtset%ppmodel==4) .and. dtset%gwcalctyp>= 10 )then 
   write(message,'(2a,i3,2a)')ch10,&
&   ' sigma : the ppmodel chosen and gwcalctyp ',dtset%gwcalctyp,ch10,&
&   ' are not yet compatible'
   call wrtout(6,message,'COLL') ; call leave_new('COLL')
  end if 

 ! Compute dimensional primitive translations rprimd (from input file)
 call mkrdim(acell,rprim,rprimd)

 ! Obtain dimensional translations in reciprocal space gprimd, metrics and unit cell volume, from rprimd.
 ! Also output rprimd, gprimd and ucvol
 call metric(gmet,gprimd,ab_out,rmet,rprimd,ucvol)

 !WARNIG here there is a possible undected error if symrel is not 
 !the same as the symrel read from the KSS file, should add a check

 ! Define consistently npw, nsh, and ecut for wavefunctions
 call setshells(dtset%ecutwfn,dtset%npwwfn,dtset%nshwfn,dtset%nsym,gmet,&
& gprimd,dtset%symrel,'wfn',ucvol)
 ! Define consistently npw, nsh, and ecut for the exchange part of sigma
 call setshells(dtset%ecutsigx,dtset%npwsigx,dtset%nshsigx,dtset%nsym,gmet,&
& gprimd,dtset%symrel,'mat',ucvol)

 sp%npwwfn=dtset%npwwfn
 sp%npwx=dtset%npwsigx
 ! Calculate npwvec as the max between dtset%npwsigx and dtset%npwwfn
 npwvec=max(sp%npwwfn,sp%npwx)

 ! Read parameters of the KSS or QPLDA or STA file and verifify them
 ibocc(:)=0 ; dtset%awtr=0
 call testlda(dtset,dtfil,nop,nkibzr,nbr,ngr,nshr,i1,i2,ntypat,natom,&
& mpsang,nsppol,mpi_enreg,localrdwf,ibocc)

 if (i2/=0 .and. i2/=1 .and. i2/=2 .and. i2/=502 .and. i2/=602) then
  write(message,'(3a)')&
&  ' sigma: ERROR -',ch10,&
&  '  unknown format found in the screening file'
  call wrtout(6,message,'PERS') ; call leave_new('COLL')
 end if

 sp%nk=nkibzr
 sp%nsppol=nsppol

! Use time reversal symmetry
! MG note that most of the equations are obtained assuming ninv==2, one cannot simply use ninv==1
 ninv= 2

 inquire(file='nop.in',exist=ltemp)
 if (ltemp) then
  open(99,file='nop.in')
  read(99,*) nop,ninv
  close(99)
 end if

 sp%nop=nop

 if (npwvec>ngr) then
  write(message,'(3a)')&
&  ' sigma: WARNING - ',ch10,&
&  '  number of G-vectors found less then required'
  call wrtout(6,message,'COLL')
  npwvec=ngr
  if (sp%npwwfn>ngr) sp%npwwfn= ngr
  if (sp%npwx>ngr) sp%npwx= ngr
  write(message,'(3(a,i8,a))')&
&  '         calculation will proceed with npwvec  = ',npwvec,ch10, &
   '         calculation will proceed with npwsigx = ',sp%npwx,ch10,&
   '         calculation will proceed with npwwfn  = ',sp%npwwfn,ch10
  call wrtout(6,message,'COLL')
 end if
 if (sp%nb>nbr) then
  write(message,'(5a,i4,a)')&
&  ' sigma: WARNING - ',ch10,                            &
&  '  number of bands found less then required     ',ch10,&
&  '         calculation will proceed with nbnds= ',nbr,ch10
  call wrtout(6,message,'COLL')
  sp%nb=nbr
 end if

 allocate(op(3,3,nop),stat=istat)
 allocate(kibz(3,sp%nk),stat=istat)
 allocate(gvec(3,npwvec),stat=istat)
  if(istat/=0) stop 'out of memory in gvec'

 ! These quantities are used to allocate valence and conductions states in case of gwpara==2  
 ! nbvw : maximum number of fully and partially occupied states over spin 
 ! nbcw : maximum number of unoccupied states over spin 
 nbvw=maxval(ibocc)
 nbcw=sp%nb-nbvw

 write(*,*)'ibocc, nbvw, nbcw =',ibocc(:),nbvw,nbcw

 ! Initialize mpi_enreg according to gwpara 
 ! in case of k-point parallelism mpi_enreg is redefined on the fly for each q-point
 ! There is a problems somewhere in initmpi_gw
 if (dtset%gwpara==1) then 
   min_band_proc= 1 ; max_band_proc=sp%nb
   nbnds_per_proc=sp%nb
 else 
 call initmpi_gw(mpi_enreg,dtset,sp%nk,sp%nb,sp%nsppol,ibocc,nbvw,nbcw,&
& min_band_proc,max_band_proc,nbnds_per_proc)
 end if 

!MG IMPORTANT NOTE : now each proc has all the occupation numbers and occ 
!   memory allocated is not large and its easier to make code reusing

 ! Allocate KS electronic structure variables
 allocate(en(sp%nk,sp%nb,sp%nsppol),stat=istat)
  if(istat/=0) stop 'out of memory in en'
 allocate(occ(sp%nk,sp%nb,sp%nsppol),stat=istat)
  if(istat/=0) stop 'out of memory in occ'
 allocate(wfg(sp%npwwfn,min_band_proc:max_band_proc,sp%nk,sp%nsppol),stat=istat)
  if(istat/=0) call memerr('sigma','wfg',dtset%npwwfn*nbnds_per_proc*sp%nk*sp%nsppol,'spc')

 en(:,:,:)=zero ; occ(:,:,:)=zero
 wfg(:,:,:,:)=(0.,0.)

 ! Read in KS band structure for Sigma operator
 if (i2==0 .or. i2==1) then
  write(message,'(2a)')' sigma: will call rdlda ',ch10
  call wrtout(6,message,'COLL')
  if (sp%nsppol==2) then
   write(message,'(4a)')ch10,&
&   ' sigma: ERROR- ',ch10,&
&   '  nsppol=2 and QPLDA file format are not compatible'
   call wrtout(6,message,'COLL') ; call leave_new('COLL')
  end if
  ! WARNING non-symmorphic ops are not treated, band parallelism doesnt work
  call rdlda(iunit,nop,sp%nb,sp%nk,npwvec,sp%npwwfn,nopr,nbr,sp%nk,ngr,&
&  title,i1,i2,i3,a1,a2,a3,op,gvec,kibz,en,occ,wfg)
 else if (i2==2) then
  write(message,'(2a)')' sigma: will call rdldaabinit ',ch10
  call wrtout(6,message,'COLL')
  if (sp%nsppol==2) then
   write(message,'(4a)')ch10,&
&   ' sigma: ERROR- ',ch10,&
&   '  nsppol=2 and old wf file format are not compatible'
   call wrtout(6,message,'COLL') ; call leave_new('COLL')
  end if
  ! WARNING non-symmorphic ops are not treated, band parallelism doesnt work
  call rdldaabinit(iunit,nop,sp%nb,sp%nk,npwvec,sp%npwwfn,&
&  title,a1,a2,a3,op,gvec,kibz,en,occ,wfg)
 else if (i2==502 .or. i2==602) then
  allocate(typat(natom),xred(3,natom))
  allocate(tnons(3,nop),stat=istat)
  ! Kleynmann-Bylander sign and matrix elements
  allocate(vkbsign(mpsang,ntypat))
  allocate(vkb(sp%npwwfn,ntypat,mpsang,sp%nk),stat=istat)
   if(istat/=0) call memerr('sigma','vkb',sp%npwwfn*ntypat*mpsang*sp%nk,'spc')
  allocate(vkbd(sp%npwwfn,ntypat,mpsang,sp%nk),stat=istat)
   if(istat/=0) call memerr('sigma','vkbd',sp%npwwfn*ntypat*mpsang*sp%nk,'spc')
   !
   ! Read KSS file
   !
  call rdkss(dtfil,hdr,nop,sp%nb,nbvw,sp%nk,npwvec,sp%nsppol,sp%npwwfn,title,a1,a2,a3,op,&
&  gvec,kibz,en,occ,wfg,ntypat,natom,mpsang,prtvol,typat,tnons,xred,vkbsign,vkb,vkbd,nel,&
&  mpi_enreg,localrdwf,min_band_proc,max_band_proc)
  deallocate(typat,xred,vkbsign,vkb,vkbd)
 end if
 ! Check whether the direct lattice defined in the input file agrees with that read in the KSS file
 if ( (any(abs(a1-rprimd(:,1))>tol6)) .or. &
      (any(abs(a2-rprimd(:,2))>tol6)) .or. &
      (any(abs(a3-rprimd(:,3))>tol6)) ) then 
  write(message,'(6a)')ch10,&
&  ' sigma : ERROR - ',ch10,&
&  ' direct lattice vectors read from the KSS file ',ch10,&
&  ' differ from the values specified in the input file'
  call wrtout(6,message,'COLL')
  write(message,'(3a,3(3es16.6),3a,3(3es16.6),3a)')ch10,&
&  ' rprimd from KSS file = ',ch10,(rprimd(:,jj),jj=1,3),ch10,&
&  ' rprimd from input file = ',ch10,a1,a2,a3,ch10,ch10,&
&  ' Please, modify the lattice vectors in the input file '
  call wrtout(6,message,'COLL') ; call leave_new('COLL')
 end if 

 call pclock(10)

 ! Read screening file (\tilde epsilon^{-1}) 
 call testscr(dtfil,ep%nq,ep%nomega,ep%npwe,ep%npwwfn,ep%nb,titem1,fform,mpi_enreg,localrdwf,hdr_scr)

 sp%nq=ep%nq
 er%nq=ep%nq
 sp%npwc=ep%npwe
 er%nomega=ep%nomega

 !MG I have added a warning but still I cannot understand this point 
 if (sp%npwc>sp%npwx) then 
  sp%npwc=sp%npwx
  write(message,'(5a)')ch10,&
&  ' sigma : WARNING - ',ch10,&
&  '  found npwc > npwx, imposing npwc=npwx',ch10
  call wrtout(6,message,'COLL')
 end if 

 er%npwe=sp%npwc
 allocate(qpoint(3,sp%nq))
 nslice_qpt=sp%nq

 if (fform==1002) then
  allocate(er%omega(ep%nomega))
  ! In case of numerical integration this array might be huge ~Gb, 
  ! if required by the user we allocate a slice for a single q-point inside csigme 
  !FIXME using userie, shoud become an input variable
  if (dtset%userie/=1) then 
   allocate(er%epsm1(sp%npwc,sp%npwc,ep%nomega,sp%nq),stat=istat)
    if(istat/=0) call memerr('sigma','er%epsm1',sp%npwc*sp%npwc*ep%nomega*sp%nq,'spc')
   verbose=.true.
   call rdscr(dtfil,sp%npwc,sp%nq,sp%nq,ep%nomega,qpoint,er%omega,er%epsm1,mpi_enreg,localrdwf,verbose)
  else 
   write(message,'(2a)')ch10,&
&   ' sigma : userie was 1, allocating single slice of screening (slower but less memory)' 
   !this is to avoid the call to cppm.., should be done in a cleaner way but I have to check all the code
   sp%ppmodel=0 
   nslice_qpt=1 !for the moment, later we can try to increase this value
   verbose=.true.
   allocate(er%epsm1(1,1,ep%nomega,sp%nq),stat=istat)
   call rdscr(dtfil,1,sp%nq,sp%nq,ep%nomega,qpoint,er%omega,er%epsm1,mpi_enreg,localrdwf,verbose)
   deallocate(er%epsm1)
  end if 

  if (ep%nomega==2) then
   ep%nomegaer= 1 ; ep%nomegaei= 1
  else
   ep%nomegaer= 1
   do io=1,ep%nomega
    !Note that this coding works only if the first frequencies are real,
    if (real(er%omega(io))>0.001*Ha_eV) ep%nomegaer= io
   end do
   ep%nomegaei= ep%nomega-ep%nomegaer
  end if

 else if (fform==1003) then
  write(message,'(3a)')&
&  ' sigma: ERROR- ',ch10,&
&  '  em1 file format not any more compatible'
  call wrtout(6,message,'COLL') ; call leave_new('COLL')
 else
  write(message,'(3a)')&
&  ' sigma : ERROR -',ch10,&
&  '  unknown epsilon^-1 file format'
  call wrtout(6,message,'COLL') ; call leave_new('COLL')
 end if !of if fform

 write(message,'(a,2(a,i4),a)')ch10,&
& ' nomegaer= ',ep%nomegaer,' nomegaei= ',ep%nomegaei,ch10
 call wrtout(6,message,'COLL')

!TODO Here we can check that the frequencies are suitable for AC or CD 

 ! Calculate phases associated to non-symmorphic operations required to symmetrize $\tilde\epsilon^{-1}$
 ! phsgt= exp{-i2\pi SG\cdot t}
 allocate(phsgt(sp%npwc,nop),stat=istat)
  if(istat/=0) call memerr('sigma','phsgt',sp%npwc*nop,'dpc')
 do ig=1,sp%npwc 
  do iop=1,nop
   sg(:)= matmul(op(:,:,iop),gvec(:,ig))
   phsgt(ig,iop)= exp(-(0.,1.) *two_pi*dot_product(sg,tnons(:,iop)))
  end do
 end do

 call pclock(70)

 ! Calculate b1, b2, b3 and ucvol, bzvol
 ! Here, unlike the main abinit code, the reciprocal vectors are defined as a_i \cdot b_j = 2pi delta_ij
 ! TODO use abinit convention throughout the code
 call lattice(a1,a2,a3,b1,b2,b3,ucvol,bzvol)

 ! Set real space mesh :
 !  enforce_sym==1 enforce a FFT mesh compatible with all the symmetry operation
 !  method==0 FFT grid read from fft.in (debugging purpose)
 !  method==1 normal FFT grid 
 !  method==2 slightly augmented FFT grid  (see setmesh)
 !  method==3 augmented FFT grid 

 enforce_sym= mod(dtset%fftgw,10) 
 if (dtset%fftgw==00 .or. dtset%fftgw==01) method=0
 if (dtset%fftgw==10 .or. dtset%fftgw==11) method=1
 if (dtset%fftgw==20 .or. dtset%fftgw==21) method=2
 if (dtset%fftgw==30 .or. dtset%fftgw==31) method=3
 addshell=2
 aliasing_factor=1 !this is used only in case of method 0 or 1  
 ! TODO setmesh should fill the ngfft array that should be passed to all the other subroutines
 call setmesh(gmet,gvec,ngfft1,ngfft2,ngfft3,ngfft1a,ngfft2a,ngfft3a,npwvec,max(sp%npwx,sp%npwc),sp%npwwfn,nr,&
& method,addshell,aliasing_factor,nop,dtset%symrel,tnons,enforce_sym,1)

 nrb=ngfft1*ngfft2*ngfft3

 ! Set up tables for FFT igfft
 allocate(igfft(npwvec,5,5,5),stat=istat)
  if(istat/=0) call memerr('sigma','igfft',5*5*5*npwvec,'i4b')
 call cigfft(npwvec,ngfft1a,ngfft1,ngfft2,ngfft3,gvec,igfft)
 
 call pclock(20)

 call timab(402,2,tsec)
 call timab(403,1,tsec)

 ! Calculate KS wavefunctions in real space using FFT
 allocate(wfr(nr,min_band_proc:max_band_proc,sp%nk,sp%nsppol),stat=istat)
  if(istat/=0) call memerr('sigma','wfr',nr*nbnds_per_proc*sp%nk*sp%nsppol,'spc')
 wfr(:,:,:,:)=(0.,0.)
 tim_fourdp= 5
 call fftwfn(sp%npwwfn,min_band_proc,max_band_proc,sp%nk,nr,sp%nsppol,wfg,wfr,&
& igfft(:,3,3,3),ngfft1,ngfft1a,ngfft2,ngfft3,tim_fourdp,mpi_enreg)

 call pclock(30)
 call timab(403,2,tsec)
 call timab(404,1,tsec)

 ! Set up table indicating rotations of r-points
 allocate(irottb(nr,nop),stat=istat)
  if(istat/=0) call memerr('screening','irottb',nr*nop,'i4b')
 allocate(grottb(npwvec,2,nop),grottbm1(npwvec,2,nop),stat=istat)
  if(istat/=0) call memerr('screening','grottb/grottbm1 ',2*(2*npwvec*nop),'i4b')

 irottb(:,:)=0
 grottb(:,:,:)=0
 grottbm1(:,:,:)=0

 call surot(op,tnons,nop,ninv,ngfft1,ngfft1a,ngfft2,ngfft3,nr,npwvec,gvec,grottb,irottb,grottbm1)
 call pclock(40)

 ! Set up required k-points in whole BZ; nkbzx maximum number of them
 nkbzx=sp%nk*nop*ninv

 allocate(kbz(3,nkbzx),wtk(sp%nk),stat=istat)
 allocate(ktab(nkbzx),ktabi(nkbzx),ktabo(nkbzx),ktabr(nr,nkbzx),stat=istat)
  if(istat/=0) stop ' out of memory in k-points tables'

 call identk(kibz,sp%nk,nkbzx,nr,nop,ninv,irottb,op,kbz,ktab,ktabr,ktabi,ktabo,sp%nkbz,wtk)

 call pclock(50)

 ! Calculate phase factors for non-symmorphic operations, required to symmetrize oscillator matrix elements
 ! u_{Sk}(r)=e^{-i 2 \pi k\cdot R{^-1}t} u_k (R^{-1}(r-t))
 ! Note that time reversal symmetry is taken into account inside rho_tw_g
 allocate(ktabp(sp%nkbz),stat=istat)
 do ik=1,sp%nkbz
  rm1t= matmul(transpose(op(:,:,ktabo(ik))),tnons(:,ktabo(ik)))
  ktabp(ik)= exp(-(0.,1.) * two_pi * dot_product(kibz(:,ktab(ik)),rm1t))
 end do  

 ! For each spin nbv gives the valence band index
 allocate(nbv(sp%nsppol))

 ! dtset%fixmom is passed to fermi.F90 to fix the problem with newocc in case of magnetic metals
 call fermi(hdr,sp%nb,sp%nk,dtset%fixmom,sp%nsppol,wtk,en,occ,nel,nbv,efermi)

 ! TODO this part should be moved at the beginning
 if (dtset%nkptgw==0) then
  sp%nkcalc= sp%nk
  allocate(sp%xkcalc(3,sp%nkcalc),sp%kcalc(sp%nkcalc))
  allocate(sp%minbnd(sp%nkcalc),sp%maxbnd(sp%nkcalc))
  sp%xkcalc(:,:)= kibz(:,:)
  sp%minbnd(:)= 1
  sp%minbdgw= 1 ; b1gw= sp%minbdgw
  ! Use maximum value over spins
  sp%maxbnd(:)= maxval(nbv)*2
  if (maxval(nbv)*2>sp%nb) sp%maxbnd(:)= sp%nb
  sp%maxbdgw= maxval(sp%maxbnd) ; b2gw= sp%maxbdgw
  !Note that for SCGW on WF we have to allocate the entire matrix
  !the problem arises in ham_changebasis, maybe we can use a smaller number!
  if (dtset%gwpara==0 .or. dtset%gwpara==1) then
   b1gw=1
   b2gw= sp%nb
  end if
  sizebdgw=b2gw-b1gw+1
 else
  sp%nkcalc=dtset%nkptgw
  allocate(sp%xkcalc(3,sp%nkcalc),sp%kcalc(sp%nkcalc))
  allocate(sp%minbnd(sp%nkcalc),sp%maxbnd(sp%nkcalc))

  sp%xkcalc(:,:)=dtset%kptgw(:,:)
  sp%minbnd(:)=dtset%bdgw(1,:)
  sp%minbdgw= minval(sp%minbnd) ; b1gw= sp%minbdgw
  sp%maxbnd(:)=dtset%bdgw(2,:)
  sp%maxbdgw= maxval(sp%maxbnd) ; b2gw= sp%maxbdgw
  !Note that for SCGW on WF we have to allocate the entire matrix
  !the problem arises in ham_changebasis, maybe we can use a smaller number!
  if (dtset%gwpara==0 .or. dtset%gwpara==1) then
   b1gw=1
   b2gw= sp%nb
  end if
  sizebdgw=b2gw-b1gw+1
  if (any(sp%maxbnd(:)>sp%nb)) then
   write(message,'(5a)')&
&   ' sigma : ERROR - ',ch10,&
&   '  At least one band where the GW corrections are required exceeds the number of treated bands.',ch10, &
&   '  Increase the number of bands in the input file (or in the KSS file) '
   call wrtout(6,message,'COLL') ; call leave_new('COLL')
  end if
 end if

!DEBUG
!write(*,*)'using b1gw and b2gw',b1gw,b2gw
!ENDDEBUG

 !MG if option==1 two kpoints are considered equivalent if they differ by a
 !reciprocal G vector, actually option is 0, but should be changed FIXME
 !verbose (last argument set to 0)
 call findk(sp%nkcalc,sp%nkbz,sp%xkcalc,kbz,sp%kcalc,0,1)
 call pclock(60)

 if (mpi_enreg%gwpara==2) then 
  ! First coding, communication and memory could be optimized (what about pointers!)
  allocate(wfr_gw(nr,b1gw:b2gw,sp%nk,sp%nsppol),stat=istat)
   if (istat/=0) call memerr('sigma','wfr_gw',nr*sizebdgw*sp%nkcalc*sp%nsppol,'spc')
  wfr_gw=(0.,0.)
  write(message,'(2a)')ch10,' communicating wavefunctions between processors'
  call wrtout(6,message,'COLL')
  do ikibz=1,sp%nk
   do ib=b1gw,b2gw
    do is=1,sp%nsppol
     ikb_proc_rank=mpi_enreg%proc_distrb(ikibz,ib,is)
     if (me==ikb_proc_rank) wfr_gw(:,ib,ikibz,is)= wfr(:,ib,ikibz,is)
     call xcast_mpi(wfr_gw,ikb_proc_rank,spaceComm,ierr)
    end do
   end do 
  end do
  call leave_test(mpi_enreg)
 end if 

 ! Calculate the density
 allocate(rho(nr,sp%nsppol),stat=istat)
  if(istat/=0) stop 'out of memory'

! If nsppol==2, crho reports the total charge in the first half and the spin up charge density in the second half
! If the  wavefunctions are not in memory we can use density.F90 presently the wfr are supposed to be in memory
!MG FIXME crho should be substituted by density since it is also possible to read wfr from file 
!   Note that I found small diff btw the density calculated in screening and sigma, should check better 
!   but probably its due to parallelism or possible BUG
!   also the density calculated using abinis and abinip is sligthly different!! 
 call crho(irottb,sp%nb,ninv,sp%nkbz,sp%nk,sp%nk,nop,nr,nrb,sp%nsppol,occ,omegaplasma,rho,ucvol,wfr,wtk,&
& mpi_enreg,min_band_proc,max_band_proc)

! call density(sp%nb,sp%nk,sp%nkbz,sp%nsppol,nop,ninv,nr,ngfft1,ngfft1a,ngfft2,ngfft3,&
!&   irottb,ucvol,wtk,occ,rho,omegaplasma,mpi_enreg,1,sp%nb,localrdwf,&
!&   wfr) !optional

!MG now we calculate only the matrix elements that are really required for the GW
!   using b1gw b2gw 

 ! vxc(b1,b2,k,s) = <b1,k,s| v_{xc} | b2,k,s>
 ! Note that these matrix elements are calculated without non linear core correction
 allocate(vxc(b1gw:b2gw,b1gw:b2gw,sp%nk,sp%nsppol),stat=istat)
  if (istat/=0) call memerr('sigma','vxc',sizebdgw**2*sp%nk*sp%nsppol,'spc')

 vxc(:,:,:,:)=(0.,0.)

!FIXME BUG IN PARALLEL VERSION
!MG here there is a problem since each processor is calculating only the matrix
!elements related to the sub-block stored in its memory should add the possibility to read from file
!anyway the diagonal matrix elements are correct
!the solution I propose consists in broadcasting the wavefunctions where QP corrections are 
!required before entering csigme, usually the number of states for QP is not so large
!this would allow to calculate the matrix elements required, however ths method requires 
!modifications in most of the arrays used in csigme to store the matrix elements of the KS hamiltonian
!Another possibility could be to calculate these quantities during the generation of the KSS file
! TODO write a sub to calculate only the diagonal elements, since in that case band parallelism can be used
 if (mpi_enreg%gwpara==2) then
  call cmevxclda(dtset,dtset%ixc,mpi_enreg,sp%nb,b1gw,b2gw,ngfft1,ngfft2,ngfft3,sp%nk,&
&  nr,nrb,sp%nsppol,rho,rprimd,wfr_gw,vxc,b1gw,b2gw)
 else 
  call cmevxclda(dtset,dtset%ixc,mpi_enreg,sp%nb,b1gw,b2gw,ngfft1,ngfft2,ngfft3,sp%nk,&
&  nr,nrb,sp%nsppol,rho,rprimd,wfr,vxc,min_band_proc,max_band_proc)
 end if

 if (dtset%gwcalctyp>=20) then 
  !
  ! Self-consistency both on eigenvalues and eigenfunctions
  !
  allocate(vhartr(b1gw:b2gw,b1gw:b2gw,sp%nk,sp%nsppol),stat=istat)
   if (istat/=0) call memerr('sigma','vhartr',sizebdgw**2*sp%nk*sp%nsppol,'spc')
  write(message,'(2a)')ch10,' *************** KS Energies *******************'
  call wrtout(6,message,'COLL')
!FIXME BUG IN PARALLEL VERSION
!MG here there is a problem since each processor is calculating only the matrix
!elements related to the sub-block stored in its memory
  if (mpi_enreg%gwpara==2) then
   write(*,*)'not coded yet'
   call leave_new('COLL')
  end if 
  ! Calculate matrix elements of the Hartree potential
  call hartrham(dtset,sp,gmet,ucvol,ngfft1,ngfft2,ngfft3,nr,npwvec,gvec,kibz,wtk,&
&  sp%nsppol,wfr,wfg,rho(:,1),occ,en,b1gw,b2gw,vhartr,mpi_enreg,min_band_proc,max_band_proc)
 end if !of gwcalctyp>=20
 ! Set up q-points in whole BZ
 nqbzx=sp%nq*nop*ninv
 allocate(qbz(3,nqbzx),qtab(nqbzx),qtabo(nqbzx),qtabi(nqbzx),stat=istat)
  if(istat/=0) stop 'out of memory in q-points tables'
 qbz(:,:)=0
 qtab(:) =0 ; qtabo(:)=0 ; qtabi(:)=0
 !MG here there is a problem since if an umklapp G_o vector is required 
 !   then during the reconstruction of \tilde\espilon^{-1} we have to calculate G-G_o see csigme.F90
 call identq(qpoint,sp%nq,nqbzx,op,nop,ninv,qbz,qtab,qtabi,qtabo,sp%nqbz)

 ! Stop if G_0 is not zero, not yet implemented
 do iq=1,sp%nqbz
  call dosym(op(:,:,qtabo(iq)),(3-qtabi(iq))/2,qpoint(:,qtab(iq)),sq(:))
  if (any(abs(qbz(:,iq)- sq(:) )>1.0d-4)) then
   write(*,*)' sigma: ERROR '
   write(*,*)' qpoint ',qbz(:,iq)
   write(*,*)' is the symmetric of ',qpoint(:,qtab(iq))
   write(*,*)' through operation ',op(:,:,qtabo(iq))
   write(*,*)' however a non zero umklapp G vector is required and this is not yet allowed'
   write(*,*)sq,qbz(:,iq)
   call leave_new('COLL')
  end if
 end do 

 call pclock(80)

 ! Calculate |q+G| for all q in IBZ and G
 ! Here we should add the treatment of the cutoff in the coulombian
 allocate(qpg(sp%npwx,sp%nq),stat=istat)
  if(istat/=0) stop 'out of memory in qpg'
 !MG FIXME qratio is only used for ppmodel==2
 !         qratio is evaluated in clcqpg even though ppmodel\=2
 !   Maybe here we can save a bit of memory
 allocate(qratio(sp%npwc,sp%npwc,sp%nq),stat=istat)
  if(istat/=0) call memerr('sigma','qratio',sp%npwc*sp%npwc*sp%nq,'dp')

 call clcqpg(sp%npwx,sp%npwc,gvec,qpoint,sp%nq,b1,b2,b3,qpg,qratio)

!DEBUG CUTOFF
!allocate (vccut(sp%nq,sp%npwx))
!cutp%cutoffmode='surface'
!cutp%rcut=10
!cutp%boxcenter=zero
!call cutoff_driver(sp%nq,qpoint,sp%npwx,gvec,cutp,gmet,gprimd,vccut)
!ENDDEBUG CUTOFF

 call pclock(90)

 if (sp%ppmodel==1 .or. sp%ppmodel==2) then
  ppmsize=sp%npwc**2*sp%nq+sp%npwc**2*sp%nq
  allocate(er%bigomegatwsq(sp%npwc,sp%npwc,sp%nq),er%omegatw(sp%npwc,sp%npwc,sp%nq),stat=istat)
 else if (sp%ppmodel==3) then
  ppmsize=sp%npwc*sp%nq+sp%npwc*sp%nq
  allocate(er%bigomegatwsq(sp%npwc,1,sp%nq),er%omegatw(sp%npwc,1,sp%nq),stat=istat)
  allocate(er%eigpot(sp%npwc,sp%npwc,sp%nq),stat=istat)
   if(istat/=0) call memerr('sigma','sr%eigpot',sp%npwc**2*sp%nq,'spc')
 else if (sp%ppmodel==4) then
  ppmsize=sp%npwc**2*sp%nq+sp%npwc*sp%nq
  allocate(er%bigomegatwsq(sp%npwc,sp%npwc,sp%nq),er%omegatw(sp%npwc,1,sp%nq),stat=istat)
 end if
  if (istat/=0) call memerr('sigma','bigomegatwsq/omegatw',ppmsize,'spc')

  ! added by Rshaltaf for the vertex correction inclusion
 if(dtset%gwgamma==1)then
  call eps1_tc(dtset,mpi_enreg,ngfft1,ngfft2,ngfft3,nr,&
&  rho,rprimd,igfft(1:sp%npwc,3,3,3),sp%npwc,gmet,gvec,ep%nomega,&
&  er%epsm1,sp%nq,ucvol,qpoint,er%omega,b1,b2,b3,dtfil,hdr,ep%npwwfn,npwvec,ep%nb)
 end if

 ! if nfreqmidm == -1000, the default, the frequency moment will not be calculated

   if (dtset%nfreqmidm/=-1000)then
     if (dtset%nfreqmidm<0)then
      write(message,'(a,i4,a,a)')&
&      'Calculating the ',dtset%nfreqmidm,'th frequency moment of imaginary part of inverse DM',ch10
     else if (dtset%nfreqmidm>0)then
      write(message,'(a,i4,a,a)')&
&      'Calculating the ',dtset%nfreqmidm,'th frequency moment of imaginary part of DM',ch10
     else
      write(message,'(a,i4,a,a)')&
&      'Calculating the ',dtset%nfreqmidm,&
&      'th frequency moment of imaginary part of full polarizibility',ch10
     call wrtout(6,message,'COLL')
    end if
    call calc_ffm(er%epsm1,sp%nq,sp%npwc,ep%nomega,er%omega,&
&    b1,b2,b3,qpoint,dtset%ppmodel,gvec,dtset%nfreqmidm)
   end if
       
  ! Calculate plasmonpole model parameters
 if (sp%ppmodel==1) then
  call cppm1par(sp%npwc,sp%nq,ep%nomega,er%epsm1,er%omega,er%bigomegatwsq,er%omegatw,omegaplasma)
 else if (sp%ppmodel==2 .or. sp%ppmodel==3 .or. sp%ppmodel==4 ) then 
  if (use_augmented) then 
   write(message,'(3a)')ch10,' sigma : using augmented FFT grid to calculate the density ',ch10
   call wrtout(6,message,'COLL')
   !FIXME Should skip this part if we are still using an augmented FFT in GW  
   !every processor should do this but everyone must have at least the occupied states
   !here we should use method 2 with a different value for npweps 
   !since the radius of the sphere for the density is 2*r_eps, for the moment:
   method= 3 ; addshell= 0
   !aliasing_factor=2
   !enforce_sym=0 !dont enforce symmetric FFT
   call setmesh(gmet,gvec,ngfft1d,ngfft2d,ngfft3d,ngfft1dd,ngfft2dd,ngfft3dd,npwvec,sp%npwc,sp%npwwfn,nrd,&
&   method,addshell,aliasing_factor,nop,dtset%symrel,tnons,enforce_sym,0)
   allocate(rhod(nrd,sp%nsppol),stat=istat) !density on the augmented FFT grid
   nbnds_occ=sp%nb !this is to test, should be the number of occupied states
   !
   ! Calculate rho using augmented FFT mesh
   !
   call crho_augmented(gmet,gvec,npwvec,sp%npwwfn,sp%npwc,nbnds_occ,ninv,nop,op,tnons,&
&   sp%nk,sp%nkbz,sp%nsppol,wfg,ngfft1d,ngfft2d,ngfft3d,nrd,ngfft1dd,ngfft2dd,ngfft3dd,&
&   occ,wtk,ucvol,mpi_enreg,min_band_proc,max_band_proc,rhod)
   !FIXME only to be clearer and avoid possible bugs we should pass also the frequencies to all these subs
   !morevover is seems that ppmodel is set to 1 as default
   if (sp%ppmodel==2) then
    call cppm2par(sp%npwc,sp%nq,ep%nomega,er%epsm1,er%bigomegatwsq, &
&    er%omegatw,ngfft1d,ngfft2d,ngfft3d,gvec,qratio,rhod(:,1),nrd,qpoint,b1,b2,b3)
   else if (sp%ppmodel==3) then
    call cppm3par(sp%npwc,sp%nq,ep%nomega,er%epsm1,er%bigomegatwsq, &
&    er%omegatw,ngfft1d,ngfft2d,ngfft3d,gvec,rhod(:,1),nrd,er%eigpot,qpoint,b1,b2,b3)
   else if (sp%ppmodel==4) then
    call cppm4par(sp%npwc,sp%nq,er%epsm1,ep%nomega,er%bigomegatwsq, &
&    er%omegatw,ngfft1d,ngfft2d,ngfft3d,gvec,rhod(:,1),nrd,qpoint,b1,b2,b3)
   end if 
   deallocate(rhod)
  else 
   !
   ! Calculate rho using normal FFT grid
   !
   if (sp%ppmodel==2) then
    !only to be clearer and avoid possible bugs we should pass also the frequencies to all these subs
    call cppm2par(sp%npwc,sp%nq,ep%nomega,er%epsm1,er%bigomegatwsq, &
&    er%omegatw,ngfft1,ngfft2,ngfft3,gvec,qratio,rho(:,1),nr,qpoint,b1,b2,b3)
   else if (sp%ppmodel==3) then
    call cppm3par(sp%npwc,sp%nq,ep%nomega,er%epsm1,er%bigomegatwsq, &
&    er%omegatw,ngfft1,ngfft2,ngfft3,gvec,rho(:,1),nr,er%eigpot,qpoint,b1,b2,b3)
   else if (sp%ppmodel==4) then
    call cppm4par(sp%npwc,sp%nq,er%epsm1,ep%nomega,er%bigomegatwsq, &
&    er%omegatw,ngfft1,ngfft2,ngfft3,gvec,rho(:,1),nr,qpoint,b1,b2,b3)
   end if 
  end if !normal or augmented FFT
 else if (sp%ppmodel/=0) then  
  write(message,'(5a)')ch10,&
&  ' sigma: BUG -',ch10,&
&  '  ppmodel can only be 0, 1, 2, 3 or 4 ',ch10
  call wrtout(6,message,'COLL') ; call leave_new('COLL')
 end if

! Write plasmon pole parameters on file if request (only master works)
! FIXME Anyway ppmodel==1 also if AC or CD is required
 if (me==0 .and. sp%ppmodel>0 .and. dtset%prtpmp/=0) then 
  if (sp%ppmodel==1 .or. sp%ppmodel==2) then 
   npwc2= sp%npwc ; npwc3= sp%npwc
  else if (sp%ppmodel==3) then
   npwc2= 1       ; npwc3= 1
  else if (sp%ppmodel==4) then 
   npwc2= sp%npwc ; npwc3= 1
  end if 
! NOTE header is from SCR file, I noted that symrel are shuffled wrt operations in the KSS 
! eigpot should be optional
! other informations like for example omegaplasma, nband used for esp, npwwfn for wfn and so on
  call wrppm(dtfil,hdr_scr,dtset%ppmodel,sp%npwc,npwc2,npwc3,gvec,sp%nq,qpoint,er%bigomegatwsq,er%omegatw,er%eigpot)
 end if 

 sr%nk=sp%nk ; sr%nb=sp%nb
 sr%nomega=sp%nomegasr
 
 if (me==0 .or. localrdwf==1) call hdr_clean(hdr_scr)
 ! FIXME qratio should be used only in the routines dealing with PPM
 deallocate(qratio)

!******************************************************************************************* 

 if (dtset%gwcalctyp<10) then 
  ! In case of one shot calculation copy the KS density, KS energies, KS occ numbers and 
  ! the Fermi level in arrays that will be passed to csigme, 
  ! keep in mind however that these are KS quantities!
  ! previously this was done inside rdqps, this new coding will speed up this part
  ! since we are skipping the double call to call_wf_qp!
  allocate(en_qp(sp%nk,sp%nb,sp%nsppol),stat=istat)
   if (istat/=0) call memerr('sigma','en_qp',sp%nk*sp%nb*sp%nsppol,'dp')
  allocate(occ_qp(sp%nk,sp%nb,sp%nsppol))
  allocate(rho_qp(nr,sp%nsppol))
  occ_qp(:,:,:)=occ(:,:,:) !this array will be passed to cisgme
  !MG FIXME this is only to pass the automatic tests!, should be removed
  call crho(irottb,sp%nb,ninv,sp%nkbz,sp%nk,sp%nk,nop,nr,nrb,sp%nsppol,occ_qp,omegaplasma,rho_qp,ucvol,wfr,wtk,&
&  mpi_enreg,min_band_proc,max_band_proc)
  !ENDMG
  en_qp(:,:,:)=en(:,:,:)
  rho_qp(:,:)=rho(:,:) !this arrays will be passed to hartrham
  efermi_qp=efermi
  deallocate(rho)
  deallocate(occ)
 else 
  !
  ! Read QP wavefunctions of the previous step (FBruneval051215)
  ! Note that each proc knows the full matrices
  !
  allocate(m_lda_to_qp(sp%nb,sp%nb,sp%nk,sp%nsppol),stat=istat)
   if (istat/=0) call memerr('sigma','m_lda_to_qp',sp%nb**2*sr%nk*sp%nsppol,'spc')
  m_lda_to_qp(:,:,:,:)= (0.,0.)
  allocate(en_qp(sp%nk,sp%nb,sp%nsppol),stat=istat)
   if (istat/=0) call memerr('sigma','en_qp',sp%nk*sp%nb*sp%nsppol,'dp')
  allocate(rho_p(nr,nsppol))
  rho_p(:,:)= rho(:,:)
  ! Now each processor reads the data for all the bands
  ! Read self-consistent transformation matrices and QP energies
  call rdqps(dtfil,sp%gwcalctyp,sr%nk,sp%nb,sp%nsppol,kibz,nscf,nr,en,&
&  nbsc,en_qp,m_lda_to_qp,rho_p)

! Compute the QP wfr and wfg using the KS representation (wfr and wfg are overwritten)
! Also here there are problems in case of parallel execution
! get rid off min_band and max_band_proc, check other indices
!FIXME HERE I HAVE TO THINK on how to parallelize this subroutine in the gwpara==2 case 

  call calc_wf_qp(sp%nk,nbnds_per_proc,nr          ,sp%nsppol,m_lda_to_qp,wfr,min_band_proc,max_band_proc)
  call calc_wf_qp(sp%nk,nbnds_per_proc,dtset%npwwfn,sp%nsppol,m_lda_to_qp,wfg,min_band_proc,max_band_proc)
  ! Compute QP occupation numbers
  allocate(occ_qp(sp%nk,sp%nb,sp%nsppol))
  occ_qp(:,:,:)= occ(:,:,:)
  write(message,'(3a)')ch10,' sigma : calculating QP occupation numbers ',ch10
  call wrtout(6,message,'COLL')
  call fermi(hdr,sp%nb,sp%nk,dtset%fixmom,sp%nsppol,wtk,en_qp,occ_qp,nel,nbv,efermi_qp)
  ! Compute QP density (total and the spin up charge density)
  allocate(rho_qp(nr,sp%nsppol))
  !MG FIXME crho should be substituted by density since it is also possible to read wfr from file 
  call crho(irottb,sp%nb,ninv,sp%nkbz,sp%nk,sp%nk,nop,nr,nrb,sp%nsppol,occ_qp,omegaplasma,rho_qp,ucvol,wfr,wtk,&
&  mpi_enreg,min_band_proc,max_band_proc)

  ! call density(sp%nb,sp%nk,sp%nkbz,sp%nsppol,nop,ninv,nr,ngfft1,ngfft1a,ngfft2,ngfft3,&
  !& irottb,ucvol,wtk,occ_qp,rho_qp,omegaplasma,mpi_enreg,1,sp%nb,localrdwf,&
  !& wfr) !optional

  ! Simple mixing of the densities in order to damp oscillations in the Hartree potential (FB061217)
  write(message,'(2a,f5.3,a)')ch10,&
&  ' sigma: mixing QP densities using rhoqpmix = ',dtset%rhoqpmix,ch10
  call wrtout(6,message,'COLL')
  rho_qp(:,:)= rho_p(:,:) + dtset%rhoqpmix*(rho_qp(:,:)-rho_p(:,:))
  deallocate(rho_p)
! MG: Since in three plasmon pole models depend on the Fourier components of the density
! I think that in case of self-consistency we should calculate here the ppm coefficients using rho_qp
! We can introduce a logical flag to avoid the calculation of the ppm parameters
! with the KS density and calculate here the coefficients
 end if

 deallocate(irottb)

 ! Setup the Hartree hamiltonian (FBruneval051215)

 ! Setup KS hamiltonian hlda(b1,b2,k,s)= <b1,k,s| H_s | b2,k,s>
 allocate(hlda(b1gw:b2gw,b1gw:b2gw,sr%nk,sp%nsppol),stat=istat)
  if (istat/=0) call memerr('sigma','hlda',sizebdgw**2*sr%nk*sp%nsppol,'spc')
 hlda(:,:,:,:)= (0.,0.)
 !
 ! Calculate hlda(b1,b1,k,s)= <b1,k,s| H_s | b1,k,s>
 !
 do is=1,sp%nsppol
  do ik=1,sp%nk
   do ib=b1gw,b2gw
    hlda(ib,ib,ik,is)= en(ik,ib,is)
   end do !ib
  end do !ik
 end do !is
 !
 ! Calculate hhartree(b1,b2,k,s)= <b1,k,s| T + v_{loc} + v_{nl} + v_{H}^s  | b2,k,s>
 !
 allocate(sr%hhartree(b1gw:b2gw,b1gw:b2gw,sr%nk,sp%nsppol)) 
 if (dtset%gwcalctyp<20) then
  ! Self-consistency only on energies 
  ! Calculate the KS Hartree Hamiltonian $ T + v_{loc} + v_{nl} + v_{H}$ in the KS representation
  !FIXME if parallelism is over bands these quantities are wrong, except for the diagonal terms, see cmevxclda
  sr%hhartree(:,:,:,:)= hlda(:,:,:,:) - vxc(:,:,:,:)
 else

! Self-consistency both on energies and wavefunctions
! Calculate the bare Hamiltonian  $ H_{bare} = T + v_{loc} + v_{nl}$ in the KS representation
  allocate(hbare(b1gw:b2gw,b1gw:b2gw,sr%nk,sp%nsppol),stat=istat)
   if (istat/=0) call memerr('sigma','hbare',sizebdgw**2*sr%nk*sp%nsppol,'spc')
  !FIXME if parallelism is over bands these quantities are wrong, see cmevxclda
  if (mpi_enreg%gwpara==2 .and. dtset%gwcalctyp>=10) then
   write(*,*)'not coded yet' ; call leave_new('COLL')
  end if 

  hbare(:,:,:,:)= hlda(:,:,:,:) - vhartr(:,:,:,:) - vxc(:,:,:,:)

  ! Change the basis from KS to QP
  ! FIXME here I have to introduce b1gw b2gw 
  call ham_changebasis(sr%nb,sr%nk,sp%nsppol,m_lda_to_qp,hbare)

  ! Calculate the QP Hartree potential
  write(message,'(2a)')ch10,' *************** QP Energies *******************'
  call wrtout(6,message,'COLL')

 ! FIXME Now each proc has all the bands, this subroutine must be modified 
 if (mpi_enreg%gwpara==2 .and. dtset%gwcalctyp>=10) then
  write(*,*)'not coded yet'
  call leave_new('COLL')
 end if 
  call hartrham(dtset,sp,gmet,ucvol,ngfft1,ngfft2,ngfft3,nr,npwvec,gvec,kibz,wtk,&
&  sp%nsppol,wfr,wfg,rho_qp(:,1),occ_qp,en_qp,b1gw,b2gw,vhartr,mpi_enreg,min_band_proc,max_band_proc)

  ! Obtain the QP Hartree Hamiltonian
  sr%hhartree(:,:,:,:)= hbare(:,:,:,:) + vhartr(:,:,:,:)
 end if !of dtset%gwcalctyp<20)

 deallocate(wfg)

 ! Prepare the storage of the new wavefunctions and new energies
 !MG Note that eigvec_qp now is stored by each proc
 allocate(sr%en_qp_diago(sp%nb,sr%nk,sp%nsppol))
 allocate(sr%eigvec_qp(sp%nb,sp%nb,sr%nk,sp%nsppol))

 sr%eigvec_qp(:,:,:,:)=(0.,0.)
 sr%en_qp_diago(:,:,:)=zero
 ! Initialize with KS wavefunctions and energies
 do ib=1,sp%nb
  sr%en_qp_diago(ib,:,:)= en(:,ib,:)
  sr%eigvec_qp(ib,ib,:,:)= (1.,0.)
 end do 

 if (allocated(vhartr)) deallocate(vhartr)
 if (allocated(hbare))  deallocate(hbare )
 if (allocated(hlda))   deallocate(hlda  )

 ! Store < nks | Vxc | nks >
 ! Here the matrix elements of vxc in the KS basis set are stored not the elements in the QP basis set!, why?
 allocate(sr%vxcme(b1gw:b2gw,sr%nk,sp%nsppol))
 do ib=b1gw,b2gw
  sr%vxcme(ib,:,:)= vxc(ib,ib,:,:)
 end do

 deallocate(vxc)

 call pclock(100)

!******************************************************************************************* 
 ! Allocate arrays in the sigma_results datatype  

 allocate(sr%sigxme (b1gw:b2gw,sr%nk,sp%nsppol),stat=istat)
 allocate(sr%sigcme (b1gw:b2gw,sr%nk,sr%nomega,sp%nsppol),stat=istat)
 allocate(sr%sigxcme(b1gw:b2gw,sr%nk,sr%nomega,sp%nsppol),stat=istat)
  if(istat/=0) stop 'out of memory sigxme/sigcme/sigxcme '
!MG from Olevano patch AC?
!allocate(sr%ame(sr%nb,sr%nk,sr%nomega),sr%ak(sr%nk,sr%nomega),stat=istat)
! if(istat/=0) stop 'out of memory ame'
 allocate(sr%sigcmee0(b1gw:b2gw,sr%nk,sp%nsppol),sr%ze0(b1gw:b2gw,sr%nk,sp%nsppol),stat=istat)
 allocate(sr%dsigmee0(b1gw:b2gw,sr%nk,sp%nsppol),sr%sigmee(b1gw:b2gw,sr%nk,sp%nsppol),stat=istat)
 allocate(sr%degw(b1gw:b2gw,sr%nk,sp%nsppol))
!MG note sp%nb
 allocate(sr%e0(sp%nb,sr%nk,sp%nsppol),sr%egw(sp%nb,sr%nk,sp%nsppol),stat=istat)
  if(istat/=0) stop 'out of memory e0,egw '

 allocate(sr%e0gap(sr%nk,sp%nsppol),sr%degwgap(sr%nk,sp%nsppol),sr%egwgap(sr%nk,sp%nsppol))

 sr%nomegasrd= sp%nomegasrd

! Used to evaluate $\Sigma(E)$ around the KS\QP eigenvalue
 allocate(sr%omegasrd  (b1gw:b2gw,sr%nk,sr%nomegasrd,sp%nsppol))
 allocate(sr%sigcmesrd (b1gw:b2gw,sr%nk,sr%nomegasrd,sp%nsppol))
 allocate(sr%sigxcmesrd(b1gw:b2gw,sr%nk,sr%nomegasrd,sp%nsppol))

 sr%e0(:,:,:)= zero
 sr%egw(:,:,:)= (zero,zero)
 sr%e0gap(:,:)= zero
 sr%sigcme(:,:,:,:)= (zero,zero)
 sr%sigxme(:,:,:)= (zero,zero)
 sr%sigxcme(:,:,:,:)= (zero,zero)
 sr%sigcmee0(:,:,:)= (zero,zero)
 sr%ze0(:,:,:)= (zero,zero)
 sr%dsigmee0(:,:,:)= (zero,zero)
 sr%sigmee(:,:,:)= (zero,zero)
 sr%omegasrd(:,:,:,:)= (zero,zero)
 sr%sigcmesrd(:,:,:,:)= (zero,zero)
 sr%sigxcmesrd(:,:,:,:)= (zero,zero)
 sr%degw(:,:,:)= (zero,zero)

 if (analytic_continuation) then 
  sr%nomegasi= sp%nomegasi
  !This is not spin dependent
  allocate(sr%omegasi(sr%nomegasi))
  allocate(sr%sigcmesi (b1gw:b2gw,sr%nk,sr%nomegasi,sp%nsppol))
  allocate(sr%sigxcmesi(b1gw:b2gw,sr%nk,sr%nomegasi,sp%nsppol))
  sr%omegasi= czero
  sr%sigcmesi= (0.,0.) 
  sr%sigxcmesi= (0.,0.)
 end if 

 do is=1,sp%nsppol
  do ik=1,sr%nk
!MG now each proc has all the bands
   do ib=1,sp%nb
    sr%e0 (ib,ik,is)= en_qp(ik,ib,is)
    sr%egw(ib,ik,is)= en_qp(ik,ib,is) !Initial guess for GW energies
   end do
!should add a check at the beginning on nbv
   sr%e0gap(ik,is)= sr%e0(nbv(is)+1,ik,is) - sr%e0(nbv(is),ik,is)
  end do
 end do

 ! Apply scissor operator if required
 ep%soenergy= dtset%soenergy
 if (ep%soenergy>0.1d-4) then
  write(message,'(5a,f10.5,a)')&
&  ' sigma : performing a first self-consistency',ch10,&
&  '         update of the energies in G by a scissor operator',ch10, &
&  '         applying a scissor operator of [eV] ',ep%soenergy*Ha_eV,ch10
   call wrtout(06,message,'COLL') ; call wrtout(ab_out,message,'COLL')
  do is=1,sp%nsppol
   if (sr%nb>=nbv(is)+1) sr%egw(nbv(is)+1:sr%nb,:,is)= sr%egw(nbv(is)+1:sr%nb,:,is)+ep%soenergy
  end do
 else if (update_energies) then
  allocate(sr_gwenergy(sr%nb,sr%nk,sp%nsppol),stat=istat)
   if(istat/=0) stop 'out of memory sr_gwenergy'
  write(message,'(3a)')&
&  ' sigma : performing a first self-consistency',ch10,&
   '         update of the energies in G by a previous GW calculation'
  call wrtout(06,message,'COLL') ; call wrtout(ab_out,message,'COLL')
  call rdgw(nkibzr,sr%nb,nbv,sp%nsppol,kibz,sr_gwenergy)
  do ib=1,sr%nb
   do ik=1,sr%nk
    do is=1,sp%nsppol
     sr%egw(ib,ik,is)= sr_gwenergy(ik,ib,is)
    end do
   end do
  end do
  deallocate(sr_gwenergy)
 !FIXME here we should call fermi to recalculate the new fermi level
 end if

 ! In case of AC refer all the energies wrt to the fermi level
 ! Take care because results from ppmodel cannot be used for AC 
 ! FIXME check en or en_qp (in case of SCGW?)
 if (analytic_continuation) then 
  do ib=1,sr%nb
   do ik=1,sr%nk
    do is=1,sp%nsppol
     ! All these quantities will be passed to csigme
     en_qp (ik,ib,is)= en_qp (ik,ib,is)-efermi_qp
     sr%egw(ib,ik,is)= sr%egw(ib,ik,is)-efermi_qp
     sr%e0 (ib,ik,is)= sr%e0 (ib,ik,is)-efermi_qp
    end do
   end do
  end do
  ! if I skipped the self-consistent part then here I have to use fermi
  oldefermi= efermi_qp
  efermi=0
 end if 

 call write_sigma_results_header(sp,ep)

 if (me==0) then
  filnam= trim(dtfil%filnam_ds(4))//'_GW'
  open(21,file=filnam,status='unknown',form='formatted')
  write(21,*) sp%nkcalc

  filnam= trim(dtfil%filnam_ds(4))//'_SIG'
  open(22,file=filnam,status='unknown',form='formatted')

  filnam= trim(dtfil%filnam_ds(4))//'_SGR'
  open(23,file=filnam,status='unknown',form='formatted')

  if (analytic_continuation) then 
   filnam= trim(dtfil%filnam_ds(4))//'_SGM'
   !Open file for Matsubara axis 
   open(20,file=filnam,status='unknown',form='formatted')
  end if 
 end if 

 call timab(404,2,tsec)
 call timab(405,1,tsec)

 lt_k%sym_flag= dtset%symsigma

 ! Allocate arrays needed to perform symmetrization (if required)
 if (lt_k%sym_flag/=0) then
  write(*,*)' switching on symmetrization'
  lt_k%nop= nop ; lt_k%ninv= ninv
  lt_k%nkbz= sp%nqbz
  !npwvec or npwx or npwc?
  allocate(lt_k%ibzq(sp%nqbz),lt_k%ltq(2,nop),lt_k%wtksym(ninv,nop,sp%nqbz))
  allocate(lt_k%iumklp(npwvec,ninv,nop),lt_k%fumklp(ninv,nop))
  allocate(lt_k%tab(sp%nqbz),lt_k%tabi(sp%nqbz),lt_k%tabo(sp%nqbz),lt_k%ibzq_ind(sp%nkbz))
  lt_k%tab=0 ; lt_k%tabi=0 ; lt_k%tabo=0
 end if

 call pclock(110)

 ! Calculate self-energy and save on disc for each k-point
 !MG FIXME here we have what I call the stupid input file BUG (or stupid man BUG):
 !if one calculates the GW corrections in the same k-point twice the results
 !for the correlation part of sigma are different, since in the second
 !calculation the starting point is updated and is different from the LDA value
 !this can be useful to update the energy but I have to add a check in chkinp
 do ikcalc=1,sp%nkcalc
  write(message,'(2a,i5,a)')ch10,&
&  ' sigma : loop over k point, treating k point number : ',ikcalc,ch10
  call wrtout(6,message,'COLL')

  ikibz=ktab(sp%kcalc(ikcalc))

  ! In case of symmetrization, find little group  
  ! FIXME check if it is possible to have sp%npwx < sp%npwc
  if (lt_k%sym_flag/=0) then
   call setup_little_group (sp%xkcalc(:,ikcalc),gmet,nop,op,ninv,sp%nqbz,qbz,&
&   lt_k%ibzq,lt_k%ibzq_ind,lt_k%ltq,lt_k%tab,lt_k%tabo,lt_k%tabi,lt_k%wtksym,&
&   lt_k%iumklp,lt_k%fumklp,npwvec,gvec,ngfft1a,ngfft1,ngfft2,ngfft3,prtvol)
  end if

  if (mpi_enreg%gwpara==1) then
! Parallelization over k-points and spin
! It must be optimized since using symmetries the number of q-points depends on k
! althought it is impossible to share equally the load btw procs
! note nkbz is this a possible BUG?, in this way we optimize the load but...
! I dont like this approach, should rewrite everything
! since it could lead to BUGS if this distribution is used in other subroutines which 
! use the IBZ
   allocate(mpi_enreg%proc_distrb(sp%nkbz,sp%nb,sp%nsppol)) !this will be passed to chi0

   mpi_enreg%parareel= 0
   mpi_enreg%paralbd=  0
   mpi_enreg%paral_level= 2

   if (lt_k%sym_flag==0) then  !no symmetries, divide full BZ
    allocate(nband_t(sp%nkbz*sp%nsppol))
    nband_t(:)= sp%nb
    call distrb2(sp%nb,nband_t,sp%nkbz,sp%nsppol,mpi_enreg)

   else if (lt_k%sym_flag/=0) then  
    nqpt4sigk=sum(lt_k%ibzq(:))   !should be saved in setup_little_group
    allocate(nband_t(nqpt4sigk*sp%nsppol)) 
    nband_t(:)= sp%nb
 
    mpi_enreg_dummy= mpi_enreg !sure?, CHECKME here the order makes the difference
    allocate(mpi_enreg_dummy%proc_distrb(nqpt4sigk,sp%nb,sp%nsppol))

    call distrb2(sp%nb,nband_t,nqpt4sigk,sp%nsppol,mpi_enreg_dummy)
    mpi_enreg%proc_distrb=-999 !to debug

    ! Identify q and G0 where q+G0=k_j-k_i
    do iq=1,sum(lt_k%ibzq(:))
     do ik=1,sp%nkbz
      do is=1,sp%nsppol
       kgwmk(:)= sp%xkcalc(:,ikcalc)-kbz(:,ik) !warn check xkcalc, whether is inside the BZ or not
       call findqg0(iqm,g0,kgwmk,sp%nqbz,qbz) 
       if (lt_k%ibzq_ind(iqm)==iq)  mpi_enreg%proc_distrb(ik,:,is)=mpi_enreg_dummy%proc_distrb(iq,:,is)
      end do
     end do 
    end do

    deallocate(mpi_enreg_dummy%proc_distrb)

   end if 

   deallocate(nband_t)

   ! Announce the treatment of k-points by each proc
   do ik=1,sp%nkbz 
    do is=1,sp%nsppol
     if (any(mpi_enreg%proc_distrb(ik,:,is)/=mpi_enreg%proc_distrb(ik,1,is))) stop 'wrong distrb'
     if (mpi_enreg%proc_distrb(ik,sp%nb,is)/=-999) then 
      write(message,'(a,i4,a,i4,a,i4)')&
&      'P sigma : treating k-point ',ik,' and spin ',is,' by node ',mpi_enreg%proc_distrb(ik,sp%nb,is)
      call wrtout(6,message,'COLL')
      !call wrtout(ab_out,message,'PERS')
     end if
    end do 
   end do 

 end if ! mpi_enreg%gwpara==2 

!MG TODO here I should include Valerio recipe

  call csigme(dtfil,sp,ep,sp%kcalc(ikcalc),sp%minbnd(ikcalc),sp%maxbnd(ikcalc),qpoint,gvec,npwvec,op,nop,phsgt,&
&  kibz,ktab,ktabr,ktabi,ktabp,kbz,ngfft1,ngfft1a,ngfft2,ngfft3,igfft,nr,en_qp,occ_qp,qbz,qtab,qtabi,qtabo,&
&  qpg,ucvol,grottb,wfr,efermi_qp,er,sr,mpi_enreg,sp%ppmodel,b1,b2,b3,nbnds_per_proc,min_band_proc, &
&  max_band_proc,lt_k,nslice_qpt)

  ! Calculating direct gap for each spin
  do is=1,sp%nsppol
   if (sp%maxbnd(ikcalc)>=nbv(is)+1) then
    sr%egwgap (ikibz,is)=  sr%egw(nbv(is)+1,ikibz,is) -  sr%egw(nbv(is),ikibz,is)
    sr%degwgap(ikibz,is)= sr%degw(nbv(is)+1,ikibz,is) - sr%degw(nbv(is),ikibz,is)
   else
    ! The "gap" cannot be computed
    sr%e0gap  (ikibz,is)= zero
    sr%egwgap (ikibz,is)= zero
    sr%degwgap(ikibz,is)= zero
   end if
  end do
  
  if (mpi_enreg%gwpara==1) deallocate (mpi_enreg%proc_distrb)

  ! Print out final results
  if (me==0) then 
   call write_sigma_results(sp,sr,ikcalc,ikibz,en)
  end if

  call pclock(200+ikcalc)

 end do !ikcalc

 write(message,'(a)')' sigma : finished the k point loop'
 call wrtout(6,message,'COLL')

 if (sp%gwcalctyp>=10) then 
  call fermi(hdr,sp%nb,sp%nk,dtset%fixmom,sp%nsppol,wtk,sr%en_qp_diago,occ_qp,nel,nbv,efermi_qp)
  write(message,'(a,es16.6,a,es16.6,a)')&
&  ' New Fermi energy : ',efermi_qp,' Ha ,',efermi_qp*Ha_eV,' eV'
  call wrtout(6,message,'COLL') ; call wrtout(ab_out,message,'COLL')
 end if

 if (sp%ppmodel==3) deallocate(er%eigpot)
 deallocate(igfft,wfr,grottb,grottbm1,ktabr,qpg)

!MG FIXME here I should introduce the spin.
!to retain backwards compatibility I suggest
!to add the Abinit header in the _QPS file,
!in such a way the nsppol variable could be read from hdr%nsppol
!if the header exists, otherwise the file is in the old format

!problems in case of band parallelisation
!Only master should do this, restoring old version of wrqps. 
!before calling wrqps we should collect all the data we need from each proc
!ie en_qp, m_lda_to_qp and eigevec_qp and also rho_qp

 ! Write out scf data in case of self-consistent calculation
 if (sp%gwcalctyp>10 .and. me==0) then 
  call wrqps(dtfil,sp%gwcalctyp,sr%nk,sr%nb,sp%nsppol,&
&   kibz,nscf,nr,sr%en_qp_diago,sr%eigvec_qp,m_lda_to_qp,rho_qp)
  write(message,'(a)')' sigma : SCF data written '
  call wrtout(6,message,'COLL')
 end if 

 ! interpolation of sigma matrix elements
 ! call rdkss4interp(dtfil,sp,sr,npwvec,sp%npwwfn,kibz,kbz,nr,wfr_qp,ktab,ktabr,ktabi,&
 !& hdr,ngfft1,ngfft1a,ngfft2,ngfft2a,ngfft3,ngfft3a,mpi_enreg)

 call timab(405,2,tsec)

 ! Previously these files were closed in write_sigma_results
 ! TODO should rationalize the file names and the unit number 
 close(21)
 close(22)
 close(23)
 if (analytic_continuation) close(20) !Matsubara file

 deallocate(phsgt)
 deallocate(op)
 deallocate(en)
 deallocate(gvec)
 if (allocated(occ)) deallocate(occ) 
 deallocate(occ_qp)
 deallocate(qpoint)
 deallocate(kibz,wtk,ktab,ktabi,ktabo,ktabp,kbz)
 deallocate(qtab,qtabo,qtabi)
 if (allocated(m_lda_to_qp)) deallocate(m_lda_to_qp)
 deallocate(en_qp)
 deallocate(sr%sigcme,sr%sigxme)
 deallocate(sr%omegasrd)
 deallocate(sr%vxcme,sr%e0,sr%hhartree,sr%eigvec_qp,sr%en_qp_diago)
 deallocate(sr%sigcmee0,sr%dsigmee0,sr%sigcmesrd,sr%sigxcmesrd,sr%sigmee,sr%ze0)
 deallocate(sr%degw,sr%egw,sr%e0gap,sr%degwgap,sr%egwgap)
 deallocate(sp%kcalc,sp%xkcalc,sp%minbnd,sp%maxbnd)
 deallocate(nbv)
 deallocate(rho_qp)
 if (allocated(rho)) deallocate(rho)

 if (sp%nomegasr>0) deallocate(sr%omega)
 if (lt_k%sym_flag/=0) then
  deallocate(lt_k%ibzq,lt_k%ltq,lt_k%wtksym)
  deallocate(lt_k%tab,lt_k%tabi,lt_k%tabo,lt_k%ibzq_ind)
  deallocate(lt_k%iumklp,lt_k%fumklp)
 end if

 if (me==0 .or. localrdwf==1) then
  if (i2==502 .or. i2==602) then
   call hdr_clean(hdr)
  end if
 end if

 if (allocated(tnons  )) deallocate(tnons)

 if (associated(er%omega             )) deallocate(er%omega)
 if (associated(er%epsm1             )) deallocate(er%epsm1)
 if (associated(sr%sigxcme           )) deallocate(sr%sigxcme)
 if (associated(er%omegatw           )) deallocate(er%omegatw)
 if (associated(er%bigomegatwsq      )) deallocate(er%bigomegatwsq)
 if (associated(mpi_enreg%proc_distrb)) deallocate(mpi_enreg%proc_distrb)

 if (analytic_continuation) then 
! MG g95 problem : sr%sigxcmesi can be associated, even thou the array is not explicitly allocated
  if (associated(sr%omegasi))deallocate(sr%omegasi)
  if (associated(sr%sigcmesi))deallocate(sr%sigcmesi)
  !write(*,*)'sr%sigxcmesi is associated = ',associated(sr%sigxcmesi)
  if (associated(sr%sigxcmesi))deallocate(sr%sigxcmesi)
 end if 

 call timab(401,2,tsec)
 call pclock(9999)

 write(message,'(a)')' sigma : ended'
 call wrtout(06,message,'COLL')

 call timab(401,2,tsec)
 call pclock(9999)

!DEBUG
!write(6,*)' sigma : end '
!ENDDEBUG

 end subroutine sigma
!!***
