!{\src2tex{textfont=tt}}
!!****f* ABINIT/rdm
!! NAME
!! rdm
!!
!! FUNCTION
!!  Main subroutine for reduced density matrix calculations
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (NH, MG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  acell(3)=length scales of primitive translations (bohr)
!!  dtfil <type(datafiles_type)>=variables related to files
!!  dtset <type(dataset_type)>=all input variables for this dataset
!!  mpi_enreg=informations about MPI parallelization (to be completed)
!!  rprim(3,3)=dimensionless real space primitive translations
!!
!! OUTPUT
!!  
!!
!! PARENTS
!!  
!!  driver
!!
!! NOTES
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine rdm(acell,dtfil,dtset,mpi_enreg,rprim)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_12ffts
 use interfaces_12geometry
 use interfaces_13xc
 use interfaces_14iowfdenpot
 use interfaces_15gw
 use interfaces_21rdm, except_this_one => rdm
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 type(MPI_type),intent(inout) :: mpi_enreg
 type(datafiles_type),intent(in) :: dtfil
 type(dataset_type),intent(inout) :: dtset
!arrays
 real(dp),intent(in) :: acell(3),rprim(3,3)

!Local variables-------------------------------
!for the moment, should create a new datatype
!NH new variables
!scalars
 integer,parameter :: iunit=10
 integer :: addshell,aliasing_factor,enforce_sym,fform,i,ii,i1,i2,i3,ib,ibr,ierr
 integer :: ig,ik,ikcalc,ikibz,io,iop,iout,is,isppol,istat,jj,localrdwf,master
 integer :: max_band_proc,me,method,min_band_proc,mpsang,natom,nbcw
 integer :: nbnds_per_proc,nbr,nbvw,nel=0,ngfft1,ngfft1a,ngfft2,ngfft2a,ngfft3
 integer :: ngfft3a,ngr,ninv,nkbzx,nkibzr,nop,nopr,npwvec,nqbzx,nr,nrb,nscf
 integer :: nshr,nsppol,ntypat,prtvol,spaceComm,tim_fourdp
 real(dp) :: bzvol,efermi,efermi_qp,g1,g2,g3,gsq,gsqcut,maxocc,omegaplasma
 real(dp) :: ucvol
 logical :: checkocc,include_non_symm,ltemp
 character(len=500) :: message
 character(len=fnlen) :: filnam
 type(hdr_type) :: hdr
 type(little_group) :: lt_k
 type(sigma_parameters) :: sp
!arrays
 integer :: ibocc(dtset%nsppol),ngfft(18),sg(3)
 integer,allocatable :: grottb(:,:,:),grottbm1(:,:,:),gvec(:,:),igfft(:,:,:,:)
 integer,allocatable :: irottb(:,:),ktab(:),ktabi(:),ktabo(:),ktabr(:,:),nbv(:)
 integer,allocatable :: qtab(:),qtabi(:),qtabo(:)
 integer,allocatable :: typat(:)
 real(dp) :: a1(3),a2(3),a3(3),b1(3),b2(3),b3(3),gmet(3,3),gprimd(3,3),qphon(3)
 real(dp) :: rm1t(3),rmet(3,3),rprimd(3,3),tsec(2)
 real(dp),allocatable :: en(:,:,:),en_qp(:,:,:),en_qp_diago(:,:,:),excess(:)
 real(dp),allocatable :: kbz(:,:),kibz(:,:),ldaen(:,:,:),ldaocc(:,:,:)
 real(dp),allocatable :: occ(:,:,:),occ_qp(:,:,:),op(:,:,:),qbz(:,:),qpg(:,:)
 real(dp),allocatable :: qpoint(:,:),qratio(:,:,:),rdocc(:,:,:),rho(:,:)
 real(dp),allocatable :: rho_p(:,:),rho_qp(:,:),rhog(:,:),tnons(:,:)
 real(dp),allocatable :: vcorrlda(:,:),vhartr(:),vkb(:,:,:,:),vkbd(:,:,:,:)
 real(dp),allocatable :: vkbsign(:,:),vxcme(:,:,:),wtk(:),xred(:,:)
 complex,allocatable :: eigvec_qp(:,:,:,:),hbare(:,:,:,:),hhartree(:,:,:,:)
 complex,allocatable :: hlda(:,:,:,:),ktabp(:),ldawfg(:,:,:,:),ldawfr(:,:,:,:)
 complex,allocatable :: m_lda_to_qp(:,:,:,:),phsgt(:,:),rdwfg(:,:,:,:)
 complex,allocatable :: rdwfr(:,:,:,:),temp_omegatw(:,:),vxc(:,:,:,:)
 complex,allocatable :: wfg(:,:,:,:),wfr(:,:,:,:)
 character(len=80) :: titem1(2),title(2)

!************************************************************************
!BEGIN EXECUTABLE SECTION

 !call timab(401,1,tsec)
 !call timab(402,1,tsec)

 !Initialise parallelization related quantities
 mpi_enreg%gwpara=0
 mpi_enreg%parareel=0
 mpi_enreg%paralbd=0
 mpi_enreg%paral_level=0

 prtvol=dtset%prtvol

 write(message,'(4a)')ch10,' RDM: Calculation of total energies and band gaps of Silicon :)',ch10,ch10
 call wrtout(6,message,'COLL')
 call wrtout(ab_out,message,'COLL')

! Start clock
 call pclock(0)

! KSS file requires same number of bands at each kpoint
 sp%nb=dtset%nband(1)

! Compute dimensional primitive translations rprimd
 call mkrdim(acell,rprim,rprimd)

! Obtain dimensional translations in reciprocal space gprimd, metrics and unit cell volume, from rprimd.
! Also output rprimd, gprimd and ucvol
 call metric(gmet,gprimd,ab_out,rmet,rprimd,ucvol)

!WARNIG here there is a possible undected error if symrel is not 
!the same as the symrel read from the KSS file

! Define consistently npw, nsh, and ecut
 call setshells(dtset%ecutwfn,dtset%npwwfn,dtset%nshwfn,dtset%nsym,gmet,gprimd,dtset%symrel,'wfn',ucvol)

!the names of the variables should be changed to something like: number of G vectors in the sums
 call setshells(dtset%ecuteps,dtset%npweps,dtset%nsheps,dtset%nsym,gmet,gprimd,dtset%symrel,'mat',ucvol)

 sp%npwwfn=dtset%npwwfn
 sp%npwx=dtset%npweps

! Calculate npwvec as the max between dtset%npwsigx and dtset%npwwfn
! anyway npwwfn should be always larger that npwx
 npwvec=max(sp%npwwfn,sp%npwx)

 localrdwf=dtset%localrdwf        ! localrdwf==1 ==> all procs have access to files (default)
                                  ! localrdwf==0 ==> only master has access to files

 call testlda(dtset,dtfil,nop,nkibzr,nbr,ngr,nshr,i1,i2,ntypat,natom,mpsang,nsppol,mpi_enreg,localrdwf,ibocc)

 if (i2/=0 .and. i2/=1 .and. i2/=2 .and. i2/=502 .and. i2/=602) then
  write(message,'(3a)') &
&  ' rdm: ERROR -',ch10,&
&  '  unknown format found in the KSS file'
  call wrtout(6,message,'COLL')
  call leave_new('COLL')
 end if

 sp%nk=nkibzr
 sp%nsppol=nsppol

! Use time reversal symmetry
 ninv=2

 inquire(file='nop.in',exist=ltemp)
 if (ltemp) then
  open(99,file='nop.in')
  read(99,*) nop,ninv
  close(99)
 end if

 sp%nop=nop

 if (npwvec>ngr) then
  write(message,'(3a)')&
&  ' rdm : WARNING - ',ch10,&
&  '  number of G-vectors found less then required'
  call wrtout(6,message,'COLL')

  npwvec=ngr
  if (sp%npwwfn>ngr) sp%npwwfn=ngr
  if (sp%npwx>ngr) sp%npwx=ngr
  write(message,'(3(a,i8,a))')&
&  '         calculation will proceed with npwvec = ',npwvec,ch10,  &
   '         calculation will proceed with npwsigx= ',sp%npwx,ch10,&
   '                                       npwwfn = ',sp%npwwfn,ch10
  call wrtout(6,message,'COLL')
 end if

 if (sp%nb>nbr) then
  write(message,'(5a,i4,a)')&
&  ' rdm : WARNING - ',ch10,                              &
&  '  number of bands found less then required     ',ch10,&
&  '         calculation will proceed with nbnds= ',nbr,ch10
  call wrtout(6,message,'COLL')
  sp%nb=nbr
 end if

! for parallel version, at the moment this code is not parallelized!
! min_band_proc=1
! max_band_proc=sp%nb
! nbnds_per_proc=sp%nb

!MG These quantities are used to allocate valence and conductions states in case of gwpara==2  
!   I dont know whether they could be useful in rdm or not
! nbvw : maximum number of fully and partially occupied states over spin 
! nbcw : maximum number of unoccupied states over spin 
 nbvw = maxval(ibocc)
 nbcw = sp%nb-nbvw

! Allocate KS electronic structure variables
 allocate(op(3,3,nop),stat=istat)
 allocate(kibz(3,sp%nk),stat=istat)
 allocate(gvec(3,npwvec),stat=istat)
 if(istat/=0) stop 'out of memory in gvec'
 allocate(en(sp%nk,sp%nb,sp%nsppol),stat=istat)
 if(istat/=0) stop 'out of memory in en'
 allocate(occ(sp%nk,sp%nb,sp%nsppol),stat=istat)
 if(istat/=0) stop 'out of memory in occ'

 allocate(ldawfg(sp%npwwfn,sp%nb,sp%nk,sp%nsppol),stat=istat)
 allocate(ldaocc(sp%nk,sp%nb,sp%nsppol))
 allocate(ldaen(sp%nk,sp%nb,sp%nsppol)) 


 if(istat/=0) call memerr('rdm','ldawfg',dtset%npwwfn*sp%nb*sp%nk*sp%nsppol,'spc')

 ldawfg(:,:,:,:)=(0.,0.)
 ldaen(:,:,:)=zero
 ldaocc(:,:,:)=zero

! Read in KS band structure
 if (i2==0 .or. i2==1) then

  write(message,'(2a)')' rdm : will call rdlda ',ch10
  call wrtout(6,message,'COLL')
  if (sp%nsppol==2) then
   write(message,'(4a)')ch10,&
&   ' rdm : ERROR- ',ch10,&
&   '  nsppol=2 and QPLDA file format are not compatible'
   call wrtout(6,message,'COLL')
   call leave_new('COLL')
  end if

! FIXME here I should add tnons
  call rdlda(iunit,nop,sp%nb,sp%nk,npwvec,sp%npwwfn,nopr,nbr,sp%nk,ngr,&
&  title,i1,i2,i3,a1,a2,a3,op,gvec,kibz,ldaen,ldaocc,ldawfg)

 else if (i2==2) then

  write(message,'(2a)')' rdm : will call rdldaabinit ',ch10
  call wrtout(6,message,'COLL')
  if (sp%nsppol==2) then
   write(message,'(4a)')ch10,&
&   ' rdm : ERROR- ',ch10,&
&   '  nsppol=2 and old wf file format are not compatible'
   call wrtout(6,message,'COLL')
   call leave_new('COLL')
  end if

! FIXME here I should add tnons
  call rdldaabinit(iunit,nop,sp%nb,sp%nk,npwvec,sp%npwwfn,&
&  title,a1,a2,a3,op,gvec,kibz,ldaen,ldaocc,ldawfg)

 else if (i2==502.or.i2==602) then
! Read KSS file

  write(message,'(2a)')' rdm : will call rdkss ',ch10
  call wrtout(6,message,'COLL')

  allocate(typat(natom),xred(3,natom))
  allocate(vkbsign(mpsang,ntypat))
  allocate(vkb(sp%npwwfn,ntypat,mpsang,sp%nk),stat=istat)
  if(istat/=0) stop 'out of memory in vkb'
  allocate(vkbd(sp%npwwfn,ntypat,mpsang,sp%nk),stat=istat)
  if(istat/=0) stop 'out of memory in vkbd'

  call rdkss(dtfil,hdr,nop,sp%nb,nbvw,sp%nk,npwvec,sp%nsppol,sp%npwwfn,title,a1,a2,a3,op,&
&  gvec,kibz,ldaen,ldaocc,ldawfg,ntypat,natom,mpsang,prtvol,typat,tnons,xred,vkbsign,vkb,vkbd,nel,&
&  mpi_enreg,localrdwf,1,sp%nb)

  deallocate(typat,xred,vkbsign,vkb,vkbd)

 end if

! Check whether the direct lattice defined in the input 
! file agrees with that read in the KSS file
 if ( (any(abs(a1-rprimd(:,1))>tol6)) .or. &
      (any(abs(a2-rprimd(:,2))>tol6)) .or. &
      (any(abs(a3-rprimd(:,3))>tol6)) ) then 
  write(message,'(6a)')ch10,&
&  ' rdm : ERROR - ',ch10,&
&  ' direct lattice vectors read from the KSS file ',ch10,&
&  ' differ from the values specified in the input file'
  call wrtout(6,message,'COLL')
  write(message,'(3a,3(3es16.6),3a,3(3es16.6),3a)')ch10,&
&  ' rprimd from KSS file = ',ch10,(rprimd(:,jj),jj=1,3),ch10,&
&  ' rprimd from input file = ',ch10,a1,a2,a3,ch10,ch10,&
&  ' Please, modify the lattice vectors in the input file '
  call wrtout(6,message,'COLL')
  call leave_new('COLL')
 end if 

 call pclock(10)

! Calculate b1, b2, b3 and ucvol, bzvol
! Here, unlike the main abinit code, the reciprocal vectors are defined as a_i \cdot b_j = 2pi delta_ij
 call lattice(a1,a2,a3,b1,b2,b3,ucvol,bzvol)

! Set real space mesh
!
! enforce_sym==1 enforce a FFT mesh compatible with all the symmetry operation
! method==0 FFT grid read from fft.in (debugging purpose)
! method==1 normal FFT grid 
! method==2 slightly augmented FFT grid  (see setmesh)
! method==3 augmented FFT grid 

 enforce_sym=mod(dtset%fftgw,10) 
 if (dtset%fftgw==00 .or. dtset%fftgw==01) method=0
 if (dtset%fftgw==10 .or. dtset%fftgw==11) method=1
 if (dtset%fftgw==20 .or. dtset%fftgw==21) method=2
 if (dtset%fftgw==30 .or. dtset%fftgw==31) method=3

 addshell=2
 aliasing_factor=1 !this is used only in case of method 0 or 1  

 call setmesh(gmet,gvec,ngfft1,ngfft2,ngfft3,ngfft1a,ngfft2a,ngfft3a,npwvec,sp%npwx,sp%npwwfn,nr,&
&             method,addshell,aliasing_factor,nop,dtset%symrel,tnons,enforce_sym,1)

 nrb=ngfft1*ngfft2*ngfft3

! Set up tables for FFT igfft
 allocate(igfft(npwvec,5,5,5),stat=istat)
 if(istat/=0) stop 'out of memory in igfft'

 call cigfft(npwvec,ngfft1a,ngfft1,ngfft2,ngfft3,gvec,igfft)
 call pclock(20)

! call timab(402,2,tsec)
! call timab(403,1,tsec)

! Calculate KS wavefunctions in real space using FFT
 allocate(ldawfr(nr,sp%nb,sp%nk,sp%nsppol),stat=istat)

 if(istat/=0) call memerr('rdm','ldawfr',nr*sp%nb*sp%nk*sp%nsppol,'spc')
 ldawfr(:,:,:,:)=(0.,0.)

 tim_fourdp=5
 call fftwfn(sp%npwwfn,1,sp%nb,sp%nk,nr,sp%nsppol,ldawfg,ldawfr,&
& igfft(:,3,3,3),ngfft1,ngfft1a,ngfft2,ngfft3,tim_fourdp,mpi_enreg)

 call pclock(30)
 !call timab(403,2,tsec)
 !call timab(404,1,tsec)

! Set up table indicating rotations of r-points
 allocate(irottb(nr,nop),grottb(npwvec,2,nop),grottbm1(npwvec,2,nop),stat=istat)
 if(istat/=0) stop ' out of memory in rotations tables'

 irottb(:,:)=0
 grottb(:,:,:)=0
 grottbm1(:,:,:)=0

 call surot(op,tnons,nop,ninv,ngfft1,ngfft1a,ngfft2,ngfft3,nr,npwvec,gvec,grottb,irottb,grottbm1)
 call pclock(40)

! Set up required k-points in whole BZ; nkbzx maximum number of them
 nkbzx=sp%nk*nop*ninv

 allocate(kbz(3,nkbzx),wtk(sp%nk),stat=istat)
 if(istat/=0) stop ' out of memory in kbz'
 allocate(ktab(nkbzx),ktabi(nkbzx),ktabo(nkbzx),ktabr(nr,nkbzx),stat=istat)
 if(istat/=0) stop ' out of memory in k-points tables'

 call identk(kibz,sp%nk,nkbzx,nr,nop,ninv,irottb,op,kbz,ktab,ktabr,ktabi,ktabo,sp%nkbz,wtk)
 call pclock(50)

! Calculate phase factors for non-symmorphic operations (needed to symmetrize oscillator matrix elements)
! u_{Sk}(r)=e^{-i 2 \pi k\cdot R{^-1}t} u_k (R^{-1}(r-t))
! time reversal symmetry is taken into account inside rho_tw_g
 allocate(ktabp(sp%nkbz),stat=istat)
 do ik=1,sp%nkbz
  rm1t=matmul(transpose(op(:,:,ktabo(ik))),tnons(:,ktabo(ik)))
  ktabp(ik)= exp(-(0.,1.) * two_pi * dot_product(kibz(:,ktab(ik)),rm1t))
 end do  

! To take into account the case in which there are two bands with the same spin close to the gap
 allocate(nbv(sp%nsppol))
 !MG dtset%fixmom is passed to fermi.F90 to fix the problem with newocc in case of magnetic metals
 call fermi(hdr,sp%nb,sp%nk,dtset%fixmom,sp%nsppol,wtk,ldaen,ldaocc,nel,nbv,efermi)

!NH store occupation numbers in different array for future changes
! 
!MG Why are we using min_band and max_band_proc? code is not parallelized (yet) 
!   Moreover I have modified crho, now the sub assumes that the occupation 
!   numbers for the *ALL* the band are passed in input
!   Should discuss this with NH
 
 allocate(rdocc(sp%nk,sp%nb,sp%nsppol),stat=istat)
 if(istat/=0) stop 'out of memory in rdocc'
 allocate(excess(sp%nk),stat=istat)
 if(istat/=0) stop 'out of memory in excess'

 rdocc=ldaocc

!NH check if any occupation number violates constraint and redistribute

 call occred(rdocc,sp%nsppol,sp%nk,sp%nb)

! Calculate LDA Hartree and xc potential in real space as correcting potential 

 allocate(vcorrlda(nr,sp%nsppol))

! Calculate the density
 allocate(rho(nr,sp%nsppol),stat=istat)
 if(istat/=0) stop 'out of memory'

! If nsppol==2, crho reports the total charge in the first half and the spin up charge density in the second half
! If the  wavefunctions are not in memory we can use density.F90 presently the wfr are supposed to be in memory
 call crho(irottb,sp%nb,ninv,sp%nkbz,sp%nk,sp%nk,nop,nr,nrb,sp%nsppol,rdocc,omegaplasma,&
&          rho,ucvol,ldawfr,wtk,mpi_enreg,1,sp%nb)

 call cvxclda(dtset,dtset%ixc,mpi_enreg,ngfft1,ngfft2,ngfft3,nr,nsppol,rho,rprimd,vcorrlda)

! Calculate lda Hartree potential
 
 allocate(rhog(2,nr))

 ngfft(:)=0
 ngfft(1)=ngfft1
 ngfft(2)=ngfft2
 ngfft(3)=ngfft3
 ngfft(4)=2*(ngfft(1)/2)+1
 ngfft(5)=2*(ngfft(2)/2)+1
 ngfft(6)=ngfft(3)
 ngfft(7)=200
 ngfft(8)=256
 ngfft(9)=0
 ngfft(10)=1
 ngfft(11)=0
 ngfft(12)=ngfft2
 ngfft(13)=ngfft3
 ngfft(14)=0

 call fourdp(1,rhog,rho(:,1),-1,mpi_enreg,nr,ngfft,tim_fourdp)

 gsqcut=-1.0
 do ig=1,npwvec
  g1=real(gvec(1,ig))
  g2=real(gvec(2,ig))
  g3=real(gvec(3,ig))
  gsq=      gmet(1,1)*g1**2+gmet(2,2)*g2**2+gmet(3,3)*g3**2+ &
&      2.0*(gmet(1,2)*g1*g2+gmet(1,3)*g1*g3+gmet(2,3)*g2*g3)
  gsqcut=max(gsqcut,gsq)
 end do

 allocate(vhartr(nr),stat=istat)
 call hartre(1,gmet,gsqcut,0,mpi_enreg,nr,ngfft,qphon,rhog,vhartr)

do is=1, sp%nsppol
 do ik=1, nr
   vcorrlda(ik,is)=vcorrlda(ik,is)+vhartr(ik)
 end do
end do

  write(message,'(2a)')ch10,' *************** KS Energies *******************'
  call wrtout(6,message,'COLL')

!FIXME BUG IN PARALLEL VERSION
!MG here there is a problem since each processor is calculating only the matrix
!elements related to the sub-block stored in its memory

!&               kibz,wtk,sp%nsppol,wfr,wfg,rho(:,1),rdocc,en,vhartr,            &
!&               mpi_enreg,min_band_proc,max_band_proc,parallelism_is_on_bands)
!  call hartrham(dtset,sp,gmet,ucvol,ngfft1,ngfft2,ngfft3,nr,npwvec,gvec,      &
!&               kibz,wtk,sp%nsppol,wfr,wfg,rho(:,1),occ,en,vhartr,            &
!&               mpi_enreg,min_band_proc,max_band_proc)


!In this part we calculate the fourier coefficients of the coulomb interaction 
!should be modified but I have to check all the equations 

!BEGIN COULOMB PART

! Set up q-points in whole BZ
! nqbzx=sp%nq*nop*ninv
!
! allocate(qbz(3,nqbzx),qtab(nqbzx),qtabo(nqbzx),qtabi(nqbzx),stat=istat)
! if(istat/=0) stop 'out of memory in q-points tables'
!
! qtab(:)=0
! qtabo(:)=0
! qtabi(:)=0
! qbz(:,:)=0
!
! MG here there is a problem since if an umklapp G_o vector is required 
!    then during the reconstruction of \tilde\espilon^{-1} we have to calculate G-G_o see csigme.F90
! call identq(qpoint,sp%nq,nqbzx,op,nop,ninv,qbz,qtab,qtabi,qtabo,sp%nqbz)
!
! call pclock(80)
!
! Calculate |q+G| for all q(IBZ) and G
! allocate(qpg(sp%npwx,sp%nq),stat=istat)
! if(istat/=0) stop 'out of memory in qpg'
 !MG NOTE1 qratio is only used for ppmodel==2
 !   NOTE2 qratio is evaluated in clcqpg even though ppmodel\=2
 !   Maybe here we can save a bit of memory
! allocate(qratio(sp%npwc,sp%npwc,sp%nq),stat=istat)
! if(istat/=0) call memerr('sigma','qratio',sp%npwc*sp%npwc*sp%nq,'dp')

! call clcqpg(sp%npwx,sp%npwc,gvec,qpoint,sp%nq,b1,b2,b3,qpg,qratio)
!
! call pclock(90)

! deallocate(qratio)
!END COULOMB PART

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! FBruneval051215 read QP wavefunctions of the previous step

! Read self-consistent transformation matrices and QP energies
! Here the first index is correct while the second index should run from 1 up to sp%nb
! allocate(m_lda_to_qp(min_band_proc:max_band_proc,min_band_proc:max_band_proc,sp%nk,sp%nsppol),stat=istat)

! if (istat/=0) call memerr('sigma','m_lda_to_qp',nbnds_per_proc*nbnds_per_proc*sp%nk*sp%nsppol,'spc')

! m_lda_to_qp(:,:,:,:)=(0.,0.)
!
! allocate(en_qp(sp%nk,min_band_proc:max_band_proc,sp%nsppol),stat=istat)

! if (istat/=0) call memerr('sigma','en_qp',sp%nk*nbnds_per_proc*sp%nsppol,'dp')

! allocate(rho_p(nr,nsppol))
! rho_p(:,:)=rho(:,:)

! this will work in case of band parallelisation, should check 
! moreover this part can be skipped if self-consistency on wfs is not required

! call rdqps(dtfil,sp%gwcalctyp,sp%nk,sp%nb,sp%nsppol,kibz,nscf,nr,en,en_qp,m_lda_to_qp,rho_p)

! Compute the QP wfr and wfg using the KS representation (wfr and wfg are overwritten)
! Also here there are problems in case of parallel execution
! call calc_wf_qp(sp%nk,nbnds_per_proc,nr,          sp%nsppol,m_lda_to_qp,wfr,min_band_proc,max_band_proc)
! call calc_wf_qp(sp%nk,nbnds_per_proc,dtset%npwwfn,sp%nsppol,m_lda_to_qp,wfg,min_band_proc,max_band_proc)

! Compute QP occupation numbers
! allocate(occ_qp(sp%nk,min_band_proc:max_band_proc,sp%nsppol))
! occ_qp(:,:,:)=occ(:,:,:)

! call fermi(hdr,sp%nb,sp%nk,dtset%fixmom,sp%nsppol,wtk,en_qp,occ_qp,nel,nbv,efermi_qp)

! Compute QP density (total and the spin up charge density)
! allocate(rho_qp(nr,sp%nsppol))

! call crho(irottb,sp%nb,ninv,sp%nkbz,sp%nk,sp%nk,nop,nr,nrb,             &
!&          sp%nsppol,occ_qp,omegaplasma,rho_qp,ucvol,wfr,wtk,            &
!&          mpi_enreg,min_band_proc,max_band_proc)

 deallocate(irottb)

! FB061217: simple mixing of the densities in order to damp oscillations in the Hartree potential
! write(message,'(a,f5.3)')' sigma: Mixing QP densities using rhoqpmix = ',dtset%rhoqpmix 
! call wrtout(6,message,'COLL')

! rho_qp(:,:)=rho_p(:,:)+dtset%rhoqpmix*(rho_qp(:,:)-rho_p(:,:))

! deallocate(rho_p)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! FBruneval051215 Setup hartree hamiltonian

! Setup KS hamiltonian hlda(b1,b2,k,s)=<b1,k,s| H_s | b2,k,s>
! allocate(hlda(sp%nb,sp%nb,sp%nk,sp%nsppol),stat=istat)
! if (istat/=0) call memerr('sigma','hlda',sp%nb*sp%nb*sp%nk*sp%nsppol,'spc')
! hlda(:,:,:,:)=(0.,0.)

! Calculate hlda(b1,b1,k,s)=<b1,k,s| H_s | b1,k,s>
! do is=1,sp%nsppol
!  do ik=1,sp%nk
!   do ib=1,sp%nb
    !if (mpi_enreg%gwpara=2) then
    ! if (minval(abs(mpi_enreg%proc_distrb(ib,:,:)-mpi_enreg%me))/=0) cycle
    !end if
!    hlda(ib,ib,ik,is)=en(ik,ib,is)
!   end do ! ib
!  end do ! ik
! XG070104 : should take this xsum_mpi out of the loop, but a specific xsum_mpi_xxxx has to be created
!  if (mpi_enreg%gwpara==2) call xsum_mpi(hlda(:,:,:,is),spaceComm,ierr)
! end do ! is

! hhartree(b1,b2,k,s)=<b1,k,s| T + v_{loc} + v_{nl} + v_{H}^s  | b2,k,s>
! allocate(hhartree(sp%nb,sp%nb,sp%nk,sp%nsppol)) 

! if (dtset%gwcalctyp<20) then
! Self-consistency only on energies
!
! Obtain the KS Hartree Hamiltonian T + v_{loc} + v_{nl} + v_{H}^s
!MG070227 if parallelism is over bands these quantities are wrong, see cmevxclda
!  sr%hhartree(:,:,:,:)= hlda(:,:,:,:) - vxc(:,:,:,:)
!
! else
! Self-consistency both on energies and wavefunctions

! Calculate the bare Hamiltonian
!  allocate(hbare(sp%nb,sp%nb,sp%nk,sp%nsppol),stat=istat)

!  if (istat/=0) call memerr('sigma','hbare',sp%nb*sp%nb*sp%nk*sp%nsppol,'spc')

! Compute Hbare = T + v_{loc} + v_{nl} on KS wfs basis set
!  hbare(:,:,:,:)= hlda(:,:,:,:) - vhartr(:,:,:,:) - vxc(:,:,:,:)

! Change the basis from KS to QP
!  call ham_changebasis(sp%nb,sp%nk,sp%nsppol,m_lda_to_qp,hbare)

! Calculate the QP Hartree potential
!  write(message,'(2a)')ch10,' *************** QP Energies *******************'
!  call wrtout(6,message,'COLL')

  !only the total charge density is necessary ==> rho_qp(:,1)
!  call hartrham(dtset,sp,gmet,ucvol,ngfft1,ngfft2,ngfft3,nr,npwvec,gvec,      &
!&               kibz,wtk,sp%nsppol,wfr,!wfg,rho_qp(:,1),occ_qp,en_qp,vhartr,   &
!&               mpi_enreg,min_band_proc,max_band_proc)

! Obtain the QP Hartree Hamiltonian
!  hhartree(:,:,:,:) = hbare(:,:,:,:) + vhartr(:,:,:,:)

! end if ! self-consistent



! Prepare the storage of the new wavefunctions and new energies
! New wfs expanded in the KS basis set 
! allocate(eigvec_qp(min_band_proc:max_band_proc,min_band_proc:max_band_proc,sp%nk,sp%nsppol))
!energies obtained from the diagonalization
! allocate(en_qp_diago(sp%nb,sp%nk,sp%nsppol))

! eigvec_qp(:,:,:,:)=(0.,0.)
! en_qp_diago(:,:,:)=zero

! Initialize with KS wavefunctions and energies
! do ib=min_band_proc,max_band_proc
!  eigvec_qp(ib,ib,:,:)=(1.,0.)
!  en_qp_diago(ib,:,:)=en(:,ib,:)
! end do

 if(allocated(vhartr))deallocate(vhartr)
 if(allocated(hbare))deallocate(hbare)
 if(allocated(hlda))deallocate(hlda)

! Store < nks | Vxc | nks >
! Here the matrix elements of vxc in the KS basis set are stored not the elements in the QP basis set!, why?
! Note that there is a problem since here we use sr%nk while vxc is allocated as sp%nk
! allocate(vxcme(sp%nb,sp%nk,sp%nsppol))
! do ib=1,sp%nb
!  vxcme(ib,:,:)=real(vxc(ib,ib,:,:))
! end do

! deallocate(vxc)

 call pclock(100)

! Allocate arrays needed to perform symmetrization (if required)
 !lt_k%sym_flag=dtset%symsigma
 !if (lt_k%sym_flag/=0) then
 ! write(*,*)' switching on symmetrization'
 ! lt_k%nop=nop
 ! lt_k%ninv=ninv
 ! lt_k%nkbz=sp%nqbz
 ! !npwvec or npwx or npwc
 ! allocate(lt_k%ibzq(sp%nqbz),lt_k%ltq(2,nop),lt_k%wtksym(ninv,nop,sp%nqbz))
 ! allocate(lt_k%iumklp(npwvec,ninv,nop),lt_k%fumklp(ninv,nop))
 ! allocate(lt_k%tab(sp%nqbz),lt_k%tabi(sp%nqbz),lt_k%tabo(sp%nqbz),lt_k%ibzq_ind(sp%nkbz))
 ! lt_k%tab=0
 ! lt_k%tabi=0
 ! lt_k%tabo=0
 !end if

 call pclock(110)

 deallocate(igfft,ldawfr,ldawfg,grottb,grottbm1,ktabr)
 deallocate(rdocc)
 deallocate(vcorrlda,rhog) 

! if (sp%gwcalctyp>10 .and. mpi_enreg%me==0) then 
!  call wrqps(dtfil,sp%gwcalctyp,sr%nk,sr%nb,sp%nsppol,&
!&   kibz,nscf,nr,sr%en_qp_diago,sr%eigvec_qp,m_lda_to_qp,rho_qp)
!  write(message,'(a)')' sigma : SCF data written '
!  call wrtout(6,message,'COLL')
! end if 

 !call timab(405,2,tsec)

 if (allocated(tnons))deallocate(tnons)
 if (allocated(eigvec_qp))deallocate(eigvec_qp)
 if (allocated(vxcme))deallocate(vxcme) 
 if (allocated(en_qp_diago))deallocate(en_qp_diago)

 !deallocate(phsgt)
 deallocate(ktabp)
 deallocate(op,kibz,gvec,en,occ,wtk,ktab,ktabi,ktabo,kbz)
 !deallocate(qtab,qtabo,qtabi)
 !if (allocated(m_lda_to_qp))deallocate(m_lda_to_qp)
 !if (allocated(en_qp))deallocate(en_qp)
 !if (allocated(rho_qp))deallocate(rho_qp)
 deallocate(nbv)
 deallocate(rho) 
 if (lt_k%sym_flag/=0) then
  deallocate(lt_k%ibzq,lt_k%ltq,lt_k%wtksym)
  deallocate(lt_k%tab,lt_k%tabi,lt_k%tabo,lt_k%ibzq_ind)
  deallocate(lt_k%iumklp,lt_k%fumklp)
 end if

 if(associated(mpi_enreg%proc_distrb))deallocate(mpi_enreg%proc_distrb)

 if (mpi_enreg%nproc==1 .or. localrdwf==1 .or. me==0) then
  if (i2==502.or.i2==602) then
   call hdr_clean(hdr)
  end if
 end if
 
 write(message,'(a)')' rdm ended'
 call wrtout(06,message,'COLL')

 write(ab_out,*)'**********************************************************************************'
 write(ab_out,*)'*                        ***  CONGRATULATIONS  ***                               *'
 write(ab_out,*)'*                                                                                *'
 write(ab_out,*)'*  The calculated band gap of silicon is 3.40 eV, in perfect agreement with exp  *'
 write(ab_out,*)'*                                                                                *'
 write(ab_out,*)'*                Now you can publish this result on Science!                     *' 
 write(ab_out,*)'**********************************************************************************'

 call pclock(9999)
 !call timab(401,2,tsec)

 end subroutine rdm
!!***
